/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import java.math.BigDecimal;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;
import com.prismtech.idt.idlmm.ConstantDef;
import com.prismtech.idt.idlmm.FixedDef;


/**
 * Only fixed point values can be assigned to fixed point type constants.
 * 
 * @see <a
 *      href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constrain 31 This constarin has been disabled after chancing model.
 */
public class CheckFixedPointConstant extends IdlmmConstraint
{
   @Override
   public IStatus validate (IValidationContext ctx)
   {
      ConstantDef target = (ConstantDef) ctx.getTarget ();
      if (target.getType () instanceof FixedDef)
      {
         if (target.getValue () instanceof BigDecimal)
         {
            return ctx.createSuccessStatus ();
         }
         else
         {
            return ctx.createFailureStatus (new Object[]
            {
               describe (target)
            });
         }
      }
      return ctx.createSuccessStatus ();
   }
}
