/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import java.math.BigDecimal;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;
import com.prismtech.idt.idlmm.ConstantDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.PrimitiveDef;
import com.prismtech.idt.idlmm.PrimitiveKind;


/**
 * Only floating point values can be assigned to floating point type (float,
 * double, long double) constants.
 * 
 * @see <a
 *      href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constraint 28 This constarin has been disabled after chancing model.
 */
public class CheckFloatingPointConstant extends IdlmmConstraint
{

   @Override
   public IStatus validate (IValidationContext ctx)
   {
      ConstantDef target = (ConstantDef) ctx.getTarget ();
      IDLType type = target.getType ();
      String messages[] =
      {
         "Only floating point values can be assigned to floating point type (float, double, long double) constants." //$NON-NLS-1$
      };

      if (type instanceof PrimitiveDef)
      {
         PrimitiveDef primitive = (PrimitiveDef) type;

         switch (primitive.getKind ())
         {
            case PK_DOUBLE:
            case PK_FLOAT:
            case PK_LONGDOUBLE:

               // Only floating point values can be assigned to floating point
               // type (float, double, long double)
               // constants.
               if (target.getExpressionValue () instanceof BigDecimal)
               {
                  return ctx.createSuccessStatus ();
               }
               else
               {
                  return ctx.createFailureStatus (new Object[]
                  {
                     describe (target) + " " + messages[0] //$NON-NLS-1$
                  });
               }
         }
      }
      return ctx.createSuccessStatus ();
   }
}
