package com.prismtech.idt.idlmm.constraints;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.validation.IValidationContext;

import com.prismtech.idt.idlmm.AttributeDef;
import com.prismtech.idt.idlmm.HomeDef;
import com.prismtech.idt.idlmm.ValueDef;

/**
 * A primary key type must be a value type derived from Components::PrimaryKeyBase.
 *   
 *   CCM Spec 6.7.2.1
 * 
 * @see <a href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * 
 * @constraint 69
 */

public class CheckHomePrimaryKey extends IdlmmConstraint
{

   @Override
   public IStatus validate (IValidationContext ctx)
   {
      HomeDef target = (HomeDef) ctx.getTarget ();
      ValueDef primaryKey = target.getPrimarykey ();
      EList<ValueDef> baseValues = primaryKey.getAllBaseValues ();
      
      for(ValueDef base : baseValues){ 
         if(base.getAbsoluteName ().equals ("::Components::PrimaryKeyBase"))
         {
            return ctx.createSuccessStatus ();
         }
      }
      

      Object[] args = new Object[]
      {
            target.getAbsoluteName (), primaryKey.getAbsoluteName ()
      };

      return ctx.createFailureStatus (args);
   }
   
}
