/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import java.math.BigInteger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;
import com.prismtech.idt.idlmm.ConstantDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.PrimitiveDef;
import com.prismtech.idt.idlmm.PrimitiveKind;


/**
 * @see <a
 *      href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 */
public class CheckIntegerConstant extends IdlmmConstraint
{
   @Override
   public IStatus validate (IValidationContext ctx)
   {
      ConstantDef target = (ConstantDef) ctx.getTarget ();
      IDLType type = target.getType ();
      Object value = target.getExpressionValue ();
      if(value == null)
      {
         return ctx.createSuccessStatus ();
      }
      if (type instanceof PrimitiveDef)
      {
         PrimitiveDef primitive = (PrimitiveDef) type;

         switch (primitive.getKind ())
         {
            case PK_ULONG:
               if (! (value instanceof BigInteger))
               {
                  return ctx.createFailureStatus (new Object[]
                  {
                        describe (target), "unsigned long"
                  });
               }
               break;
            case PK_ULONGLONG:
               if (! (value instanceof BigInteger))
               {
                  return ctx.createFailureStatus (new Object[]
                  {
                        describe (target), "unsigned long long"
                  });
               }
               break;
            case PK_USHORT:
               if (! (value instanceof BigInteger))
               {
                  return ctx.createFailureStatus (new Object[]
                  {
                        describe (target), "unsigned short"
                  });
               }
               break;
            case PK_LONG:
               if (! (value instanceof BigInteger))
               {
                  return ctx.createFailureStatus (new Object[]
                  {
                        describe (target), "long"
                  });
               }
               break;
            case PK_LONGLONG:
               if (! (value instanceof BigInteger))
               {
                  return ctx.createFailureStatus (new Object[]
                  {
                        describe (target), "long long"
                  });
               }
               break;
            case PK_SHORT:
               if (! (value instanceof BigInteger))
               {
                  return ctx.createFailureStatus (new Object[]
                  {
                        describe (target), "short"
                  });
               }
               break;
            case PK_OCTET:
               if (! (value instanceof BigInteger))
               {
                  return ctx.createFailureStatus (new Object[]
                  {
                        describe (target), "octet"
                  });
               }
               break;
         }
      }
      return ctx.createSuccessStatus ();
   }
}
