/*
 * COPYRIGHT NOTICE 
 *
 * The information contained herein and which follows is proprietary
 * information and is the property of PrismTech Corporation.  This
 * information is not to be divulged, released, copied, reproduced, or
 * conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech
 * Corporation.
 *
 * DISCLAIMER
 * 
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")
 * are under any obligation to support the use of this code which is
 * provided without warranty of any kind. PrismTech shall (except as
 * otherwise prohibited by law) have no liability whatsoever in relation
 * to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;

import com.prismtech.idt.idlmm.Interface;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.InterfaceModifier;


/**
 * An abstract interface may only inherit from other abstract interfaces.
 * 
 * @see <a href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constrain 7, 14
 */
public class CheckInterfaceAbstractInheritance extends IdlmmConstraint
{
   @Override
   public IStatus validate (IValidationContext ctx)
   {
      InterfaceDef target = (InterfaceDef) ctx.getTarget ();

      if (target.getModifier ().equals (InterfaceModifier.IF_ABSTRACT))
      {
         for (Interface base : target.getBaseInterfaces ())
         {
            if (!target.getModifier ().equals (InterfaceModifier.IF_ABSTRACT))
            {
               Object[] args = new Object[]
               {
                     describe (target), describe (base)
               };

               return ctx.createFailureStatus (args);
            }
         }
      }

      return ctx.createSuccessStatus ();
   }
}
