/*
 * COPYRIGHT NOTICE 
 *
 * The information contained herein and which follows is proprietary
 * information and is the property of PrismTech Corporation.  This
 * information is not to be divulged, released, copied, reproduced, or
 * conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech
 * Corporation.
 *
 * DISCLAIMER
 * 
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")
 * are under any obligation to support the use of this code which is
 * provided without warranty of any kind. PrismTech shall (except as
 * otherwise prohibited by law) have no liability whatsoever in relation
 * to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;

import com.prismtech.idt.idlmm.InterfaceDef;


/**
 * An interface may not be specified as a direct base interface of a derived interface more than once; 
 * it may be an indirect base interface more than once.
 * @see <a href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constrain 8 
 */
public class CheckInterfaceInheritanceCycles extends IdlmmConstraint
{
   @Override
   public IStatus validate (IValidationContext ctx)
   {
      InterfaceDef target = (InterfaceDef) ctx.getTarget ();

      if (target.getAllBaseInterfaces ().contains (target) == true)
      {
         return ctx.createFailureStatus (new Object[]
         {
            describe (target)
         });
      }
      else
      {
         return ctx.createSuccessStatus ();
      }
   }
}
