/*
 * COPYRIGHT NOTICE 
 *
 * The information contained herein and which follows is proprietary
 * information and is the property of PrismTech Corporation.  This
 * information is not to be divulged, released, copied, reproduced, or
 * conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech
 * Corporation.
 *
 * DISCLAIMER
 * 
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")
 * are under any obligation to support the use of this code which is
 * provided without warranty of any kind. PrismTech shall (except as
 * otherwise prohibited by law) have no liability whatsoever in relation
 * to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;

import com.prismtech.idt.idlmm.EnumDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.UnionDef;
import com.prismtech.idt.idlmm.UnionMember;

/**
 * If the&lt;switch_type_spec&gt; is an &lt;enum_type&gt;, 
 * the identifier for the enumeration is in the scope of the union; as a result, 
 * it must be distinct from the element declarators.
 * 
 * @see <a href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constraint 58
 */
public class CheckUnionEnumIdentifierDistinctFromElementDeclarators extends IdlmmConstraint
{

   /* (non-Javadoc)
    * @see org.eclipse.emf.validation.AbstractModelConstraint#validate(org.eclipse.emf.validation.IValidationContext)
    */
   @Override
   public IStatus validate (IValidationContext ctx)
   {
      UnionDef target = UnionDef.class.cast (ctx.getTarget ());
      IDLType type = target.getType ();
      boolean result = false;
      String enumName = null;
      if (type instanceof EnumDef)
      {
         enumName = ((EnumDef)type).getName ();
         if ( (enumName != null) && (enumName.length () > 0))
         {
            for (UnionMember unionMember : target.getMembers ())
            {
               String name = unionMember.getName ();
               if ( (name != null) && (name.length () > 0) && enumName.equals (name))
               {
                  result = true;
                  break;
               }
            }
         }
      }
      if(result)
      {
       return ctx.createFailureStatus (new Object[]
               {
                     this.describe (target), enumName
               });
      }      
      return ctx.createSuccessStatus ();
   }

}
