/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;

import com.prismtech.idt.idlmm.Expression;
import com.prismtech.idt.idlmm.Named;
import com.prismtech.idt.idlmm.UnionDef;
import com.prismtech.idt.idlmm.UnionMember;


/**
 * The values of the constant expressions for the case labels of a single union
 * definition must be distinct.
 * 
 * @see <a
 *      href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constrain 60
 */
public class CheckUnionHasDistinctConstantExpressionsForCaseLabels extends IdlmmConstraint
{

   /*
    * (non-Javadoc)
    * 
    * @see org.eclipse.emf.validation.AbstractModelConstraint#validate(org.eclipse.emf.validation.IValidationContext)
    */
   @Override
   public IStatus validate (IValidationContext ctx)
   {
      UnionDef target = UnionDef.class.cast (ctx.getTarget ());
      List<Object> values = new ArrayList<Object> ();
      Object expValue = null;
      for (UnionMember unionMember : target.getMembers ())
      {
         if (unionMember.isDefault ())
         {
            continue;
         }
         for (Expression label : unionMember.getLabels ())
         {
            expValue = label.getExpressionValue ();
            if (values.contains (expValue))
            {
               return ctx.createFailureStatus (new Object[]
               {
                     this.describe (target),
                     ( (label instanceof Named) ? Named.class.cast (label).getName () : "?")
               });
            }
            values.add (expValue);
         }
      }
      return ctx.createSuccessStatus ();
   }

}
