/*
 * COPYRIGHT NOTICE 
 *
 * The information contained herein and which follows is proprietary
 * information and is the property of PrismTech Corporation.  This
 * information is not to be divulged, released, copied, reproduced, or
 * conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech
 * Corporation.
 *
 * DISCLAIMER
 * 
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")
 * are under any obligation to support the use of this code which is
 * provided without warranty of any kind. PrismTech shall (except as
 * otherwise prohibited by law) have no liability whatsoever in relation
 * to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;

import com.prismtech.idt.idlmm.UnionDef;
import com.prismtech.idt.idlmm.UnionMember;


/**
 * A default case can appear at most once.
 * 
 * @see <a href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constrain 54
 */
public class CheckUnionHasOnlyOneDefault extends IdlmmConstraint
{
   @Override
   public IStatus validate (IValidationContext ctx)
   {
      UnionDef target = (UnionDef) ctx.getTarget ();

      int defaults = 0;

      for (UnionMember member : target.getMembers ())
      {
         if (member.isDefault ())
         {
            defaults++;
         }
      }

      if (defaults > 1)
      {
         return ctx.createFailureStatus (new Object[]
         {
               describe (target), defaults
         });
      }
      else
      {
         return ctx.createSuccessStatus ();
      }
   }
}
