/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.validation.IValidationContext;

import com.prismtech.idt.idlmm.UnionDef;
import com.prismtech.idt.idlmm.UnionMember;


/**
 * Checks that the element declarators in a particular union are unique.
 * 
 * @see <a
 *      href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constrain 57
 */
public class CheckUnionHasUniqueElementDeclarators extends IdlmmConstraint
{

   /*
    * (non-Javadoc)
    * 
    * @see org.eclipse.emf.validation.AbstractModelConstraint#validate(org.eclipse.emf.validation.IValidationContext)
    */
   @Override
   public IStatus validate (IValidationContext ctx)
   {
      UnionDef target = UnionDef.class.cast (ctx.getTarget ());
      List<String> names = new ArrayList<String> ();
      String name = null;
      for (UnionMember unionMember : target.getMembers ())
      {
         name = unionMember.getName ();
         if ( (name != null) && (name.length () > 0))
         {
            if (names.contains (name))
            {
               return ctx.createFailureStatus (new Object[]
               {
                     this.describe (target), name
               });
            }
            names.add (name);
         }
      }
      return ctx.createSuccessStatus ();
   }

}
