/*
 * COPYRIGHT NOTICE 
 *
 * The information contained herein and which follows is proprietary
 * information and is the property of PrismTech Corporation.  This
 * information is not to be divulged, released, copied, reproduced, or
 * conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech
 * Corporation.
 *
 * DISCLAIMER
 * 
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")
 * are under any obligation to support the use of this code which is
 * provided without warranty of any kind. PrismTech shall (except as
 * otherwise prohibited by law) have no liability whatsoever in relation
 * to its use.
 */
package com.prismtech.idt.idlmm.constraints;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.validation.IValidationContext;
import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.Container;



/**
 * Name scoping rules require that the member declarators in a particular structure be unique. 
 * The value of a struct is the value of all of its members.
 * 
 * @see <a href="https://scarab.prismtech.com/twiki/bin/view/MDD/IdtModelChecker">IDT-ModelConstraints</a>
 * @constrain 52, 63
 */
public class CheckUniqueStructMemberDeclarators extends IdlmmConstraint
{

   @Override
   public IStatus validate (IValidationContext ctx)
   {
      Container target = (Container) ctx.getTarget ();
      
      EList<Contained> memberList = target.getContents ();
      
      for (int i = 0; i < memberList.size (); i++)
      {
         String name = memberList.get (0).getName ();
         for (int j = i+1; j < memberList.size (); j++)
         {
            if (name.equalsIgnoreCase (memberList.get (j).getName ()))
            {
                return ctx.createFailureStatus (new Object[]
                {
                      describe (target), describe (memberList.get(j)) 
                });
            }
         }
      }
      return ctx.createSuccessStatus ();
   }

}
