/*
 * COPYRIGHT NOTICE 
 *
 * The information contained herein and which follows is proprietary
 * information and is the property of PrismTech Corporation.  This
 * information is not to be divulged, released, copied, reproduced, or
 * conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech
 * Corporation.
 *
 * DISCLAIMER
 * 
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")
 * are under any obligation to support the use of this code which is
 * provided without warranty of any kind. PrismTech shall (except as
 * otherwise prohibited by law) have no liability whatsoever in relation
 * to its use.
 */
package com.prismtech.idt.idlmm.impl;

import java.util.AbstractSet;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

/**
 * The {@link Set} implementation that preserves elements order
 * 
 * FIXME: There should be only one copy of this class in the product
 * however, it is implementation class that there is no reason to export it from
 * this plugin nor we have mechanism to share classes between two other plugins without
 * introducing too much new plugins.    
 */
public class ArraySet<E> extends AbstractSet<E> implements Set<E>
{
   private final ArrayList<E> set;
   
   /**
    * 
    * Constructor <code>ArraySet</code><br/>
    *
    */
   public ArraySet()
   {
      set = new ArrayList<E> ();      
   }
   
   /**
    * 
    * Constructor <code>ArraySet</code><br/>
    * 
    * @param c
    */
   public ArraySet(Collection<? extends E> c)
   {
      set = new ArrayList<E> (c.size ());
      for (E o : c)
      {
         add(o);
      }
   }  
   
   @Override   
   public boolean add(E o)
   {
      if (set.contains (o))
      {
         return false;
      }
      return set.add (o); 
   }

   @Override
   public boolean contains(Object elem)
   {
      return set.contains (elem);
   }

   @Override
   public Iterator<E> iterator()
   {
      return set.iterator ();
   }

   @Override
   public boolean remove(Object o)
   {
      return set.remove (o);
   }

   @Override
   public int size()
   {
      return set.size ();
   }

   @Override
   public Object[] toArray()
   {
      return set.toArray ();
   }

   @Override
   public <T> T[] toArray(T[] a)
   {
      return set.toArray (a);
   }

   @Override
   public String toString()
   {
      return set.toArray ().toString ();
   }
}
