package com.prismtech.idt.idlmm.impl;

import java.util.Set;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.UniqueEList;

import com.prismtech.idt.idlmm.AliasDef;
import com.prismtech.idt.idlmm.AttributeDef;
import com.prismtech.idt.idlmm.ComponentDef;
import com.prismtech.idt.idlmm.ComponentPortDef;
import com.prismtech.idt.idlmm.ConsumesDef;
import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.EmitsDef;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.ProvidesDef;
import com.prismtech.idt.idlmm.PublishesDef;
import com.prismtech.idt.idlmm.UsesDef;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.DelegatingEcoreEList.UnmodifiableEList;

public class ComponentDefExt
{

   public static EList<IRObject> getPrerequisites (ComponentDef self)
   {
      Set<IRObject> result = new ArraySet<IRObject> (ContainerExt.getPrerequisites (self));
      result.addAll (self.getAllBaseComponents ());
      return new BasicEList.UnmodifiableEList<IRObject> (result.size (), result.toArray ());
   }

   public static EList<ComponentDef> getAllBaseComponents (ComponentDef self)
   {
      BasicEList<ComponentDef> bases = new BasicEList<ComponentDef> ();
      ComponentDef base = self.getBaseComponent ();
      while (base != null)
      {
         bases.add (base);
         base = base.getBaseComponent ();
      }
      return new UnmodifiableEList<ComponentDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllBaseComponents (), 
            bases);
   }

  private static BasicEList<InterfaceDef> getAllSupportedInterfaces (ComponentDef self, BasicEList<InterfaceDef> supportedList)
  {
     // get all supported interfaces from inherited components
     ComponentDef base = self.getBaseComponent ();
     if (base != null)
     {
        getAllSupportedInterfaces (base, supportedList);
     }
     
     // get all supported interfaces and inheriting interfaces
     for (InterfaceDef supported : self.getSupportedInterfaces ())
     {
        if (!supportedList.contains (supported))
        {
           // add all inherited interfaces of supported interface
           supportedList.addAll (InterfaceDefExt.getAllBaseInterfaces (supported));
           
           // add supported interface itself
           supportedList.add (supported);
        }
     }
     return supportedList;
  }

  public static EList<InterfaceDef> getAllSupportedInterfaces (ComponentDef self)
   {
      BasicEList<InterfaceDef> bases = getAllSupportedInterfaces (self, new UniqueEList<InterfaceDef> ());
      return new UnmodifiableEList<InterfaceDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllSupportedInterfaces (), 
            bases);
   }

  public static EList<Contained> getAllContents (ComponentDef self)
  {
     BasicEList<Contained> result = new UniqueEList<Contained> ();

     // get content from inherited components
     for (ComponentDef base : getAllBaseComponents (self))
     {
        result.addAll ( base.getContents ());
     }

     // get content from supported interfaces
     for (InterfaceDef base : getAllSupportedInterfaces (self))
     {
        result.addAll ( base.getContents ());
     }

     // add self content
     result.addAll (self.getContents ());

     return new UnmodifiableEList<Contained> (
           (EObjectImpl)self, 
           (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllContents (), 
           result);
  }

   public static EList<AttributeDef> getAttributes (ComponentDef self)
   {
      return new UnmodifiableEList<AttributeDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getAttributeContainer_Attributes (), 
            ContainerExt.getContents (self, AttributeDef.class));
   }

   private static BasicEList<AttributeDef> getAllAttributes (ComponentDef self, BasicEList<AttributeDef> attributes)
   {
      // get attributes from inherited components
      for (ComponentDef base : getAllBaseComponents (self))
      {
         getAllAttributes (base, attributes);
      }

      // get attributes from supported interfaces
      for (InterfaceDef base : getAllSupportedInterfaces (self))
      {
         attributes.addAll ( base.getAttributes ());
      }
      
      // get attributes from the component itself
      attributes.addAll (self.getAttributes ());
      
      return attributes;
   }
   
   public static EList<AttributeDef> getAllAttributes (ComponentDef self)
   {
      BasicEList<AttributeDef> result = getAllAttributes (self, new UniqueEList<AttributeDef> ());
      return new UnmodifiableEList<AttributeDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getAttributeContainer_AllAttributes (), 
            result);
   }

   public static EList<ComponentPortDef> getPorts (ComponentDef self)
   {
      return new UnmodifiableEList<ComponentPortDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_Ports (), 
            ContainerExt.getContents (self, ComponentPortDef.class));
   }

   public static EList<ComponentPortDef> getAllPorts (ComponentDef self)
   {
      EList<ComponentPortDef> result = new BasicEList<ComponentPortDef> ();
      // get ports from inherited components
      for (ComponentDef base : getAllBaseComponents (self))
      {
          result.addAll ( base.getPorts ());
      }
      // get ports from the component itself
      result.addAll (getPorts (self));
      return new UnmodifiableEList<ComponentPortDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllPorts (), 
            result);
   }

   public static EList<UsesDef> getUsesPorts (ComponentDef self)
   {
      return new UnmodifiableEList<UsesDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_UsesPorts (), 
            ContainerExt.getContents (self, UsesDef.class));
   }

   public static EList<UsesDef> getAllUsesPorts (ComponentDef self)
   {
      EList<UsesDef> result = new BasicEList<UsesDef> ();
      // get ports from inherited components
      for (ComponentDef base : getAllBaseComponents (self))
      {
          result.addAll ( base.getUsesPorts ());
      }
      // get ports from the component itself
      result.addAll (getUsesPorts (self));
      return new UnmodifiableEList<UsesDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllUsesPorts (), 
            result);
   }

   public static EList<ProvidesDef> getProvidesPorts (ComponentDef self)
   {
      return new UnmodifiableEList<ProvidesDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_ProvidesPorts (), 
            ContainerExt.getContents (self, ProvidesDef.class));
   }

   public static EList<ProvidesDef> getAllProvidesPorts (ComponentDef self)
   {
      EList<ProvidesDef> result = new BasicEList<ProvidesDef> ();
      // get ports from inherited components
      for (ComponentDef base : getAllBaseComponents (self))
      {
          result.addAll ( base.getProvidesPorts ());
      }
      // get ports from the component itself
      result.addAll (getProvidesPorts (self));
      return new UnmodifiableEList<ProvidesDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllProvidesPorts (), 
            result);
   }

   public static EList<ConsumesDef> getConsumesPorts (ComponentDef self)
   {
      return new UnmodifiableEList<ConsumesDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_ConsumesPorts (), 
            ContainerExt.getContents (self, ConsumesDef.class));
   }

   public static EList<ConsumesDef> getAllConsumesPorts (ComponentDef self)
   {
      EList<ConsumesDef> result = new BasicEList<ConsumesDef> ();
      // get ports from inherited components
      for (ComponentDef base : getAllBaseComponents (self))
      {
          result.addAll ( base.getConsumesPorts ());
      }
      // get ports from the component itself
      result.addAll (getConsumesPorts (self));
      return new UnmodifiableEList<ConsumesDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllConsumesPorts (), 
            result);
   }

   public static EList<PublishesDef> getPublishesPorts (ComponentDef self)
   {
      return new UnmodifiableEList<PublishesDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_PublishesPorts (), 
            ContainerExt.getContents (self, PublishesDef.class));
   }

   public static EList<PublishesDef> getAllPublishesPorts (ComponentDef self)
   {
      EList<PublishesDef> result = new BasicEList<PublishesDef> ();
      // get ports from inherited components
      for (ComponentDef base : getAllBaseComponents (self))
      {
          result.addAll ( base.getPublishesPorts ());
      }
      // get ports from the component itself
      result.addAll (getPublishesPorts (self));
      return new UnmodifiableEList<PublishesDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllPublishesPorts (), 
            result);
   }

   public static EList<EmitsDef> getEmitsPorts (ComponentDef self)
   {
      return new UnmodifiableEList<EmitsDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_EmitsPorts (), 
            ContainerExt.getContents (self, EmitsDef.class));
   }

   public static EList<EmitsDef> getAllEmitsPorts (ComponentDef self)
   {
      EList<EmitsDef> result = new BasicEList<EmitsDef> ();
      // get ports from inherited components
      for (ComponentDef base : getAllBaseComponents (self))
      {
          result.addAll ( base.getEmitsPorts ());
      }
      // get ports from the component itself
      result.addAll (getEmitsPorts (self));
      return new UnmodifiableEList<EmitsDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getComponentDef_AllEmitsPorts (), 
            result);
   }


}
