package com.prismtech.idt.idlmm.impl;

import java.util.Set;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.UniqueEList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.DelegatingEcoreEList.UnmodifiableEList;

import com.prismtech.idt.idlmm.AttributeDef;
import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.EmitsDef;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.Initializer;
import com.prismtech.idt.idlmm.Interface;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.OperationDef;
import com.prismtech.idt.idlmm.EventDef;
import com.prismtech.idt.idlmm.ValueContained;
import com.prismtech.idt.idlmm.ValueDef;
import com.prismtech.idt.idlmm.ValueMemberDef;

public class EventDefExt
{

   public static EList<IRObject> getPrerequisites (EventDefImpl self)
   {
      Set<IRObject> result = new ArraySet<IRObject> (ContainerExt.getPrerequisites (self));
      result.addAll (self.getBaseValues ());
      return new BasicEList.UnmodifiableEList<IRObject> (result.size (), result.toArray ());
   }

   public static EList<OperationDef> getOperations (EventDef self)
   {
      return new UnmodifiableEList<OperationDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getOperationContainer_Operations (), 
            ContainerExt.getContents (self, OperationDef.class));
   }

   public static EList<OperationDef> getAllOperations (EventDef self)
   {
      BasicEList<OperationDef> result = new UniqueEList<OperationDef> ();

      // get operations from inherited values
      for (ValueDef base : self.getAllBaseValues ())
      {
          result.addAll ( base.getOperations ());
      }

      // get operations from supported interfaces
      for (Interface base : self.getAllSupportedInterfaces ())
      {
          if (base instanceof InterfaceDef)
          {
             result.addAll ( ((InterfaceDef) base).getOperations ());
          }
      }

      // get operations from the home itself
      result.addAll (self.getOperations ());
      return new UnmodifiableEList<OperationDef>(
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getOperationContainer_AllOperations (), 
            result);
   }

   public static EList<ValueContained> getEventContents (EventDef self)
   {
      // EventDef inherits from ValueDef. eventContents is the same than valueContents
      return self.getValueContents ();
   }

   public static EList<Contained> getAllContents (EventDef self)
   {
      // EventDef inherits from ValueDef. ValueExt operations can be used.
      return ValueExt.getAllContents(self);
   }

   public static EList<ValueDef> getAllBaseValues (EventDef self)
   {
      // EventDef inherits from ValueDef. ValueExt operations can be used.
      return ValueExt.getAllBaseValues(self);
   }

   public static EList<Interface> getAllSupportedInterfaces (EventDef self)
   {
      // EventDef inherits from ValueDef. ValueExt operations can be used.
      return ValueExt.getAllSupportedInterfaces(self);
   }

   public static EList<AttributeDef> getAttributes (EventDef self)
   {
      // EventDef inherits from ValueDef. ValueExt operations can be used.
      return ValueExt.getAttributes(self);
   }

   public static EList<AttributeDef> getAllAttributes (EventDef self)
   {
      // EventDef inherits from ValueDef. ValueExt operations can be used.
      return ValueExt.getAllAttributes(self);
   }

   public static EList<Initializer> getInitializers (EventDef self)
   {
      // EventDef inherits from ValueDef. ValueExt operations can be used.
      return ValueExt.getInitializers(self);
   }

   public static EList<ValueMemberDef> getMembers (EventDef self)
   {
      // EventDef inherits from ValueDef. ValueExt operations can be used.
      return ValueExt.getMembers(self);
   }

}
