package com.prismtech.idt.idlmm.impl;

import java.util.Set;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.UniqueEList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.DelegatingEcoreEList.UnmodifiableEList;

import com.prismtech.idt.idlmm.AliasDef;
import com.prismtech.idt.idlmm.AttributeDef;
import com.prismtech.idt.idlmm.ConstantDef;
import com.prismtech.idt.idlmm.EnumDef;
import com.prismtech.idt.idlmm.ExceptionDef;
import com.prismtech.idt.idlmm.FactoryOperation;
import com.prismtech.idt.idlmm.FinderOperation;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.HomeDef;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.NativeDef;
import com.prismtech.idt.idlmm.OperationDef;
import com.prismtech.idt.idlmm.StructDef;
import com.prismtech.idt.idlmm.UnionDef;

public class HomeDefExt
{

   public static EList<IRObject> getPrerequisites (HomeDefImpl self)
   {
      Set<IRObject> result = new ArraySet<IRObject> (ContainerExt.getPrerequisites (self));
      result.add (self.getBaseHome ());
      return new BasicEList.UnmodifiableEList<IRObject> (result.size (), result.toArray ());
   }

   public static EList<HomeDef> getAllBaseHomes (HomeDef self)
   {
      BasicEList<HomeDef> bases = new BasicEList<HomeDef> ();
      HomeDef base = self.getBaseHome ();
      while (base != null)
      {
         bases.add (base);
         base = base.getBaseHome ();
      }
      return new UnmodifiableEList<HomeDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getHomeDef_AllBaseHomes (), 
            bases);
   }

   private static BasicEList<InterfaceDef> getAllSupportedInterfaces (HomeDef self, BasicEList<InterfaceDef> supportedList)
   {
      // get all supported interfaces from inherited homes
      HomeDef base = self.getBaseHome ();
      if (base != null)
      {
         getAllSupportedInterfaces (base, supportedList);
      }
      
      // get all supported interfaces and inheriting interfaces
      for (InterfaceDef supported : self.getSupportedInterfaces ())
      {
         if (!supportedList.contains (supported))
         {
            // add all inherited interfaces of supported interface
            supportedList.addAll (InterfaceDefExt.getAllBaseInterfaces (supported));
            
            // add supported interface itself
            supportedList.add (supported);
         }
      }
      return supportedList;
   }

   public static EList<InterfaceDef> getAllSupportedInterfaces (HomeDef self)
   {
      BasicEList<InterfaceDef> bases = getAllSupportedInterfaces (self, new UniqueEList<InterfaceDef> ());
      return new UnmodifiableEList<InterfaceDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getHomeDef_AllSupportedInterfaces (), 
            bases);
   }

   public static EList<AttributeDef> getAttributes (HomeDef self)
   {
      return ContainerExt.getContents (self, AttributeDef.class);
   }

   public static EList<AttributeDef> getAllAttributes (HomeDef self)
   {
      EList<AttributeDef> result = new UniqueEList<AttributeDef> ();

      // get attributes from inherited homes
      for (HomeDef base : getAllBaseHomes (self))
      {
          result.addAll ( base.getAttributes ());
      }

      // get attributes from supported interfaces
      for (InterfaceDef base : getAllSupportedInterfaces (self))
      {
          result.addAll ( base.getAttributes ());
      }

      // get attributes from the home itself
      result.addAll (self.getAttributes ());

      return new UnmodifiableEList<AttributeDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getAttributeContainer_AllAttributes (), 
            result);
   }

   public static EList<OperationDef> getOperations (HomeDef self)
   {
      return ContainerExt.getContents (self, OperationDef.class);
   }

   public static EList<OperationDef> getAllOperations (HomeDef self)
   {
      EList<OperationDef> result = new UniqueEList<OperationDef> ();

      // get operations from inherited homes
      for (HomeDef base : getAllBaseHomes (self))
      {
          result.addAll ( base.getOperations ());
      }

      // get operations from supported interfaces
      for (InterfaceDef base : getAllSupportedInterfaces (self))
      {
          result.addAll ( base.getOperations ());
      }

      // get operations from the home itself
      result.addAll (self.getOperations ());

      return new UnmodifiableEList<OperationDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getOperationContainer_AllOperations (), 
            result);      
   }

   public static EList<FactoryOperation> getFactoryOperations (HomeDef self)
   {
      return ContainerExt.getContents (self, FactoryOperation.class);
   }

   public static EList<FactoryOperation> getAllFactoryOperations (HomeDef self)
   {
      EList<FactoryOperation> result = new UniqueEList<FactoryOperation> ();

      // get factory operations from inherited homes
      for (HomeDef base : getAllBaseHomes (self))
      {
          result.addAll ( base.getFactoryOperations ());
      }

      // get factory operations from the home itself
      result.addAll (self.getFactoryOperations ());

      return new UnmodifiableEList<FactoryOperation> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getHomeDef_AllFactoryOperations (), 
            result); 
   }

   public static EList<FinderOperation> getFinderOperations (HomeDef self)
   {
      return ContainerExt.getContents (self, FinderOperation.class);
   }

   public static EList<FinderOperation> getAllFinderOperations (HomeDef self)
   {
      EList<FinderOperation> result = new UniqueEList<FinderOperation> ();

      // get finder operations from inherited homes
      for (HomeDef base : getAllBaseHomes (self))
      {
          result.addAll ( base.getFinderOperations ());
      }

      // get finder operations from the home itself
      result.addAll (self.getFinderOperations ());

      return new UnmodifiableEList<FinderOperation> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getHomeDef_AllFinderOperations (), 
            result); 
   }

   public static EList<ConstantDef> getConstants (HomeDef self)
   {
      return ContainerExt.getContents (self, ConstantDef.class);
   }

   private static BasicEList<ConstantDef> getAllConstants (HomeDef self, UniqueEList<ConstantDef> constants)
   {
      // get all constants from inherited home
      if (self.getBaseHome() != null) {
         getAllConstants (self.getBaseHome(), constants);
      }
      
      // get all constants from all supported interfaces
      for (InterfaceDef supported : self.getSupportedInterfaces()) {
         constants.addAll (InterfaceDefExt.getAllConstants(supported));
      }
      
      // add constants from current interface
      constants.addAll (getConstants(self));
      return constants;
   }

   public static EList<ConstantDef> getAllConstants (HomeDef self)
   {
      BasicEList<ConstantDef> constants = getAllConstants (self, new UniqueEList<ConstantDef> ());
      return new UnmodifiableEList<ConstantDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getConstantContainer_AllConstants (), 
            constants);
   }

   public static EList<ExceptionDef> getExceptions (HomeDef self)
   {
      return ContainerExt.getContents (self, ExceptionDef.class);
   }

   private static BasicEList<ExceptionDef> getAllExceptions (HomeDef self, UniqueEList<ExceptionDef> exceptions)
   {
      // get all exceptions from inherited home
      if (self.getBaseHome() != null) {
         getAllExceptions (self.getBaseHome(), exceptions);
      }
      
      // get all exceptions from all supported interfaces
      for (InterfaceDef supported : self.getSupportedInterfaces()) {
         exceptions.addAll (InterfaceDefExt.getAllExceptions(supported));
      }
      
      // add exceptions from current interface
      exceptions.addAll (getExceptions(self));
      return exceptions;
   }

   public static EList<ExceptionDef> getAllExceptions (HomeDef self)
   {
      BasicEList<ExceptionDef> exceptions = getAllExceptions (self, new UniqueEList<ExceptionDef> ());
      return new UnmodifiableEList<ExceptionDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getExceptionContainer_AllExceptions (), 
            exceptions);
   }

   public static EList<AliasDef> getAliases (HomeDef self)
   {
      return ContainerExt.getContents (self, AliasDef.class);
   }

   private static BasicEList<AliasDef> getAllAliases (HomeDef self, UniqueEList<AliasDef> aliases)
   {
      // get all aliases from inherited home
      if (self.getBaseHome() != null) {
         getAllAliases (self.getBaseHome(), aliases);
      }
      
      // get all aliases from all supported interfaces
      for (InterfaceDef supported : self.getSupportedInterfaces()) {
         aliases.addAll (InterfaceDefExt.getAllAliases(supported));
      }
      
      // add aliases from current interface
      aliases.addAll (getAliases(self));
      return aliases;
   }

   public static EList<AliasDef> getAllAliases (HomeDef self)
   {
      BasicEList<AliasDef> aliases = getAllAliases (self, new UniqueEList<AliasDef> ());
      return new UnmodifiableEList<AliasDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getAliasContainer_AllAliases (), 
            aliases);
   }

   public static EList<StructDef> getStructs (HomeDef self)
   {
      return ContainerExt.getContents (self, StructDef.class);
   }

   private static BasicEList<StructDef> getAllStructs (HomeDef self, UniqueEList<StructDef> structs)
   {
      // get all structs from inherited home
      if (self.getBaseHome() != null) {
         getAllStructs (self.getBaseHome(), structs);
      }
      
      // get all structs from all supported interfaces
      for (InterfaceDef supported : self.getSupportedInterfaces()) {
         structs.addAll (InterfaceDefExt.getAllStructs(supported));
      }
      
      // add structs from current interface
      structs.addAll (getStructs(self));
      return structs;
   }

   public static EList<StructDef> getAllStructs (HomeDef self)
   {
      BasicEList<StructDef> structs = getAllStructs (self, new UniqueEList<StructDef> ());
      return new UnmodifiableEList<StructDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getStructContainer_AllStructs (), 
            structs);
   }

   public static EList<UnionDef> getUnions (HomeDef self)
   {
      return ContainerExt.getContents (self, UnionDef.class);
   }

   private static BasicEList<UnionDef> getAllUnions (HomeDef self, UniqueEList<UnionDef> unions)
   {
      // get all unions from inherited home
      if (self.getBaseHome() != null) {
         getAllUnions (self.getBaseHome(), unions);
      }
      
      // get all unions from all supported interfaces
      for (InterfaceDef supported : self.getSupportedInterfaces()) {
         unions.addAll (InterfaceDefExt.getAllUnions(supported));
      }
      
      // add unions from current interface
      unions.addAll (getUnions(self));
      return unions;
   }

   public static EList<UnionDef> getAllUnions (HomeDef self)
   {
      BasicEList<UnionDef> unions = getAllUnions (self, new UniqueEList<UnionDef> ());
      return new UnmodifiableEList<UnionDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getUnionContainer_AllUnions (), 
            unions);
   }

   public static EList<EnumDef> getEnums (HomeDef self)
   {
      return ContainerExt.getContents (self, EnumDef.class);
   }

   private static BasicEList<EnumDef> getAllEnums (HomeDef self, UniqueEList<EnumDef> enums)
   {
      // get all enums from inherited home
      if (self.getBaseHome() != null) {
         getAllEnums (self.getBaseHome(), enums);
      }
      
      // get all enums from all supported interfaces
      for (InterfaceDef supported : self.getSupportedInterfaces()) {
         enums.addAll (InterfaceDefExt.getAllEnums(supported));
      }
      
      // add enums from current interface
      enums.addAll (getEnums(self));
      return enums;
   }

   public static EList<EnumDef> getAllEnums (HomeDef self)
   {
      BasicEList<EnumDef> enums = getAllEnums (self, new UniqueEList<EnumDef> ());
      return new UnmodifiableEList<EnumDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getEnumContainer_AllEnums (), 
            enums);
   }

   public static EList<NativeDef> getNatives (HomeDef self)
   {
      return ContainerExt.getContents (self, NativeDef.class);
   }

   private static BasicEList<NativeDef> getAllNatives (HomeDef self, UniqueEList<NativeDef> natives)
   {
      // get all natives from inherited home
      if (self.getBaseHome() != null) {
         getAllNatives (self.getBaseHome(), natives);
      }
      
      // get all natives from all supported interfaces
      for (InterfaceDef supported : self.getSupportedInterfaces()) {
         natives.addAll (InterfaceDefExt.getAllNatives(supported));
      }
      
      // add natives from current interface
      natives.addAll (getNatives(self));
      return natives;
   }

   public static EList<NativeDef> getAllNatives (HomeDef self)
   {
      BasicEList<NativeDef> natives = getAllNatives (self, new UniqueEList<NativeDef> ());
      return new UnmodifiableEList<NativeDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getNativeContainer_AllNatives (), 
            natives);
   }
   
}
