package com.prismtech.idt.idlmm.impl;

import java.util.Set;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.ECollections;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.UniqueEList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.DelegatingEcoreEList.UnmodifiableEList;

import com.prismtech.idt.idlmm.AliasDef;
import com.prismtech.idt.idlmm.AttributeDef;
import com.prismtech.idt.idlmm.ConstantDef;
import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.EnumDef;
import com.prismtech.idt.idlmm.ExceptionDef;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.Interface;
import com.prismtech.idt.idlmm.InterfaceContained;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.NativeDef;
import com.prismtech.idt.idlmm.OperationDef;
import com.prismtech.idt.idlmm.StructDef;
import com.prismtech.idt.idlmm.UnionDef;


/**
 * This is an 'extension class' for {@link InterfaceDef}.
 * <p>
 * It provides implementations for the operation signatures defined in the UML
 * model.
 */
public class InterfaceDefExt
{
   public static EList<OperationDef> getAllOperations (InterfaceDef self)
   {
      EList<OperationDef> result = new BasicEList<OperationDef> ();

      for (Interface base : getAllBaseInterfaces (self))
      {
         if (base instanceof InterfaceDef)
         {
            result.addAll ( ((InterfaceDef) base).getOperations ());
         }
      }

      for (InterfaceContained obj : self.getInterfaceContents ())
      {
         if (obj instanceof OperationDef)
         {
            result.add ((OperationDef) obj);
         }
      }

      return new UnmodifiableEList<OperationDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getOperationContainer_AllOperations (), 
            result);
   }

   public static EList<AttributeDef> getAllAttributes (InterfaceDef self)
   {
      EList<AttributeDef> result = new BasicEList<AttributeDef> ();

      for (Interface base : getAllBaseInterfaces (self))
      {
         if (base instanceof InterfaceDef)
         {
            result.addAll ( ((InterfaceDef) base).getAttributes ());
         }
      }

      result.addAll (self.getAttributes ());

      return new UnmodifiableEList<AttributeDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getAttributeContainer_AllAttributes (), 
            result);
   }

   public static EList<AliasDef> getAliases (InterfaceDef self)
   {
      return ContainerExt.getContents (self, AliasDef.class);
   }

   private static BasicEList<AliasDef> getAllAliases (InterfaceDef self, UniqueEList<AliasDef> aliases)
   {
      // get all aliases from inherited interfaces
      for (InterfaceDef base : self.getBaseInterfaces ())
      {
         getAllAliases (base, aliases);
      }
      // add aliases from current interface
      aliases.addAll (getAliases(self));
      return aliases;
   }

   public static EList<AliasDef> getAllAliases (InterfaceDef self)
   {
      BasicEList<AliasDef> aliases = getAllAliases (self, new UniqueEList<AliasDef> ());
      return new UnmodifiableEList<AliasDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getAliasContainer_AllAliases (), 
            aliases);
   }

   public static EList<ConstantDef> getConstants (InterfaceDef self)
   {
      return ContainerExt.getContents (self, ConstantDef.class);
   }

   private static BasicEList<ConstantDef> getAllConstants (InterfaceDef self, UniqueEList<ConstantDef> constants)
   {
      // get all constants from inherited interfaces
      for (InterfaceDef base : self.getBaseInterfaces ())
      {
         getAllConstants (base, constants);
      }
      // add constants from current interface
      constants.addAll (getConstants(self));
      return constants;
   }

   public static EList<ConstantDef> getAllConstants (InterfaceDef self)
   {
      BasicEList<ConstantDef> constants = getAllConstants (self, new UniqueEList<ConstantDef> ());
      return new UnmodifiableEList<ConstantDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getConstantContainer_AllConstants (), 
            constants);
   }

   public static EList<NativeDef> getNatives (InterfaceDef self)
   {
      return ContainerExt.getContents (self, NativeDef.class);
   }

   private static BasicEList<NativeDef> getAllNatives (InterfaceDef self, UniqueEList<NativeDef> natives)
   {
      // get all natives from inherited interfaces
      for (InterfaceDef base : self.getBaseInterfaces ())
      {
         getAllNatives (base, natives);
      }
      // add natives from current interface
      natives.addAll (getNatives(self));
      return natives;
   }

   public static EList<NativeDef> getAllNatives (InterfaceDef self)
   {
      BasicEList<NativeDef> natives = getAllNatives (self, new UniqueEList<NativeDef> ());
      return new UnmodifiableEList<NativeDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getNativeContainer_AllNatives (), 
            natives);
   }

   public static EList<StructDef> getStructs (InterfaceDef self)
   {
      return ContainerExt.getContents (self, StructDef.class);
   }

   private static BasicEList<StructDef> getAllStructs (InterfaceDef self, UniqueEList<StructDef> structs)
   {
      // get all structs from inherited interfaces
      for (InterfaceDef base : self.getBaseInterfaces ())
      {
         getAllStructs (base, structs);
      }
      // add structs from current interface
      structs.addAll (getStructs(self));
      return structs;
   }

   public static EList<StructDef> getAllStructs (InterfaceDef self)
   {
      BasicEList<StructDef> structs = getAllStructs (self, new UniqueEList<StructDef> ());
      return new UnmodifiableEList<StructDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getStructContainer_AllStructs (), 
            structs);
   }

   public static EList<EnumDef> getEnums (InterfaceDef self)
   {
      return ContainerExt.getContents (self, EnumDef.class);
   }

   private static BasicEList<EnumDef> getAllEnums (InterfaceDef self, UniqueEList<EnumDef> enums)
   {
      // get all enums from inherited interfaces
      for (InterfaceDef base : self.getBaseInterfaces ())
      {
         getAllEnums (base, enums);
      }
      // add enums from current interface
      enums.addAll (getEnums(self));
      return enums;
   }

   public static EList<EnumDef> getAllEnums (InterfaceDef self)
   {
      BasicEList<EnumDef> enums = getAllEnums (self, new UniqueEList<EnumDef> ());
      return new UnmodifiableEList<EnumDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getEnumContainer_AllEnums (), 
            enums);
   }

   public static EList<UnionDef> getUnions (InterfaceDef self)
   {
      return ContainerExt.getContents (self, UnionDef.class);
   }

   private static BasicEList<UnionDef> getAllUnions (InterfaceDef self, UniqueEList<UnionDef> unions)
   {
      // get all unions from inherited interfaces
      for (InterfaceDef base : self.getBaseInterfaces ())
      {
         getAllUnions (base, unions);
      }
      // add unions from current interface
      unions.addAll (getUnions(self));
      return unions;
   }

   public static EList<UnionDef> getAllUnions (InterfaceDef self)
   {
      BasicEList<UnionDef> unions = getAllUnions (self, new UniqueEList<UnionDef> ());
      return new UnmodifiableEList<UnionDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getUnionContainer_AllUnions (), 
            unions);
   }

   public static EList<ExceptionDef> getExceptions (InterfaceDef self)
   {
      return ContainerExt.getContents (self, ExceptionDef.class);
   }

   private static BasicEList<ExceptionDef> getAllExceptions (InterfaceDef self, UniqueEList<ExceptionDef> exceptions)
   {
      // get all exceptions from inherited interfaces
      for (InterfaceDef base : self.getBaseInterfaces ())
      {
         getAllExceptions (base, exceptions);
      }
      // add exceptions from current interface
      exceptions.addAll (getExceptions(self));
      return exceptions;
   }

   public static EList<ExceptionDef> getAllExceptions (InterfaceDef self)
   {
      BasicEList<ExceptionDef> exceptions = getAllExceptions (self, new UniqueEList<ExceptionDef> ());
      return new UnmodifiableEList<ExceptionDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getExceptionContainer_AllExceptions (), 
            exceptions);
   }

   public static EList<AttributeDef> getAttributes (InterfaceDef self)
   {
      return ContainerExt.getContents (self, AttributeDef.class);
   }

   public static EList<OperationDef> getOperations (InterfaceDef self)
   {
      return ContainerExt.getContents (self, OperationDef.class);
   }

   /**
    * @see InterfaceDef#getAllContents()
    */
   public static EList<Contained> getAllContents (InterfaceDef self)
   {
      EList<Contained> result = new BasicEList<Contained> ();

      for (Interface base : getAllBaseInterfaces (self))
      {
         if (base instanceof InterfaceDef)
         {
            result.addAll ( ((InterfaceDef) base).getInterfaceContents ());
         }
      }

      result.addAll (self.getContents ());

      return new UnmodifiableEList<Contained> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getInterfaceDef_AllContents (), 
            result);
   }

   /**
    * Helper for {@link #getAllBaseInterfaces(InterfaceDef)}
    */
   private static BasicEList<InterfaceDef> getAllBaseInterfaces (InterfaceDef self, BasicEList<InterfaceDef> bases)
   {
      for (InterfaceDef base : self.getBaseInterfaces ())
      {
         if (!bases.contains (base))
         {
            getAllBaseInterfaces (base, bases);
            bases.add (base);
         }
      }
      return bases;
   }

   /**
    * @see InterfaceDef#getAllBaseInterfaces()
    */
   public static EList<InterfaceDef> getAllBaseInterfaces (InterfaceDef self)
   {
      BasicEList<InterfaceDef> bases = getAllBaseInterfaces (self, new BasicEList<InterfaceDef> ());
      return new UnmodifiableEList<InterfaceDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getInterfaceDef_AllBaseInterfaces (), 
            bases);
   }

   public static EList<IRObject> getPrerequisites (InterfaceDef self)
   {
      Set<IRObject> result = new ArraySet<IRObject> (ContainerExt.getPrerequisites (self));
      result.addAll (self.getBaseInterfaces ());
      return new BasicEList.UnmodifiableEList<IRObject> (result.size (), result.toArray ());
   }
}
