package com.prismtech.idt.idlmm.impl;


import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;

import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.Container;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.IdlmmFactory;
import com.prismtech.idt.idlmm.OwnedIDLType;
import com.prismtech.idt.idlmm.PrimitiveDef;
import com.prismtech.idt.idlmm.PrimitiveKind;
import com.prismtech.idt.idlmm.SharedIDLType;
import com.prismtech.idt.idlmm.Typed;


public class TypedExt
{

   private static final Map<String, PrimitiveKind> m_primitives = new HashMap<String, PrimitiveKind> ();

   static
   {
      m_primitives.put ("float", PrimitiveKind.PK_FLOAT);
      m_primitives.put ("double", PrimitiveKind.PK_DOUBLE);
      m_primitives.put ("long double", PrimitiveKind.PK_LONGDOUBLE);
      m_primitives.put ("short", PrimitiveKind.PK_SHORT);
      m_primitives.put ("long", PrimitiveKind.PK_LONG);
      m_primitives.put ("long long", PrimitiveKind.PK_LONGLONG);
      m_primitives.put ("unsigned short", PrimitiveKind.PK_USHORT);
      m_primitives.put ("unsigned long", PrimitiveKind.PK_ULONG);
      m_primitives.put ("unsigned long long", PrimitiveKind.PK_ULONGLONG);
      m_primitives.put ("char", PrimitiveKind.PK_CHAR);
      m_primitives.put ("wchar", PrimitiveKind.PK_WCHAR);
      m_primitives.put ("boolean", PrimitiveKind.PK_BOOLEAN);
      m_primitives.put ("octet", PrimitiveKind.PK_OCTET);
      m_primitives.put ("any", PrimitiveKind.PK_ANY);
   }

   private static IdlmmFactory getFactory (Typed self)
   {
      return IdlmmFactory.eINSTANCE;
      // return (IdlmmFactory) self.eClass ().getEPackage ().getEFactoryInstance
      // ();
   }

   /**
    * Implementation of {@link Typed#setType(String)}
    */
   public static IDLType setType (Typed self, String name)
   {
      IDLType result = null;

      if ( (name == null) || (name.length () == 0))
      {
         self.setType ((IDLType) null);
         result = self.getType ();
      }
      else
      {
         PrimitiveKind kind = m_primitives.get (name);

         if (kind != null)
         {
            result = self.setType (kind);
         }
         else
         {
            for (EObject eobject = self; null != eobject; eobject = eobject.eContainer ())
            {
               if (eobject instanceof Container)
               {
                  Contained found = ((Container) eobject).lookup (name, true, true);

                  if (found instanceof IDLType)
                  {
                     self.setType ((IDLType) found);
                     result = self.getType ();
                     break;
                  }
               }
            }
         }
      }

      return result;
   }

   public static IDLType setType (Typed self, PrimitiveKind kind)
   {
      PrimitiveDef primitive = getFactory (self).createPrimitiveDef ();

      primitive.setKind (kind);
      self.setOwnedType (primitive);

      return primitive;
   }

   public static void setType (Typed self, IDLType type)
   {
      if (null == type)
      {
         self.setOwnedType (null);
         self.setSharedType (null);
      }
      else if (type instanceof OwnedIDLType)
      {
         self.setOwnedType ((OwnedIDLType) type);
      }
      else if (type instanceof SharedIDLType)
      {
         self.setSharedType ((SharedIDLType) type);
      }
   }

   public static IDLType getType (Typed self)
   {
      IDLType result = self.getOwnedType ();
      
      if (null != result)
      {
         return result;
      }
      return self.getSharedType (); 
   }

   public static EList<IRObject> getPrerequisites (Typed typed)
   {
      IDLType type = typed.getType ();
      if (type instanceof SharedIDLType)
      {
         EList<IRObject> result = new BasicEList<IRObject> ();
         result.add (type);
         return result;
      }
      return type.getPrerequisites ();
   }
}
