package com.prismtech.idt.idlmm.impl;

import java.util.Set;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.UniqueEList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.DelegatingEcoreEList.UnmodifiableEList;

import com.prismtech.idt.idlmm.AttributeDef;
import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.Initializer;
import com.prismtech.idt.idlmm.Interface;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.OperationDef;
import com.prismtech.idt.idlmm.Value;
import com.prismtech.idt.idlmm.ValueDef;
import com.prismtech.idt.idlmm.ValueMemberDef;


public class ValueExt
{
   public static EList<IRObject> getPrerequisites (ValueDef self)
   {
      Set<IRObject> result = new ArraySet<IRObject> (ContainerExt.getPrerequisites (self));
      result.addAll (self.getBaseValues ());
      result.addAll (self.getSupportedInterfaces ());
      return new BasicEList<IRObject> (result);
   }

   private static BasicEList<ValueDef> getAllBaseValues (ValueDef self, BasicEList<ValueDef> bases)
   {
      for (ValueDef base : self.getBaseValues ())
      {
         if (!bases.contains (base))
         {
            getAllBaseValues (base, bases);
            bases.add (base);
         }
      }
      return bases;
   }

   public static EList<ValueDef> getAllBaseValues (ValueDef self)
   {
      BasicEList<ValueDef> bases = getAllBaseValues (self, new BasicEList<ValueDef> ());
      return new UnmodifiableEList<ValueDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getValueDef_AllBaseValues (), 
            bases);
   }

   private static BasicEList<Interface> getAllSupportedInterfaces (ValueDef self, BasicEList<Interface> supportedList)
   {
      // get all supported interfaces from inherited values
      for (ValueDef base : self.getBaseValues ())
      {
         getAllSupportedInterfaces(base, supportedList);
      }
      
      // get all supported interfaces and inheriting interfaces
      for (Interface supported : self.getSupportedInterfaces ())
      {
         if (!supportedList.contains (supported))
         {
            // add all inherited interfaces of supported interface
            if (supported instanceof InterfaceDef)
            {
               supportedList.addAll (InterfaceDefExt.getAllBaseInterfaces ((InterfaceDef)supported));
            }
            
            // add supported interface itself
            supportedList.add (supported);
         }
      }
      return supportedList;
   }

   public static EList<Interface> getAllSupportedInterfaces (ValueDef self)
   {
      BasicEList<Interface> bases = getAllSupportedInterfaces (self, new UniqueEList<Interface> ());
      return new UnmodifiableEList<Interface> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getValueDef_AllSupportedInterfaces (), 
            bases);
   }

   public static EList<ValueMemberDef> getMembers (ValueDef self)
   {
      return ContainerExt.getContents (self, ValueMemberDef.class);
   }

   public static EList<AttributeDef> getAttributes (ValueDef self)
   {
      return ContainerExt.getContents (self, AttributeDef.class);
   }

   private static BasicEList<AttributeDef> getAllAttributes (ValueDef self, UniqueEList<AttributeDef> attributes)
   {
      // get all attributes from inherited values
      for (ValueDef base : self.getBaseValues ())
      {
         getAllAttributes (base, attributes);
      }
      // get all attributes from supported interfaces
      for (Interface base : self.getSupportedInterfaces ())
      {
         if (base instanceof InterfaceDef)
         {
            attributes.addAll ( ((InterfaceDef)base).getAllAttributes ());
         }
      }
      // add attributes from current value
      attributes.addAll (getAttributes (self));
      return attributes;
   }

   public static EList<AttributeDef> getAllAttributes (ValueDef self)
   {
      BasicEList<AttributeDef> attributes = getAllAttributes (self, new UniqueEList<AttributeDef> ());
      return new UnmodifiableEList<AttributeDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getAttributeContainer_AllAttributes (), 
            attributes);
   }

   public static EList<OperationDef> getOperations (ValueDef self)
   {
      return ContainerExt.getContents (self, OperationDef.class);
   }

   private static BasicEList<OperationDef> getAllOperations (ValueDef self, UniqueEList<OperationDef> operations)
   {
      BasicEList<OperationDef> result = new UniqueEList<OperationDef> ();

      // get operations from inherited values
      for (ValueDef base : self.getAllBaseValues ())
      {
          result.addAll ( base.getOperations ());
      }

      // get operations from supported interfaces
      for (Interface base : self.getAllSupportedInterfaces ())
      {
         if (base instanceof InterfaceDef)
         {
            result.addAll ( ((InterfaceDef) base).getOperations ());
         }
      }

      // get operations from the current value
      result.addAll (self.getOperations ());

      return new BasicEList.UnmodifiableEList<OperationDef> (result.size (), result.data ());
   }

   public static EList<OperationDef> getAllOperations (ValueDef self)
   {
      BasicEList<OperationDef> operations = getAllOperations (self, new UniqueEList<OperationDef> ());
      return new UnmodifiableEList<OperationDef> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getOperationContainer_AllOperations (), 
            operations);
   }

   public static EList<Initializer> getInitializers (ValueDef self)
   {
      return ContainerExt.getContents (self, Initializer.class);
   }

   public static EList<Contained> getAllContents (ValueDef self)
   {
      EList<Contained> result = new BasicEList<Contained> ();
      for (Value base : self.getBaseValues ())
      {
         if (base instanceof ValueDef)
         {
            result.addAll ( ((ValueDef) base).getAllContents ());
         }
      }
      for (Interface iface : self.getSupportedInterfaces ())
      {
         if (iface instanceof InterfaceDef)
         {
            result.addAll ( ((InterfaceDef) iface).getAllContents ());
         }
      }
      result.addAll (self.getContents ());

      return new UnmodifiableEList<Contained> (
            (EObjectImpl)self, 
            (EStructuralFeature)IdlmmPackageImpl.init ().getValueDef_AllContents (), 
            result);
   }
}
