package com.prismtech.idt.idlmm.impl;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;

import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IdlmmPackage;
import com.prismtech.idt.idlmm.OwnedIDLType;
import com.prismtech.idt.idlmm.SharedIDLType;
import com.prismtech.idt.idlmm.Typed;


/**
 * This {@link Adapter} keeps the {@link Typed#getSharedType() sharedType} and
 * {@link Typed#getOwnedType() ownedType} references synchronised with each
 * other. An instance of this adapter should be added in the constructor of
 * every subtype of {@link Typed}.
 * <p>
 * Note that this is not reentrant, but then neither is the generated EMF code
 */
public class TypedTypeOwnedTypeAdapter implements Adapter
{
   /**
    * Type {@link Typed} object whos {@link Typed#getSharedType() sharedType} and
    * {@link Typed#getOwnedType() ownedType} references we are keeping in sync.
    */
   private Typed m_typed = null;

   /**
    * When we receive a notification of either the {@link Typed#getSharedType() sharedType}
    * or {@link Typed#getOwnedType() ownedType} reference changing, this flag
    * indicates if we should also make the corresponding change to
    * {@link Typed#getOwnedType() ownedType} or {@link Typed#getSharedType() sharedType}.
    * We use this to prevent recursion.
    */
   private boolean m_sync = true;

   public void setTarget (Notifier target)
   {
      m_typed = (Typed) target;
      m_sync = true;
   }

   public Notifier getTarget ()
   {
      return m_typed;
   }

   public boolean isAdapterForType (Object type)
   {
      return false;
   }

   public void notifyChanged (Notification notification)
   {
      switch (notification.getFeatureID (Typed.class))
      {
         case IdlmmPackage.TYPED__SHARED_TYPE:
         {
            if (m_sync == true)
            {
               m_sync = false;

               Typed typed = (Typed) notification.getNotifier ();
               IDLType type = (IDLType) notification.getNewValue ();

               if (type instanceof SharedIDLType)
               {
                  typed.setOwnedType (null);
               }

               m_sync = true;
            }

            break;
         }

         case IdlmmPackage.TYPED__OWNED_TYPE:
         {
            if (m_sync == true)
            {
               m_sync = false;

               Typed typed = (Typed) notification.getNotifier ();
               IDLType type = (IDLType) notification.getNewValue ();

               if (type instanceof OwnedIDLType)
               {
                  typed.setSharedType (null);
               }

               m_sync = true;
            }

            break;
         }

         default:
         {
            break;
         }
      }
   }
}
