package com.prismtech.idt.idlmm.util;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.IdlmmPackage;
import com.prismtech.idt.idlmm.Named;
import com.prismtech.idt.idlmm.Repository;


public class IdlmmDescriber
{
   private static final Map<EClass, String> TYPENAMES = new HashMap<EClass, String> ();

   static
   {
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getAliasDef (), "typedef");
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getArrayDef (), "array");
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getAttributeDef (), "attribute");
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getConstantDef (), "const");
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getInterfaceDef (), "interface");
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getModuleDef (), "module");
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getStructDef (), "struct");
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getStructMember (), "struct member");
      TYPENAMES.put (IdlmmPackage.eINSTANCE.getEnumDef (), "enum");
      // etc....
   }

   /**
    * Returns a descriptive string for the specified {@link Object}, for use in
    * error messages
    */
   public static String describe (Object object)
   {
      if (object == null)
      {
         return "null";
      }

      String tname = getTypename (object);
      String oname = getObjectName (object);

      return tname + ( (tname == null) ? "" : " ") + "'" + oname + "'";
   }

   public static String[] describe (Object[] objects)
   {
      String[] result = new String[objects.length];

      for (int i = 0; i < objects.length; i++)
      {
         result[i] = describe (objects[i]);
      }

      return result;
   }

   private static String getTypename (Object object)
   {
      String result = null;

      if (object instanceof EObject)
      {
         EObject eobject = (EObject) object;
         EClass eclass = eobject.eClass ();

         result = TYPENAMES.get (eclass);

         if (result == null)
         {
            result = eclass.getName ();
         }
      }

      if (result == null)
      {
         result = object.getClass ().getSimpleName ();
      }

      return result;
   }

   private static String getObjectName (Object object)
   {
      String result = null;

      if (object instanceof Contained)
      {
         result = ((Contained) object).getAbsoluteName ();

         if (result.endsWith ("::"))
         {
            result += "<noname>";
         }
      }
      else if (object instanceof Named)
      {
         result = ((Named) object).getName ();

         if (result == null)
         {
            result = "<noname>";
         }
      }
      else if (object instanceof Repository)
      {
         result = "Repository";
      }
      else
      {
         result = object.toString ();
      }

      return result;
   }
}
