package com.prismtech.idt.idlmm.util;

public final class ScopedName
{
   private final String[] m_identifiers;

   public ScopedName (String name)
   {
      assert (name != null);
      assert (name.length () > 0);

      m_identifiers = name.split ("::");

      assert (m_identifiers != null);
      assert (m_identifiers.length > 0);
   }

   public ScopedName (String[] identifiers)
   {
      assert (identifiers != null);
      assert (identifiers.length > 0);

      m_identifiers = identifiers;
   }

   public boolean isBlank ()
   {
      // FIXME: according to asserts it is impossible
      return (m_identifiers.length == 0);
   }

   public boolean isAbsolute ()
   {
      return (m_identifiers.length > 0) && (m_identifiers[0].length () == 0);
   }

   public boolean isRelative ()
   {
      return (m_identifiers.length > 0) && (m_identifiers[0].length () > 0);
   }

   public int size ()
   {
      return m_identifiers.length;
   }

   public String get (int index)
   {
      return m_identifiers[index];
   }

   public ScopedName ltrim (int begin) // inclusive
   {
      assert (begin <= m_identifiers.length);

      String[] identifiers = new String[m_identifiers.length - begin];

      System.arraycopy (m_identifiers, begin, identifiers, 0, identifiers.length);

      return new ScopedName (identifiers);
   }

   public ScopedName rtrim (int count)
   {
      assert (count <= m_identifiers.length);
      
      String[] identifiers = new String[m_identifiers.length - count];

      System.arraycopy (m_identifiers, 0, identifiers, 0, identifiers.length);
      
      return new ScopedName (identifiers);      
   }

   @Override
   public String toString ()
   {
      StringBuilder builder = new StringBuilder ();

      for (int i = 0; i < m_identifiers.length; i++)
      {
         if (i > 0)
         {
            builder.append ("::");
         }

         builder.append (m_identifiers[i]);
      }

      return builder.toString ();
   }
}
