/***************************************************************************************************
 * - COPYRIGHT NOTICE - * * The information contained herein and which follows is proprietary *
 * information and is the property of PrismTech Corporation. This * information is not to be
 * divulged, released, copied, reproduced, or * conveyed to unauthorized personnel, companies, or
 * other institutions * without the direct and expressed approval in writing of PrismTech *
 * Corporation. * * * - DISCLAIMER - * Neither PrismTech Limited nor PrismTech Corporation
 * ("PrismTech") * are under any obligation to support the use of this code which is * provided
 * without warranty of any kind. PrismTech shall (except as * otherwise prohibited by law) have no
 * liability whatsoever in relation * to its use. * *
 **************************************************************************************************/
package com.prismtech.idt.imports.idl.driver;

import java.io.File;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;

import com.prismtech.idt.idlmm.Repository;
import com.prismtech.idt.imports.idl.importer.IIdlImporterParams;
import com.prismtech.idt.imports.idl.parser.IIdlParserParams;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;
import com.prismtech.idt.imports.idl.preprocessor.IIdlPreprocessorParams;


/**
 * This interface encapsulates an IDL file to be processed by the {@link IIdlImportDriver}. As well
 * as specifying the file to be processed, it also allows the user to specify options that affect
 * the preprocessing, parsing, repository creation and validation for that specific file.
 * <p>
 * Once the user has invoked the {@link IIdlImportDriver#build(IIdlImportDriverParams)} method, this
 * interface provides access to the results of the build process. This includes the files
 * dependencies, the preprocessor output, the parsers AST, the {@link Repository} and the
 * {@link IStatus} of the build process, for the specific {@link #getIdlFile() file}.
 * <p>
 * This interface is not designed to be implemented by clients. Instances can be created using the
 * {@link IIdlImportDriverParams#add(File)} method.
 * 
 * @see IIdlImportDriverParams#add(File)
 * @see IIdlImportDriverParams#get(File)
 */
public interface IIdlImportDriverParam
{
   public File getIdlFile ();

   public IIdlPreprocessorParams getPreprocessorParams ();

   public IIdlParserParams getParserParams ();

   public IIdlImporterParams getImporterParams ();

   public Map<File, List<File>> getDependencies ();

   public String getPreprocessorOutput ();

   public IdlParserAST getAST ();

   public Repository getRepository ();

   public IStatus getStatus ();
}
