package com.prismtech.idt.imports.idl.internal.driver;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;

import com.prismtech.idt.idlmm.Repository;
import com.prismtech.idt.imports.idl.driver.IIdlImportDriver;
import com.prismtech.idt.imports.idl.driver.IIdlImportDriverParam;
import com.prismtech.idt.imports.idl.importer.IIdlImporterParams;
import com.prismtech.idt.imports.idl.parser.IIdlParserParams;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;
import com.prismtech.idt.imports.idl.preprocessor.IIdlPreprocessorParams;

class IdlImportDriverParam implements IIdlImportDriverParam
{
   private final File idlFile;

   private List<IIdlImportDriverParam> driverParams;

   private IIdlPreprocessorParams preprocessorParams;

   private IIdlParserParams parserParams;

   private IIdlImporterParams importerParams;

   private String preprocessorOutput;

   private Map<File, List<File>> dependencies;

   private IdlParserAST parserAST;

   private Repository repository;

   private List<IStatus> stati = new ArrayList<IStatus> ();

   public IdlImportDriverParam (File idlFile)
   {
      this.idlFile = idlFile;
   }

   public IdlParserAST getAST ()
   {
      return parserAST;
   }

   public Map<File, List<File>> getDependencies ()
   {
      return dependencies;
   }

   public List<IIdlImportDriverParam> getDriverParams ()
   {
      return driverParams;
   }

   public File getIdlFile ()
   {
      return idlFile;
   }

   public String getPreprocessorOutput ()
   {
      return preprocessorOutput;
   }

   public IIdlPreprocessorParams getPreprocessorParams ()
   {
      return preprocessorParams;
   }

   public void setPreprocessorParams (IIdlPreprocessorParams params)
   {
      this.preprocessorParams = params;
   }

   public IIdlParserParams getParserParams ()
   {
      return parserParams;
   }

   public void setParserParams (IIdlParserParams params)
   {
      this.parserParams = params;
   }

   public IIdlImporterParams getImporterParams ()
   {
      return importerParams;
   }

   public void setImporterParams (IIdlImporterParams params)
   {
      this.importerParams = params;
   }

   public Repository getRepository ()
   {
      return repository;
   }

   public void setAST (IdlParserAST parserAST)
   {
      this.parserAST = parserAST;
   }

   public void setDependencies (Map<File, List<File>> dependencies)
   {
      this.dependencies = dependencies;
   }

   public void setPreprocessorOutput (String preprocessorOutput)
   {
      this.preprocessorOutput = preprocessorOutput;
   }

   public void setRepository (Repository repository)
   {
      this.repository = repository;
   }

   public IStatus getStatus ()
   {
      MultiStatus status = new MultiStatus (IIdlImportDriver.PLUGIN_ID, 0, "", null);
      for (IStatus s : stati)
         status.add (s);
      return status;
   }

   public void addStatus (IStatus status)
   {
      stati.add (status);
   }
   
   @Override
   public String toString ()
   {
      return getClass ().getSimpleName () + "[" + idlFile + "]";
   }
}
