package com.prismtech.idt.imports.idl.internal.driver;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import com.prismtech.idt.imports.idl.driver.IIdlImportDriverParam;
import com.prismtech.idt.imports.idl.driver.IIdlImportDriverParams;


class IdlImportDriverParams implements IIdlImportDriverParams
{
   private final IdlImportDriver m_driver;

   private final List<IIdlImportDriverParam> m_params = new ArrayList<IIdlImportDriverParam> ();

   IdlImportDriverParams (IdlImportDriver driver)
   {
      m_driver = driver;
   }

   IdlImportDriver getDriver ()
   {
      return m_driver;
   }

   public IIdlImportDriverParam add (File idlfile)
   {
      IdlImportDriverParam result = (IdlImportDriverParam) get (idlfile);

      if (result == null)
      {
         result = new IdlImportDriverParam (idlfile);

         result.setPreprocessorParams (m_driver.getPreprocessor ().createPreprocessorParams ());
         result.setParserParams (m_driver.getParser ().createParserParams ());
         result.setImporterParams (m_driver.getImporter ().createImporterParams ());

         m_params.add (result);
      }

      return result;
   }

   public IIdlImportDriverParam get (File idlfile)
   {
      for (IIdlImportDriverParam file : m_params)
      {
         if (file.getIdlFile ().equals (idlfile))
         {
            return file;
         }
      }

      return null;
   }

   public boolean contains (File idlfile)
   {
      return get (idlfile) != null;
   }

   public boolean remove (File idlfile)
   {
      IIdlImportDriverParam param = get (idlfile);

      if (param == null)
      {
         return false;
      }

      return m_params.remove (param);
   }

   public int size ()
   {
      return m_params.size ();
   }

   public Iterator<IIdlImportDriverParam> iterator ()
   {
      return Collections.unmodifiableList (m_params).iterator ();
   }
   
   @Override
   public String toString ()
   {
      return getClass().getSimpleName () + "[" + m_params.size () + "]";
   }
}
