package com.prismtech.idt.imports.idl.internal.driver;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;

import com.prismtech.idt.imports.idl.parser.IIdlParser;
import com.prismtech.idt.imports.idl.parser.IIdlParserStatus;

public class IdlParserStatusAdapter extends MultiStatus implements IIdlParserStatus
{
   private final IIdlParserStatus m_status;
   
   public IdlParserStatusAdapter(IIdlParserStatus status)
   {
      super(IIdlParser.PLUGIN_ID, 0, status.getMessage (), status.getException ());
      m_status = status;
      if(status.isMultiStatus ())
      {
         for(IIdlParserStatus child : status.getChildStati ())
         {
            this.add(new IdlParserStatusAdapter(child));
         }
      }
      else
      {
         switch(status.getSeverity())
         {
            case IIdlParserStatus.OK:
               setSeverity (IStatus.OK);
               break;
            case IIdlParserStatus.INFO:
               setSeverity (IStatus.INFO);
               break;
            case IIdlParserStatus.WARNING:
               setSeverity (IStatus.WARNING);
               break;
            case IIdlParserStatus.ERROR:
               setSeverity (IStatus.ERROR);
               break;
            case IIdlParserStatus.CANCEL:
               setSeverity (IStatus.CANCEL);
               break;
         }
      }
   }

   public boolean isMultiStatus ()
   {
      return m_status.isMultiStatus ();
   }

   public IIdlParserStatus[] getChildStati ()
   {
      return m_status.getChildStati ();
   }

   public int getColumn ()
   {
      return m_status.getColumn ();
   }

   public String getFilename ()
   {
      return m_status.getFilename ();
   }

   public int getLine ()
   {
      return m_status.getLine ();
   }
}
