/*******************************************************************************
 * - COPYRIGHT NOTICE - * * The information contained herein and which follows
 * is proprietary * information and is the property of PrismTech Corporation.
 * This * information is not to be divulged, released, copied, reproduced, or *
 * conveyed to unauthorized personnel, companies, or other institutions *
 * without the direct and expressed approval in writing of PrismTech *
 * Corporation. * * * - DISCLAIMER - * Neither PrismTech Limited nor PrismTech
 * Corporation ("PrismTech") * are under any obligation to support the use of
 * this code which is * provided without warranty of any kind. PrismTech shall
 * (except as * otherwise prohibited by law) have no liability whatsoever in
 * relation * to its use. * *
 ******************************************************************************/

package com.prismtech.idt.imports.idl.importer;

import java.util.Collection;

import org.eclipse.core.runtime.IStatus;

import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.Repository;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


/**
 * IDL model importer interface
 */
public interface IIdlImporter
{
   public IIdlImporterParams createImporterParams ();

   /**
    * Sets the current {@link IIdlSourceComposer} that will be used during
    * {@link #build(IdlParserAST, Collection) build()} to translate the
    * information from {@link IdlParserAST} attributes into a string suitable
    * for {@link IRObject#setSource(String)}
    * 
    * @param composer
    *           The {@link IIdlSourceComposer} to be used during
    *           {@link #build(IdlParserAST, Collection) build()}
    */
   public void setSourceComposer (IIdlSourceComposer composer);

   /**
    * Registers the type of pragmas matching a specific pattern
    * 
    * @param pattern
    *           a regular expression identifying this pragma
    * @param type
    *           the model type of this pragma; can be ScopePragma, PrePragma or
    *           PostPragma
    */
   public void addPragmaHandler (IIdlPragmaHandler handler);

   /**
    * Builds IDL model from abstract syntax tree.
    * <code>dependencyResolver</code> is used to resolve model elements
    * contained in repositories which are prerequisites for the repository being
    * built.
    * 
    * @param ast
    *           IDL abstract syntax tree
    * @param includes
    * @param sourceComposer
    *           composer which composes "source" attribute of an object of the
    *           model
    * @return resulting repository
    */
   public IStatus build (IIdlImporterParams params, IdlParserAST ast);

   /**
    * Returns import status. Calling this method only has sense after <code>
    * build</code>
    * was called, status is empty otherwise.
    * 
    * @return build status
    */
   public Repository getRepository ();

   /**
    * Returns import status. Calling this method only has sense after <code>
    * build</code>
    * was called, status is empty otherwise.
    * 
    * @return build status
    */
   public IStatus getStatus ();
}
