package com.prismtech.idt.imports.idl.internal.importer;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import antlr.RecognitionException;

import com.prismtech.idt.imports.idl.importer.IIdlImporterStatus;
import com.prismtech.idt.imports.idl.importer.IdlImporterPlugin;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class IdlImporterStatus extends Status implements IIdlImporterStatus
{
   private final String m_filename;

   private final int m_line;

   private final int m_column;

   public IdlImporterStatus (IdlTreeParser parser, int severity, String message, Throwable exception)
   {
      super (severity, IdlImporterPlugin.PLUGIN_ID, message, exception);

      m_filename = ((IdlParserAST) parser.getAST ()).getFilename ();
      m_line = parser.getAST ().getLine ();
      m_column = parser.getAST ().getColumn ();
   }

   public IdlImporterStatus (IdlParserAST ast, int severity, String message, Throwable exception)
   {
      super (severity, IdlImporterPlugin.PLUGIN_ID, message, exception);

      m_filename = ast.getFilename ();
      m_line = ast.getLine ();
      m_column = ast.getColumn ();
   }

   public IdlImporterStatus (IdlParserAST ast, int severity, String message)
   {
      super (severity, IdlImporterPlugin.PLUGIN_ID, message);

      m_filename = ast.getFilename ();
      m_line = ast.getLine ();
      m_column = ast.getColumn ();
   }

   public IdlImporterStatus (RecognitionException exception)
   {
      super (IStatus.ERROR, IdlImporterPlugin.PLUGIN_ID, exception.getMessage (), exception);

      m_filename = exception.getFilename ();
      m_line = exception.getLine ();
      m_column = exception.getColumn ();
   }

   public String getFilename ()
   {
      return m_filename;
   }

   public int getLine ()
   {
      return m_line;
   }

   public int getColumn ()
   {
      return m_column;
   }

}
