/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */
package com.prismtech.idt.imports.idl.internal.importer;

/**
 * @author Yuriy Kravchenko
 */

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.ListIterator;
import java.util.Set;
import java.util.Stack;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.EList;

import com.prismtech.idt.idlmm.AddOperator;
import com.prismtech.idt.idlmm.AliasDef;
import com.prismtech.idt.idlmm.BitwiseAndOperator;
import com.prismtech.idt.idlmm.BitwiseComplementOperator;
import com.prismtech.idt.idlmm.BitwiseLeftShiftOperator;
import com.prismtech.idt.idlmm.BitwiseOrOperator;
import com.prismtech.idt.idlmm.BitwiseRightShiftOperator;
import com.prismtech.idt.idlmm.BitwiseXorOperator;
import com.prismtech.idt.idlmm.ComponentDef;
import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.Container;
import com.prismtech.idt.idlmm.DivideOperator;
import com.prismtech.idt.idlmm.EnumDef;
import com.prismtech.idt.idlmm.EventDef;
import com.prismtech.idt.idlmm.ExceptionDef;
import com.prismtech.idt.idlmm.Expression;
import com.prismtech.idt.idlmm.HomeDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.IdlmmFactory;
import com.prismtech.idt.idlmm.Interface;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.InterfaceModifier;
import com.prismtech.idt.idlmm.Literal;
import com.prismtech.idt.idlmm.Member;
import com.prismtech.idt.idlmm.ModuleDef;
import com.prismtech.idt.idlmm.ModuloOperator;
import com.prismtech.idt.idlmm.MultiplyOperator;
import com.prismtech.idt.idlmm.NegativeOperator;
import com.prismtech.idt.idlmm.OperationMode;
import com.prismtech.idt.idlmm.PositiveOperator;
import com.prismtech.idt.idlmm.PrimitiveDef;
import com.prismtech.idt.idlmm.PrimitiveKind;
import com.prismtech.idt.idlmm.Repository;
import com.prismtech.idt.idlmm.SharedIDLType;
import com.prismtech.idt.idlmm.StructDef;
import com.prismtech.idt.idlmm.SubtractOperator;
import com.prismtech.idt.idlmm.Typed;
import com.prismtech.idt.idlmm.UnionDef;
import com.prismtech.idt.idlmm.UnionMember;
import com.prismtech.idt.idlmm.ValueDef;
import com.prismtech.idt.idlmm.ValueMemberDef;
import com.prismtech.idt.idlmm.Visibility;
import com.prismtech.idt.imports.idl.importer.IIdlPragmaHandler;
import com.prismtech.idt.imports.idl.importer.IIdlSourceComposer;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ArrayNodePropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.AttributeDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.AttributeExceptionDeclarant;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ComponentDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ComponentDefPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.CompoundNodePropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ConstTypePropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ConsumesPortDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ContainerType;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ContainerTypeHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.DeclaratorHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.EmitsPortDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.EnumPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.EventDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.EventPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ExceptionDeclarant;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ExpressionHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ExpressionOperable;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.FactoryOperationPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.FinderOperationPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.HomeDefPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.InitializerDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.InitializerParamDeclHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.InterfaceDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.InterfaceDefPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.LiteralHelper;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.LiteralHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.NativeDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.NodePropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.OperationContextHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.OperationDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ParamDeclHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.PragmaContext;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.PragmaDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.PrimitiveDefPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ProvidesPortDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.PublishesPortDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.RepositoryPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ResolvingContext;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ScopedNameNodePropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ScopedNameResolver;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.SequenceTypePropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.StringDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.TypeIdHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.TypePrefixHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.TypePropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.TypedefTypePropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.UnionDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.UsesPortDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ValueBoxDclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ValueDeclPropertiesHolder;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.ValuePropertiesHolder;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class IdlmmBuilder
{
   private final IdlmmFactory m_factory = IdlmmFactory.eINSTANCE;

   private Repository m_repository = null;

   private final Stack<NodePropertiesHolder> m_stack = new Stack<NodePropertiesHolder> ();

   private final Set<SharedIDLType> m_forwards = new HashSet<SharedIDLType> ();

   private final ResolvingContext m_resolvingContext = new ResolvingContext ();

   private final ScopedNameResolver m_scNameResolver;

   private final IIdlSourceComposer m_sourceAttrComposer;

   private List<IIdlPragmaHandler> m_pragmaHandlers;

   private List<PragmaDeclPropertiesHolder> pragmas = new ArrayList<PragmaDeclPropertiesHolder> ();

   private StatusHolder m_statusHolder;

   /* package */IdlmmBuilder (StatusHolder statusHolder, IIdlSourceComposer sourceAttrComposer,
         List<IIdlPragmaHandler> pragmaHandlers)
   {
      m_sourceAttrComposer = sourceAttrComposer;
      m_pragmaHandlers = pragmaHandlers;
      m_statusHolder = statusHolder;
      m_scNameResolver = new ScopedNameResolver (m_resolvingContext, m_statusHolder);
   }

   // ---------------------------------------------------------------------------------------------------
   private <T extends NodePropertiesHolder> void push (T object)
   {
      m_stack.push (object);
   }

   private <T extends NodePropertiesHolder> T pop (Class<T> type)
   {
      assert (type != null);
      assert (m_stack.size () > 0);
      assert (type.isInstance (m_stack.peek ()));

      return type.cast (m_stack.pop ());
   }

   private <T extends NodePropertiesHolder> T peek (Class<T> type)
   {
      assert (type != null);
      assert (m_stack.size () > 0);
      assert (type.isInstance (m_stack.peek ()));

      return type.cast (m_stack.peek ());
   }

   /**
    * Returns the first object that was pushed onto the stack
    */
   /* package */<T extends Object> T getRootObject (Class<T> type)
   {
      assert (type != null);
      assert (type == Repository.class);

      // FIXME:
      if (this.m_forwards.size () > 0)
      {
         throw new RuntimeException ("Unresolved forward declarations " + this.m_forwards);
      }

      return type.cast (m_repository);
   }

   /* package */void startRepository (IdlParserAST ast)
   {
      assert (m_stack.size () == 0);
      assert (m_resolvingContext.getSize () == 0);
      assert (m_repository == null);

      RepositoryPropertiesHolder repHolder = new RepositoryPropertiesHolder (this.composeSourceAttribute (ast));
      m_resolvingContext.setCurrentContext ((Container) repHolder.getDefinition ());

      m_repository = Repository.class.cast (repHolder.getDefinition ());

      this.push (repHolder);
   }

   /* package */void endRepository ()
   {
      assert (m_stack.size () == 1);
      assert (m_resolvingContext.getSize () == 1);

      for (PragmaDeclPropertiesHolder pragma : pragmas)
      {
         IStatus status = pragma.bindPragma ();
         if (!status.isOK ())
         {
            m_statusHolder.addStatusItem (status);
         }
      }

      m_resolvingContext.leaveCurrentContext ();
      this.pop (TypePropertiesHolder.class);
   }

   // -------------------------------------------------------------------------------
   // MODULE
   /* package */void startModule (IdlParserAST ast, IdlParserAST module_literal)
   {
      String name = ast.getText ();
      // we check if module with such name is already defined in the current
      // scope. If so - current module is
      // a continuation of the module defined above, and we do not create a new
      // module, just continue filling previously defined one
      ModuleDef mod = this.findLocalDeclaration (name, ModuleDef.class);

      if (mod != null)
      {
         // module with such name is already defined in the current scope
         TypePropertiesHolder typeHolder = new ContainerTypeHolder (mod, this.composeSourceAttribute (ast), ast, module_literal);
         this.push (typeHolder);
         m_resolvingContext.setCurrentContext (mod);
      }
      else
      {
         // We bind this module with the parent container in this point - when
         // module is started.
         // It is important that we expect that TOS should be an instance of
         // Container (module, repository) since all inner
         // definitions of that container have to already been binded with the
         // container.
         Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();

         // create module holder
         ContainerTypeHolder typeHolder = new ContainerTypeHolder (ast.getText (), ContainerType.MODULE, this
               .composeSourceAttribute (ast), ast, module_literal);
         this.push (typeHolder);
         // get ModuleDef from the module holder
         List<IRObject> defs = typeHolder.getDefinitions ();

         this.bindDefinitionsWithContainer (defs, parent);
         m_resolvingContext.setCurrentContext ((Container) defs.get (0));
      }
   }

   /* package */void endModule ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   // -------------------------------------------------------------------------------
   // STRUCT
   /* package */void startStruct (IdlParserAST ast, IdlParserAST literal)
   {
      String name = ast.getText ();
      StructDef forward = this.findLocalDeclaration (name, StructDef.class);

      if (this.m_forwards.contains (forward))
      {
         // this struct has forward declaration. So we create StructDeclHolder
         // and put resolved type inside
         this.m_forwards.remove (forward);
         TypePropertiesHolder typeHolder = new ContainerTypeHolder (forward, this.composeSourceAttribute (ast), ast, literal);
         this.push (typeHolder);
         m_resolvingContext.setCurrentContext (forward);
      }
      else
      {
         Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
         TypePropertiesHolder typeHolder = new ContainerTypeHolder (name, ContainerType.STRUCT, this.composeSourceAttribute (ast),
               ast, literal);
         this.push (typeHolder);
         // get StructDef from the Struct holder
         List<IRObject> defs = typeHolder.getDefinitions ();

         this.bindDefinitionsWithContainer (defs, parent);
         m_resolvingContext.setCurrentContext ((Container) defs.get (0));
      }
   }

   /* package */void endStruct ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   // -------------------------------------------------------------------------------------
   // MEMBER
   /* package */void startMember ()
   {
   }

   /* package */void endMember (IdlParserAST semi)
   {
      TypePropertiesHolder memberHolder = this.pop (TypePropertiesHolder.class);

      TypePropertiesHolder structHolder = this.peek (TypePropertiesHolder.class);
      Container container = (Container) structHolder.getDefinition ();

      for (DeclaratorHolder decl : memberHolder.getDeclarators ())
      {
         this.createMember (decl, container, memberHolder.getDefinition (), semi);
      }
   }

   // ------------------------------------------------------------------------------primitive
   // TYPES
   // --------------------------------------------------------- float type
   /* package */void startFloatType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_FLOAT, m_repository, this
            .composeSourceAttribute (ast), ast);
      this.push (nodeHolder);
   }

   /* package */void endFloatType ()
   {
   }

   // --------------------------------------------------------- double type
   /* package */void startDoubleType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_DOUBLE, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endDoubleType ()
   {
   }

   // --------------------------------------------------------- short type
   /* package */void startShortType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_SHORT, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endShortType ()
   {
   }

   // --------------------------------------------------------- long type
   /* package */void startLongType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_LONG, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endLongType ()
   {
   }

   // --------------------------------------------------------- longlong type
   /* package */void startLongLongType (IdlParserAST ast, IdlParserAST i2)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_LONGLONG, m_repository, this
            .composeSourceAttribute (ast), ast, i2);

      this.push (nodeHolder);

   }

   /* package */void endLongLongType ()
   {
   }

   // --------------------------------------------------------- ushort type
   /* package */void startUShortType (IdlParserAST ast, IdlParserAST i2)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_USHORT, m_repository, this
            .composeSourceAttribute (ast), ast, i2);

      this.push (nodeHolder);
   }

   /* package */void endUShortType ()
   {
   }

   // --------------------------------------------------------- ulong type
   /* package */void startULongType (IdlParserAST ast, IdlParserAST i2)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_ULONG, m_repository, this
            .composeSourceAttribute (ast), ast, i2);

      this.push (nodeHolder);
   }

   /* package */void endULongType ()
   {
   }

   // --------------------------------------------------------- ulonglong type
   /* package */void startULongLongType (IdlParserAST ast, IdlParserAST i2, IdlParserAST i3)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_ULONGLONG, m_repository, this
            .composeSourceAttribute (ast), ast, i2, i3);

      this.push (nodeHolder);
   }

   /* package */void endULongLongType ()
   {
   }

   // --------------------------------------------------------- long double type
   /* package */void startLongDoubleType (IdlParserAST ast, IdlParserAST i2)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_LONGDOUBLE, m_repository, this
            .composeSourceAttribute (ast), ast, i2);

      this.push (nodeHolder);
   }

   /* package */void endLongDoubleType ()
   {
   }

   // --------------------------------------------------------- char type
   /* package */void startCharType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_CHAR, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endCharType ()
   {
   }

   // --------------------------------------------------------- wchar type
   /* package */void startWCharType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_WCHAR, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endWCharType ()
   {
   }

   // --------------------------------------------------------- boolean type
   /* package */void startBooleanType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_BOOLEAN, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endBooleanType ()
   {
   }

   // --------------------------------------------------------- octet type
   /* package */void startOctetType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_OCTET, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endOctetType ()
   {
   }

   // --------------------------------------------------------- any type
   /* package */void startAnyType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_ANY, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endAnyType ()
   {
   }

   // --------------------------------------------------------- object type
   /* package */void startObjectType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_OBJREF, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endObjectType ()
   {
   }

   // --------------------------------------------------------- valuebase type
   /* package */void startValueBaseType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_VALUE_BASE, m_repository, this
            .composeSourceAttribute (ast), ast);

      this.push (nodeHolder);
   }

   /* package */void endValueBaseType ()
   {
   }

   // -----------------------------------------------------------string type
   /* package */void startStringType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new StringDeclPropertiesHolder (PrimitiveKind.PK_STRING, this.composeSourceAttribute (ast),
            ast);
      this.push (nodeHolder);
   }

   /* package */void endStringType ()
   {
      this.endStringTypeImpl ();
   }

   // -----------------------------------------------------------wstring type
   /* package */void startWStringType (IdlParserAST ast)
   {
      TypePropertiesHolder nodeHolder = new StringDeclPropertiesHolder (PrimitiveKind.PK_WSTRING,
            this.composeSourceAttribute (ast), ast);
      this.push (nodeHolder);
   }

   /* package */void endWStringType ()
   {
      this.endStringTypeImpl ();
   }

   private void endStringTypeImpl ()
   {
      TypePropertiesHolder typeHolder = this.peek (TypePropertiesHolder.class);
      if (typeHolder instanceof ScopedNameNodePropertiesHolder)
      {
         ScopedNameNodePropertiesHolder scHolder = this.pop (ScopedNameNodePropertiesHolder.class);

         StringDeclPropertiesHolder stringHolder = this.peek (StringDeclPropertiesHolder.class);

         // resolving expression and setting "source" attribute to it
         Expression ex = m_scNameResolver.resolveExpression (scHolder);
         stringHolder.setExpression (ex);
      }

   }

   // ----------------------------------------------------------------------------------
   // Fixed //n.r. for the DCPS
   // /*package*/ void startFixedType(IdlParserAST ast1, IdlParserAST ast2)
   // {
   // // this.m_currNodeProperties = new
   // // FixedNodePropertiesHolder(ast1.getText(), ast2.getText());
   // // this.m_currNodeProperties = new
   // // NodePropertiesHolder(PrimitiveKind.PK_FLOAT);
   // }
   //
   // /*package*/ void endFixedType()
   // {
   // }

   // ---------------------------------------------------------------------------
   // SCOPED NAME
   /* package */void startScopedName ()
   {
   }

   /* package */void endScopedName (StringBuilder sb, IdlParserAST... asts)
   {
      NodePropertiesHolder nodeHolder = this.peek (NodePropertiesHolder.class);

      String scopedName = sb.toString ();
      String sourceVal = this.composeSourceAttribute (asts[0]);

      if (nodeHolder instanceof ExpressionHolder)
      {
         // we should add resolved scoped name to expression right now;
         // create dummy ScopedName holder and pass it to ScopedName resolver
         Expression expr = m_scNameResolver.resolveExpression (new ScopedNameNodePropertiesHolder (scopedName, null, sourceVal,
               asts));
         ExpressionHolder eHolder = (ExpressionHolder) nodeHolder;
         eHolder.addOperand (expr);
      }
      else
      {
         TypePropertiesHolder nHolder = new ScopedNameNodePropertiesHolder (scopedName, m_scNameResolver, sourceVal, asts);
         this.push (nHolder);
      }
   }

   // ---------------------------------------------------------------------------
   // SEQUENCE
   /* package */void startSequenceType ()
   {
   }

   /* package */void endSequenceType ()
   {
      // If in this moment Scoped name Holder is stored in the top of the stack
      // it means that a bound of the sequence is defined
      // using scoped name.
      TypePropertiesHolder typeHolder = this.peek (TypePropertiesHolder.class);
      if (typeHolder instanceof ScopedNameNodePropertiesHolder)
      {
         ScopedNameNodePropertiesHolder scHolder = this.pop (ScopedNameNodePropertiesHolder.class);
         SequenceTypePropertiesHolder seqHolder = this.peek (SequenceTypePropertiesHolder.class);

         // resolving expression and setting "source" attribute to it
         Expression ex = m_scNameResolver.resolveExpression (scHolder);
         seqHolder.setExpression (ex);
      }

   }

   /* package */void sequenceMetaPoint (IdlParserAST ast)
   {
      TypePropertiesHolder nHolder = this.pop (TypePropertiesHolder.class);
      SequenceTypePropertiesHolder seqHolder = new SequenceTypePropertiesHolder (nHolder, this.composeSourceAttribute (ast), ast);
      this.push (seqHolder);
   }

   // ---------------------------------------------------------------------------
   // ARRAY
   /* package */void startArrayDeclarator (IdlParserAST ast)
   {
      ArrayNodePropertiesHolder nodeHolder = null;

      TypePropertiesHolder prevHolder = this.peek (TypePropertiesHolder.class);
      // check whether parent holder is an array holder

      if (prevHolder instanceof TypedefTypePropertiesHolder)
      {
         // declaration an array via typedef declaration is the special case.
         // Hence we need to handle this situation
         // in a special way
         TypedefTypePropertiesHolder typedefHolder = (TypedefTypePropertiesHolder) prevHolder;
         TypePropertiesHolder typedefInnerHolder = typedefHolder.getTypeHolder ();
         // if an inner holder is not an ArrayHolder - it means that we have to
         // create ArrayHolder
         // and put it to the typedef holder as inner type holder
         if (! (typedefInnerHolder instanceof ArrayNodePropertiesHolder))
         {
            nodeHolder = new ArrayNodePropertiesHolder (typedefInnerHolder, this.composeSourceAttribute (ast), ast);
            nodeHolder.addDeclaratorHolder (new DeclaratorHolder (ast, this.composeSourceAttribute (ast)));
            typedefHolder.setTypeHolder (nodeHolder);
         }
         else
         {
            ((ArrayNodePropertiesHolder) typedefInnerHolder).addDeclaratorHolder (new DeclaratorHolder (ast, this
                  .composeSourceAttribute (ast)));
         }
      }
      else if (! (prevHolder instanceof ArrayNodePropertiesHolder))
      {
         // this is the first array declaration after type declaration. Then we
         // pop type declaration, create array declaration holder with type
         // holder inside.
         prevHolder = this.pop (TypePropertiesHolder.class);

         // checking whether entity which is "type" of the array is needs
         // binding with parent container. It needs in
         // case when the entity is defined straight ahead of the array's
         // declaration. E.g. " enum E {a,b } arr[10] ";
         TypePropertiesHolder containerHolder = this.peek (TypePropertiesHolder.class);
         this.bindDefinitionsWithContainer (prevHolder.getDefinitions (), (Container) containerHolder.getDefinition ());

         nodeHolder = new ArrayNodePropertiesHolder (prevHolder, this.composeSourceAttribute (ast), ast);
         nodeHolder.addDeclaratorHolder (new DeclaratorHolder (ast, this.composeSourceAttribute (ast)));

         this.push (nodeHolder);
      }
      else
      {
         nodeHolder = (ArrayNodePropertiesHolder) prevHolder;
         nodeHolder.addDeclaratorHolder (new DeclaratorHolder (ast, this.composeSourceAttribute (ast)));
      }
   }

   /* package */void endArrayDeclarator ()
   {
      ArrayNodePropertiesHolder aHolder = null;
      TypePropertiesHolder typeHolder = this.peek (TypePropertiesHolder.class);

      if (typeHolder instanceof ScopedNameNodePropertiesHolder)
      {
         List<ScopedNameNodePropertiesHolder> scNames = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);
         aHolder = this.findArrayPropertiesHolder ();
         for (int i = scNames.size () - 1; i >= 0; --i)
         {
            ScopedNameNodePropertiesHolder scHolder = scNames.get (i);
            Expression ex = m_scNameResolver.resolveExpression (scHolder);
            aHolder.setExpression (ex);
         }
      }
      else
      {
         aHolder = this.findArrayPropertiesHolder ();
      }

      aHolder.startNextBoundList ();
   }

   private ArrayNodePropertiesHolder findArrayPropertiesHolder ()
   {
      ArrayNodePropertiesHolder aHolder = null;
      TypePropertiesHolder typeHolder = this.peek (TypePropertiesHolder.class);

      if (typeHolder instanceof ArrayNodePropertiesHolder)
      {
         aHolder = (ArrayNodePropertiesHolder) typeHolder;
      }
      else if (typeHolder instanceof TypedefTypePropertiesHolder)
      {
         // special case
         aHolder = (ArrayNodePropertiesHolder) ((TypedefTypePropertiesHolder) typeHolder).getTypeHolder ();
      }
      else
      {
         // TODO: add error reporting
         throw new RuntimeException ("Array definition expected.");
      }

      return aHolder;
   }

   // -------------------------------------------------------------------------------------
   // UNION
   /* package */void startUnionType (IdlParserAST ast, IdlParserAST literal_union)
   {
      String name = ast.getText ();

      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      UnionDef forward = this.findLocalDeclaration (name, UnionDef.class);

      if (this.m_forwards.contains (forward))
      {
         // this union has forward declaration. So we create UnionDeclHolder and
         // put resolved type inside
         this.m_forwards.remove (forward);
         TypePropertiesHolder typeHolder = new UnionDeclPropertiesHolder (forward, this.composeSourceAttribute (ast), ast,
               literal_union);
         this.push (typeHolder);
         m_resolvingContext.setCurrentContext (forward);
      }
      else
      {
         TypePropertiesHolder nodeHolder = new UnionDeclPropertiesHolder (ast.getText (), this.composeSourceAttribute (ast), ast,
               literal_union);
         this.push (nodeHolder);

         // get UnionDef from the Union holder
         List<IRObject> defs = nodeHolder.getDefinitions ();

         this.bindDefinitionsWithContainer (defs, parent);
         m_resolvingContext.setCurrentContext ((Container) defs.get (0));
      }
   }

   /* package */void endUnionType ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   // -----------------------------------------------------------
   // switch_type_spec
   /* package */void startSwitchTypeSpec ()
   {
   }

   /* package */void endSwitchTypeSpec (IdlParserAST ast)
   {
      // pop node - it should be union descriminator
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      UnionDeclPropertiesHolder unionHolder = this.peek (UnionDeclPropertiesHolder.class);

      IDLType type = (IDLType) typeHolder.getDefinition ();
      UnionDef unionObj = (UnionDef) unionHolder.getDefinition ();

      // TODO: check for unknown type?
      unionObj.setType (type);

      // special case - if discriminant is a EnumDef we need to bind the enum
      // with the parent container.
      // It is done in order to bind EnumDef which defined straight in "switch"
      // declaration of the union
      if (type instanceof EnumDef)
      {
         this.bindDefinitionWithContainer (type, unionObj);
      }
   }

   // ----------------------------------------------------------- switch_body
   /* package */void startSwitchBody ()
   {
   }

   /* package */void endSwitchBody ()
   {
   }

   // ----------------------------------------------------------- case_stmt
   /* package */void startCaseStmt (IdlParserAST ast1)
   {
      TypePropertiesHolder nodeHolder = this.peek (TypePropertiesHolder.class);
      UnionDeclPropertiesHolder unionHolder = null;

      if (nodeHolder instanceof ScopedNameNodePropertiesHolder)
      {
         // If a scoped name holder is stored at the top of the stack - it means
         // that "case"
         // value holds in the scoped name holder
         ScopedNameNodePropertiesHolder scHolder = (ScopedNameNodePropertiesHolder) this.pop (TypePropertiesHolder.class);
         // resolving expression and setting "source" attribute to it
         Expression ex = m_scNameResolver.resolveExpression (scHolder);

         unionHolder = this.peek (UnionDeclPropertiesHolder.class);
         unionHolder.setExpression (ex);
      }
      else
      {
         unionHolder = (UnionDeclPropertiesHolder) nodeHolder;
      }

      this.addCaseASTToUnion (unionHolder, ast1);
   }

   /* package */void endCaseStmt ()
   {
   }

   private void addCaseASTToUnion (UnionDeclPropertiesHolder unionHolder, IdlParserAST ast)
   {
      unionHolder.addCaseAST (ast);
   }

   // ------------------------------------------------------------- element_spec
   /* package */void startElementSpec ()
   {
   }

   /* package */void endElementSpec ()
   {
   }

   // --------------------------------------------------------------
   // union_member
   /* package */void startUnionMember ()
   {
   }

   /* package */void endUnionMember ()
   {
      TypePropertiesHolder memberHolder = this.pop (TypePropertiesHolder.class);
      UnionDeclPropertiesHolder unionHolder = this.peek (UnionDeclPropertiesHolder.class);

      IDLType memberType = (IDLType) memberHolder.getDefinition ();
      DeclaratorHolder declHolder = memberHolder.getDeclarators ().get (0);

      assert memberHolder.getDeclarators ().size () == 1;

      this.createUnionMember (unionHolder, memberType, declHolder);

      unionHolder.endUnionMember ();
   }

   // -------------------------------------------------------------------------
   // union member def
   private void createUnionMember (UnionDeclPropertiesHolder unionHolder, IDLType type, DeclaratorHolder declHolder)
   {
      UnionDef unionDef = (UnionDef) unionHolder.getDefinition ();
      UnionMember unionMember = this.m_factory.createUnionMember ();

      unionMember.setContainer (unionDef);
      unionMember.setName (declHolder.getName ());
      unionMember.setSource (declHolder.getSourceAttrValue ());
      unionMember.setType (type);

      List<Expression> values = unionHolder.getCurrentLabels ();
      EList<Expression> labels = unionMember.getLabels ();

      // set value to the member
      for (Expression val : values)
      {
         labels.add (val);
      }

      // TODO: FIXME: we should bind inner definitions in simpler way
      if (type instanceof Contained && ((Contained) type).getContainer () == null)
      {
         ((Contained) type).setContainer (unionDef);
      }
   }

   // ----------------------------------------------------------------------------
   // ENUM
   /* package */void startEnumType (IdlParserAST ast, IdlParserAST literal_enum)
   {
      TypePropertiesHolder nodeHolder = new EnumPropertiesHolder (ast.getText (), this.composeSourceAttribute (ast), ast,
            literal_enum);
      this.push (nodeHolder);
   }

   /* package */void endEnumType ()
   {

   }

   /* package */void startEnumerator ()
   {
   }

   /* package */void endEnumerator (IdlParserAST ast)
   {
      EnumPropertiesHolder enumHolder = this.peek (EnumPropertiesHolder.class);
      enumHolder.addMember (ast.getText (), this.composeSourceAttribute (ast), ast);
   }

   // -----------------------------------------------------------------------------
   // CONST TYPE
   /* package */void startConstType ()
   {

   }

   /* package */void constMetaPoint (IdlParserAST ast1, IdlParserAST ast2)
   {
      TypePropertiesHolder nHolder = this.pop (TypePropertiesHolder.class);
      ConstTypePropertiesHolder constHolder = new ConstTypePropertiesHolder (nHolder, ast1, this.composeSourceAttribute (ast2),
            ast1, ast2);
      this.push (constHolder);
   }

   /* package */void endConstType ()
   {
      TypePropertiesHolder typeHolder = this.peek (TypePropertiesHolder.class);
      if (typeHolder instanceof ScopedNameNodePropertiesHolder)
      {
         // scoped name is used as value of the const. So we have to put the
         // scoped name to const holder as const literal
         ScopedNameNodePropertiesHolder scHolder = this.pop (ScopedNameNodePropertiesHolder.class);

         ConstTypePropertiesHolder constHolder = this.peek (ConstTypePropertiesHolder.class);

         // resolve expression and setting "source" attribute to it
         Expression ex = m_scNameResolver.resolveExpression (scHolder);
         constHolder.setExpression (ex);
      }
   }

   // ------------------------------------------------------------------------------
   // TYPEDEF
   /* package */void startTypedefType ()
   {
   }

   /* package */void typedefMetaPoint (IdlParserAST ast)
   {
      TypePropertiesHolder nHolder = this.pop (TypePropertiesHolder.class);
      TypedefTypePropertiesHolder typedefHolder = new TypedefTypePropertiesHolder (nHolder, ast);
      this.push (typedefHolder);
   }

   /* package */void endTypedefType ()
   {
   }

   // ----------------------------------------------------------------------------
   // VALUE

   void startValueDcl (IdlParserAST cst, IdlParserAST id, IdlParserAST ast)
   {
      String name = id.getText ();

      boolean isCustom = false;
      if (cst != null && cst.getText () != null)
      {
         isCustom = cst.getText ().equals ("custom");
      }

      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      ValuePropertiesHolder typeHolder = new ValuePropertiesHolder (name, false, isCustom, this.composeSourceAttribute (ast), cst,
            id, ast);
      this.push (typeHolder);

      // get InterfaceDef from the holder and bind it with the parent container
      List<IRObject> defs = typeHolder.getDefinitions ();
      this.bindDefinitionsWithContainer (defs, parent);

      m_resolvingContext.setCurrentContext ((Container) defs.get (0));
   }

   void endValueDcl ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   void startValueAbsDcl (IdlParserAST id, IdlParserAST ast)
   {
      String name = id.getText ();

      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      ValuePropertiesHolder typeHolder = new ValuePropertiesHolder (name, true, false, this.composeSourceAttribute (ast), id, ast);
      this.push (typeHolder);

      // get InterfaceDef from the holder and bind it with the parent container
      List<IRObject> defs = typeHolder.getDefinitions ();
      this.bindDefinitionsWithContainer (defs, parent);

      m_resolvingContext.setCurrentContext ((Container) defs.get (0));
   }

   void endValueAbsDcl ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   void startValueForwardDcl ()
   {

   }

   void endValueBoxDcl (IdlParserAST id, IdlParserAST ast)
   {
      TypePropertiesHolder nHolder = this.pop (TypePropertiesHolder.class);
      ValueBoxDclPropertiesHolder vboxHolder = new ValueBoxDclPropertiesHolder (nHolder, id, ast);
      this.push (vboxHolder);
   }

   void endValueForwardDcl (IdlParserAST abs, IdlParserAST id, IdlParserAST ast)
   {
      String name = id.getText ();
      boolean isAbstract = false;
      if (abs != null && abs.getText () != null)
      {
         isAbstract = abs.getText ().equals ("abstract");
      }

      ValueDeclPropertiesHolder typeHolder = new ValueDeclPropertiesHolder (name, isAbstract, this.composeSourceAttribute (id),
            abs, id, ast);
      this.push (typeHolder);
   }

   /* package */void startValueBases ()
   {
   }

   /* package */void endValueBases (IdlParserAST trn)
   {
      List<ScopedNameNodePropertiesHolder> bases = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);
      ValueDef iDef = (ValueDef) this.peek (ContainerTypeHolder.class).getDefinition ();
      if (trn != null && trn.getText () != null && trn.getText ().equals ("truncatable"))
      {
         iDef.setTruncatable (true);
      }

      EList<ValueDef> baseValueList = iDef.getBaseValues ();
      for (int i = bases.size () - 1; i >= 0; --i)
      {
         ScopedNameNodePropertiesHolder scHolder = bases.get (i);
         ValueDef value = m_scNameResolver.resolveValueDef (scHolder);
         if (value != null)
         {
            baseValueList.add (value);
         }
      }

   }

   void startValueSupports ()
   {

   }

   void endValueSupports ()
   {
      List<ScopedNameNodePropertiesHolder> supports = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);
      ValueDef iDef = (ValueDef) this.peek (ContainerTypeHolder.class).getDefinition ();

      EList<Interface> supportedInterfList = iDef.getSupportedInterfaces ();
      for (int i = supports.size () - 1; i >= 0; --i)
      {
         ScopedNameNodePropertiesHolder scHolder = supports.get (i);
         InterfaceDef type = m_scNameResolver.resolveInterfaceDef (scHolder);
         supportedInterfList.add (type);
      }
   }

   void startInitDcl (IdlParserAST id)
   {
      TypePropertiesHolder typeHolder = new InitializerDeclPropertiesHolder (id.getText (), this.composeSourceAttribute (id), id);
      this.push (typeHolder);
   }

   void endInitDcl ()
   {
      this.endDefinition ();
   }

   void startInitParamDcl ()
   {

   }

   void endInitParamDcl (IdlParserAST dec)
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      InitializerDeclPropertiesHolder initHolder = this.peek (InitializerDeclPropertiesHolder.class);
      InitializerParamDeclHolder paramHolder = new InitializerParamDeclHolder (typeHolder, dec.getText (), this
            .composeSourceAttribute (dec), dec);
      initHolder.addParameterHolder (paramHolder);
   }

   // ----------------------------------------------------------------------------
   // INTERFACE

   /* package */void startInterfaceForwardDecl ()
   {
      // does nothing

   }

   /* package */void endInterfaceForwardDecl (IdlParserAST i1, IdlParserAST ast, IdlParserAST ast2)
   {
      InterfaceDeclPropertiesHolder typeHolder = new InterfaceDeclPropertiesHolder (ast.getText (),
            determineInterfaceModifier (i1), this.composeSourceAttribute (ast), i1, ast, ast2);
      this.push (typeHolder);
   }

   /* package */void startInterfaceType (IdlParserAST i, IdlParserAST ast, IdlParserAST interface_literal)
   {
      String name = ast.getText ();

      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      InterfaceModifier im = this.determineInterfaceModifier (i);
      InterfaceDefPropertiesHolder typeHolder = new InterfaceDefPropertiesHolder (name, im, this.composeSourceAttribute (ast), i,
            ast, interface_literal);
      this.push (typeHolder);

      // get InterfaceDef from the holder and bind it with the parent container
      List<IRObject> defs = typeHolder.getDefinitions ();
      this.bindDefinitionsWithContainer (defs, parent);

      m_resolvingContext.setCurrentContext ((Container) defs.get (0));
   }

   /* package */void endInterfaceType ()
   {
      m_resolvingContext.leaveCurrentContext ();

   }

   private InterfaceModifier determineInterfaceModifier (IdlParserAST i1)
   {
      if (i1 == null)
      {
         return InterfaceModifier.IF_REGULAR;
      }
      else if (i1.getText ().equals ("abstract"))
      {
         return InterfaceModifier.IF_ABSTRACT;
      }
      else if (i1.getText ().equals ("local"))
      {
         return InterfaceModifier.IF_LOCAL;
      }

      throw new IllegalStateException ("Illegal interface modifier: " + i1);
   }

   // -----------------------------------------------------------------------
   // interface_header
   /* package */void startInterfaceHeader ()
   {
   }

   /* package */void endInterfaceHeader ()
   {
   }

   /* package */void startInterfaceInherSpec ()
   {
   }

   /* package */void endInterfaceInherSpec ()
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      InterfaceDef iDef = (InterfaceDef) this.peek (InterfaceDefPropertiesHolder.class).getDefinition ();
      EList<InterfaceDef> baseInterfList = iDef.getBaseInterfaces ();

      for (int i = scHolderList.size () - 1; i >= 0; --i)
      {
         ScopedNameNodePropertiesHolder scHolder = scHolderList.get (i);
         String scName = scHolder.getScopedName ();
         InterfaceDef type = m_scNameResolver.resolveInterfaceDef (scHolder);
         
         if (type != null)
         {
            baseInterfList.add (type);
         }
      }
   }

   // ------------------------------------------------------------------
   // interface_body
   /* package */void startInterfaceBody ()
   {

   }

   /* package */void endInterfaceBody ()
   {

   }

   // ------------------------------------------------------------------
   // interface_export_decl
   /* package */void startExportDcl ()
   {

   }

   /* package */void endExportDcl ()
   {
      this.endDefinition ();
   }

   /** type_id and type_prefix_id is not supported */
   // -------------------------------------------------------------------
   // type_id_dcl
   /* package */void startTypeIdDcl()
   {
   }
   
   /* package */void endTypeIdDcl(IdlParserAST ast)
   {
      LiteralHolder litHolder = this.pop (LiteralHolder.class);
      ScopedNameNodePropertiesHolder scHolder = this.pop (ScopedNameNodePropertiesHolder.class);
      this.push(new TypeIdHolder(m_scNameResolver,scHolder, litHolder, ast));
   }
   
   //-------------------------------------------------------------------
   //type_prefix_dcl
   /* package */void startTypePrefixDcl()
   {
   }
   
   /* package */void endTypePrefixDcl(IdlParserAST ast)
   {
      LiteralHolder litHolder = this.pop (LiteralHolder.class);
      ScopedNameNodePropertiesHolder scHolder = this.pop (ScopedNameNodePropertiesHolder.class);
      this.push(new TypePrefixHolder(m_scNameResolver,scHolder, litHolder, ast));
   }
   
   // ---------------------------------------------------------------------------
   // ATTRIBUTE
   /* package */void startAttrDecl ()
   {

   }

   /* package */void endAttrDecl ()
   {

   }

   // ------------------------------------------------------------------
   // attr_spec
   /* package */void startAttrSpec ()
   {

   }

   /* package */void endAttrSpec ()
   {

   }

   // ------------------------------------------------------------------
   // readonly_attr_spec
   /* package */void startReadonlyAttrSpec (IdlParserAST ast1, IdlParserAST ast2)
   {
      TypePropertiesHolder typeHolder = new AttributeDeclPropertiesHolder (true, this.composeSourceAttribute (ast1), ast1, ast2);
      this.push (typeHolder);
   }

   /* package */void endReadonlyAttrSpec ()
   {

   }

   // ------------------------------------------------------------------
   // normal_attr_spec
   /* package */void startNormalAttrSpec (IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = new AttributeDeclPropertiesHolder (false, this.composeSourceAttribute (ast), ast);
      this.push (typeHolder);
   }

   /* package */void endNormalAttrSpec ()
   {

   }

   // --------------------------------------------------------------------------------
   // INTERFACE OPERATION
   /* package */void startOperationDecl ()
   {

   }

   /* package */void endOperationDecl ()
   {

   }

   /* package */void endOperationDeclAttribute (IdlParserAST ast1, IdlParserAST ast2)
   {
      OperationMode om = this.getOperationMode (ast2);

      TypePropertiesHolder typeHolder = new OperationDeclPropertiesHolder (ast1.getText (), om, this.composeSourceAttribute (ast1),
            ast1, ast2);
      this.push (typeHolder);
   }

   private OperationMode getOperationMode (IdlParserAST ast)
   {
      return (ast == null) ? OperationMode.OP_NORMAL : OperationMode.OP_ONEWAY;
   }

   // ------------------------------------------------------------------
   // op_attribute
   /* package */void startOpAttribute ()
   {

   }

   /* package */void endOpAttribute ()
   {

   }

   // ------------------------------------------------------------------
   // op_type_spec
   /* package */void startOpTypeSpec ()
   {

   }

   /* package */void endOpTypeSpec ()
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      CompoundNodePropertiesHolder operHolder = this.peek (CompoundNodePropertiesHolder.class);

      operHolder.setTypeHolder (typeHolder);
   }

   // --------------------------------------------------------------------
   // void_literal
   /* package */void startVoidLiteral ()
   {

   }

   /* package */void endVoidLiteral (IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = new PrimitiveDefPropertiesHolder (PrimitiveKind.PK_VOID, m_repository, this
            .composeSourceAttribute (ast), ast);
      this.push (typeHolder);
   }

   // -------------------------------------------------------------------
   // context_expr
   /* package */void startContextExpr ()
   {
      OperationContextHolder cxHolder = new OperationContextHolder ();
      this.push (cxHolder);
   }

   /* package */void endContextExpr ()
   {
      OperationContextHolder cxHolder = this.pop (OperationContextHolder.class);

      OperationDeclPropertiesHolder opHolder = this.peek (OperationDeclPropertiesHolder.class);
      opHolder.addContextHolder (cxHolder);
   }

   // ------------------------------------------------------------------
   // param_dcl
   /* package */void startParamDcl ()
   {

   }

   /* package */void endParamDcl (IdlParserAST ast1, IdlParserAST ast2)
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      OperationDeclPropertiesHolder operHolder = this.peek (OperationDeclPropertiesHolder.class);

      ParamDeclHolder paramHolder = new ParamDeclHolder (typeHolder, ast1.getText (), ast2.getText (), this
            .composeSourceAttribute (ast2), ast1, ast2);

      operHolder.addParameterHolder (paramHolder);
   }

   // ------------------------------------------------------------------
   // param_type_spec
   /* package */void startParamTypeSpec ()
   {

   }

   /* package */void endParamTypeSpec ()
   {

   }

   /* package */void endAttrParamSpec ()
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      AttributeDeclPropertiesHolder attrHolder = this.peek (AttributeDeclPropertiesHolder.class);
      attrHolder.setTypeHolder (typeHolder);
   }

   // ---------------------------------------------------------------------------
   // DECLARATORS
   /* package */void startSimpleDeclarator (IdlParserAST ast)
   {
      DeclaratorHolder declHolder = new DeclaratorHolder (ast, this.composeSourceAttribute (ast));

      TypePropertiesHolder nodeHolder = this.peek (TypePropertiesHolder.class);
      nodeHolder.addDeclaratorHolder (declHolder);
   }

   /* package */void endSimpleDeclarator ()
   {
   }

   // -----------------------------------------------------------------
   // readonly_attr_declarators
   /* package */void startReadonlyAttrDeclarator ()
   {

   }

   /* package */void endReadonlyAttrDeclarator ()
   {

   }

   // -----------------------------------------------------------------
   // attr_declarators
   /* package */void startAttrDeclarator ()
   {

   }

   /* package */void endAttrDeclarator ()
   {

   }

   // -----------------------------------------------------------------------------EXCEPTIONS
   /* package */void startRaisesExpr ()
   {

   }

   /* package */void endRaisesExpr (IdlParserAST ast)
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      ExceptionDeclarant exDeclarant = this.peek (ExceptionDeclarant.class);

      // TODO it needs to be done the case when the exception is unresolved
      for (ListIterator<ScopedNameNodePropertiesHolder> li = scHolderList.listIterator (scHolderList.size ()); li.hasPrevious ();)
      {
         ScopedNameNodePropertiesHolder scHolder = li.previous ();
         String scName = scHolder.getScopedName ();

         ExceptionDef type = m_scNameResolver.resolveException (scHolder);
         exDeclarant.addRaisedException (type);

         // ASTIdlmmBinder.bindIRObjectToAST (type, ast);

         exDeclarant.addRaisedException (type);
      }

   }

   // ----------------------------------------------------------------
   // get_raised_exception
   /* package */void startGetRaisedException ()
   {
   }

   /* package */void endGetRaisedException (IdlParserAST ast)
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      AttributeExceptionDeclarant exDeclarant = this.peek (AttributeExceptionDeclarant.class);

      // TODO it needs to be done the case when the exception is unresolved
      for (ListIterator<ScopedNameNodePropertiesHolder> li = scHolderList.listIterator (scHolderList.size ()); li.hasPrevious ();)
      {
         ScopedNameNodePropertiesHolder scHolder = li.previous ();

         ExceptionDef exception = m_scNameResolver.resolveException (scHolder);
         // ASTIdlmmBinder.bindIRObjectToAST (exception, ast);

         exDeclarant.addRaisedException (exception);
      }

   }

   // ----------------------------------------------------------------
   // set_raised_exception
   /* package */void startSetRaisedException ()
   {
   }

   /* package */void endSetRaisedException (IdlParserAST ast)
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      AttributeExceptionDeclarant exDeclarant = this.peek (AttributeExceptionDeclarant.class);

      // TODO it needs to be done the case when the exception is unresolved
      for (ListIterator<ScopedNameNodePropertiesHolder> li = scHolderList.listIterator (scHolderList.size ()); li.hasPrevious ();)
      {
         ScopedNameNodePropertiesHolder scHolder = li.previous ();

         ExceptionDef exception = m_scNameResolver.resolveException (scHolder);
         // ASTIdlmmBinder.bindIRObjectToAST (exception, ast);

         exDeclarant.addRaisedSetException (exception);
      }
   }

   // ------------------------------------------------------------------
   // native_decl
   /* package */void startNativeDecl ()
   {

   }

   /* package */void endNativeDecl (IdlParserAST ast1, IdlParserAST ast2)
   {
      NativeDeclPropertiesHolder nHolder = new NativeDeclPropertiesHolder (ast2.getText (), this.composeSourceAttribute (ast1),
            ast1, ast2);
      this.push (nHolder);
   }

   // ------------------------------------------------------------------
   // struct_forward_decl
   /**
    * In case of forward declaration we need to create StructDef without field
    * and bind the StructDef with the parent container ContainerHolder(Struct)
    * is not created. When "real" definition of the Struct is met the previously
    * created definition is found and put to ContainerHolder(Struct)
    */
   /* package */void startStructForwardDecl ()
   {

   }

   /* package */void endStructForwardDecl (IdlParserAST ast)
   {
      // create struct and bind it with parent container
      ContainerTypeHolder typeHolder = new ContainerTypeHolder (ast.getText (), ContainerType.STRUCT, this
            .composeSourceAttribute (ast));
      this.push (typeHolder);
      this.m_forwards.add ((StructDef) typeHolder.getDefinition ());
   }

   // ------------------------------------------------------------------
   // union_forward_decl
   /* package */void startUnionForwardDecl ()
   {

   }

   /* package */void endUnionForwardDecl (IdlParserAST ast)
   {
      UnionDeclPropertiesHolder unionHolder = new UnionDeclPropertiesHolder (ast.getText (), this.composeSourceAttribute (ast));
      this.push (unionHolder);
      this.m_forwards.add ((UnionDef) unionHolder.getDefinition ());
   }

   // -----------------------------------------------------------------------------
   // EXCEPTIONS
   /* package */void startExceptionDecl (IdlParserAST ast, IdlParserAST exception_literal)
   {
      String name = ast.getText ();
      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      TypePropertiesHolder typeHolder = new ContainerTypeHolder (name, ContainerType.EXCEPTION, this.composeSourceAttribute (ast),
            ast, exception_literal);
      Container def = (Container) typeHolder.getDefinition ();
      this.push (typeHolder);
      List<IRObject> defs = typeHolder.getDefinitions ();

      this.bindDefinitionsWithContainer (defs, parent);
      this.m_resolvingContext.setCurrentContext (def);

   }

   /* package */void endExceptionDecl ()
   {
      this.m_resolvingContext.leaveCurrentContext ();
   }

   // ----------------------------------------------------------------------------
   // OPERATIONS
   /* package */void startConstExpr ()
   {
   }

   /* package */void endConstExpr ()
   {
   }

   // ------------------------------------ or
   /* package */void startOrExpr (IdlParserAST ast)
   {
      BitwiseOrOperator bitwiseOrOperator = m_factory.createBitwiseOrOperator ();
      this.push (new ExpressionHolder (bitwiseOrOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endOrExpr ()
   {
      this.endExpression ();
   }

   // ------------------------------------- xor
   /* package */void startXorExpr (IdlParserAST ast)
   {
      BitwiseXorOperator bitwiseXorOperator = m_factory.createBitwiseXorOperator ();
      this.push (new ExpressionHolder (bitwiseXorOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endXorExpr ()
   {
      this.endExpression ();
   }

   // ------------------------------------- and
   /* package */void startAndExpr (IdlParserAST ast)
   {
      BitwiseAndOperator bitwiseAndOperator = m_factory.createBitwiseAndOperator ();
      this.push (new ExpressionHolder (bitwiseAndOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endAndExpr ()
   {
      this.endExpression ();
   }

   // ------------------------------------ plus
   /* package */void startPlusExpr (IdlParserAST ast)
   {
      // test new operation holder
      AddOperator addOp = m_factory.createAddOperator ();
      ExpressionHolder opHolder = new ExpressionHolder (addOp, this.composeSourceAttribute (ast));
      this.push (opHolder);
   }

   /* package */void endPlusExpr ()
   {
      ExpressionHolder holder = this.peek (ExpressionHolder.class);

      assert holder.getExpression () instanceof AddOperator;

      AddOperator exp = (AddOperator) holder.getExpression ();

      if (exp.getRhs () == null)
      {
         PositiveOperator pos = m_factory.createPositiveOperator ();
         pos.setOperand (exp.getLhs ());
         pos.setSource (exp.getSource ());

         holder.setExpression (pos);
      }

      this.endExpression ();
   }

   // ----------------------------------- minus
   /* package */void startMinusExpr (IdlParserAST ast)
   {
      SubtractOperator subOp = m_factory.createSubtractOperator ();
      ExpressionHolder opHolder = new ExpressionHolder (subOp, this.composeSourceAttribute (ast));
      this.push (opHolder);
   }

   /* package */void endMinusExpr ()
   {
      ExpressionHolder holder = this.peek (ExpressionHolder.class);

      assert holder.getExpression () instanceof SubtractOperator;

      SubtractOperator exp = (SubtractOperator) holder.getExpression ();

      if (exp.getRhs () == null)
      {
         NegativeOperator neg = m_factory.createNegativeOperator ();
         neg.setOperand (exp.getLhs ());
         neg.setSource (exp.getSource ());

         holder.setExpression (neg);
      }
      this.endExpression ();
   }

   // ------------------------------------- rshift
   /* package */void startRShiftExpr (IdlParserAST ast)
   {
      BitwiseRightShiftOperator bitwiseRightShiftOperator = m_factory.createBitwiseRightShiftOperator ();
      this.push (new ExpressionHolder (bitwiseRightShiftOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endRShiftExpr ()
   {
      this.endExpression ();
   }

   // ------------------------------------- lshift
   /* package */void startLShiftExpr (IdlParserAST ast)
   {
      BitwiseLeftShiftOperator bitwiseLeftShiftOperator = m_factory.createBitwiseLeftShiftOperator ();
      this.push (new ExpressionHolder (bitwiseLeftShiftOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endLShiftExpr ()
   {
      this.endExpression ();
   }

   // ------------------------------------- tilde
   /* package */void startTildeExpr (IdlParserAST ast)
   {
      BitwiseComplementOperator bitwiseComplementOperator = m_factory.createBitwiseComplementOperator ();
      this.push (new ExpressionHolder (bitwiseComplementOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endTildeExpr ()
   {
      this.endExpression ();
   }

   // ------------------------------------- star
   /* package */void startStarExpr (IdlParserAST ast)
   {
      MultiplyOperator multiplyOperator = m_factory.createMultiplyOperator ();
      this.push (new ExpressionHolder (multiplyOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endStarExpr ()
   {
      this.endExpression ();
   }

   // ------------------------------------- div
   /* package */void startDivExpr (IdlParserAST ast)
   {
      DivideOperator divideOperator = m_factory.createDivideOperator ();
      this.push (new ExpressionHolder (divideOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endDivExpr ()
   {
      this.endExpression ();
   }

   // ------------------------------------- mod
   /* package */void startModExpr (IdlParserAST ast)
   {
      ModuloOperator moduloOperator = m_factory.createModuloOperator ();
      this.push (new ExpressionHolder (moduloOperator, this.composeSourceAttribute (ast)));
   }

   /* package */void endModExpr ()
   {
      this.endExpression ();
   }

   // ---------------------------------- unary
   /* package */void startUnaryExpr ()
   {
   }

   /* package */void endUnaryExpr ()
   {
   }

   // ----------------------------------
   /* package */void startPrimaryExpr ()
   {
   }

   /* package */void endPrimaryExpr ()
   {
   }

   private void endExpression ()
   {
      ExpressionHolder opHolder = this.pop (ExpressionHolder.class);
      NodePropertiesHolder parentHolder = this.peek (NodePropertiesHolder.class);

      if (parentHolder instanceof TypedefTypePropertiesHolder)
      {
         // special case - typedef holder
         parentHolder = ((TypedefTypePropertiesHolder) parentHolder).getTypeHolder ();
      }

      if (parentHolder instanceof ExpressionHolder)
      {
         // Popped expression is just an operand of another expression. So we
         // add popped expression
         // from this holder to the parent one
         ((ExpressionHolder) parentHolder).addOperand (opHolder.getExpression ());
      }
      else if (parentHolder instanceof ExpressionOperable)
      {
         ((ExpressionOperable) parentHolder).setExpression (opHolder.getExpression ());
      }
      else
      {
         // TODO: report an error
         throw new RuntimeException ("Unexpected expression");
      }
   }

   // -------------------------------------------------------------------------
   // CONST
   /* package */void startPositiveIntConst ()
   {
   }

   /* package */void endPositiveIntConst ()
   {
   }

   // ------------------------------------------------------------------------------
   // "GRAMMAR LISTENERS"
   // ---------------------------------------------------------- definition
   /* package */void startDefinition ()
   {
   }

   /* package */void endDefinition ()
   {
      // pop definition
      TypePropertiesHolder definitionHolder = this.pop (TypePropertiesHolder.class);
      List<IRObject> defs = definitionHolder.getDefinitions ();

      if(definitionHolder instanceof TypePrefixHolder)
      {
         IStatus status = ((TypePrefixHolder)definitionHolder).bind ();
         if (!status.isOK ())
         {
            m_statusHolder.addStatusItem (status);
         }

      }
      else if(definitionHolder instanceof TypeIdHolder)
      {
         IStatus status = ((TypeIdHolder)definitionHolder).bind ();
         if (!status.isOK ())
         {
            m_statusHolder.addStatusItem (status);
         }
      }
      else if (definitionHolder instanceof PragmaDeclPropertiesHolder)
      {
         
      }
      else
      {
         // peek parent container
         TypePropertiesHolder parentHolder = this.peek (TypePropertiesHolder.class);
         Container parent = (Container) parentHolder.getDefinition ();

         this.bindDefinitionsWithContainer (defs, parent);
      }
   }

   // -------------------------------------------------------- type_spec
   /* package */void startTypeSpec ()
   {
   }

   /* package */void endTypeSpec ()
   {
   }

   // -------------------------------------------------------- simple_type_spec
   /* package */void startSimpleTypeSpec ()
   {
   }

   /* package */void endSimpleTypeSpec ()
   {
   }

   // --------------------------------------------------------- constr_type_spec
   /* package */void startConstrTypeSpec ()
   {
   }

   /* package */void endConstrTypeSpec ()
   {
   }

   // ------------------------------------------------------------------------------CREATING
   // DEFINITIONS
   private void createMember (DeclaratorHolder declHolder, Container container, IRObject type, IdlParserAST ast)
   {
      Member member = null;
      if (container instanceof StructDef)
      {
         member = this.m_factory.createStructMember ();
      }
      else if (container instanceof ExceptionDef)
      {
         member = this.m_factory.createExceptionMember ();
      }
      else if (container instanceof ValueDef)
      {
         member = this.m_factory.createValueMemberDef ();
         if (ast != null && ast.getText () != null)
         {
            if (ast.getText ().endsWith ("public"))
            {
               ((ValueMemberDef) member).setAccess (Visibility.PUBLIC_MEMBER);
            }
            else if (ast.getText ().endsWith ("private"))
            {
               ((ValueMemberDef) member).setAccess (Visibility.PRIVATE_MEMBER);
            }
         }
      }
      else
      {
         // TODO throw error
      }

      member.setContainer (container);
      member.setName (declHolder.getName ());
      // set "source" value from DeclaratorHolder
      member.setSource (declHolder.getSourceAttrValue ());
      this.bindDefinitionWithContainer (type, container);
      member.setType ((IDLType) type);
   }

   // ------------------------------------------------------------------------------------
   // -----------------------------------------------------------------------------
   // LITERAL

   private IDLType extractEssentialType (IDLType type)
   {
      while (type instanceof AliasDef)
      {
         type = ((AliasDef) type).getType ();
      }
      return type;
   }

   private ExpressionOperable getExpressionContext ()
   {
      ExpressionOperable c1 = this.findInstanceOfType (ExpressionOperable.class);
      TypedefTypePropertiesHolder c2 = this.findInstanceOfType (TypedefTypePropertiesHolder.class);

      if (c2 != null && ! (c2.getTypeHolder () instanceof ExpressionOperable))
      {
         return c1;
      }

      return (this.m_stack.indexOf (c1) < this.m_stack.indexOf (c2)) ? ExpressionOperable.class.cast (c2.getTypeHolder ()) : c1;
   }

   private boolean isLongLiteralContext ()
   {
      ExpressionOperable context = getExpressionContext ();

      if (context == null)
      {
         throw new IllegalStateException ();
      }

      PrimitiveKind kind;

      if (context instanceof ArrayNodePropertiesHolder || context instanceof SequenceTypePropertiesHolder
            || context instanceof StringDeclPropertiesHolder)
      {
         // force default long for arrays, sequences and strings
         return false;
      }

      IDLType contextType;

      if (context instanceof ConstTypePropertiesHolder)
      {
         contextType = extractEssentialType ((IDLType) ((ConstTypePropertiesHolder) context).getTypeHolder ().getDefinition ());

      }
      else if (context instanceof UnionDeclPropertiesHolder)
      {
         contextType = extractEssentialType ( ((Typed) ((UnionDeclPropertiesHolder) context).getDefinition ()).getType ());
      }
      else
      {
         throw new IllegalStateException ();
      }

      if (! (contextType instanceof PrimitiveDef))
      {
         // FIXME: add error status report: "Unallowed constant type, using
         // default expression constraints..."
         return false;
      }

      kind = ((PrimitiveDef) contextType).getKind ();

      return kind == PrimitiveKind.PK_LONGLONG || kind == PrimitiveKind.PK_ULONGLONG || kind == PrimitiveKind.PK_LONGDOUBLE;
   }

   /* package */void startIntegerLiteral ()
   {
   }

   /* package */void endIntegerLiteral (IdlParserAST ast)
   {
      Literal il = LiteralHelper.createIntegerLiteral (ast.getText (), this.composeSourceAttribute (ast));
      this.endLiteral (il);
   }

   /* package */void startBooleanLiteral ()
   {

   }

   /* package */void endBooleanLiteral (IdlParserAST ast)
   {
      Literal bl = LiteralHelper.createBooleanLiteral (ast.getText (), this.composeSourceAttribute (ast));
      this.endLiteral (bl);
   }

   /* package */void startStringLiteral ()
   {
   }

   /* package */void endStringLiteral (IdlParserAST ast)
   {
      Literal l = LiteralHelper.createStringLiteral (ast.getText (), this.composeSourceAttribute (ast));
      this.endLiteral (l);
   }

   /* package */void startWStringLiteral ()
   {

   }

   /* package */void endWStringLiteral (IdlParserAST ast)
   {
      Literal l = LiteralHelper.createWStringLiteral (ast.getText (), this.composeSourceAttribute (ast));
      this.endLiteral (l);
   }

   /* package */void startCharLiteral ()
   {

   }

   /* package */void endCharLiteral (IdlParserAST ast)
   {
      Literal l = LiteralHelper.createCharLiteral (ast.getText (), this.composeSourceAttribute (ast));
      this.endLiteral (l);
   }

   /* package */void startWCharLiteral ()
   {

   }

   /* package */void endWCharLiteral (IdlParserAST ast)
   {
      Literal l = LiteralHelper.createWCharLiteral (ast.getText (), this.composeSourceAttribute (ast));
      this.endLiteral (l);
   }

   /* package */void startFloatLiteral ()
   {
   }

   /* package */void endFloatLiteral (IdlParserAST ast)
   {
      Literal l = LiteralHelper.createFloatingPointLiteral (ast.getText (), this.composeSourceAttribute (ast));
      this.endLiteral (l);
   }

   private void endLiteral (Literal lit)
   {
      NodePropertiesHolder nodeHolder = this.peek (NodePropertiesHolder.class);

      if (nodeHolder instanceof ExpressionHolder)
      {
         ((ExpressionHolder) nodeHolder).addOperand (lit);
      }
      else if (nodeHolder instanceof ExpressionOperable)
      {
         ((ExpressionOperable) nodeHolder).setExpression (lit);
      }
      else if (nodeHolder instanceof OperationContextHolder)
      {
         ((OperationContextHolder) nodeHolder).addLiteral (lit);
      }
      else if (nodeHolder instanceof TypedefTypePropertiesHolder)
      {
         // special case - typedef holder
         TypePropertiesHolder inHolder = ((TypedefTypePropertiesHolder) nodeHolder).getTypeHolder ();
         if (inHolder instanceof ExpressionOperable)
         {
            ((ExpressionOperable) inHolder).setExpression (lit);
         }
      }
      else
      {
         this.push (new LiteralHolder(lit));
      }
   }

   // -----------------------------------------------------------------------------
   // PREPROCESSOR
   // ------------------------------------------------------------------------------
   // PRAGMA
   /* package */void startPragmaType ()
   {
   }

   /* package */void endPragmaType (IdlParserAST ast)
   {
      Container parent = m_resolvingContext.getCurrentContext ();
      PragmaContext ctx = new PragmaContext ();
      ctx.setAst (ast);
      ctx.setPragmaText (ast.getText ());
      ctx.setParent (parent);

      PragmaDeclPropertiesHolder typeHolder = new PragmaDeclPropertiesHolder (ctx, this.composeSourceAttribute (ast),
            m_pragmaHandlers);
      typeHolder.getDefinition ();
      m_stack.push (typeHolder);
      pragmas.add (typeHolder);
   }

   
   // ----------------------------------------------------------------------------
   // VALUE

   void startEventDcl (IdlParserAST cst, IdlParserAST id, IdlParserAST ast)
   {
      String name = id.getText ();

      boolean isCustom = false;
      if (cst != null && cst.getText () != null)
      {
         isCustom = cst.getText ().equals ("custom");
      }

      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      EventPropertiesHolder typeHolder = new EventPropertiesHolder (name, false, isCustom, this.composeSourceAttribute (ast), cst,
            id, ast);
      this.push (typeHolder);

      // get InterfaceDef from the holder and bind it with the parent container
      List<IRObject> defs = typeHolder.getDefinitions ();
      this.bindDefinitionsWithContainer (defs, parent);

      m_resolvingContext.setCurrentContext ((Container) defs.get (0));
   }

   void endEventDcl ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   void startEventAbsDcl (IdlParserAST id, IdlParserAST ast)
   {
      String name = id.getText ();

      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      EventPropertiesHolder typeHolder = new EventPropertiesHolder (name, true, false, this.composeSourceAttribute (ast), id, ast);
      this.push (typeHolder);

      // get InterfaceDef from the holder and bind it with the parent container
      List<IRObject> defs = typeHolder.getDefinitions ();
      this.bindDefinitionsWithContainer (defs, parent);

      m_resolvingContext.setCurrentContext ((Container) defs.get (0));
   }

   void endEventAbsDcl ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   void startEventForwardDcl ()
   {

   }

   void endEventForwardDcl (IdlParserAST abs, IdlParserAST id, IdlParserAST ast)
   {
      String name = id.getText ();
      boolean isAbstract = false;
      if (abs != null && abs.getText () != null)
      {
         isAbstract = abs.getText ().equals ("abstract");
      }

      EventDeclPropertiesHolder typeHolder = new EventDeclPropertiesHolder (name, isAbstract, this.composeSourceAttribute (id),
            abs, id, ast);
      this.push (typeHolder);
   }

   
   // ---------------------------------------------------------------------------
   // UTILITY FUNCTIONS
   private void bindDefinitionsWithContainer (List<IRObject> defs, Container container)
   {
      for (IRObject def : defs)
      {
         bindDefinitionWithContainer (def, container);
      }
   }

   private void bindDefinitionWithContainer (IRObject def, Container container)
   {
      if (def instanceof Contained)
      {
         Contained ct = (Contained) def;
         if (ct.getContainer () == null)
         {
            ct.setContainer (container);
         }
      }
      if (def instanceof AliasDef)
      {
         // special case. We need to bind container not merely with AliasDef,
         // but with AliasDef's inner object as well
         AliasDef alDef = (AliasDef) def;
         IDLType inObj = alDef.getType ();
         if (inObj instanceof Contained && ((Contained) inObj).getContainer () == null)
         {
            ((Contained) inObj).setContainer (container);
         }
      }
   }

   private <T extends NodePropertiesHolder> List<T> popInstancesOfType (Class<T> type)
   {
      List<T> result = new ArrayList<T> ();

      while (this.m_stack.peek ().getClass ().equals (type))
      {
         result.add (type.cast (this.m_stack.pop ()));
      }
      return result;
   }

   private <T> T findInstanceOfType (Class<T> type)
   {
      for (int i = this.m_stack.size () - 1; i >= 0; --i)
      {
         if (type.isAssignableFrom (this.m_stack.elementAt (i).getClass ()))
         {
            return type.cast (this.m_stack.elementAt (i));
         }
      }
      return null;
   }

   private <T extends Contained> T findLocalDeclaration (String name, Class<T> type)
   {
      final Container lastContainer = m_resolvingContext.getCurrentContext ();

      final EList<Contained> currentContextContents = lastContainer.getContents ();

      for (Contained c : currentContextContents)
      {
         if (c.getName ().equalsIgnoreCase (name))
         {
            if (type.isInstance (c))
            {
               return type.cast (c);
            }
            break;
         }

      }

      return null;
   }

   private String composeSourceAttribute (IdlParserAST ast)
   {
      try
      {
         return m_sourceAttrComposer.compose (ast);
      }
      catch (Exception e)
      {
         m_statusHolder.addStatusItem (new IdlImporterStatus (ast, IStatus.ERROR, e.getMessage (), e));
         return "";
      }
   }

   void startComponentForwardDecl ()
   {
      // do nothing
      
   }

   void endComponentForwardDecl (IdlParserAST i1, IdlParserAST ast)
   {
      ComponentDeclPropertiesHolder typeHolder = new ComponentDeclPropertiesHolder (i1.getText (),
            this.composeSourceAttribute (i1), i1, ast);
      this.push (typeHolder);
   }

   void startComponentType (IdlParserAST i1, IdlParserAST ast)
   {
      String name = i1.getText ();

      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      ComponentDefPropertiesHolder typeHolder = new ComponentDefPropertiesHolder (name, 
            this.composeSourceAttribute (ast), i1, ast);
      this.push (typeHolder);

      // get ComponentDef from the holder and bind it with the parent container
      List<IRObject> defs = typeHolder.getDefinitions ();
      this.bindDefinitionsWithContainer (defs, parent);

      m_resolvingContext.setCurrentContext ((Container) defs.get (0));
      
   }

   void endComponentType ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   void startComponentHeader ()
   {
      // nothing todo
   }

   void endComponentHeader ()
   {
      // nothing todo
   }

   void startComponentInherSpec ()
   {
      // nothing to do
   }

   void endComponentInherSpec ()
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      // according to grammar, a component could only inherit from 1 other component
      ScopedNameNodePropertiesHolder scHolder = scHolderList.get (0);
      
      ComponentDef cDef = (ComponentDef) this.peek (ComponentDefPropertiesHolder.class).getDefinition ();

      ComponentDef type = m_scNameResolver.resolveComponentDef (scHolder);
      if (type != null)
      {
         cDef.setBaseComponent (type);
      }
   }

   void startComponentSupportSpec ()
   {
      // nothing to do
   }

   void endComponentSupportSpec ()
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      ComponentDef cDef = (ComponentDef) this.peek (ComponentDefPropertiesHolder.class).getDefinition ();
      EList<InterfaceDef> supportedInterfList = cDef.getSupportedInterfaces ();

      for (int i = scHolderList.size () - 1; i >= 0; --i)
      {
         ScopedNameNodePropertiesHolder scHolder = scHolderList.get (i);
         InterfaceDef type = m_scNameResolver.resolveInterfaceDef (scHolder);
         
         if (type != null)
         {
            supportedInterfList.add (type);
         }
      }
   }

   void startComponentBody ()
   {
      // nothing to do
   }

   void endComponentBody ()
   {
      // nothing to do
   }

   void startComponentExportDcl ()
   {
      // nothing to do
   }

   void endComponentExportDcl ()
   {
      this.endDefinition ();
   }

   void startProvidesDcl (IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = new ProvidesPortDeclPropertiesHolder (this.composeSourceAttribute (ast), ast);
      this.push (typeHolder);
   }

   void endProvidesParamSpec(IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      ProvidesPortDeclPropertiesHolder providesHolder = this.peek (ProvidesPortDeclPropertiesHolder.class);
      
      // test if port type is an interface or an Object (at this time Object as port type is not supported by meta-model)
      IRObject type = typeHolder.getDefinition();
      if ((type instanceof InterfaceDef) /*||
          (type instanceof PrimitiveDef) && ((PrimitiveDef)type).getKind().equals(PrimitiveKind.PK_OBJREF)*/)
      {
         providesHolder.setTypeHolder (typeHolder);
      } else {
         // try to get port name for more info
         String portName = "";
         if (ast.getNextSibling() != null) {
            portName = "'" + ast.getNextSibling().getText() + "' ";
         }
         
         m_statusHolder.addStatusItem (new IdlImporterStatus (ast, IStatus.ERROR, 
               "Type for provides port " + portName + "is not an interface"));
      }
   }
   
   void endProvidesDcl ()
   {
      // nothing to do
   }

   void startUsesDcl (IdlParserAST ast, boolean multiple)
   {
      TypePropertiesHolder typeHolder = new UsesPortDeclPropertiesHolder (multiple, this.composeSourceAttribute (ast), ast);
      this.push (typeHolder);
   }

   void endUsesParamSpec(IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      UsesPortDeclPropertiesHolder usesHolder = this.peek (UsesPortDeclPropertiesHolder.class);
      if (typeHolder.getDefinition () instanceof InterfaceDef) 
      {
         usesHolder.setTypeHolder (typeHolder);
      } else {
         // try to get port name for more info
         String portName = "";
         if (ast.getNextSibling() != null) {
            portName = "'" + ast.getNextSibling().getText() + "' ";
         }
         
         m_statusHolder.addStatusItem (new IdlImporterStatus (ast, IStatus.ERROR, 
               "Type for uses port " + portName + "is not an interface"));
      }
   }
   
   void endUsesDcl ()
   {
      // nothing to do
   }

   void startEmitsDcl (IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = new EmitsPortDeclPropertiesHolder (this.composeSourceAttribute (ast), ast);
      this.push (typeHolder);
   }

   void endEmitsParamSpec(IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      EmitsPortDeclPropertiesHolder emitsHolder = this.peek (EmitsPortDeclPropertiesHolder.class);
      if (typeHolder.getDefinition () instanceof EventDef) 
      {
         emitsHolder.setTypeHolder (typeHolder);
      } else {
         // try to get port name for more info
         String portName = "";
         if (ast.getNextSibling() != null) {
            portName = "'" + ast.getNextSibling().getText() + "' ";
         }
         
         m_statusHolder.addStatusItem (new IdlImporterStatus (ast, IStatus.ERROR, 
               "Type for emits port " + portName + "is not an eventtype"));
      }
   }
   
   void endEmitsDcl ()
   {
      // nothing to do
   }

   void startPublishesDcl (IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = new PublishesPortDeclPropertiesHolder (this.composeSourceAttribute (ast), ast);
      this.push (typeHolder);
   }

   void endPublishesParamSpec(IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      PublishesPortDeclPropertiesHolder publishesHolder = this.peek (PublishesPortDeclPropertiesHolder.class);
      if (typeHolder.getDefinition () instanceof EventDef) 
      {
         publishesHolder.setTypeHolder (typeHolder);
      } else {
         // try to get port name for more info
         String portName = "";
         if (ast.getNextSibling() != null) {
            portName = "'" + ast.getNextSibling().getText() + "' ";
         }
         
         m_statusHolder.addStatusItem (new IdlImporterStatus (ast, IStatus.ERROR, 
               "Type for publishes port " + portName + "is not an eventtype"));
      }
   }
   
   void endPublishesDcl ()
   {
      // nothing to do
   }

   void startConsumesDcl (IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = new ConsumesPortDeclPropertiesHolder (this.composeSourceAttribute (ast), ast);
      this.push (typeHolder);
   }

   void endConsumesParamSpec(IdlParserAST ast)
   {
      TypePropertiesHolder typeHolder = this.pop (TypePropertiesHolder.class);
      ConsumesPortDeclPropertiesHolder consumesHolder = this.peek (ConsumesPortDeclPropertiesHolder.class);
      if (typeHolder.getDefinition () instanceof EventDef) 
      {
         consumesHolder.setTypeHolder (typeHolder);
      } else {
         // try to get port name for more info
         String portName = "";
         if (ast.getNextSibling() != null) {
            portName = "'" + ast.getNextSibling().getText() + "' ";
         }
         
         m_statusHolder.addStatusItem (new IdlImporterStatus (ast, IStatus.ERROR, 
               "Type for consumes port " + portName + "is not an eventtype"));
      }
   }
   
   void endConsumesDcl ()
   {
      // nothing to do
   }

   void startHomeType (IdlParserAST i1, IdlParserAST ast)
   {
      String name = i1.getText ();

      Container parent = (Container) this.peek (ContainerTypeHolder.class).getDefinition ();
      HomeDefPropertiesHolder typeHolder = new HomeDefPropertiesHolder (name, 
            this.composeSourceAttribute (ast), i1, ast);
      this.push (typeHolder);

      // get HomeDef from the holder and bind it with the parent container
      List<IRObject> defs = typeHolder.getDefinitions ();
      this.bindDefinitionsWithContainer (defs, parent);

      m_resolvingContext.setCurrentContext ((Container) defs.get (0));
   }

   void endHomeType ()
   {
      m_resolvingContext.leaveCurrentContext ();
   }

   void startHomeHeader ()
   {
      // nothing to do
   }

   void endHomeHeader ()
   {
      // nothing to do
   }

   void startHomeInherSpec ()
   {
      // nothing to do
   }

   void endHomeInherSpec ()
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      // according to grammar, a home could only inherit from 1 other home
      ScopedNameNodePropertiesHolder scHolder = scHolderList.get (0);
      
      HomeDef hDef = (HomeDef) this.peek (HomeDefPropertiesHolder.class).getDefinition ();

      HomeDef type = m_scNameResolver.resolveHomeDef (scHolder);
      if (type != null)
      {
         hDef.setBaseHome (type);
      }
   }

   void startHomeSupportSpec ()
   {
      // nothing to do
   }

   void endHomeSupportSpec ()
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      HomeDef hDef = (HomeDef) this.peek (HomeDefPropertiesHolder.class).getDefinition ();
      EList<InterfaceDef> supportedInterfList = hDef.getSupportedInterfaces ();

      for (int i = scHolderList.size () - 1; i >= 0; --i)
      {
         ScopedNameNodePropertiesHolder scHolder = scHolderList.get (i);
         InterfaceDef type = m_scNameResolver.resolveInterfaceDef (scHolder);
         
         if (type != null)
         {
            supportedInterfList.add (type);
         }
      }
   }

   void startHomeManagesSpec ()
   {
      // nothing to do
   }

   void endHomeManagesSpec ()
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      // according to grammar, a home could only manage 1 component
      ScopedNameNodePropertiesHolder scHolder = scHolderList.get (0);
      
      HomeDef hDef = (HomeDef) this.peek (HomeDefPropertiesHolder.class).getDefinition ();

      ComponentDef type = m_scNameResolver.resolveComponentDef (scHolder);
      if (type != null)
      {
         hDef.setManagedComponent (type);
      }
   }

   void startHomePrimaryKeySpec ()
   {
      // nothing to do
   }

   void endHomePrimaryKeySpec ()
   {
      List<ScopedNameNodePropertiesHolder> scHolderList = this.popInstancesOfType (ScopedNameNodePropertiesHolder.class);

      // according to grammar, a home could only have 1 valuetype as primarykey
      ScopedNameNodePropertiesHolder scHolder = scHolderList.get (0);
      
      HomeDef hDef = (HomeDef) this.peek (HomeDefPropertiesHolder.class).getDefinition ();

      ValueDef type = m_scNameResolver.resolveValueDef (scHolder);
      if (type != null)
      {
         hDef.setPrimarykey (type);
      }
   }

   void startHomeBody ()
   {
      // nothing to do
   }

   void endHomeBody ()
   {
      // nothing to do
   }

   void startHomeExportDcl ()
   {
      // nothing to do
   }

   void endHomeExportDcl ()
   {
      this.endDefinition ();
   }

   void startFactoryDcl (IdlParserAST id)
   {
      TypePropertiesHolder typeHolder = new FactoryOperationPropertiesHolder (id.getText (), this.composeSourceAttribute (id), id);
      this.push (typeHolder);
   }

   void endFactoryDcl ()
   {
      // nothing to do
   }

   void startFinderDcl (IdlParserAST id)
   {
      TypePropertiesHolder typeHolder = new FinderOperationPropertiesHolder (id.getText (), this.composeSourceAttribute (id), id);
      this.push (typeHolder);
   }

   void endFinderDcl ()
   {
      // nothing to do
   }

   
   
   
   
   
   
   
}