package com.prismtech.idt.imports.idl.internal.importer;

import java.util.StringTokenizer;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.EList;

import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.IdlmmFactory;
import com.prismtech.idt.imports.idl.importer.IIdlPragmaHandler;
import com.prismtech.idt.imports.idl.importer.IdlImporterPlugin;
import com.prismtech.idt.imports.idl.internal.importer.builderutil.RepositoryIDHelper;


public class PrefixPragmaHandler implements IIdlPragmaHandler
{
   protected static final IdlmmFactory FACTORY = IdlmmFactory.eINSTANCE;

   public boolean matches (String pragmaText)
   {
      return pragmaText.matches ("prefix +\\\"[^\\\"]*\\\"");
   }

   public IStatus create (CreateContext ctx)
   {
      String prefix = ctx.getPragmaText ();

      assert (matches (prefix));

      /*
       * Remove the opening quote and everything preceding it
       */
      int openQuote = prefix.indexOf ('"');
      prefix = prefix.substring (openQuote + 1);

      /*
       * Remove the closing quote
       */
      int closeQuote = prefix.indexOf ('"');
      prefix = prefix.substring (0, closeQuote);
      
      if(!RepositoryIDHelper.validPrefix (prefix))
      {
         return new IdlImporterStatus (ctx.getAst (), IStatus.ERROR, "Invalid prefix " + prefix);
      }

      ctx.setData ("PREFIX", prefix);
      ctx.setPragma (FACTORY.createPrePragma ());
      
      return new Status (IStatus.OK, IdlImporterPlugin.PLUGIN_ID, "");
   }

   public IStatus bind (CreateContext ctx)
   {
      String prefix = (String) ctx.getData ("PREFIX");
      
      EList<Contained> contents = ctx.getParent ().getContents ();

      String pragmaString = ctx.getPragma ().getSource ();
      StringTokenizer pragmaST = new StringTokenizer (pragmaString, "#,");
      String pragmaSource = pragmaST.nextToken ();
      int pragmaLine = Integer.parseInt (pragmaST.nextToken ());

      for (Contained obj : contents)
      {
         String objString = obj.getSource ();
         StringTokenizer objST = new StringTokenizer (objString, "#,");
         String objSource = objST.nextToken ();
         int objLine = Integer.parseInt (objST.nextToken ());

         if (pragmaSource.equals (objSource) && objLine > pragmaLine)
         {
            obj.setPrefix (prefix);
         }
      }

      return new Status (IStatus.OK, IdlImporterPlugin.PLUGIN_ID, "");
   }
}
