package com.prismtech.idt.imports.idl.internal.importer;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.IdlmmFactory;
import com.prismtech.idt.idlmm.IdlmmPackage;
import com.prismtech.idt.idlmm.Pragma;
import com.prismtech.idt.imports.idl.importer.IIdlPragmaHandler;
import com.prismtech.idt.imports.idl.importer.IdlImporterPlugin;


public class VersionPragmaHandler implements IIdlPragmaHandler
{
   protected static final IdlmmFactory FACTORY = IdlmmFactory.eINSTANCE;

   private String pattern = "([[::]?[\\w]*]*) ([\\d]*\\.[\\d]*)";

   public boolean matches (String pragmaText)
   {
      return pragmaText.matches ("version .*");
   }

   public IStatus create (CreateContext ctx)
   {
      String versionStr = ctx.getPragmaText ().replace ("version ", "");
      Pattern p = Pattern.compile (pattern);
      Matcher m = p.matcher (versionStr);
      if (!m.matches ())
      {
         return new IdlImporterStatus (ctx.getAst (), IStatus.ERROR, "Invalid version pragma: #pragma " + ctx.getPragmaText ());
      }

      String name = m.group (1);
      String version = m.group (2);

      Contained obj = ctx.getParent ().lookup (name);
      if (obj == null)
      {
         return new IdlImporterStatus (ctx.getAst (), IStatus.ERROR, "Can't find " + name + " from #pragma " + ctx.getPragmaText ());
      }
      else if (obj.eIsSet (IdlmmPackage.Literals.CONTAINED__ID))
      {
         return new IdlImporterStatus (ctx.getAst (), IStatus.ERROR, "ID already set to " + obj.getID ()
               + " so can't apply #pragma " + ctx.getPragmaText ());
      }
      else if (obj.getVersion () != null && !obj.getVersion ().equals ("") && !obj.getVersion ().equals (version))
      {
         return new IdlImporterStatus (ctx.getAst (), IStatus.ERROR, "Version already set to " + obj.getVersion ()
               + " so can't apply #pragma " + ctx.getPragmaText ());
      }
      else
      {
         obj.setVersion (version);
      }

      Pragma def = FACTORY.createScopePragma ();
      ctx.setPragma (def);
      return new Status (IStatus.OK, IdlImporterPlugin.PLUGIN_ID, "");
   }

   public IStatus bind (CreateContext ctx)
   {
      return new Status (IStatus.OK, IdlImporterPlugin.PLUGIN_ID, "");
   }
}
