/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.LinkedList;

import com.prismtech.idt.idlmm.ArrayDef;
import com.prismtech.idt.idlmm.Expression;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class ArrayNodePropertiesHolder extends CompoundNodePropertiesHolder implements ExpressionOperable
{
   private LinkedList<LinkedList<Expression>> bounds = new LinkedList<LinkedList<Expression>> ();

   private LinkedList<Expression> currentBounds = null;

   public ArrayNodePropertiesHolder (TypePropertiesHolder nh, String sourceVal, IdlParserAST... ast)
   {
      super (sourceVal, ast);

      m_innerTypeHolder = nh;

      this.createCurrentBoundList ();
   }

   public Expression getExpression ()
   {
      throw new UnsupportedOperationException ();
   }

   public void setExpression (Expression bound)
   {
      this.currentBounds.add (bound);
   }

   /**
    * overriding is neeeded as a new instance of StringDef should be returned at
    * every invocation of getDefinition() method Without overriding new instance
    * of StringDef is returned only at first invocation of getDefinition()
    */
   @Override
   public IRObject getDefinition ()
   {
      return this.createDefinition ();
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      return this.createArrayDef ();
   }

   public LinkedList<LinkedList<Expression>> getBounds ()
   {
      return this.bounds;
   }

   public void startNextBoundList ()
   {
      this.bounds.add (this.currentBounds);
      this.createCurrentBoundList ();
   }

   private void createCurrentBoundList ()
   {
      this.currentBounds = new LinkedList<Expression> ();
   }

   private ArrayDef createArrayDef ()
   {
      IDLType type = (IDLType) m_innerTypeHolder.getDefinition ();

      LinkedList<Expression> dims = this.getBounds ().removeFirst ();
      return this.createArrayDef (type, dims);
   }

   private ArrayDef createArrayDef (Object type, LinkedList<Expression> dims)
   {
      Expression length = dims.removeLast ();
      ArrayDef def = FACTORY.createArrayDef ();
      def.setLength (length);
      this.setSourceAttribute (def);
      def.setType ((IDLType) type);

      if (!dims.isEmpty ())
      {
         return this.createArrayDef (def, dims);
      }

      return def;

   }

}
