/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.prismtech.idt.idlmm.AttributeDef;
import com.prismtech.idt.idlmm.AttributeMode;
import com.prismtech.idt.idlmm.ExceptionDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class AttributeDeclPropertiesHolder extends CompoundNodePropertiesHolder implements AttributeExceptionDeclarant
{
   private AttributeMode m_aMode;

   private List<ExceptionDef> m_getExceptionList = new ArrayList<ExceptionDef> ();

   private List<ExceptionDef> m_setExceptionList = new ArrayList<ExceptionDef> ();

   public AttributeDeclPropertiesHolder (boolean isReadonly, String sourceVal, IdlParserAST... asts)
   {
      super (sourceVal, asts);

      this.determineAttributeMode (isReadonly);
   }

   @Override
   public List<IRObject> getDefinitions ()
   {
      if (this.m_definitions == null)
      {
         this.m_definitions = this.createDefinitions ();
      }

      return this.m_definitions;
   }

   @Override
   public IRObject getDefinition ()
   {
      if (this.m_definitions == null)
      {
         this.m_definitions = this.createDefinitions ();
      }
      return this.m_definitions.get (0);
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      return null;
   }

   private List<IRObject> createDefinitions ()
   {
      List<IRObject> defs = new ArrayList<IRObject> ();

      for (DeclaratorHolder decl : m_declarators)
      {
         AttributeDef attrDef = this.createAttributeDef (decl.getName ());
         attrDef.setSource (decl.getSourceAttrValue ());
         defs.add (attrDef);
      }

      return defs;
   }

   public void addRaisedException (ExceptionDef ex)
   {
      this.m_getExceptionList.add (ex);
   }

   public void addRaisedSetException (ExceptionDef ex)
   {
      this.m_setExceptionList.add (ex);
   }

   private AttributeDef createAttributeDef (String name)
   {
      AttributeDef atDef = FACTORY.createAttributeDef ();

      atDef.setMode (this.m_aMode);
      IDLType innerType = (IDLType) m_innerTypeHolder.getDefinition ();
      atDef.setName (name);

      // TODO: check for unknown type?
      atDef.setType (innerType);

      this.addDeclaredExceptionToAttribute (atDef);

      return atDef;
   }

   private void addDeclaredExceptionToAttribute (AttributeDef attrDef)
   {
      EList<ExceptionDef> getExList = attrDef.getGetExceptions ();
      EList<ExceptionDef> setExList = attrDef.getSetExceptions ();

      for (ExceptionDef ex : this.m_getExceptionList)
      {
         getExList.add (ex);
      }

      for (ExceptionDef ex : this.m_setExceptionList)
      {
         setExList.add (ex);
      }
   }

   private void determineAttributeMode (boolean ir)
   {
      m_aMode = (ir) ? AttributeMode.ATTR_READONLY : AttributeMode.ATTR_NORMAL;
   }

}
