/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */
package com.prismtech.idt.imports.idl.internal.importer.builderutil;

import antlr.collections.AST;

import com.prismtech.idt.idlmm.ConstantDef;
import com.prismtech.idt.idlmm.Expression;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.Literal;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


/**
 * @author Yuriy Kravchenko
 */
public class ConstTypePropertiesHolder extends CompoundNodePropertiesHolder implements ExpressionOperable
{
   private final String m_name;

   private Expression m_value;

   public ConstTypePropertiesHolder (TypePropertiesHolder nh, AST ast, String sourceVal, IdlParserAST... asts)
   {
      super (nh, sourceVal, asts);
      m_name = ast.getText ();
   }

   public void setExpression (Expression value)
   {
      if (this.m_value != null)
      {
         if (this.m_value instanceof Literal && value instanceof Literal)
         {
            if (this.m_value.getExpressionValue () instanceof String && value.getExpressionValue () instanceof String)
            {
               this.m_value = LiteralHelper.concatenateStringLiterals ((Literal) this.m_value, (Literal) value);
               return;
            }
         }
         throw new IllegalArgumentException ("Multiple const values allowed only for strings of one type");
      }
      else
      {
         this.m_value = value;
      }
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      IDLType innerType = (IDLType) m_innerTypeHolder.getDefinition ();

      ConstantDef definition = FACTORY.createConstantDef ();
      definition.setName (this.m_name);

      definition.setValue (m_value);

      // TODO: check for unknown type?
      definition.setType (innerType);

      return definition;
   }

   public Expression getExpression ()
   {
      return m_value;
   }

}
