/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.Collections;

import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.Named;
import com.prismtech.idt.idlmm.ValueDef;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class ContainerTypeHolder extends TypeDeclarationPropertiesHolder
{
   private String m_name;

   private ContainerType m_type;

   public ContainerTypeHolder (String name, ContainerType type, String sourceValue, IdlParserAST... ast)
   {
      super (sourceValue, ast);

      m_type = type;
      m_name = name;
   }

   public ContainerTypeHolder (IRObject definition, String sourceValue, IdlParserAST... ast)
   {
      super (sourceValue, ast);

      /**
       * setSourceValue() method should be invoked here because
       * createDefinition() method will not be invoked (definition is already
       * created)
       */
      this.setSourceAttribute (definition);

      m_definitions = Collections.singletonList (definition);
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      IRObject result = null;
      switch (m_type)
      {
         case REPOSITORY:
            result = FACTORY.createRepository ();
            break;
         case MODULE:
            result = FACTORY.createModuleDef ();
            break;
         case STRUCT:
            result = FACTORY.createStructDef ();
            break;
         case UNION:
            result = FACTORY.createUnionDef ();
            break;
         case INTERFACE:
            result = FACTORY.createInterfaceDef ();
            break;
         case INTERFACE_FORWARD:
            result = FACTORY.createInterfaceDecl ();
            break;
         case VALUE:
            result = FACTORY.createValueDef ();
            break;
         case VALUE_FORWARD:
            result = FACTORY.createValueDecl ();
            break;
         case VALUE_ABS:
            result = FACTORY.createValueDef ();
            break;
         case EXCEPTION:
            result = FACTORY.createExceptionDef ();
            break;
         case EVENT:
            result = FACTORY.createEventDef ();
            break;
         case EVENT_FORWARD:
            result = FACTORY.createEventDecl ();
            break;
         case EVENT_ABS:
            result = FACTORY.createEventDef ();
            break;
         case COMPONENT:
            result = FACTORY.createComponentDef ();
            break;
         case COMPONENT_FORWARD:
            result = FACTORY.createComponentDecl ();
            break;
         case HOME:
            result = FACTORY.createHomeDef ();
            break;
        default:
            break;
      }

      this.setName (result);

      return result;
   }

   protected void setName (IRObject result)
   {
      if (result instanceof Named)
      {
         ((Named) result).setName (m_name);
      }
   }
}
