/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;

import com.prismtech.idt.idlmm.Expression;
import com.prismtech.idt.idlmm.OwnedExpression;
import com.prismtech.idt.idlmm.SharedExpression;


/**
 * @author Yuriy Kravchenko
 */
public class EntityCloner
{
   public static Expression cloneExpression (Expression src)
   {
      return EntityCloner.cloneExpressionImpl (src);
   }

   @SuppressWarnings("unchecked")
   private static <T extends Object> T cloneExpressionImpl (T srcObject)
   {
      assert srcObject != null;

      // we wan't copy simple Java classes.
      if (! (srcObject instanceof EObject))
      {
         return srcObject;
      }

      // !
      if (srcObject instanceof SharedExpression)
      {
         return srcObject;
      }
      // to make a real deep copy of all containers etc.
      // we need to process EList srcObject here that comes from
      // EStructuralFeature with upper bound > 1

      EObject src = EObject.class.cast (srcObject);

      EClass srcClass = src.eClass ();
      EFactory factory = srcClass.getEPackage ().getEFactoryInstance ();

      EObject clone = factory.create (srcClass);

      EList<EReference> references = srcClass.getEAllReferences ();
      EList<EAttribute> attributes = srcClass.getEAllAttributes ();

      for (EReference ref : references)
      {
         if ((ref.isDerived () == false) && (ref.isChangeable () == true))
         {
            Object obj = src.eGet (ref);
            if (obj instanceof OwnedExpression)
            {
               // feature "by value", we need to make a copy
               clone.eSet (ref, cloneExpressionImpl (src.eGet (ref)));
            }
            else
            {
               // just a shared reference, we can refer to it too.
               clone.eSet (ref, obj);
            }
         }
      }

      for (EAttribute attr : attributes)
      {
         if ( (attr.isDerived () == false) && (attr.isChangeable () == true))
         {
            // Attribute seems to be a plain Java classes, but
            // we will try to clone it anyway.
            // first condition in this method will sort it out.
            clone.eSet (attr, cloneExpressionImpl (src.eGet (attr)));
         }
      }

      return (T) clone;
   }

}
