/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.prismtech.idt.idlmm.ExceptionDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.Initializer;
import com.prismtech.idt.idlmm.InitializerParameterDef;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class InitializerDeclPropertiesHolder extends CompoundNodePropertiesHolder implements ExceptionDeclarant
{
   protected final String m_name;

   protected List<InitializerParamDeclHolder> m_parameters = new ArrayList<InitializerParamDeclHolder> ();

   protected OperationContextHolder m_cxHolder;

   protected final List<ExceptionDef> m_exceptionList = new ArrayList<ExceptionDef> ();

   public InitializerDeclPropertiesHolder (String name, String sourceVal, IdlParserAST... asts)
   {
      super (sourceVal, asts);
      m_name = name;
   }

   public void addParameterHolder (InitializerParamDeclHolder paramHolder)
   {
      this.m_parameters.add (paramHolder);
   }

   public void addRaisedException (ExceptionDef ex)
   {
      this.m_exceptionList.add (ex);
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      Initializer initializer = FACTORY.createInitializer ();
      initializer.setName (this.m_name);
      for (InitializerParamDeclHolder paramHolder : this.m_parameters)
      {
         InitializerParameterDef param = this.createParameter (paramHolder);
         param.setInitializer (initializer);
         initializer.getParameters ().add (param);
      }
      this.addRaisedExceptionsToInitializer (initializer);
      return initializer;
   }

   public void addContextHolder (OperationContextHolder cxHolder)
   {
      this.m_cxHolder = cxHolder;
   }

   protected InitializerParameterDef createParameter (InitializerParamDeclHolder paramHolder)
   {
      InitializerParameterDef param = FACTORY.createInitializerParameterDef ();
      param.setName (paramHolder.getName ());
      param.setSource (paramHolder.getSourceVal ());
      IDLType innerType = (IDLType) paramHolder.getTypeHolder ().getDefinition ();
      param.setType (innerType);
      return param;
   }

   private void addRaisedExceptionsToInitializer (Initializer initializer)
   {
      if (this.m_exceptionList == null)
      {
         return;
      }

      EList<ExceptionDef> el = initializer.getExceptions ();

      for (ExceptionDef ex : this.m_exceptionList)
      {
         el.add (ex);
      }
   }

}
