/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.math.BigDecimal;
import java.math.BigInteger;

import com.prismtech.idt.idlmm.BooleanLiteral;
import com.prismtech.idt.idlmm.CharacterLiteral;
import com.prismtech.idt.idlmm.FloatingPointLiteral;
import com.prismtech.idt.idlmm.IdlmmFactory;
import com.prismtech.idt.idlmm.IntegerLiteral;
import com.prismtech.idt.idlmm.Literal;
import com.prismtech.idt.idlmm.StringLiteral;
import com.prismtech.idt.idlmm.WCharacterLiteral;
import com.prismtech.idt.idlmm.WStringLiteral;


public class LiteralHelper
{
   private static final IdlmmFactory m_factory = IdlmmFactory.eINSTANCE;

   //
   // Literals
   // 

   public static Literal createBooleanLiteral (String value, String source)
   {
      BooleanLiteral l = m_factory.createBooleanLiteral ();
      l.setExpressionValue (Boolean.parseBoolean (value));
      l.setSource (source);
      return l;
   }

   public static Literal createStringLiteral (String value, String source)
   {
      StringLiteral l = m_factory.createStringLiteral ();
      l.setExpressionValue (value);
      l.setSource (source);
      return l;
   }

   public static Literal createWStringLiteral (String value, String source)
   {
      WStringLiteral l = m_factory.createWStringLiteral ();
      l.setExpressionValue (value);
      l.setSource (source);
      return l;
   }

   public static Literal createCharLiteral (String value, String source)
   {
      CharacterLiteral l = m_factory.createCharacterLiteral ();
      l.setExpressionValue (value.charAt (0));
      l.setSource (source);
      return l;
   }

   public static Literal createWCharLiteral (String value, String source)
   {
      WCharacterLiteral l = m_factory.createWCharacterLiteral ();
      l.setExpressionValue (value.charAt (0));
      l.setSource (source);
      return l;
   }

   public static Literal createIntegerLiteral (String value, String source)
   {
      IntegerLiteral l = m_factory.createIntegerLiteral ();
      if (value.startsWith ("0x") || value.startsWith ("0X"))
      {
         l.setExpressionValue (new BigInteger (value.substring (2), 16));
      }
      else if (value.startsWith ("0") && !value.equals ("0"))
      {
         l.setExpressionValue (new BigInteger (value, 8));
      }
      else
      {
         l.setExpressionValue (new BigInteger (value));
      }
      l.setSource (source);
      return l;
   }

   public static Literal createFloatingPointLiteral (String value, String source)
   {
      FloatingPointLiteral l = m_factory.createFloatingPointLiteral ();
      l.setExpressionValue (new BigDecimal (value));
      l.setSource (source);
      return l;
   }

   //
   // Utilites
   //

   public static Literal concatenateStringLiterals (Literal sl1, Literal sl2)
   {
      String data = (String) sl1.getExpressionValue () + (String) sl2.getExpressionValue ();
      return createStringLiteral (data, sl1.getSource ());
   }
}
