/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;

import com.prismtech.idt.idlmm.ExceptionDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.Literal;
import com.prismtech.idt.idlmm.OperationDef;
import com.prismtech.idt.idlmm.OperationMode;
import com.prismtech.idt.idlmm.ParameterDef;
import com.prismtech.idt.idlmm.ParameterMode;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class OperationDeclPropertiesHolder extends CompoundNodePropertiesHolder implements ExceptionDeclarant
{
   private static final String IN_PARAM_MODE = "in";

   private static final String OUT_PARAM_MODE = "out";

   private static final String INOUT_PARAM_MODE = "inout";

   private final OperationMode m_om;

   private final String m_name;

   private List<ParamDeclHolder> m_parameters = new ArrayList<ParamDeclHolder> ();

   private OperationContextHolder m_cxHolder;

   private final List<ExceptionDef> m_exceptionList = new ArrayList<ExceptionDef> ();

   public OperationDeclPropertiesHolder (String name, OperationMode om, String sourceVal, IdlParserAST... asts)
   {
      super (sourceVal, asts);

      m_om = om;
      m_name = name;
   }

   public void addParameterHolder (ParamDeclHolder paramHolder)
   {
      this.m_parameters.add (paramHolder);
   }

   public void addRaisedException (ExceptionDef ex)
   {
      this.m_exceptionList.add (ex);
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      OperationDef operation = FACTORY.createOperationDef ();

      operation.setMode (this.m_om);
      operation.setName (this.m_name);
      IDLType innerType = (IDLType) m_innerTypeHolder.getDefinition ();

      // TODO: check for unknown type?
      operation.setType (innerType);

      for (ParamDeclHolder paramHolder : this.m_parameters)
      {
         ParameterDef param = this.createParameter (paramHolder);
         param.setOperation (operation);
      }

      this.addContextToOperation (operation);
      this.addRaisedExceptionsToOperation (operation);

      return operation;
   }

   public void addContextHolder (OperationContextHolder cxHolder)
   {
      this.m_cxHolder = cxHolder;
   }

   private ParameterDef createParameter (ParamDeclHolder paramHolder)
   {
      ParameterDef param = FACTORY.createParameterDef ();

      param.setName (paramHolder.getName ());
      param.setMode (this.createParameterMode (paramHolder.getOperationMode ()));

      param.setSource (paramHolder.getSourceVal ());
      IDLType innerType = (IDLType) paramHolder.getTypeHolder ().getDefinition ();
      param.setType (innerType);

      return param;
   }

   private ParameterMode createParameterMode (String mode)
   {
      if (mode.equals (IN_PARAM_MODE))
      {
         return ParameterMode.PARAM_IN;
      }
      else if (mode.equals (OUT_PARAM_MODE))
      {
         return ParameterMode.PARAM_OUT;
      }
      else if (mode.equals (INOUT_PARAM_MODE))
      {
         return ParameterMode.PARAM_INOUT;
      }

      return null;
   }

   private void addContextToOperation (OperationDef op)
   {
      if (this.m_cxHolder == null)
      {
         return;
      }

      EList<String> contexts = op.getContexts ();

      for (Literal lit : this.m_cxHolder.getContext ())
      {
         contexts.add ((String) lit.getExpressionValue ());
      }
   }

   private void addRaisedExceptionsToOperation (OperationDef op)
   {
      if (this.m_exceptionList == null)
      {
         return;
      }

      EList<ExceptionDef> el = op.getExceptions ();

      for (ExceptionDef ex : this.m_exceptionList)
      {
         if (ex != null)
         {
            el.add (ex);
         }
      }
   }

}
