/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;

import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.imports.idl.importer.IIdlImporterStatus;
import com.prismtech.idt.imports.idl.importer.IIdlPragmaHandler;
import com.prismtech.idt.imports.idl.internal.importer.IdlImporterStatus;
import com.prismtech.idt.imports.idl.internal.importer.UnknownPragmaHandler;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class PragmaDeclPropertiesHolder extends TypeDeclarationPropertiesHolder
{
   private final PragmaContext ctx;

   private IIdlPragmaHandler handler;

   private final List<IIdlPragmaHandler> pragmaHandlers;

   public PragmaDeclPropertiesHolder (PragmaContext ctx, String sourceVal, List<IIdlPragmaHandler> pragmaHandlers)
   {
      super (sourceVal);

      this.ctx = ctx;
      this.pragmaHandlers = pragmaHandlers;
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      IStatus status;
      for (IIdlPragmaHandler handler : pragmaHandlers)
      {
         if (handler.matches (ctx.getPragmaText ()))
         {
            this.handler = handler;
            status = convertStatus (this.handler.create (ctx), ctx.getAst ());
            
            ctx.setData ("CreateStatus", status);
            return ctx.getPragma ();
         }
      }
      this.handler = new UnknownPragmaHandler ();
      status = this.handler.create (ctx);
      ctx.setData ("CreateStatus", status);
      return ctx.getPragma ();
   }
   
   private IStatus convertStatus (IStatus status, IdlParserAST ast)
   {
      if (status == null)
      {
         return status;
      }
      else if (status.isMultiStatus ())
      {
         MultiStatus multiStatus = new MultiStatus (status.getPlugin (), status.getCode (), status.getMessage (), status.getException ());
         
         for (IStatus childStatus : status.getChildren ())
         {
            multiStatus.add (convertStatus (childStatus, ast));
         }
         status = multiStatus;
      }
      else if ((status instanceof IIdlImporterStatus) == false)
      {
         status = new IdlImporterStatus (ctx.getAst (), status.getSeverity (), status.getMessage (), status.getException ());
      }
      return status;
   }

   public PragmaContext getContext ()
   {
      return ctx;
   }

   public IStatus bindPragma ()
   {
      IStatus status = ((IStatus) ctx.getData ("CreateStatus"));
      
      if (status.isOK ())
      {
         status = convertStatus (handler.bind (ctx), ctx.getAst ());
      }
      
      return status;
   }
}
