/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import com.prismtech.idt.idlmm.Expression;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.PrimitiveDef;
import com.prismtech.idt.idlmm.PrimitiveKind;
import com.prismtech.idt.idlmm.StringDef;
import com.prismtech.idt.idlmm.WStringDef;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class StringDeclPropertiesHolder extends TypeDeclarationPropertiesHolder implements ExpressionOperable
{
   private Expression m_bound = null;

   private final PrimitiveKind m_pKind;

   public StringDeclPropertiesHolder (PrimitiveKind pk, String sourceVal, IdlParserAST... ast)
   {
      super (sourceVal, ast);
      this.m_pKind = pk;
   }

   /**
    * overriding is neeeded as a new instance of StringDef should be returned at
    * every invocation of getDefinition() method Without overriding new instance
    * of StringDef is returned only at first invocation of getDefinition()
    */
   @Override
   public IRObject getDefinition ()
   {
      return this.createDefinition ();
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      IDLType result = null;

      if (m_bound != null)
      {
         Expression bound = EntityCloner.cloneExpression (m_bound);

         if (this.m_pKind == PrimitiveKind.PK_STRING)
         {
            StringDef strDef = FACTORY.createStringDef ();
            strDef.setBound (bound);

            result = strDef;
         }
         else if (this.m_pKind == PrimitiveKind.PK_WSTRING)
         {
            WStringDef strDef = FACTORY.createWStringDef ();
            strDef.setBound (bound);

            result = strDef;
         }
         else
         {
            // TODO raising exception
         }
      }
      else
      {
         PrimitiveDef strDef = FACTORY.createPrimitiveDef ();
         strDef.setKind (this.m_pKind);
         result = strDef;
      }

      return result;
   }

   public void setExpression (Expression ex)
   {
      m_bound = ex;
   }

   public Expression getExpression ()
   {
      throw new UnsupportedOperationException ();
   }

}
