/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.IdlmmPackage;
import com.prismtech.idt.imports.idl.importer.IdlImporterPlugin;
import com.prismtech.idt.imports.idl.internal.importer.IdlImporterStatus;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;

/**
 * @author Yuriy Kravchenko
 */

public class TypeIdHolder extends TypeDeclarationPropertiesHolder
{
   private final LiteralHolder id;
   private final ScopedNameNodePropertiesHolder name;
   private final ScopedNameResolver scNameResolver;
   private final IdlParserAST ast;
   
   public TypeIdHolder(ScopedNameResolver scNameResolver, ScopedNameNodePropertiesHolder name, LiteralHolder prefix, IdlParserAST ast)
   {
      this.name = name;
      this.id = prefix;
      this.scNameResolver = scNameResolver;
      this.ast = ast;
   }
   
   public IStatus bind ()
   {
      IRObject obj = scNameResolver.resolveContained (name);
      String idStr = (String)id.getLiteral ().getExpressionValue ();
      if(!RepositoryIDHelper.validRepositoryID (idStr))
      {
         return new IdlImporterStatus (ast, IStatus.ERROR, "Invalid repository id " + idStr);
      }
      else if(obj == null)
      {
         return new IdlImporterStatus (ast, IStatus.ERROR, "Could not find " + name.getScopedName ());
      }
      else if(obj instanceof Contained)
      {
         Contained target = (Contained)obj;
         if (target.eIsSet (IdlmmPackage.Literals.CONTAINED__ID))
         {
            if (!idStr.equals (target.getID ()))
            {
               return new IdlImporterStatus (ast, IStatus.ERROR, "Cannot set repository id on '" + name + "' to '" + id + "'. It's already set to '" + target.getID () + "'");
            }
         }
         target.setID (idStr);
         return new Status (IStatus.OK, IdlImporterPlugin.PLUGIN_ID, "");
      }
      else
      {
         return new IdlImporterStatus (ast, IStatus.ERROR, "Invalid component " + name.getScopedName ());
      }
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      return null;
   }
}
