/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.EventDef;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.IdlmmPackage;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.ModuleDef;
import com.prismtech.idt.idlmm.ValueDef;
import com.prismtech.idt.imports.idl.importer.IdlImporterPlugin;
import com.prismtech.idt.imports.idl.internal.importer.IdlImporterStatus;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;

/**
 * @author Yuriy Kravchenko
 */

public class TypePrefixHolder extends TypeDeclarationPropertiesHolder
{
   private final LiteralHolder prefix;
   private final ScopedNameNodePropertiesHolder name;
   private final ScopedNameResolver scNameResolver;
   private final IdlParserAST ast;
   
   public TypePrefixHolder(ScopedNameResolver scNameResolver, ScopedNameNodePropertiesHolder name, LiteralHolder prefix, IdlParserAST ast)
   {
      this.name = name;
      this.prefix = prefix;
      this.scNameResolver = scNameResolver;
      this.ast = ast;
   }
   
   public IStatus bind ()
   {
      Contained obj = scNameResolver.resolveContained (name);
      String prefixStr = (String)prefix.getLiteral ().getExpressionValue ();
      if(!RepositoryIDHelper.validPrefix (prefixStr))
      {
         return new IdlImporterStatus (ast, IStatus.ERROR, "Invalid repository id " + prefixStr);
      }
      else if(obj == null)
      {
         return new IdlImporterStatus (ast, IStatus.ERROR, "Could not find " + name.getScopedName ());
      }
      else if (obj.eIsSet (IdlmmPackage.Literals.CONTAINED__ID))
      {
         return new IdlImporterStatus (ast, IStatus.ERROR, "ID already set to " + obj.getID ());
      }
      else if (obj.getPrefix () != null && !obj.getPrefix ().equals ("") && !obj.getPrefix ().equals (prefixStr))
      {
         return new IdlImporterStatus (ast, IStatus.ERROR, "Prefix already set to " + obj.getPrefix ());
      }
      else if(obj instanceof ModuleDef || obj instanceof InterfaceDef || obj instanceof ValueDef || obj instanceof EventDef)
      {
         ((Contained)obj).setPrefix (prefixStr);
         return new Status (IStatus.OK, IdlImporterPlugin.PLUGIN_ID, "");
      }
      else
      {
         return new IdlImporterStatus (ast, IStatus.ERROR, "typeprefix can only be applied to a module, interface, valetype or eventtype");
      }
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      return null;
   }
}
