/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public abstract class TypePropertiesHolder implements NodePropertiesHolder
{
   protected List<DeclaratorHolder> m_declarators = new ArrayList<DeclaratorHolder> ();

   protected IdlParserAST[] m_asts = new IdlParserAST[] {};

   protected List<IRObject> m_definitions = null;

   protected String m_sourceAttrValue;

   public TypePropertiesHolder ()
   {
   }

   public TypePropertiesHolder (String sourceVal)
   {
      m_sourceAttrValue = sourceVal;
   }

   public TypePropertiesHolder (String sourceVal, IdlParserAST... ast)
   {
      m_sourceAttrValue = sourceVal;
      m_asts = ast;
   }

   public TypePropertiesHolder (IdlParserAST... ast)
   {
      m_asts = ast;
   }

   public void addDeclaratorHolder (DeclaratorHolder declHolder)
   {
      this.m_declarators.add (declHolder);
   }

   public List<DeclaratorHolder> getDeclarators ()
   {
      return this.m_declarators;
   }

   protected IRObject createDefinition ()
   {
      IRObject iro = this.createDefinitionImpl ();
      
      if (iro != null)
      {
         this.setSourceAttribute (iro);
      }
      
      return iro;
   }

   // template method
   protected abstract IRObject createDefinitionImpl ();

   public List<IRObject> getDefinitions ()
   {
      if (this.m_definitions == null)
      {
         this.m_definitions = Collections.singletonList (this.createDefinition ());
      }
      return this.m_definitions;
   }

   public IRObject getDefinition ()
   {
      if (this.m_definitions == null)
      {
         this.m_definitions = Collections.singletonList (this.createDefinition ());
      }
      return this.m_definitions.get (0);
   }

   @Override
   public String toString ()
   {
      return String.valueOf (m_definitions) + String.valueOf (m_declarators);
   }

   protected void setSourceAttribute (IRObject obj)
   {
      obj.setSource (m_sourceAttrValue);
   }
}
