/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.ArrayList;
import java.util.List;

import com.prismtech.idt.idlmm.AliasDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class TypedefTypePropertiesHolder extends CompoundNodePropertiesHolder
{
   public TypedefTypePropertiesHolder (TypePropertiesHolder typeHolder, IdlParserAST asts)
   {
      super (typeHolder, asts);
   }

   @Override
   public List<IRObject> getDefinitions ()
   {
      if (this.m_definitions == null)
      {
         this.m_definitions = this.createDefinitions ();
      }

      return this.m_definitions;
   }

   @Override
   public IRObject getDefinition ()
   {
      if (this.m_definitions == null)
      {
         this.m_definitions = this.createDefinitions ();
      }
      return this.m_definitions.get (0);
   }

   @Override
   protected IRObject createDefinition ()
   {
      return null;
   }

   private List<IRObject> createDefinitions ()
   {
      List<IRObject> defs = new ArrayList<IRObject> ();

      List<DeclaratorHolder> decls = this.getTypeDeclarators ();
      for (DeclaratorHolder decl : decls)
      {
         IDLType innerType = (IDLType) m_innerTypeHolder.getDefinition ();

         AliasDef alDef = FACTORY.createAliasDef ();
         alDef.setName (decl.getName ());

         // TODO: check for unknown type?
         alDef.setType (innerType);

         alDef.setSource (decl.getSourceAttrValue ());
         /**
          * bind created AliasDef with all accessible AST nodes
          */
         defs.add (alDef);
      }

      return defs;
   }

   private List<DeclaratorHolder> getTypeDeclarators ()
   {
      if (this.m_innerTypeHolder instanceof ArrayNodePropertiesHolder)
      {
         /**
          * array is the special case. In this case we need to work with Array
          * Holder declarators because typedef declarators list is empty
          */
         return m_innerTypeHolder.getDeclarators ();
      }

      return m_declarators;
   }

   @Override
   public String toString ()
   {
      return super.toString () + " " + String.valueOf (m_declarators);
   }

}
