/*
 * COPYRIGHT NOTICE The information contained herein and which follows is
 * proprietary information and is the property of PrismTech Corporation. This
 * information is not to be divulged, released, copied, reproduced, or conveyed
 * to unauthorized personnel, companies, or other institutions without the
 * direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech") are under
 * any obligation to support the use of this code which is provided without
 * warranty of any kind. PrismTech shall (except as otherwise prohibited by law)
 * have no liability whatsoever in relation to its use.
 */

package com.prismtech.idt.imports.idl.internal.importer.builderutil;

/**
 * @author Yuriy Kravchenko
 */

import java.util.ArrayList;
import java.util.List;

import com.prismtech.idt.idlmm.IRObject;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.UsesDef;
import com.prismtech.idt.idlmm.UsesMode;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


public class UsesPortDeclPropertiesHolder extends CompoundNodePropertiesHolder
{
   boolean m_multiple;
   
   public UsesPortDeclPropertiesHolder (boolean multiple, String sourceVal, IdlParserAST... asts)
   {
      super (sourceVal, asts);
      m_multiple = multiple;
   }

   @Override
   public List<IRObject> getDefinitions ()
   {
      if (this.m_definitions == null)
      {
         this.m_definitions = this.createDefinitions ();
      }

      return this.m_definitions;
   }

   @Override
   public IRObject getDefinition ()
   {
      if (this.m_definitions == null)
      {
         this.m_definitions = this.createDefinitions ();
      }
      return this.m_definitions.get (0);
   }

   @Override
   protected IRObject createDefinitionImpl ()
   {
      return null;
   }

   private List<IRObject> createDefinitions ()
   {
      List<IRObject> defs = new ArrayList<IRObject> ();

      for (DeclaratorHolder decl : m_declarators)
      {
         UsesDef def = this.createUsesDef (decl.getName ());
         if (def != null) {
            def.setSource (decl.getSourceAttrValue ());
            defs.add (def);
         }
      }

      return defs;
   }

   private UsesDef createUsesDef (String name)
   {
      UsesDef def = FACTORY.createUsesDef ();

      def.setName (name);
      
      if (m_multiple) {
         def.setMode (UsesMode.USES_MULTIPLE);
      }

      // Note: m_innerTypeHolder could be null if wrong type was detected
      if ((m_innerTypeHolder != null) && (m_innerTypeHolder.getDefinition() instanceof InterfaceDef)) {
         def.setInterface_type((InterfaceDef) m_innerTypeHolder.getDefinition());
      }
      return def;
   }

}
