package com.prismtech.idt.imports.idl.internal.parser;

import java.util.ArrayList;
import java.util.List;

import com.prismtech.idt.imports.idl.parser.IIdlParserStatus;

class IdlParserStatus implements IIdlParserStatus
{
   private int m_severity;
   
   private String m_message;
   
   private String m_filename;

   private int m_line;

   private int m_column;
 
   private Throwable m_cause;

   private List<IIdlParserStatus> m_children;
   
   public IdlParserStatus()
   {
      m_children = new ArrayList<IIdlParserStatus>();
   }

   public IdlParserStatus (int severity)
   {
      m_severity = severity;
      m_children = new ArrayList<IIdlParserStatus>();
   }

   public IdlParserStatus (int severity, String message, Throwable cause, String filename)
   {
      m_severity = severity;
      m_message = message;
      m_filename = filename;
      m_line = -1;
      m_column = -1;
      m_cause = cause;
      m_children = new ArrayList<IIdlParserStatus>();
   }

   public IdlParserStatus (IdlParser parser, int severity, String message, Throwable cause, String filename)
   {
      this (parser, severity, message, cause, filename, -1, -1);
   }

   public IdlParserStatus (IdlParser parser, int severity, String message, Throwable cause, String filename, int line, int column)
   {
      m_severity = severity;
      m_message = message;
      m_filename = filename;
      m_line = line;
      m_column = column;
      m_cause = cause;
      m_children = new ArrayList<IIdlParserStatus>();
   }

   public int getSeverity ()
   {
      return m_severity;
   }

   public String getMessage ()
   {
      return m_message;
   }

   public String getFilename ()
   {
      return m_filename;
   }

   public int getLine ()
   {
      return m_line;
   }

   public int getColumn ()
   {
      return m_column;
   }

   public Throwable getException ()
   {
      return m_cause;
   }

   public IIdlParserStatus[] getChildStati ()
   {
      return (IIdlParserStatus[]) m_children.toArray (new IIdlParserStatus[m_children.size ()]);
   }

   public void add (IIdlParserStatus status)
   {
      m_children.add (status);
   }

   public boolean isMultiStatus ()
   {
      return !m_children.isEmpty ();
   }

   public boolean isOK ()
   {
      if(isMultiStatus())
      {
         for(IIdlParserStatus child : m_children)
         {
            if(!child.isOK ())
            {
               return false;
            }
         }
      }
      return m_severity == IIdlParserStatus.OK;
   }

   public String toString ()
   {
      return m_filename + ':' + m_line + ':' + m_column + ':' + getMessage ();
   }
}
