/***************************************************************************************************
 * - COPYRIGHT NOTICE - * * The information contained herein and which follows is proprietary *
 * information and is the property of PrismTech Corporation. This * information is not to be
 * divulged, released, copied, reproduced, or * conveyed to unauthorized personnel, companies, or
 * other institutions * without the direct and expressed approval in writing of PrismTech *
 * Corporation. * * * - DISCLAIMER - * Neither PrismTech Limited nor PrismTech Corporation
 * ("PrismTech") * are under any obligation to support the use of this code which is * provided
 * without warranty of any kind. PrismTech shall (except as * otherwise prohibited by law) have no
 * liability whatsoever in relation * to its use. * *
 **************************************************************************************************/

package com.prismtech.idt.imports.idl.internal.parser;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;

import antlr.collections.AST;

import com.prismtech.idt.imports.idl.parser.IIdlParser;
import com.prismtech.idt.imports.idl.parser.IIdlParserParams;
import com.prismtech.idt.imports.idl.parser.IIdlParserStatus;
import com.prismtech.idt.imports.idl.parser.IIdlPragma;
import com.prismtech.idt.imports.idl.parser.IdlParserAST;


/**
 * This is a wrapper for idl parser.
 */
public class IdlParserWrapper implements IIdlParser
{
   private IIdlParserParams params;

   private List<IIdlParserStatus> m_status = new ArrayList<IIdlParserStatus> ();

   private IdlLexer lexer;

   private AST m_ast = null;

   public IIdlParserParams createParserParams ()
   {
      return new IdlParserParams ();
   }

   public IIdlParserStatus parse (IIdlParserParams params, File input)
   {
      IIdlParserStatus result;
      FileInputStream str = null;

      try
      {
         str = new FileInputStream (input);
         result = parse (params, str);
      }
      catch (FileNotFoundException e)
      {
         result = new IdlParserStatus (IdlParserStatus.ERROR, e.getMessage (), e, input.toString ());
      }
      finally
      {
         try
         {
            if (str != null)
            {
               str.close ();
            }
         }
         catch (IOException e)
         {
         }
      }

      return result;
   }

   public IIdlParserStatus parse (IIdlParserParams params, InputStream input)
   {
      this.params = params;
      lexer = new IdlLexer (input);
      return parse ();
   }

   public IIdlParserStatus parse (IIdlParserParams params, Reader input)
   {
      this.params = params;
      lexer = new IdlLexer (input);
      return parse ();
   }

   private IIdlParserStatus parse ()
   {
      m_status.clear ();
      m_ast = null;

      lexer.setTokenObjectClass (IdlLexerToken.class);

      IdlParser parser = new IdlParser (lexer);
      parser.setFilename (lexer.getFilename ());

      this.addStatus (parser.parse ());
      m_ast = parser.getAST ();

      return getStatus ();
   }

   public List<IIdlPragma> getPragmas ()
   {
      return lexer.getPragmas ();
   }

   /*
    * (non-Javadoc)
    * 
    * @see com.prismtech.idt.imports.idl.driver.IParser#getStatus()
    */
   public IIdlParserStatus getStatus ()
   {
      switch (m_status.size ())
      {
         case 0 :
         {
            return new IdlParserStatus (IIdlParserStatus.OK); //$NON-NLS-1$
         }
         case 1 :
         {
            return m_status.get (0);
         }
         default :
         {
            IdlParserStatus status = new IdlParserStatus ();//$NON-NLS-1$

            for (IIdlParserStatus s : m_status)
            {
               status.add (s);
            }
            return status;
         }
      }
   }

   public IdlParserAST getAST ()
   {
      return (IdlParserAST) m_ast;
   }

   /**
    * Adds the specified {@link IStatus} to our {@link #m_status} list. If the specified
    * {@link IStatus} is a {@link IStatus#isMultiStatus() multi status}, then its children are
    * added.
    */
   private void addStatus (IIdlParserStatus status)
   {
      if (status.isMultiStatus ())
      {
         for (IIdlParserStatus s : status.getChildStati ())
         {
            this.addStatus (s);
         }
      }
      else
      {
         m_status.add (status);
      }
   }
}
