/*
 * COPYRIGHT NOTICE The information contained herein and which follows is proprietary information
 * and is the property of PrismTech Corporation. This information is not to be divulged, released,
 * copied, reproduced, or conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech Corporation. DISCLAIMER Neither
 * PrismTech Limited nor PrismTech Corporation ("PrismTech") are under any obligation to support the
 * use of this code which is provided without warranty of any kind. PrismTech shall (except as
 * otherwise prohibited by law) have no liability whatsoever in relation to its use.
 */
package com.prismtech.idt.imports.idl.internal.parser;

import com.prismtech.idt.imports.idl.parser.IIdlPragma;


/**
 * This is a implementation of the interface {@link IIdlPragma}.
 */
public class IdlPragma implements IIdlPragma
{
   private String m_file;

   private int m_line;

   private int m_column;

   private String m_text;

   private String m_type;

   /**
    * Constructor <code>IdlPragma</code><br/>
    * 
    * @param file
    * @param line
    * @param column
    * @param text
    */
   IdlPragma (String file, int line, int column, String text)
   {
      m_file = file;
      m_line = line;
      m_column = column;
      m_text = text == null ? null : text.trim ();
      ;
   }

   /**
    * Constructor <code>IdlPragma</code><br/>
    * 
    * @param token
    */
   IdlPragma (IdlLexerToken token)
   {
      assert (token != null);
      m_file = token.getFilename ();
      m_line = token.getLine ();
      m_column = token.getColumn ();
      m_text = token.getText () == null ? null : token.getText ().trim ();
      m_type = this.extractType (token.getText ());
   }

   /*
    * (non-Javadoc)
    * 
    * @see com.prismtech.idt.imports.idl.parser.IIdlPragma#getColumn()
    */
   public int getColumn ()
   {
      return m_column;
   }

   /*
    * (non-Javadoc)
    * 
    * @see com.prismtech.idt.imports.idl.parser.IIdlPragma#getFilename()
    */
   public String getFilename ()
   {
      return m_file;
   }

   /*
    * (non-Javadoc)
    * 
    * @see com.prismtech.idt.imports.idl.parser.IIdlPragma#getLine()
    */
   public int getLine ()
   {
      return m_line;
   }

   /*
    * (non-Javadoc)
    * 
    * @see com.prismtech.idt.imports.idl.parser.IIdlPragma#getText()
    */
   public String getText ()
   {
      return m_text;
   }

   /*
    * (non-Javadoc)
    * 
    * @see com.prismtech.idt.imports.idl.parser.IIdlPragma#getType()
    */
   public String getType ()
   {
      return m_type;
   }

   @Override
   public String toString ()
   {
      StringBuffer b = new StringBuffer ();
      b.append ('[');
      b.append (this.getText ());
      b.append (",<");
      b.append ("file: ").append (this.getFilename ());
      b.append (" # col:").append (this.getColumn ());
      b.append (" line:").append (this.getLine ());
      b.append (">]");
      return b.toString ();
   }

   /*
    * (non-Javadoc)
    * 
    * @see java.lang.Object#equals(java.lang.Object)
    */
   @Override
   public boolean equals (Object obj)
   {
      if (obj == null)
         return false;
      if (obj == this)
         return true;
      if (obj instanceof IdlPragma)
      {
         IdlPragma pr = IdlPragma.class.cast (obj);
         return (m_file.equals (pr.m_file) && m_text.equals (pr.m_text) && m_line == pr.m_line && m_column == pr.m_column);
      }
      return super.equals (obj);
   }

   /*
    * (non-Javadoc)
    * 
    * @see java.lang.Object#hashCode()
    */
   @Override
   public int hashCode ()
   {
      return (m_file.hashCode () + m_text.hashCode () + m_line + m_column);
   }

   private String extractType (String text)
   {
      String pragmaType = null;
      if (text != null)
      {
         pragmaType = text.indexOf (" ") > 0 ? text.substring (0, text.indexOf (" ")).trim () : text.trim (); //$NON-NLS-1$ //$NON-NLS-2$
      }
      return pragmaType;
   }
}
