package com.prismtech.idt.imports.idl.parser;

import java.io.File;
import java.io.InputStream;
import java.io.Reader;
import java.util.List;

import com.prismtech.idt.imports.idl.internal.parser.IdlParser;

import antlr.collections.AST;


/**
 * IDL parser interface. Its main function is to build abstract syntax from preprocessed IDL. It is
 * expected to have the following characteristics: - ANTLR-based - uses information from (# )
 * directives - keeps #pragma's - keeps comments
 */
public interface IIdlParser
{
   static final String PLUGIN_ID = IIdlParser.class.getPackage ().getName ();
   
   public IIdlParserParams createParserParams ();

   /**
    * Entry point for parsing preprocessed IDL
    * 
    * @param input
    *           preprocessed IDL
    * @return The {@link IIdlParserStatus status} of the parsing operation.
    */
   public IIdlParserStatus parse (IIdlParserParams params, File input);

   /**
    * Entry point for parsing preprocessed IDL
    * 
    * @param input
    *           preprocessed IDL
    * @return The {@link IIdlParserStatus status} of the parsing operation.
    */
   public IIdlParserStatus parse (IIdlParserParams params, InputStream input);

   /**
    * Entry point for parsing preprocessed IDL
    * 
    * @param input
    *           preprocessed IDL
    * @return The {@link IIdlParserStatus status} of the parsing operation.
    */
   public IIdlParserStatus parse (IIdlParserParams params, Reader input);

   /**
    * TODO Javadoc please
    */
   public List<IIdlPragma> getPragmas ();

   /**
    * Returns parser status. Calling this method has sense only after parse method was invoked,
    * returned status is empty otherwise.
    * 
    * @return parsing status
    */
   public IIdlParserStatus getStatus ();

   /**
    * After calling any of the <code>parse (input)</code> methods, this method can be used to
    * obtain the resulting {@link AST}. The result may be <code>null</code> if parsing failed to
    * the degree where an AST could not be produced. Details of any parsing failure(s) are indicated
    * by {@link #getStatus()}
    * 
    * @return IdlParserAST
    */
   public IdlParserAST getAST ();
}
