package com.prismtech.trace;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;


public class Trace
{
   private static int nextid = 0;

   private static class ThreadInfo
   {
      String name = "[" + nextid++ + "] ";

      String indent = "";

      boolean newline = true;
   };

   private static ThreadLocal<ThreadInfo> m_threadinfo = new ThreadLocal<ThreadInfo> ()
   {
      protected ThreadInfo initialValue ()
      {
         return new ThreadInfo ();
      }
   };

   private static PrintStream m_stream = System.out;

   /**
    * Contains all registered dumpers
    * 
    * @see #register(com.prismtech.trace.Trace.IHelper)
    * @see #unregister(com.prismtech.trace.Trace.IHelper)
    */
   private static final List<IDumper> m_dumpers = new ArrayList<IDumper> ();

   /**
    * Contains all registered printers
    * 
    * @see #register(com.prismtech.trace.Trace.IHelper)
    * @see #unregister(com.prismtech.trace.Trace.IHelper)
    */
   private static final List<IPrinter> m_printers = new ArrayList<IPrinter> ();

   public static void register (IHelper helper)
   {
      if (helper instanceof IDumper)
      {
         m_dumpers.add ((IDumper) helper);
      }

      if (helper instanceof IPrinter)
      {
         m_printers.add ((IPrinter) helper);
      }
   }

   public static void unregister (IHelper helper)
   {
      if (helper instanceof IDumper)
      {
         m_dumpers.remove ((IDumper) helper);
      }

      if (helper instanceof IPrinter)
      {
         m_printers.remove ((IPrinter) helper);
      }
   }

   public static PrintStream getStream ()
   {
      return m_stream;
   }

   public static String getIndent ()
   {
      return m_threadinfo.get ().indent;
   }

   public static void begin ()
   {
      ThreadInfo info = m_threadinfo.get ();

      if (info.newline == false)
      {
         println ();
      }

      println ("{");

      info.newline = true;

      info.indent = info.indent + "   ";
   }

   public static void begin (Object p1)
   {
      println (p1);
      begin ();
   }

   public static void begin (Object p1, Object p2)
   {
      println (p1, p2);
      begin ();
   }

   public static void begin (Object p1, Object p2, Object p3)
   {
      println (p1, p2, p3);
      begin ();
   }

   public static void begin (Object p1, Object p2, Object p3, Object p4)
   {
      println (p1, p2, p3, p4);
      begin ();
   }

   public static void begin (Object p1, Object p2, Object p3, Object p4, Object p5)
   {
      println (p1, p2, p3, p4, p5);
      begin ();
   }

   public static void begin (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6)
   {
      println (p1, p2, p3, p4, p5, p6);
      begin ();
   }

   public static void begin (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6, Object p7)
   {
      println (p1, p2, p3, p4, p5, p6, p7);
      begin ();
   }

   public static void begin (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6, Object p7, Object p8)
   {
      println (p1, p2, p3, p4, p5, p6, p7, p8);
      begin ();
   }

   public static void end ()
   {
      ThreadInfo info = m_threadinfo.get ();

      if (info.indent.length () >= 3)
      {
         info.indent = info.indent.substring (3);
      }
      else
      {
         info.indent = info.indent;
      }

      if (info.newline == false)
      {
         m_stream.println ();
      }

      m_stream.print (info.name);
      m_stream.print (info.indent);
      m_stream.println ('}');

      info.newline = true;
   }

   public static void print (String string)
   {
      ThreadInfo info = m_threadinfo.get ();

      if (info.newline == true)
      {
         m_stream.print (info.name);
         m_stream.print (info.indent);
      }

      m_stream.print (string);

      info.newline = false;
   }

   public static void println (String string)
   {
      ThreadInfo info = m_threadinfo.get ();

      if (info.newline == true)
      {
         m_stream.print (info.name);
         m_stream.print (info.indent);
      }

      m_stream.println (string);

      info.newline = true;
   }

   public static void println ()
   {
      ThreadInfo info = m_threadinfo.get ();

      m_stream.println ();
      info.newline = true;
   }

   /**
    * This prints a simple string representation of the specified object. If no
    * {@link IPrinter} can be found for the object then this will use
    * {@link Object#toString()}
    */
   @SuppressWarnings("unchecked")
   public static void print (Object p1)
   {
      if (p1 == null)
      {
         print ("null");
      }
      else if (p1 instanceof String)
      {
         print ((String) p1);
      }
      else
      {
         final Class clazz = p1.getClass ();

         if (clazz.isArray ())
         {
            if (clazz.getComponentType ().isPrimitive () == true)
            {
               if (clazz.getComponentType () == Integer.TYPE)
               {
                  print ((int[]) p1);
               }
               else
               {
                  print (clazz.getComponentType ().getSimpleName () + "[]");
               }
            }
            else
            {
               print ((Object[]) p1);
            }
         }
         else
         {
            for (int i = m_printers.size () - 1; i >= 0; i--)
            {
               IPrinter printer = m_printers.get (i);

               if (printer.type ().isAssignableFrom (clazz) == true)
               {
                  printer.print (p1);
                  return;
               }
            }

            print (p1.toString ());
         }
      }
   }

   public static void print (Object p1, Object p2)
   {
      print (p1);
      print (p2);
   }

   public static void print (Object p1, Object p2, Object p3)
   {
      print (p1);
      print (p2);
      print (p3);
   }

   public static void print (Object p1, Object p2, Object p3, Object p4)
   {
      print (p1);
      print (p2);
      print (p3);
      print (p4);
   }

   public static void print (Object p1, Object p2, Object p3, Object p4, Object p5)
   {
      print (p1);
      print (p2);
      print (p3);
      print (p4);
      print (p5);
   }

   public static void print (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6)
   {
      print (p1);
      print (p2);
      print (p3);
      print (p4);
      print (p5);
      print (p6);
   }

   public static void print (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6, Object p7)
   {
      print (p1);
      print (p2);
      print (p3);
      print (p4);
      print (p5);
      print (p6);
      print (p7);
   }

   public static void print (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6, Object p7, Object p8)
   {
      print (p1);
      print (p2);
      print (p3);
      print (p4);
      print (p5);
      print (p6);
      print (p7);
      print (p8);
   }

   public static void println (Object object)
   {
      print (object);
      println ();
   }

   public static void print (int[] array)
   {
      if (array == null)
      {
         print ("null");
      }
      else
      {
         if (array.length == 0)
         {
            print ("{}");
         }
         else
         {
            print ("{ ");

            for (int i = 0; i < array.length; i++)
            {
               if (i > 0)
               {
                  print (", ");
               }

               print (array[i]);
            }

            print (" }");
         }
      }
   }

   public static void print (Object[] array)
   {
      if (array == null)
      {
         print ("null");
      }
      else
      {
         if (array.length == 0)
         {
            print ("{}");
         }
         else
         {
            print ("{ ");

            for (int i = 0; i < array.length; i++)
            {
               if (i > 0)
               {
                  print (", ");
               }

               print (array[i]);
            }

            print (" }");
         }
      }
   }

   public static void println (Object p1, Object p2)
   {
      print (p1, p2);
      println ();
   }

   public static void println (Object p1, Object p2, Object p3)
   {
      print (p1, p2, p3);
      println ();
   }

   public static void println (Object p1, Object p2, Object p3, Object p4)
   {
      print (p1, p2, p3, p4);
      println ();
   }

   public static void println (Object p1, Object p2, Object p3, Object p4, Object p5)
   {
      print (p1, p2, p3, p4, p5);
      println ();
   }

   public static void println (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6)
   {
      print (p1, p2, p3, p4, p5, p6);
      println ();
   }

   public static void println (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6, Object p7)
   {
      print (p1, p2, p3, p4, p5, p6, p7);
      println ();
   }

   public static void println (Object p1, Object p2, Object p3, Object p4, Object p5, Object p6, Object p7, Object p8)
   {
      print (p1, p2, p3, p4, p5, p6, p7, p8);
      println ();
   }

   /**
    * This prints out the detailed contents of the specified <code>object</code>
    * in a structured form. If no {@link IDumper} can be found for the object
    * then this will use {@link Object#toString()}
    */
   @SuppressWarnings("unchecked")
   public static void dump (Object object)
   {
      if (object == null)
      {
         println ("null");
      }
      else
      {
         final Class clazz = object.getClass ();

         for (int i = m_dumpers.size () - 1; i >= 0; i--)
         {
            IDumper dumper = m_dumpers.get (i);

            if (dumper.type ().isAssignableFrom (clazz) == true)
            {
               dumper.dump (object);
               return;
            }
         }

         println (object.toString ());
      }
   }

   /**
    * Dumps the stack of the calling thread.
    */
   public static void stack ()
   {
      begin ("BACKTRACE");

      try
      {
         for (StackTraceElement frame : Thread.currentThread ().getStackTrace ())
         {
            println (frame.toString ());
         }

      }
      finally
      {
         end ();
      }
   }
}
