package com.thalesgroup.ccm.codegenerator.container;

import java.util.Hashtable;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeList;

public class CommonServices {

	// Property name to set container interceptors
	public static final String CONTAINER_INTERCEPTOR_PROPERTY = 
		"com.prismtech.idt.acceleo.ccm.has_container_interceptors";
	
	// By default set CONTAINER_INTERCEPTOR_PROPERTY to false.
	// (this property is overwritten with an option)
	static {
		if (System.getProperty(CONTAINER_INTERCEPTOR_PROPERTY) == null)
			System.setProperty(CONTAINER_INTERCEPTOR_PROPERTY, "false");
	}

	/**
	 * Return true if lifecycle interceptors for components 
	 * creation and removal have to be generated.
	 * By default this option is set.
	 * It can be changed setting the 
	 * "com.prismtech.idt.acceleo.ccm.has_lifecycle_interceptors"
	 * System property to "false".
	 * @param foo unused parameter (just to have this operation 
	 * considered as an Acceleo service)
	 * @return true if lifecycle interceptors have to be generated
	 */
	public boolean hasContainerInterceptors(EObject foo) {
		return System.getProperty(CONTAINER_INTERCEPTOR_PROPERTY).equals("true");
	}
	

	
	public static Map<String, String> SelectedHomes = new Hashtable<String, String>();
	
	
	public static void selectHome(String home, String containerType) 
		throws Exception
	{
		if (!containerType.toUpperCase().equals("SESSION") &&
			!containerType.toUpperCase().equals("EXTENSION"))
		{
			throw new Exception("Unknown container type: " + containerType);
		}
		
		// normalize home name (java-like absolute names are accepted)
		String homeName = home.replaceAll("[.]", "::");
		if (!homeName.startsWith("::")) {
			homeName = "::" + homeName;
		}
		
		SelectedHomes.put(homeName, containerType.toUpperCase());
	}
	
	
	
	/**
	 * Return the list of selected homes.
	 * @param foo unused parameter (just to have this operation 
	 * considered as an Acceleo service)
	 * @return the list of selected homes
	 */
    public ENodeList selectedHomes(EObject foo) {
    	ENodeList list = new ENodeList();
    	for (String home : SelectedHomes.keySet()) {
			list.add(new ENode(home, null));
		}
    	return list;
	}
	
    /**
     * Return the container type for a selectes home.
     * @param home
     * @return the container type.
     */
    public String getContainerType(String home) {
    	return SelectedHomes.get(home);
    }
	
	
	
	/**
	 *  Add a prefix at the beginning of a string
	 * @param s the string
	 * @param prefix the prefix to be added
	 * @return the prefixed string
	 */
	public String prepend(String s, String prefix) {
		return prefix + s;
	}
	
}
