package com.thalesgroup.ccm.codegenerator.corba;

import java.util.Date;

import com.prismtech.idt.idlmm.IRObject;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeList;

public class CommonServices {

	/**
	 * Return a list of integers from 1 to max.
	 * This allows to create for loop with:
	 * <%for (intList(5)) {%>
	 * <%}%>
	 * @param foo unused argument, for operation to be detected 
	 *        as an Acceleo service
	 * @param max the maximum int of the list (and thus length of list)
	 * @return the list of integers
	 */
	public ENodeList intList(String foo, int max) {
		ENodeList list = new ENodeList();
		for (int i = 1; i <= max; i++) {
			list.add(new ENode(i, null));
		}
		return list;
	}
	
	/**
	 * Returns the current date as a string
	 * @param foo unused argument, for operation to be detected 
	 *        as an Acceleo service
	 * @return current date
	 */
	public String getDate(String foo) {
		return new Date().toString();
	}
	
	/**
	 * Throws an new Exception with specified message, provoking
	 * Acceleo generation to stop.
	 * @param foo unused argument, for operation to be detected 
	 *        as an Acceleo service
	 * @param message the message
	 * @return throws a new Exception
	 */
	public String abort(String foo, String message) 
		throws Exception
	{
		System.err.println("Aborting code generation! Cause:");
		System.err.println(message);
		throw new Exception(message);
	}
	
	/**
	 * Return the source filename (without path) where 'self' IRObject is defined.
	 * @param self IRObject
	 * @return the source filename (without path)
	 */
	public String sourceFilename(IRObject self) {
		String source = self.getSource();
		int hashIndex = source.indexOf('#');
		int slashIndex = source.lastIndexOf('/');
		return source.substring(slashIndex+1, hashIndex);
	}
	
	/**
	 * Return the source directory is IDL file defining the 'self' IRObject.
	 * @param self IRObject
	 * @return the source directory
	 */
	public String sourceDirectory(IRObject self) {
		String source = self.getSource();
		int slashIndex = source.lastIndexOf('/');
		return source.substring(0, slashIndex);
	}
	
	
	/**
	 * Return the System property indicated by the specified key.
	 * @param foo unused argument, for operation to be detected 
	 *        as an Acceleo service
	 * @param key the name of the System property
	 * @return the string value of the system property, 
	 *         or an empty string if there is no property with that key.
	 */
	public String getSystemProperty(String foo, String key) {
		String result = System.getProperty(key);
		return (result == null ? "" : result);
	}
	
	/**
	 * Return last token of a string made of strings separated with a separator.
	 * @param self the string containing tokens
	 * @param separator the separator
	 * @return the last token
	 */
	public String getLastToken(String self, String separator) {
		String[] tokens = self.split(separator);
		return tokens[tokens.length-1];
	}
	
	/**
	 * Add the specified extension to a String, and returns it.
	 * If the String already has such extension, return the unmodified String
	 * @param self the String to add extension
	 * @param extension the extension
	 * @return the String with extension
	 */
	public String addFileExtension(String self, String extension) {
		if (!self.endsWith(extension)) {
			return self + extension;
		} else {
			return self;
		}
	}
}
