package com.thalesgroup.ccm.codegenerator.corba.cpp;

import com.prismtech.idt.idlmm.AliasDef;
import com.prismtech.idt.idlmm.ArrayDef;
import com.prismtech.idt.idlmm.Component;
import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.Container;
import com.prismtech.idt.idlmm.EnumDef;
import com.prismtech.idt.idlmm.FixedDef;
import com.prismtech.idt.idlmm.Home;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.Interface;
import com.prismtech.idt.idlmm.Named;
import com.prismtech.idt.idlmm.PrimitiveDef;
import com.prismtech.idt.idlmm.PrimitiveKind;
import com.prismtech.idt.idlmm.SequenceDef;
import com.prismtech.idt.idlmm.StringDef;
import com.prismtech.idt.idlmm.StructDef;
import com.prismtech.idt.idlmm.StructMember;
import com.prismtech.idt.idlmm.UnionDef;
import com.prismtech.idt.idlmm.UnionMember;
import com.prismtech.idt.idlmm.Value;
import com.prismtech.idt.idlmm.ValueBoxDef;
import com.prismtech.idt.idlmm.WStringDef;


public class CppMappingServices {

	/**
	 *  Return the same string but with all "." replaces by "::"
	 *  @param the string
	 *  @return the string with c++ style scope
	 */
	public String withCppScope(String self) {
		return self.replaceAll("[.]", "::");
	}
	
	/** 
	 * Return the C++ mapped name of a Named element.
	 * If it's name is a C++ keyword, the name is prefixed with "_cxx_" 
	 * (see C++ mapping §1.1.2).
	 * @param name
	 * @return
	 */
	public String cppName(Named self) {
		String name = self.getName();
		for (String keyword : CPP_KEYWORDS) {
			if (keyword.equals(name)) {
				return "_cxx_"+name;
			}
		}
		return name;
	}
	
	// C++ keywords list
	private final static String[] CPP_KEYWORDS = {
		"and",       "and_eq",  "asm",          "auto",     "bitand",   "bitor",
		"bool",      "break",   "case",         "catch",    "char",     "class",
		"compl",     "const",   "const_cast",   "continue", "default",  "delete",
		"do",        "double",  "dynamic_cast", "else",     "enum",     "explicit",
		"export",    "extern",  "false",        "float",    "for",      "friend",
		"goto",      "if",      "inline",       "int",      "long",     "mutable",
		"namespace", "new",     "not",          "not_eq",   "operator", "or",
		"or_eq",     "private", "protected",    "public",   "register", "reinterpret_cast",
		"return",    "short",   "signed",       "sizeof",   "static",   "static_cast",
		"struct",    "switch",  "template",     "this",     "throw",    "true",
		"try",       "typedef", "typeid",       "typename", "union",    "unsigned",
		"using",     "virtual", "void",         "volatile", "wchar_t",  "while",
		"xor",       "xor_eq"
	};
	
	/**
	 * Return the C++ mapped classname with full scope 
	 * @param c the type to be mapped
	 * @return the C++ mapped classname
	 */
	public String cppFullClassname(Contained self) {
	      String name = cppName(self);

	      if (name == null)
	      {
	         name = "<noname>";
	      }

	      if (! (self.getContainer () instanceof Contained))
	      {
	         return "::" + name;
	      }

	      return cppFullClassname ((Contained) self.getContainer ()) + "::" + name;
	}
	
	/**
	 * Return the C++ full scope of specified type
	 * @param c the type
	 * @return the C++ full scope
	 */
	public String cppScope(Contained self) {
		Container c = self.getContainer();
		
		if (! (c instanceof Contained)) {
			if (c instanceof Named) {
				return "::" + cppName((Named) c); 
			} else {
				return "::";
			}
		} else {
			return cppFullClassname((Contained) c);
		}
	}
	
	/**
	 * Remove "::" prefix to specified string (if exist)
	 * @param self the string
	 * @return the string without "::" prefix
	 */
	public String noColonPrefix(String self) {
		if (self != null && self.startsWith("::")) {
			return self.substring(2);
		} else {
			return self;
		}
	}
	
	/************************/
	/** isString() service **/
	/************************/
	public boolean isString(IDLType self) {
		if (self instanceof PrimitiveDef) {
			PrimitiveKind kind = ((PrimitiveDef) self).getKind();
			if (kind.equals(PrimitiveKind.PK_STRING)) {
				return true;
			} else {
				return false;
			}
		}
		if (self instanceof StringDef) {
			return true;
		}
		if (self instanceof AliasDef) {
			// check aliased type
			return isString(((AliasDef) self).getType());
		}
		return false;
	}
	
	/************************/
	/** isWString() service **/
	/************************/
	public boolean isWString(IDLType self) {
		if (self instanceof PrimitiveDef) {
			PrimitiveKind kind = ((PrimitiveDef) self).getKind();
			if (kind.equals(PrimitiveKind.PK_WSTRING)) {
				return true;
			} else {
				return false;
			}
		}
		if (self instanceof WStringDef) {
			return true;
		}
		if (self instanceof AliasDef) {
			// check aliased type
			return isWString(((AliasDef) self).getType());
		}
		return false;
	}
	
	/*********************************/
	/** isVariableLength() services **/
	/*********************************/
	public boolean isVariableLength(IDLType self) {
		if (self instanceof PrimitiveDef) {
			return isVariableLength((PrimitiveDef) self);
		}
		if (self instanceof StringDef) {
			return true;
		}
		if (self instanceof WStringDef) {
			return true;
		}
		if (self instanceof FixedDef) {
			return false;
		}
		if (self instanceof SequenceDef) {
			return true;
		}
		if (self instanceof ArrayDef) {
			// depend content type
			return isVariableLength(((ArrayDef) self).getType());
		}
		if (self instanceof EnumDef) {
			return false;
		}
		if (self instanceof StructDef) {
			// depend content type
			return isVariableLength((StructDef) self);
		}
		if (self instanceof UnionDef) {
			// depend content type
			return isVariableLength((UnionDef) self);
		}
		if (self instanceof Interface) {
			return true;
		}
		if (self instanceof Interface) {
			return true;
		}
		if (self instanceof Value) {
			return true;
		}
		if (self instanceof ValueBoxDef) {
			return true;
		}
		if (self instanceof Component) {
			return true;
		}
		if (self instanceof Home) {
			return true;
		}
		if (self instanceof AliasDef) {
			// check aliased type
			return isVariableLength(((AliasDef) self).getType());
		}
		
		return true;
	}

	public boolean isVariableLength(PrimitiveDef self) {
		PrimitiveKind kind = self.getKind();
		// Only string, wstring, any and object references are variable-length
		if (kind.equals(PrimitiveKind.PK_STRING) ||
			kind.equals(PrimitiveKind.PK_WSTRING) ||
			kind.equals(PrimitiveKind.PK_ANY) ||
			kind.equals(PrimitiveKind.PK_TYPE_CODE) ||
			kind.equals(PrimitiveKind.PK_PRINCIPAL) ||
			kind.equals(PrimitiveKind.PK_OBJREF) ||
			kind.equals(PrimitiveKind.PK_VALUE_BASE)) 
		{
			return true;
		} else {
			return false;
		}
	}
		
	public boolean isVariableLength(UnionDef self) {
		// search if 1 member is a variable-length type
		for(UnionMember member : self.getMembers()) {
			if (isVariableLength(member.getType())) {
				return true;
			}
		}
		return false;
	}
	
	public boolean isVariableLength(StructDef self) {
		// search if 1 member is a variable-length type
		for(StructMember member : self.getMembers()) {
			if (isVariableLength(member.getType())) {
				return true;
			}
		}
		return false;
	}
	
	
	public boolean isExtractedFromAnyAsPointer(IDLType self) {
		// Only Any, struct, union and sequences are extracted as pointer
		// (actually CORBA objects are, but _ptr types are used) 
		if (self instanceof PrimitiveDef) {
			return ((PrimitiveDef) self).getKind().equals(PrimitiveKind.PK_ANY);
		}
		if (self instanceof StructDef || self instanceof UnionDef || self instanceof SequenceDef) {
			return true;
		}		
		
		if (self instanceof AliasDef) {
			// check aliased type
			return isExtractedFromAnyAsPointer(((AliasDef) self).getType());
		}
		
		return false;
	}
	
}
