package com.thalesgroup.ccm.codegenerator.corba.java;

import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.Container;
import com.prismtech.idt.idlmm.ModuleDef;

import fr.obeo.acceleo.gen.template.eval.ENodeCastException;

public class JavaMappingServices {

	/**
	 * Revert a pargma prefix. 
	 * E.g. "acme.com" is reverted as "com.acme"
	 */
	public String revertPrefix(String prefix) {
		String[] names = prefix.split("[.]");
		String result = names[names.length-1];
		for (int i = names.length-2; i >= 0; i--) {
			result += "." + names[i];
		}
		return result;
	}

	/**
	 * Return the java mapped classname with full scope 
	 * (including reverted pragma prefix if specified)
	 * @param c the type to be mapped
	 * @return the java mapped classname
	 */
	public String javaFullClassname(Contained self) {
		return javaPackage(self) + '.' + self.getName();
	}

	
	/**
	 * Return the package of the java mapped class.
	 * (including reverted pragma prefix if specified)
	 * @param self the type to be mapped
	 * @return the package of the mapped class
	 */
	public String javaPackage(Contained self) {
		return javaPackage(self, true);
	}

		
	/**
	 * Return the package of the java mapped class, 
	 * but ignoring any specified pragma prefix
	 * @param self the type to be mapped
	 * @return the package of the mapped class
	 */
	public String javaPackageNoPrefix(Contained self) {
		return javaPackage(self, false);
	}

	
	private String javaPackage(Contained self, boolean addPrefix) {
		// Java mapping specification (02-08-05) doesn't specify
		// the way pragma prefix are used in mapped package of a type.
		// Cardamom choice is to prepend the package with the reverted 
		// pragma prefix (as for "org.omg.CORBA").
		// But this rule well apply only if pragma prefix is specified
		// before the first module, and not after.
		// E.g. with following specification, we don't know what to do:
		//    #pragma prefix "foo.com"
		//    module M {
		//    #pragma prefix "bar.com"
		//        interface I {};
		//    };
		//
		// Thus, we choose to consider only the pragma prefix for the very
		// first scope of the IDL (most of the time a module)
		// In above example, generated classname will be:
		//    com.foo.M.I

        Container container = self.getContainer();
        
        if (container instanceof Contained) {
        	// get parent packages
        	Contained contained = (Contained) container;
        	String parentPackages = javaPackage(contained, addPrefix);
        	
        	String currentPackage = contained.getName();
        	if (!(contained instanceof ModuleDef)) {
        		// self is nested within a non-module definition.
        		// Scope package name is constructed by	appending Package to the IDL type name
        		currentPackage += "Package";
        	}
        	
        	// return full scoped name
        	if (parentPackages.length() > 0) {
        		return parentPackages + '.' + currentPackage;
        	} else {
        		return currentPackage;
        	}
        	
        } else {
        	if (addPrefix) {
        		// it's the first container, return reverted prefix
        		// (if specified) as package name
        		String prefix = self.getPrefix();
        		if (prefix == null || prefix.length() == 0) {
        			return "";
        		} else {
        			return revertPrefix(prefix);
        		}
        	} else {
        		// don't add prefix
        		return "";
        	}
        }
	}
	
}
