package com.thalesgroup.ccm.codegenerator;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.acceleo.engine.EMT;
import org.acceleo.engine.context.EvalContext;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.gen.AcceleoGenerate;
import org.acceleo.gen.extension.Extension;
import org.acceleo.template.Resource;
import org.acceleo.template.core.Script;
import org.acceleo.template.core.Template;
import org.acceleo.tools.resources.Resources;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;

import fr.obeo.acceleo.gen.template.eval.ENode;


public class AcceleoDriver {

	private ResourceSet resourceSet;
	private EMT emt;
	private AcceleoGenerate generator;
	
	private Map<String, Script> scriptsCache;
	
	
	public AcceleoDriver(String emtFile) 
		throws Exception
	{
		resourceSet = Extension.getResourceSet();		
		emt = new EMT(resourceSet, getEMTURI(emtFile),Thread.currentThread().getContextClassLoader());
		generator = new AcceleoGenerate();
		scriptsCache = new Hashtable<String, Script>();
	}
	
	
	public void setOutputDir(String outputDir) {
		// set output directory
		generator.setGenRootDir(outputDir);
	}
	
	
	/*
	 * find a EMT file
	 */
	private static URI getEMTURI(String name) 
		throws Exception 
	{
		URI ret = Resources.getResourceURI(name);
		if(ret == null){
			throw new Exception("Unable to find emt " + name);
		}
		return ret;
	}
	
	
	/*
	 * Find a script with specified name in Templates model
	 */
	private Script findScript(String name) 
		throws Exception
	{
		// try to find script in cache
		Script script = scriptsCache.get(name);
		if (script != null) {
			return script;
		}
		
		// try in top level template
		for (Script s : emt.getTemplates().get(0).getScripts()) {
			if (s.getDescriptor().getName().equals(name)) {
				scriptsCache.put(name, s);
				return s;
			}
		}
		
		// try in imported templates
		for (Resource r : emt.getTemplates().get(0).getImports()) {
			if (r instanceof Template) {
				Template t = (Template) r;
				for (Script s : t.getScripts()) {
					if (s.getDescriptor().getName().equals(name)) {
						scriptsCache.put(name, s);
						return s;
					}
				}
			}
		}
		
		throw new Exception("Script " + name + " not found !!!");
	}

	/*
	 * Code copied from org.acceleo.engine.EMT
	 */
	private List<Template> getExecutionPath(Template template) {
		List<Template> res = new ArrayList<Template>();
		boolean found = false;
		Template rootTemplate = emt.getTemplates().get(0);
		res.add(rootTemplate);

		Iterator<Resource> it = rootTemplate.getImports().iterator();
		while (it.hasNext() && !found) {
			Resource resource = it.next();
			if (resource == template) {
				found = true;
				res.add((Template) resource);
			}
		}

		// if (!found) {
		// it = rootTemplate.getImports().iterator();
		// while (it.hasNext() && !found) {
		// Resource resource = it.next();
		// if (resource instanceof Template) {
		// found = subGetExecutionPath((Template) resource, template, res);
		// if (found) {
		// res.add((Template) resource);
		// }
		// }
		// }
		// }
		return res;
	}
	
	

	public void generate(String scriptName, EObject obj) 
	throws Exception
	{
		generate(scriptName, obj, null);
	}
	
	
	public void generate(String scriptName, EObject obj, String outputFile) 
		throws Exception
	{
		// find script
		Script script = findScript(scriptName);
		
		EvalContext evalContext = new EvalContext();
		// set-up evalContext (no idea what for...)
		List<Template> execPath = getExecutionPath(script.getTemplate());
		for (Template template : execPath) {
			evalContext.startTemplate(template);
		}

		// generate file content
		ENode content = new ENode(obj, (IEvaluable) script, evalContext.getLaunchManager().isSynchronize());
		content = ((IEvaluable) script).gen(content, evalContext);

		if (outputFile == null || outputFile.length() == 0) {
			// generate filename from "file" script's attribute
			final ENode fileName = ((IEvaluable) script.getDescriptor().getFile()).gen(
					new ENode(obj, 
							  (IEvaluable) script.getDescriptor().getFile(), 
							  evalContext.getLaunchManager().isSynchronize()), 
					evalContext);

			outputFile = fileName.asString().trim();
			
			if (outputFile.length() ==0 ) {
				// "file" script's attribute doesn't exist or returned ""
				throw new Exception("Cannot guess in wich filename to generate with script '" + scriptName + "'");
			}
		}
		
		
		System.out.println("  ...Generating file: " + outputFile);
		
		// write generated file
		generator.writeFile(outputFile, content.asString(), evalContext.getLaunchManager());
	}
	
	
	
	
}
