package com.thalesgroup.cdmw.codegenerator.cdmwinit;

import java.util.List;
import java.util.Vector;

import com.prismtech.idt.idlmm.HomeDef;
import com.prismtech.idt.idlmm.IDLType;

import fr.obeo.acceleo.gen.template.eval.ENode;


/**
 * This Acceleo Services class allows to access data
 * loaded from a XML code generation file.
 * Those data are stored in static variables and should have 
 * been set by the CdmwXMLFile class for the <datastore-types> element.
 */
public class DatastoreTypesServices {

	/**
	 * Data retrieved from a <datastore-type> element 
	 */
	public static class DatastoreTypeData {
		// datastore-type/@typename
		public String typename = "";
		// datastore-type/@oid-type
		public IDLType oidType = null;
		// datastore-type/@data-type
		public IDLType dataType = null;
		// datastore-type/@idl-filename
		public String idlFilename = "";
		// datastore-type/@filename
		public String filename = "";
	}
	
	// List of DatastoreTypeData, each one corresponding to a 
	// <datastore-type> element
	// Note: A List is used instead of a Map with typename as key,
	//       in order to keep declaration order.
	public static List<DatastoreTypeData> datastoreTypes;
	
	// current datastore-type to consider for 
	// DataStoreTypeTraits file generation
	public static String currentDatastoreTypename;
	
	
	static {
		reset();
	}
	
	public static void reset() {
		datastoreTypes = new Vector<DatastoreTypeData>();
		currentDatastoreTypename = "";
	}
	

	public static DatastoreTypeData getDatastoreTypeData(String dsTypename) {
		for (DatastoreTypeData dsData : datastoreTypes) {
			if (dsData.typename.equals(dsTypename))
				return dsData;
		}

		System.err.println("INTERNAL ERROR: datastore-type '" + 
				dsTypename + "' required but was not loaded");
		throw new Error("INTERNAL ERROR: datastore-type '" + 
				dsTypename + "' required but was not loaded");
	}
	
	public List<String> getDatastoreTypenames(ENode foo) {
		List<String> ids = new Vector<String>(datastoreTypes.size());
		for (DatastoreTypeData datastore : datastoreTypes) {
			ids.add(datastore.typename);
		}
		return ids;
	}
	
	public IDLType getDatastoreOidType(ENode foo, String datastoreId) {
		return getDatastoreTypeData(datastoreId).oidType;
	}
	
	public IDLType getDatastoreDataType(ENode foo, String datastoreId) {
		return getDatastoreTypeData(datastoreId).dataType;
	}
	
	public String getDatastoreIdlFilename(ENode foo, String datastoreId) {
		return getDatastoreTypeData(datastoreId).idlFilename;
	}
	
	public String getDatastoreFilename(ENode foo, String datastoreId) {
		return getDatastoreTypeData(datastoreId).filename;
	}
	
	public String getCurrentDatastoreTypename(ENode foo) {
		return currentDatastoreTypename;
	}
	
}