package com.thalesgroup.cdmw.codegenerator.cdmwinit;

import java.util.List;
import java.util.Vector;

import com.prismtech.idt.idlmm.HomeDef;
import com.thalesgroup.cdmw.codegenerator.cdmwinit.DatastoreTypesServices.DatastoreTypeData;

import fr.obeo.acceleo.gen.template.eval.ENode;


/**
 * This Acceleo Services class allows to access data
 * loaded from a XML code generation file.
 * Those data are stored in static variables and should have 
 * been set by the CdmwXMLFile class for a <program> element.
 * To store data for another <program> element, the reset()
 * operation should be called before.  
 */
public class ProgramServices {

	/**
	 * Data retrieved from a <framework> element 
	 */
	public static class FrameworkData {
		// framework/@idref
		public String idref = "";
		// framework/@occurrences
		public int occurrences = 1;
		// corresponding factory/@idl-interface
		public String factoryIdlInterface = "";
		// if lifecycle-framework/user-factory-impl exists
		public boolean isUserFactory = false;
		// if factory/@kind = HOME
		public boolean isHome = false;
		// home-impl/@category
		public String homeCategory = "";
		// HomeDef model element corresponding to factory/@idl-interface (in case of a Home)
		public HomeDef homeDef = null;
		// if factory/@kind = NAMED
		public boolean isNamedFactory = false;
		// factory/*/@classname
		public String factoryClassname = "";
		// factory/*/@filename
		public String factoryFilename = "";
		// factory/*/*/@classname
		public String allocatorClassname = "";
		// factory/*/*/@filename
		public String allocatorFilename = "";
		// corresponding lifecycle-framework category
		public String category = "USER_DEFINED";
		// corresponding lifecycle-framework threading
		public String threading = "";
	}
	
	/**
	 * Data retrieved from a <datastore> element 
	 */
	public static class DatastoreData {
		// DatastoreTypeData with tymename == datastore/@idref
		public DatastoreTypeData datastoreType = null;
		// datastore/@occurrences
		public int occurrences = 0;
	}
	
	// program/@filename
	public static String filename;
	// program/@init-dtd
	public static String initDTD;
	// program/@directory
	public static String directory;
	// set from program/@classname (if no classname attribute, a default package is set)
	public static String cdmwInterfacePackage;
	// set from program/@classname (or from program/@filename if no classname)
	public static String classname;
	
	// services
	public static boolean hasRepositoryInterface;
	public static boolean hasContainerInterception;
	public static boolean hasPerformance;
	public static boolean hasTrace;
	public static boolean hasRequestFollowup;
	public static boolean hasBufferingEnabled;
	public static boolean hasSystemManagement;
	public static boolean hasTime;
	public static boolean hasRecording;
	public static boolean hasWatchdog;
	public static boolean hasFaultTolerance;
	public static boolean hasFaultToleranceServer;
	public static boolean hasFaultToleranceReplyLogging;
	public static boolean hasFaultToleranceWithDatastore;
	public static String  faultToleranceDatastoreKind;
	public static boolean hasLoadBalancing;
	public static boolean hasLoadBalancingServer;
	public static boolean hasLifeCycle;
	public static boolean hasHomeImpl;
	public static boolean hasEventPorts;
	
	// List of FrameworkData, each one corresponding to a <framework> element
	// Note: A List is used instead of a Map with idref as key,
	//       in order to keep declaration order.
	public static List<FrameworkData> frameworks;
	
	// List of DatastoreData actually used, each one corresponding to a 
	// <fault-tolerance>/<datastores>/<datastore> element
	// Note: A List is used instead of a Map with idref as key,
	//       in order to keep declaration order.
	public static List<DatastoreData> usedDatastores;
	
	
	static {
		reset();
	}
	
	public static void reset() {
		filename = "";
		initDTD = "";
		directory = "";
		cdmwInterfacePackage = "";
		classname = "";
		hasRepositoryInterface = false;
		hasContainerInterception = false;
		hasPerformance = false;
		hasTrace = false;
		hasRequestFollowup = false;
		hasBufferingEnabled = true;
		hasSystemManagement = true;
		hasTime = false;
		hasRecording = false;
		hasWatchdog = false;
		hasFaultTolerance = false;
		hasFaultToleranceServer = false;
		hasFaultToleranceReplyLogging = false;
		hasFaultToleranceWithDatastore = false;
		faultToleranceDatastoreKind = "";
		hasLoadBalancing = false;
		hasLoadBalancingServer = false;
		hasLifeCycle = false;
		hasHomeImpl = false;
		hasEventPorts = false;
		frameworks = new Vector<FrameworkData>();
		usedDatastores = new Vector<DatastoreData>();
	}
	

	public String filename(ENode foo) {
		return filename;
	}
	
	public String initDTD(ENode foo) {
		return initDTD;
	}
	
	public String directory(ENode foo) {
		return directory;
	}
	
	public String cdmwInterfacePackage(ENode foo) {
		return cdmwInterfacePackage;
	}
	
	public String classname(ENode foo) {
		return classname;
	}
	
	
	public boolean hasRepositoryInterface(ENode foo) {
		return hasRepositoryInterface;
	}
	
	public boolean hasContainerInterception(ENode foo) {
		return hasContainerInterception;
	}
	
	public boolean hasPerformance(ENode foo) {
		return hasPerformance;
	}
	
	public boolean hasTrace(ENode foo) {
		return hasTrace;
	}
	
	public boolean hasRequestFollowup(ENode foo) {
		return hasRequestFollowup;
	}
	
	public boolean hasBufferingEnabled(ENode foo) {
		return hasBufferingEnabled;
	}
	
	public boolean hasSystemManagement(ENode foo) {
		return hasSystemManagement;
	}
	
	public boolean hasTime(ENode foo) {
		return hasTime;
	}
	
	public boolean hasRecording(ENode foo) {
		return hasRecording;
	}
	
	public boolean hasWatchdog(ENode foo) {
		return hasWatchdog;
	}
	
	public boolean hasFaultTolerance(ENode foo) {
		return hasFaultTolerance;
	}
	
	public boolean hasFaultToleranceServer(ENode foo) {
		return hasFaultToleranceServer;
	}
	
	public boolean hasFaultToleranceReplyLogging(ENode foo) {
		return hasFaultToleranceReplyLogging;
	}
	
	public boolean hasFaultToleranceWithDatastore(ENode foo) {
		return hasFaultToleranceWithDatastore;
	}
	
	public String faultToleranceDatastoreKind(ENode foo) {
		return faultToleranceDatastoreKind;
	}
	
	public boolean hasLoadBalancing(ENode foo) {
		return hasLoadBalancing;
	}
	
	public boolean hasLoadBalancingServer(ENode foo) {
		return hasLoadBalancingServer;
	}
	
	public boolean hasLifeCycle(ENode foo) {
		return hasLifeCycle;
	}
	
	public boolean hasHomeImpl(ENode foo) {
		return hasHomeImpl;
	}
	
	public boolean hasEventPorts(ENode foo) {
		return hasEventPorts;
	}
	
	
	private static FrameworkData getFrameworkData(String frameworkName) {
		for (FrameworkData fwkData : frameworks) {
			if (fwkData.idref.equals(frameworkName))
				return fwkData;
		}

		System.err.println("INTERNAL ERROR: framework '" + 
				frameworkName + "' required but was not loaded");
		throw new Error("INTERNAL ERROR: framework '" + 
				frameworkName + "' required but was not loaded");
	}
	
	public List<String> getFrameworksIds(ENode foo) {
		List<String> ids = new Vector<String>(frameworks.size());
		for (FrameworkData fwkData : frameworks) {
			ids.add(fwkData.idref);
		}
		return ids;
	}
	
	public int getOccurrences(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).occurrences;
	}
	
	public String getFactoryIdlInterface(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).factoryIdlInterface;
	}
	
	public boolean isUserFactory(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).isUserFactory;
	}
	
	public boolean isHome(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).isHome;
	}
	
	public String homeCategory(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).homeCategory;
	}
	
	public HomeDef getHome(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).homeDef;
	}
	
	public boolean isNamedFactory(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).isNamedFactory;
	}
	
	public String getFactoryClassname(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).factoryClassname;
	}
	
	public String getFactoryFilename(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).factoryFilename;
	}
	
	public String getAllocatorClassname(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).allocatorClassname;
	}
	
	public String getAllocatorFilename(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).allocatorFilename;
	}
	
	public String getCategory(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).category;
	}
	
	public String getThreading(ENode foo, String frameworkName) {
		return getFrameworkData(frameworkName).threading;
	}
	


	private static DatastoreData getUsedDatastoreData(String dsTypename) {
		for (DatastoreData dsData : usedDatastores) {
			if (dsData.datastoreType.typename.equals(dsTypename))
				return dsData;
		}

		System.err.println("INTERNAL ERROR: datastore '" + 
				dsTypename + "' required but was not loaded");
		throw new Error("INTERNAL ERROR: datastore '" + 
				dsTypename + "' required but was not loaded");
	}
	
	public List<String> getUsedDatastoreTypenames(ENode foo) {
		List<String> ids = new Vector<String>(usedDatastores.size());
		for (DatastoreData datastore : usedDatastores) {
			ids.add(datastore.datastoreType.typename);
		}
		return ids;
	}
	
	public int getUsedDatastoreOccurrences(ENode foo, String datastoreId) {
		return getUsedDatastoreData(datastoreId).occurrences;
	}
		
	public int countAllDatastoreOccurrences(ENode foo) {
		int count = 0;
		for (DatastoreData datastore : usedDatastores) {
			count += datastore.occurrences;
		}
		return count;
	}
	
	
	
	
	public List<String> dtdServicesList(ENode foo) {
		List<String> result = new Vector<String>();
		
		if (hasRepositoryInterface)
			result.add("repository-interface");
		if (hasPerformance)
			result.add("performance");
		if (hasTrace)
			result.add("trace");
		if (hasTime)
			result.add("time");
		if (hasRecording)
			result.add("recording");
		if (hasWatchdog)
			result.add("watchdog");
		if (hasFaultTolerance)
			result.add("fault-tolerance");
		if (hasLoadBalancing)
			result.add("load-balancing");
		if (hasLifeCycle && !hasHomeImpl)
			result.add("lifecycle");
		
		return result;
	}
}
