package com.thalesgroup.cdmw.codegenerator.lifecycle;

import java.util.List;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;

import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.OperationDef;


public class ModelServices {

	/**
	 * Return all InterfaceDef types returned by operations of specified interface.
	 * (result also includes base interfaces of returned interfaces)
	 * @param intf The specified interface
	 * @return the resulting list
	 */
	public EList<InterfaceDef> getAllReturnedInterfaces(InterfaceDef intf) {
		EList<InterfaceDef> result = new BasicEList<InterfaceDef>();
		
		EList<OperationDef> operations = intf.getOperations();
	
		for (OperationDef operation : operations) {
			if (operation.getType() instanceof InterfaceDef) {
				InterfaceDef returnedIntf = (InterfaceDef)operation.getType();
				addUnique(returnedIntf, result);
				for (InterfaceDef baseIntf : returnedIntf.getAllBaseInterfaces()) {
					addUnique(baseIntf, result);
				}
			}
		}
		return result;
	}

	/**
	 * Add an object to specified list only if not already 
	 * existing in the list. (avoid duplicate)
	 * @param obj the object to add
	 * @param list the list for addition
	 */
	private <E> void addUnique(E obj, List<E> list) {
		if (!list.contains(obj)) {
			list.add(obj);
		}
	}
	
	
	/**
	 * Find specified InterfaceDef's operations with an InterfaceDef as return type.
	 * (also return factory operations from inherited interfaces)
	 */
	public EList<OperationDef> findAllFactoryOperations(InterfaceDef intf) {
		EList<OperationDef> result = new BasicEList<OperationDef>();
		
		EList<OperationDef> operations = intf.getAllOperations();
		for (OperationDef operation : operations) {
			if (operation.getType() instanceof InterfaceDef) {
				result.add(operation);
			}
		}
		
		return result;
	}
	
}
