package com.thalesgroup.cdmw.codegenerator;

import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.acceleo.engine.eval.imports.services.EObjectServices;
import org.eclipse.emf.ecore.EObject;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import com.prismtech.idt.idlmm.Contained;
import com.prismtech.idt.idlmm.HomeDef;
import com.prismtech.idt.idlmm.IDLType;
import com.prismtech.idt.idlmm.IdlmmFactory;
import com.prismtech.idt.idlmm.InterfaceDef;
import com.prismtech.idt.idlmm.PrimitiveDef;
import com.prismtech.idt.idlmm.PrimitiveKind;
import com.prismtech.idt.idlmm.Repository;
import com.thalesgroup.cdmw.codegenerator.cdmwinit.DatastoreTypesServices;
import com.thalesgroup.cdmw.codegenerator.cdmwinit.ProgramServices;
import com.thalesgroup.cdmw.codegenerator.cdmwinit.DatastoreTypesServices.DatastoreTypeData;
import com.thalesgroup.cdmw.codegenerator.cdmwinit.ProgramServices.DatastoreData;
import com.thalesgroup.cdmw.codegenerator.cdmwinit.ProgramServices.FrameworkData;
import com.thalesgroup.cdmw.codegenerator.lifecycle.LifeCycleServices;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeIterator;
import fr.obeo.acceleo.gen.template.eval.ENodeList;



public class CdmwXMLFile {

	class HomeData {
		public String idlInterface;
		public String category;
		
		public HomeData(String idlInterface, String category) {
			this.idlInterface = idlInterface;
			this.category = category;
		}
		
		public String toString() {
			return idlInterface + "," + category;
		}
	}
	
	class FactoryImplData {
		public String classname;
		public String category;
		public boolean hasServantProvider;

		public FactoryImplData(String classname, 
							   String category, 
							   boolean hasServantProvider) 
		{
			this.classname = classname;
			this.category = category;
			this.hasServantProvider = hasServantProvider;
		}
	}
	
	
	
    /**
     * Local resolver to find DTD in specified directory 
     */
    class LocalDTDResolver implements EntityResolver {
        
        private String dtdDirectory = null;
        
        public LocalDTDResolver(String dtdDirectory) {
            this.dtdDirectory = dtdDirectory;
        }

        public InputSource resolveEntity(String publicID, String systemID)
            throws org.xml.sax.SAXException {

            int lastSlash = systemID.lastIndexOf("/");
            String fileName = systemID.substring(lastSlash);
            
            return new InputSource(dtdDirectory + fileName);
        }
    }
	
    
    /**
     * Local error handler 
     */
    class LocalErrorHandler extends DefaultHandler {
        
        private SAXParseException ex = null;
        
        public void fatalError(SAXParseException e) {
            ex = e;
        }

        public void error(SAXParseException e) {
            ex = e;
        }

        public void warning(SAXParseException e) {
            ex = e;
        }
        
        public SAXParseException getError() {
            return ex;
        }
    }
	
    
    private Document doc;
    
    private XPath xpathResolver;
    
    
	public CdmwXMLFile(String xmlFile, String dtdDir) 
		throws Exception
	{
		try {
			LocalErrorHandler errorHandler = new LocalErrorHandler();
	        
	        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	        dbf.setIgnoringComments(true);
	        dbf.setNamespaceAware(true);
	        dbf.setValidating(true);
	
	        DocumentBuilder db = dbf.newDocumentBuilder();
	        db.setErrorHandler(errorHandler);
	        db.setEntityResolver(new LocalDTDResolver(dtdDir));
	        
	        doc = db.parse(xmlFile);
	        
	        if(errorHandler.getError() != null)
	        {
	            throw errorHandler.getError();
	        }
	     
	        xpathResolver = XPathFactory.newInstance().newXPath();
	        
		} catch(IOException e) {
			e.printStackTrace();
			throw new Exception("Error loading " + xmlFile + " : " + 
								e.getMessage());
		} catch(ParserConfigurationException e) {
			e.printStackTrace();
			throw new Exception("Error in parser configuration: " + 
					e.getMessage());
		} catch(SAXParseException e) {
			e.printStackTrace();
			throw new Exception("Error loading " + xmlFile + " : " + 
					e.getMessage());
		}
	}
	
	
	/**
	 * This function returns true when 
	 * /cdmw/factories contains at least one child element 
	 * while /cdmw/lifecycle-frameworks and /cdmw/programs are empty.	 
	 * @return true if only /cdmw/factories contains elements
	 */
	public boolean hasOnlyFactoriesDefs() {
		try {
			// get all children of /cdmw/factories
			NodeList factoriesNodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/factories/*", 
					doc, 
					XPathConstants.NODESET);
		
			NodeList lfcNodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/lifecycle-frameworks/*", 
					doc, 
					XPathConstants.NODESET);
		
			NodeList progNodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/programs/*", 
					doc, 
					XPathConstants.NODESET);
		
			return factoriesNodes.getLength() > 0 && 
				   lfcNodes.getLength() == 0 &&
				   progNodes.getLength() == 0;
					
			
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return false;
		}
	}
	
	
	/**
	 * Return content list of all &lt;idl&gt; elements
	 * @return idl files to be converted
	 */
	public List<String> getIdlFilesToConvert() {
		try {
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/convert-idls/idl/@idl-filename",
					doc,
					XPathConstants.NODESET);
			
			List<String> result = new Vector<String>();
			for (int i = 0; i < nodes.getLength(); i++) {
				Attr a = (Attr) nodes.item(i);
				result.add(a.getValue());
			}
			
			return result;
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}
	}
	
	
	/**
	 * Return content list of all &lt;idl-search-dir&gt; elements
	 * @return idl search directories
	 */
	public List<String> getIdlSearchDirs() 
		throws Exception
	{
		try {
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"//idl-search-dir/child::text()",
					doc,
					XPathConstants.NODESET);
			
			List<String> result = new Vector<String>();
			for (int i = 0; i < nodes.getLength(); i++) {
				Text t = (Text) nodes.item(i);
				result.add(t.getData());
			}
			
			// replace environment variables in all elements
			replaceEnvVar(result);
			
			return result;
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}
	}
	
	/**
	 * Return content list of all idl-filename attributes 
	 * from all &lt;factory&gt; elements
	 * @return idl files to be converted
	 */
	public List<String> getFactoriesIdlFilenames() {
		try {
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/factories/factory/@idl-filename",
					doc,
					XPathConstants.NODESET);
			
			List<String> result = new Vector<String>();
			for (int i = 0; i < nodes.getLength(); i++) {
				Attr a = (Attr) nodes.item(i);
				result.add(a.getValue());
			}
			
			return result;
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}
	}
	

	/**
	 * Return content list of all idl-filename attributes 
	 * from all &lt;datastore-type&gt; elements
	 * @return idl files to be converted
	 */
	public List<String> getDatastoresIdlFilenames() {
		try {
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/datastore-types/datastore-type/@idl-filename",
					doc,
					XPathConstants.NODESET);
			
			List<String> result = new Vector<String>();
			for (int i = 0; i < nodes.getLength(); i++) {
				Attr a = (Attr) nodes.item(i);
				result.add(a.getValue());
			}
			
			return result;
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}
	}


	
	/**
	 * Return content list of all &lt;excluded-idl&gt; elements
	 * @return excluded idl files
	 */
	public List<String> getIdlFilesToExclude() {
		try {
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/convert-idls/excluded-idls/excluded-idl/child::text()",
					doc,
					XPathConstants.NODESET);
			
			List<String> result = new Vector<String>();
			for (int i = 0; i < nodes.getLength(); i++) {
				Text t = (Text) nodes.item(i);
				result.add(t.getData());
			}
			
			return result;
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}
	}
	
	/**
	 * Return content of &lt;preprocessing-flags&gt; elements
	 * @return preprocessing flags
	 */
	public String getPreprocessingFlags() {
		try {
			Text t = (Text) xpathResolver.evaluate(
					"//preprocessing-flags/child::text()",
					doc,
					XPathConstants.NODE);
			
			if (t == null)
				return "";
			else
				return t.getData();
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}
	}
	
	
	
	/**
	 * Replace all environment variables (${...}) in all elements of the list
	 * @param list the list
	 * @throws Exception if an environment variable is not defined
	 */
	private static void replaceEnvVar(List<String> list) 
		throws Exception
	{
		// replace all ${.*}
		for (int i = 0; i < list.size(); i++) {
			String s = list.get(i);
			String s2 = replaceEnvVar(s);
			list.set(i, s2);
		}
	}
	
	/**
	 * Replace all environment variables (${...}) in the string
	 * @param s the string
	 * @return the string with all variables replaced
	 * @throws Exception if an environment variable is not defined
	 */
	private static String replaceEnvVar(String s) 
		throws Exception
	{
		Matcher m = Pattern.compile("\\$\\{([\\w]*)\\}").matcher(s);
		
		if (m.find()) {
			String envVar = s.substring(m.start(1), m.end(1));
			String envVarValue = System.getenv(envVar);
			if (envVarValue == null) {
				throw new Exception("Unknown environment variable " +
						            s.substring(m.start(), m.end()));
			}
			String s2 = s.substring(0, m.start()) + 
						envVarValue +
						s.substring(m.end(), s.length());
			
			return replaceEnvVar(s2);
		}
		
		return s;
	}


	public Map<String, HomeData> getHomesData() 
		throws Exception
	{
		try {
			Map<String, HomeData> result = new Hashtable<String, HomeData>();

			// get all <lifecycle-framework> nodes
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/lifecycle-frameworks/lifecycle-framework",
					doc,
					XPathConstants.NODESET);
			
			for (int i = 0; i < nodes.getLength(); i++) {
				Node n = nodes.item(i);
				// get id
				String id = ((Attr) n.getAttributes().getNamedItem("id")).getValue();
				// get factory-idref
				String factIdref = ((Attr) n.getAttributes().getNamedItem("factory-idref")).getValue();
				
				// check if corresponding factory is a Home
				Node f = (Node) xpathResolver.evaluate(
						"/cdmw/factories/factory[@idl-interface = '"+factIdref+"']",
						doc,
						XPathConstants.NODE);
				if (f == null) {
					throw new Exception("Cannot find factory '" + factIdref + "'");
				}
				
				String kind = ((Attr) f.getAttributes().getNamedItem("kind")).getValue();
				if ("HOME".equals(kind)) {
					
					// get home category
					Attr a = (Attr) xpathResolver.evaluate(
							"home-impl/@category",
							n,
							XPathConstants.NODE);
					String category = a.getValue();
					
					// add to result
					result.put(id, new HomeData(factIdref, category));
				}
			}
			
			return result;
			
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}
	}
	
	/**
	 * Return true is at least one &lt;program&gt; tag contains
	 * a &lt;container-interception&gt; tag.
	 * TODO: possible evolution; make distinction between home+component
	 * requiring interception, and those which don't.
	 * @return true or false
	 * @throws Exception
	 */
	public boolean hasContainerInterception()
		throws Exception
	{
		try {
			// get all <container-interception> nodes
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"//container-interception",
					doc,
					XPathConstants.NODESET);
			
			return (nodes.getLength() > 0);
			
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return false;
		}
		
	}
	
	
	public List<FactoryImplData> getFactoriesData() 
		throws Exception
	{
		try {
			List<FactoryImplData> result = new Vector<FactoryImplData>();

			// get all <lifecycle-framework> nodes
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/lifecycle-frameworks/lifecycle-framework",
					doc,
					XPathConstants.NODESET);
			
			for (int i = 0; i < nodes.getLength(); i++) {
				Node lfcFwk = nodes.item(i);
				// get id
				String id = ((Attr) lfcFwk.getAttributes().getNamedItem("id")).getValue();
				// get factory-idref
				String factIdref = ((Attr) lfcFwk.getAttributes().getNamedItem("factory-idref")).getValue();
				
				// check if corresponding factory is a not a Home
				Node factory = (Node) xpathResolver.evaluate(
					"/cdmw/factories/factory[@idl-interface = '"+factIdref+"']",
					doc,
					XPathConstants.NODE);
				if (factory == null) {
					throw new Exception("Cannot find factory '" + factIdref + "'");
				}
				
				String kind = ((Attr) factory.getAttributes().getNamedItem("kind")).getValue();
				if (!"HOME".equals(kind)) {
					
					// check it's a <factory-impl>
					// (nothing to generate for <user-factory-impl>)
					Node factoryImpl = (Node) xpathResolver.evaluate(
						"factory-impl",
						lfcFwk,
						XPathConstants.NODE);
					if (factoryImpl != null) {
					
						// get factory-impl classname
						Attr a = (Attr) xpathResolver.evaluate(
								"@classname",
								factoryImpl,
								XPathConstants.NODE);
						String classname = a.getValue();
						
						// get factory-impl category
						a = (Attr) xpathResolver.evaluate(
								"@category",
								factoryImpl,
								XPathConstants.NODE);
						String category = a.getValue();
						
						// check if corresponding <factory> has a <servant-provider> child
						Node n = (Node) xpathResolver.evaluate(
								"servant-provider",
								factory,
								XPathConstants.NODE);
						boolean hasServantProvider = (n != null);
						
						// add to result
						result.add(new FactoryImplData(classname, category, hasServantProvider));
					}
				}
			}
			
			return result;
			
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}

	}
	
	
	
	public List<String> getProgramsFilenames() {
		try {
			List<String> result = new Vector<String>();
			
			// get all filenames attributes of <program> nodes
			NodeList nodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/programs/program/@filename",
					doc,
					XPathConstants.NODESET);
			
			for (int i = 0; i < nodes.getLength(); i++) {
				Attr a = (Attr) nodes.item(i);
				result.add(a.getValue());
			}
			
			return result;
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
			return null;
		}
	}

	/**
	 * Prepare Acceleo ProgramService with all info
	 * it requires for code generation of the specified program
	 * @param program the program filename attribute
	 * @param model the IDL model to find Homes
	 */
	public void prepareDatastoreTypesService(Repository model) 
		throws Exception
	{
		// reset DatastoreTypesServices
		DatastoreTypesServices.reset();
		
		try {
		
			// get all <datastore-type> nodes
			NodeList dsTypeNodes = (NodeList) xpathResolver.evaluate(
					"/cdmw/datastore-types/datastore-type",
					doc,
					XPathConstants.NODESET);
			for (int i = 0; i < dsTypeNodes.getLength(); i++) {
				loadDatastoreTypeNode(dsTypeNodes.item(i), model);
			}
		
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	
	
	/**
	 * Prepare Acceleo ProgramService with all info
	 * it requires for code generation of the specified program
	 * @param program the program filename attribute
	 * @param model the IDL model to find Homes
	 */
	public void prepareProgramServices(String program, Repository model) 
		throws Exception
	{
		// reset ProgramServices
		ProgramServices.reset();
		
		try {
			// store filename attribute
			ProgramServices.filename = program;

			// get <program> node with specified filename
			Node prog = (Node) xpathResolver.evaluate(
					"/cdmw/programs/program[@filename = '"+program+"']",
					doc,
					XPathConstants.NODE);
			NamedNodeMap attributes = prog.getAttributes();

			// get init-dtd attribute
			Attr dtdAttr = (Attr) attributes.getNamedItem("init-dtd");
			ProgramServices.initDTD = dtdAttr.getValue();
			
			// get directory attribute
			Attr dirAttr = (Attr) attributes.getNamedItem("directory");
			if (dirAttr != null)
				ProgramServices.directory = dirAttr.getValue();
			
			
			// Set ProgramServices.cdmwInterfacePackage
			// from classname attribute
			Attr attr = (Attr) attributes.getNamedItem("classname");
			if (attr != null) {
				String classname = attr.getValue();
				if (classname.length() == 0) {
					throw new Exception("classname attribute of '" + program + 
										"' program should not be empty");
				}
				int idx = classname.lastIndexOf('.');
				if (idx == -1) {
					// classname without package
					ProgramServices.cdmwInterfacePackage = "";
					ProgramServices.classname = classname;
				} else {
					ProgramServices.cdmwInterfacePackage = classname.substring(0, idx);
					ProgramServices.classname = classname.substring(idx+1);
				}
			} else {
				// set default package
				ProgramServices.cdmwInterfacePackage = "cdmw.generated";
				// and use filename attribute as classname
				ProgramServices.classname = program;
			}
			

			// check <repository-interface> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/repository-interface",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasRepositoryInterface = true;
				}
			}
			
			// check <container-interception> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/container-interception",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasContainerInterception = true;
				}
			}
			
			// check <performance> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/performance",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasPerformance = true;
				}
			}
			
			// check <trace> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/trace",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasTrace = true;
				}
			}
			
			// check <trace><request-followup> service
			{
				Attr a = (Attr) xpathResolver.evaluate(
						"services/trace/@request-followup",
						prog,
						XPathConstants.NODE);
				if (a != null && "yes".equalsIgnoreCase(a.getValue())) {
					ProgramServices.hasRequestFollowup = true;
				}
			}
			
			// check <trace><enable-buffering> service
			{
				Attr a = (Attr) xpathResolver.evaluate(
						"services/trace/@enable-buffering",
						prog,
						XPathConstants.NODE);
				if (a != null && "no".equalsIgnoreCase(a.getValue())) {
					ProgramServices.hasBufferingEnabled = false;
				}
			}
			
			// check <system-management> service
			{
				Attr a = (Attr) xpathResolver.evaluate(
						"services/system-management/@fatal-handler",
						prog,
						XPathConstants.NODE);
				if (a != null) {
					ProgramServices.hasSystemManagement = ("yes".equalsIgnoreCase(a.getValue()));
				} else {
					ProgramServices.hasSystemManagement = true;
				}
			}
			
			// check <time> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/time",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasTime = true;
				}
			}
			
			// check <recording> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/recording",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasRecording = true;
				}
			}
			
			// check <watchdog> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/watchdog",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasWatchdog = true;
				}
			}
			
			// check <fault-tolerance> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/fault-tolerance",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasFaultTolerance = true;
				
					// check <fault-tolerance>/@server
					Attr a = (Attr) xpathResolver.evaluate(
							"services/fault-tolerance/@server",
							prog,
							XPathConstants.NODE);
					if (a != null) {
						ProgramServices.hasFaultToleranceServer = ("yes".equalsIgnoreCase(a.getValue()));
					}

					// check <fault-tolerance>/@reply-logging
					Attr a2 = (Attr) xpathResolver.evaluate(
							"services/fault-tolerance/@reply-logging",
							prog,
							XPathConstants.NODE);
					if (a2 != null) {
						ProgramServices.hasFaultToleranceReplyLogging = ("yes".equalsIgnoreCase(a2.getValue()));
					}

					// check <fault-tolerance>/<datastores>
					Node datastoresNode = (Node) xpathResolver.evaluate(
							"services/fault-tolerance/datastores",
							prog,
							XPathConstants.NODE);
					if (datastoresNode != null) {
						ProgramServices.hasFaultToleranceWithDatastore = true;
						
						// get <fault-tolerance>/<datastores>/@kind
						Attr kindAttr = (Attr) xpathResolver.evaluate(
								"@kind",
								datastoresNode,
								XPathConstants.NODE);
						ProgramServices.faultToleranceDatastoreKind = kindAttr.getValue();
						
						// load all <fault-tolerance>/<datastores>/<datastore> nodes
						{
							NodeList datastoreNodes = (NodeList) xpathResolver.evaluate(
									"datastore",
									datastoresNode,
									XPathConstants.NODESET);
							for (int i = 0; i < datastoreNodes.getLength(); i++) {
								loadDatastoreNode(datastoreNodes.item(i), model);
							}
						}
					}
				}
			}
			
			// check <load-balancing> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/load-balancing",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasLoadBalancing = true;
				}
				
				// check <load-balancing>/@server
				Attr a = (Attr) xpathResolver.evaluate(
						"services/load-balancing/@server",
						prog,
						XPathConstants.NODE);
				if (a != null) {
					ProgramServices.hasLoadBalancingServer = ("yes".equalsIgnoreCase(a.getValue()));
				}

			}
			
			// check <lifecycle> service
			{
				Node n = (Node) xpathResolver.evaluate(
						"services/lifecycle",
						prog,
						XPathConstants.NODE);
				if (n != null) {
					ProgramServices.hasLifeCycle = true;
				}
			}
			
			// load all <framework> nodes
			{
				NodeList frameworkNodes = (NodeList) xpathResolver.evaluate(
						"services/lifecycle/framework",
						prog,
						XPathConstants.NODESET);
				for (int i = 0; i < frameworkNodes.getLength(); i++) {
					loadFrameworkNode(frameworkNodes.item(i), model);
				}
			}
			
			
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
		}
	}

	
	private void loadDatastoreTypeNode(Node datastoreType, Repository model) 
		throws Exception
	{
		// DatastoreData to fill:
		DatastoreTypeData data = new DatastoreTypeData();

		// read datastore-type attributes
		NamedNodeMap attrs = datastoreType.getAttributes();

		// get <datastore-type> typename attribute 
		String typename = ((Attr) attrs.getNamedItem("typename")).getValue();
		data.typename = typename;

		// get <datastore-type> filename attribute 
		if (attrs.getNamedItem("filename") != null) {
			data.filename = ((Attr) attrs.getNamedItem("filename")).getValue();
		}
		
		// get <datastore-type> idl-filename attribute 
		if (attrs.getNamedItem("idl-filename") != null) {
			data.idlFilename = ((Attr) attrs.getNamedItem("idl-filename")).getValue();
		}
		
		// get <datastore-type> oid-type attribute 
		if (attrs.getNamedItem("oid-type") != null) {
			String type = ((Attr) attrs.getNamedItem("oid-type")).getValue();
			data.oidType = findIDLType(type, model);
		}
		
		// get <datastore-type> data-type attribute 
		if (attrs.getNamedItem("data-type") != null) {
			String type = ((Attr) attrs.getNamedItem("data-type")).getValue();
			data.dataType = findIDLType(type, model);
		}
		
		// store DatastoreData in ProgramServices
		DatastoreTypesServices.datastoreTypes.add(data);
	}
	
	
	private IDLType findIDLType(String type, Repository model) 
		throws Exception
	{
		if (type.equals("short")) {
			return createPrimitiveDef(PrimitiveKind.PK_SHORT);
		} else if (type.equals("long")) {
			return createPrimitiveDef(PrimitiveKind.PK_LONG);
		} else if (type.equals("unsigned short")) {
			return createPrimitiveDef(PrimitiveKind.PK_USHORT);
		} else if (type.equals("unsigned long")) {
			return createPrimitiveDef(PrimitiveKind.PK_ULONG);
		} else if (type.equals("float")) {
			return createPrimitiveDef(PrimitiveKind.PK_FLOAT);
		} else if (type.equals("double")) {
			return createPrimitiveDef(PrimitiveKind.PK_DOUBLE);
		} else if (type.equals("boolean")) {
			return createPrimitiveDef(PrimitiveKind.PK_BOOLEAN);
		} else if (type.equals("char")) {
			return createPrimitiveDef(PrimitiveKind.PK_CHAR);
		} else if (type.equals("octet")) {
			return createPrimitiveDef(PrimitiveKind.PK_OCTET);
		} else if (type.equals("any")) {
			return createPrimitiveDef(PrimitiveKind.PK_ANY);
		} else if (type.equals("string")) {
			return createPrimitiveDef(PrimitiveKind.PK_STRING);
		} else if (type.equals("object")) {
			return createPrimitiveDef(PrimitiveKind.PK_OBJREF);
		} else if (type.equals("long long")) {
			return createPrimitiveDef(PrimitiveKind.PK_LONGLONG);
		} else if (type.equals("unsigned long long")) {
			return createPrimitiveDef(PrimitiveKind.PK_ULONGLONG);
		} else if (type.equals("long double")) {
			return createPrimitiveDef(PrimitiveKind.PK_LONGDOUBLE);
		} else if (type.equals("wchar")) {
			return createPrimitiveDef(PrimitiveKind.PK_WCHAR);
		} else if (type.equals("wstring")) {
			return createPrimitiveDef(PrimitiveKind.PK_WSTRING);
		} else if (type.equals("ValueBase")) {
			return createPrimitiveDef(PrimitiveKind.PK_VALUE_BASE);
		} else {
			// type should be a SharedIDLType specified in the model.
			return findSharedIDLType(type, model);
		}
	}


	private IDLType findSharedIDLType(String type, Repository model) 
		throws Exception
	{
		// replace all '.' with '::' in type
		String typeAbsName = type.replaceAll("[.]", "::");
		if (!typeAbsName.startsWith("::"))
			typeAbsName = "::" + typeAbsName;
		
		// search SharedIDLType with corresponding absoluteName
		ENode modelNode = new ENode(model, null, true);
		ENodeList list = (new EObjectServices()).eAllContents(modelNode, "idlmm.SharedIDLType");
		for (ENodeIterator it = list.iterator(); it.hasNext(); ) {
			EObject eobj = it.next().getEObject();
			// test is EObject is a Contained (to have absoluteName)
			if (eobj instanceof Contained) {
				Contained sharedIdlType = (Contained) eobj;
				if (sharedIdlType.getAbsoluteName().equals(typeAbsName)) {
					return (IDLType) sharedIdlType;
				}
			}
		}
		
		throw new Exception("Cannot find type '" + typeAbsName + "' in IDL");
	}


	private IDLType createPrimitiveDef(PrimitiveKind kind) {
		PrimitiveDef result = IdlmmFactory.eINSTANCE.createPrimitiveDef();
		result.setKind(kind);
		return result;
	}


	private void loadDatastoreNode(Node datastore, Repository model) 
		throws Exception
	{
		// DatastoreData to fill:
		DatastoreData data = new DatastoreData();

		// read datastore attributes
		NamedNodeMap attrs = datastore.getAttributes();
		String idref = ((Attr) attrs.getNamedItem("idref")).getValue();

		// find DatastoreTypeData from DatastoreTypesServices
		// typename == idref
		data.datastoreType = DatastoreTypesServices.getDatastoreTypeData(idref);

		// read occurrences attribute
		String occurrences = ((Attr) attrs.getNamedItem("occurrences")).getValue();
		data.occurrences = Integer.parseInt(occurrences);

		// store DatastoreData in ProgramServices
		ProgramServices.usedDatastores.add(data);
	}


	private void loadFrameworkNode(Node framework, Repository model) 
		throws Exception
	{
		try {
			// FrameworkData to fill:
			FrameworkData data = new FrameworkData();
			
			// read framework attributes
			NamedNodeMap attrs = framework.getAttributes();
			String fmwkId = ((Attr) attrs.getNamedItem("idref")).getValue();
			data.idref = fmwkId;
			data.occurrences = Integer.parseInt(((Attr) attrs.getNamedItem("occurrences")).getValue());
			
			// find corresponding <lifecycle-framework> node
			NodeList lfcFrameworks = (NodeList) xpathResolver.evaluate(
					"/cdmw/lifecycle-frameworks/lifecycle-framework[@id = '"+fmwkId+"']",
					doc,
					XPathConstants.NODESET);
			if (lfcFrameworks.getLength() == 0) {
				throw new Exception("Cannot find <lifecycle-framework> element with '" +
									fmwkId + "' as id.");
			}
			Node lfcFramework = lfcFrameworks.item(0);
			
			// get <lifecycle-framework> factory-idref attribute 
			// (correspond to <factory> idl-interface attribute) 
			NamedNodeMap lfcFmwkAttrs = lfcFramework.getAttributes();
			data.factoryIdlInterface = ((Attr) lfcFmwkAttrs.getNamedItem("factory-idref")).getValue();
			
			// find corresponding <factory> node
			NodeList factories = (NodeList) xpathResolver.evaluate(
					"/cdmw/factories/factory[@idl-interface = '"+data.factoryIdlInterface+"']",
					doc,
					XPathConstants.NODESET);
			if (factories.getLength() == 0) {
				throw new Exception("Cannot find <factory> element with '" +
									data.factoryIdlInterface + "' as idl-interface.");
			}
			Node factory = factories.item(0);
			
			// load <factory> kind attribute
			NamedNodeMap factoryAttrs = factory.getAttributes();
			String kind = ((Attr) factoryAttrs.getNamedItem("kind")).getValue();
			
			// check if it's a HOME with <home-impl>
			Node homeImplNode = (Node) xpathResolver.evaluate("home-impl", lfcFramework, XPathConstants.NODE);
			if ("HOME".equals(kind)) 
			{
				if (homeImplNode != null) {
					ProgramServices.hasHomeImpl = true;
					data.isHome = true;
				} else {
					throw new Exception("<factory> '" + data.factoryIdlInterface + 
										"' is declared as a HOME, but <lifecycle-framework> '" + 
										fmwkId +"' doesn't have <home-impl>");
				}
				
				// load <home-impl> category attribute
				Attr homeCategoryAttr = (Attr) xpathResolver.evaluate(
						"@category",
						homeImplNode,
						XPathConstants.NODE);
				if (homeCategoryAttr != null) {
					data.homeCategory = homeCategoryAttr.getValue();
				}
				
				
			} else if ("NAMED".equals(kind)) {
				data.isNamedFactory = true;
			}
			
			if (data.isHome) {
				// idref should be Home name. Search HomeDef in Repository
				data.homeDef = findHome(data.factoryIdlInterface, model);
				
				// search if the manages component has EventPorts
				if (hasEventPorts(data.homeDef)) {
					ProgramServices.hasEventPorts = true;
				}
			} else {
				// check if it's a user factory with <user-factory-impl>
				data.isUserFactory = (xpathResolver.evaluate("user-factory-impl", 
															 lfcFramework, 
															 XPathConstants.NODE) 
									  != null);
				
				// find classname attribute in <lifecycle-framework> children
				Attr classnameAttr = (Attr) xpathResolver.evaluate(
						"*/@classname",
						lfcFramework,
						XPathConstants.NODE);
				if (classnameAttr != null) {
					data.factoryClassname = classnameAttr.getValue();
				}
				
				// find filename attribute in <lifecycle-framework> children
				Attr filenameAttr = (Attr) xpathResolver.evaluate(
						"*/@filename",
						lfcFramework,
						XPathConstants.NODE);
				if (filenameAttr != null) {
					data.factoryFilename = filenameAttr.getValue();
				}
				
				// find allocator classname attribute in <lifecycle-framework> grandchildren
				Attr allocClassnameAttr = (Attr) xpathResolver.evaluate(
						"*/*/@classname",
						lfcFramework,
						XPathConstants.NODE);
				if (allocClassnameAttr != null) {
					data.allocatorClassname = allocClassnameAttr.getValue();
				}
				
				// find allocator filename attribute in <lifecycle-framework> grandchildren
				Attr allocFilenameAttr = (Attr) xpathResolver.evaluate(
						"*/*/@filename",
						lfcFramework,
						XPathConstants.NODE);
				if (allocFilenameAttr != null) {
					data.allocatorFilename = allocFilenameAttr.getValue();
				}
				
			}
			
			// load category attribute
			Attr categoryAttr = (Attr) xpathResolver.evaluate(
					".//@category",
					lfcFramework,
					XPathConstants.NODE);
			if (categoryAttr != null) {
				data.category = categoryAttr.getValue();
			} else {
				data.category = "USER_DEFINED";
			}
			
			// load threading attribute
			Attr threadingAttr = (Attr) xpathResolver.evaluate(
					".//@threading",
					lfcFramework,
					XPathConstants.NODE);
			if (threadingAttr != null) {
				data.threading = threadingAttr.getValue();
			}
			
			// store FrameworkData in ProgramServices
			ProgramServices.frameworks.add(data);
			
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
		}
	}


	private HomeDef findHome(String idlInterface, Repository model) 
		throws Exception
	{
		// replace all '.' with '::' in homeName
		String homeAbsName = idlInterface.replaceAll("[.]", "::");
		if (!homeAbsName.startsWith("::"))
			homeAbsName = "::" + homeAbsName;
		
		// search HomeDef with corresponding absoluteName
		ENode modelNode = new ENode(model, null, true);
		ENodeList list = (new EObjectServices()).eAllContents(modelNode, "idlmm.HomeDef");
		for (ENodeIterator it = list.iterator(); it.hasNext(); ) {
			HomeDef home = (HomeDef) it.next().getEObject();
			if (home.getAbsoluteName().equals(homeAbsName)) {
				return home;
			}
		}
		
		throw new Exception("Cannot find Home '" + homeAbsName + "' in IDL");
	}
	
	
	private boolean hasEventPorts(HomeDef homeDef) {
		return (homeDef.getManagedComponent().getAllEmitsPorts().size() > 0) ||
			   (homeDef.getManagedComponent().getAllPublishesPorts().size() > 0);
	}


	
	public void prepareLifeCycleServices(String factoryImplClassname, Repository model) 
		throws Exception
	{
		try {
			// reset ProgramServices
			LifeCycleServices.reset();
			
			// get <factory-impl> with specified classname attribute
			Node factoryImpl = (Node) xpathResolver.evaluate(
					"/cdmw/lifecycle-frameworks/lifecycle-framework/factory-impl[@classname = '"+factoryImplClassname+"']",
					doc,
					XPathConstants.NODE);
			if (factoryImpl == null) {
				throw new Exception("Cannot find <factory-impl> with classname: " + factoryImplClassname);
			}
			if (factoryImplClassname.lastIndexOf('.') == -1) {
				LifeCycleServices.factoryImplClassname = factoryImplClassname;
			} else {
				LifeCycleServices.factoryImplClassname = 
					factoryImplClassname.substring(factoryImplClassname.lastIndexOf('.')+1);
				LifeCycleServices.factoryImplClassnameScope =
					factoryImplClassname.substring(0, factoryImplClassname.lastIndexOf('.'));
			}

			// get directory attribute
			Attr a = (Attr) xpathResolver.evaluate(
					"@directory",
					factoryImpl,
					XPathConstants.NODE);
			if (a != null) {
				LifeCycleServices.factoryImplDirectory = a.getValue();
			}
			// get filename attribute
			a = (Attr) xpathResolver.evaluate(
					"@filename",
					factoryImpl,
					XPathConstants.NODE);
			if (a != null) {
				LifeCycleServices.factoryImplFilename = a.getValue();
			}
			
			// get threading attribute
			a = (Attr) xpathResolver.evaluate(
					"@threading",
					factoryImpl,
					XPathConstants.NODE);
			if (a != null) {
				LifeCycleServices.isMultithread = a.getValue().equals("MULTITHREAD");
			}
			
			// get <servant-provider-allocator> classname attribute
			a = (Attr) xpathResolver.evaluate(
					"servant-provider-allocator/@classname",
					factoryImpl,
					XPathConstants.NODE);
			if (a != null) {
				LifeCycleServices.servantProviderAllocatorClassname = a.getValue();
			}
			
			// get <servant-provider-allocator> filename attribute
			a = (Attr) xpathResolver.evaluate(
					"servant-provider-allocator/@filename",
					factoryImpl,
					XPathConstants.NODE);
			if (a != null) {
				LifeCycleServices.servantProviderAllocatorFilename = a.getValue();
			}
			
			// get factory-idref from parent <lifecycle-framework>
			a = (Attr) xpathResolver.evaluate(
					"../@factory-idref",
					factoryImpl,
					XPathConstants.NODE);
			String factoryId = a.getValue();
			
			// find corresponding <factory> element
			Node factory = (Node) xpathResolver.evaluate(
					"/cdmw/factories/factory[@idl-interface = '"+factoryId+"']",
					doc,
					XPathConstants.NODE);

			// get idl-interface attribute
			a = (Attr) xpathResolver.evaluate(
					"@idl-interface",
					factory,
					XPathConstants.NODE);
			LifeCycleServices.idlInterface = findInterface(a.getValue(), model);
			
			// get <servant-provider> element (if exists)
			Node servantProvider = (Node) xpathResolver.evaluate(
					"servant-provider",
					factory,
					XPathConstants.NODE);
			if (servantProvider != null) {
				// get classname attribute
				a = (Attr) xpathResolver.evaluate(
						"@classname",
						servantProvider,
						XPathConstants.NODE);
				if (a != null) {
					String classname = a.getValue();
					if (classname.lastIndexOf('.') == -1) {
						LifeCycleServices.servantProviderClassname = classname;
					} else {
						LifeCycleServices.servantProviderClassname = 
							classname.substring(classname.lastIndexOf('.')+1);
						LifeCycleServices.servantProviderClassnameScope =
							classname.substring(0, classname.lastIndexOf('.'));
					}
					
				}
				// get directory attribute
				a = (Attr) xpathResolver.evaluate(
						"@directory",
						servantProvider,
						XPathConstants.NODE);
				if (a != null) {
					LifeCycleServices.servantProviderDirectory = a.getValue();
				}
				// get filename attribute
				a = (Attr) xpathResolver.evaluate(
						"@filename",
						servantProvider,
						XPathConstants.NODE);
				if (a != null) {
					LifeCycleServices.servantProviderFilename = a.getValue();
				}
			}
			
			
		} catch (XPathExpressionException e) {
			// shouldn't happen
			e.printStackTrace();
			System.exit(1);
		}
	}


	private InterfaceDef findInterface(String idlInterface, Repository model) 
		throws Exception
	{
		// replace all '.' with '::' in homeName
		String intfAbsName = idlInterface.replaceAll("[.]", "::");
		if (!intfAbsName.startsWith("::"))
			intfAbsName = "::" + intfAbsName;
		
		// search InterfaceDef with corresponding absoluteName
		ENode modelNode = new ENode(model, null, true);
		ENodeList list = (new EObjectServices()).eAllContents(modelNode, "idlmm.InterfaceDef");
		for (ENodeIterator it = list.iterator(); it.hasNext(); ) {
			InterfaceDef interfaceDef = (InterfaceDef) it.next().getEObject();
			if (interfaceDef.getAbsoluteName().equals(intfAbsName)) {
				return interfaceDef;
			}
		}
		
		throw new Exception("Cannot find Interface '" + intfAbsName + "' in IDL");
	}


	
}
