<%
metamodel http://www.prismtech.com/idlmm

import com.thalesgroup.ccm.codegenerator.corba.cpp.CppMappingServices
import com.thalesgroup.ccm.codegenerator.corba.cpp.Utils
import com.thalesgroup.ccm.codegenerator.container.Utils
import com.thalesgroup.ccm.codegenerator.container.cpp.Utils
import com.thalesgroup.ccm.codegenerator.container.cpp.Names
import com.thalesgroup.ccm.codegenerator.container.cpp.SessionComponent_cpp
%>

<%script type="idlmm.ComponentDef" name="filePath" post="trim()"%>
<%generatedExtensionServantClassname%>.cpp    

<%--
  -- Generation template  
  --%>
<%script type="idlmm.ComponentDef" name="ExtensionComponent_cpp" file="<%filePath%>"%>
<%copyrightHeader%>


#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcif/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>

#include "<%generatedExtensionServantClassname%>.hpp"

#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Foundation/idllib/CdmwDeployment.stub.hpp>

#include <CCMContainer/idllib/CdmwCcmCif.skel.hpp>

#include "<%generatedExtensionContextClassname%>.hpp"


namespace {
    const char* COMP_REP_ID = "<%ID%>";
    const char* COMP_NAME   = "<%name%>";

    /*
     * If a home, when creating a component, returns an EnterpriseComponent, we map it 
     * in an ExecutorLocator for code optimisation purpose. Thus the code is the same
     * for monolithic executor and segmented executor.
     */
    class <%name%>ExecutorLocator : 
        public CdmwCcmCif::ExtensionExecutorLocator,
        public Cdmw::OrbSupport::RefCountLocalObject
    {
        public:
            <%name%>ExecutorLocator(
                    <%equivalentLocalInterface%>_ptr executor,
                    CdmwLogging::Logger_ptr logger)
                : is_a_extension_component(false),
                  m_comp_executor(<%equivalentLocalInterface%>::_duplicate(executor)),
                  m_logger(CdmwLogging::Logger::_duplicate(logger))
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                try 
                {
                    Components::ExtensionComponent_var extension_comp
                        = Components::ExtensionComponent::_narrow(m_comp_executor.in());
            
                    if (!CORBA::is_nil(extension_comp.in()))
                    {
                        is_a_extension_component = true;
                    }
                }
                catch (...)
                {
                }
            };

            ~<%name%>ExecutorLocator() 
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
            };

            /*
             * the cif will invoke its obtain_executor operation prior to each invocation to
             * retrieve the implementation for a port. The port name, given in the name
             * parameter, is the same as used in the components interface description 
             * in IDL, or the components name for the main executor. The obtain_executor
             * operation returns a local object reference of the expected type. The 
             * CCMException exception may be raised in case of a system error that 
             * prohibits locating the requested executor.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/obtain_executor:1.0
            //
            virtual CORBA::Object_ptr obtain_executor(const char* name)
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "name="<<name);
                std::string exec_name(name);
                CORBA::Object_var result;

                if (exec_name == "<%name%>")
                {
                    result = <%equivalentLocalInterface%>::_duplicate(m_comp_executor.in());
                }
                <%for (allProvidesPorts){%>
                else if (exec_name == "<%name%>") {        
                    result = m_comp_executor->get_<%name%>();
                }
                <%}%>
                <%push()%>
                <%for (allConsumesPorts){%>
                else if (exec_name == <%portNameConstName%>) {        
                    result = <%peek.equivalentLocalInterface%>::_duplicate(m_comp_executor.in());
                }
                <%}%>
                <%pop()%>
                else 
                {
                    CDMW_ERROR(m_logger.in(), 
                      "'"<<name<<"' is unkown: cannot obtain executor for it");
                    CORBA::BAD_PARAM ex(Cdmw::OrbSupport::BAD_PARAMInvalidExecutorName, CORBA::COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(ex);
                    throw ex;
                }

                return result._retn();
            };

            /* 
             * The release_executor operation is called by the cif once the current
             * invocation on an executor that was obtained through the obtain_executor 
             * operation has finished. The locator can thus release any resources that
             * were acquired as part of the obtain_executor operation.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/release_executor:1.0
            //
            virtual void release_executor(CORBA::Object_ptr exc)
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
            };

            /* 
             * The configuration_complete operation is called to propagate the
             * configuration_complete operation on the CCMObject interface to the
             * component implementation.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/configuration_complete:1.0
            //
            virtual void configuration_complete()
                throw(Components::InvalidConfiguration,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                // No equivalent operation for monolithic executor
            };

            //
            // IDL:omg.org/Components/ExtensionComponent/set_extension_context:1.0
            //
            virtual void set_extension_context(Components::ExtensionContext_ptr ctx)
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                // check if component implements the ExtensionComponent local interface
                if (is_a_extension_component)
                {
                    CDMW_INFO(m_logger.in(), "Call set_extension_context()");
                    Components::ExtensionComponent_var extension_comp 
                        = Components::ExtensionComponent::_narrow(m_comp_executor.in());
                    extension_comp->set_extension_context(ctx);
                }
            };

            /* 
             * The ccm_remove operation is called by the container when the servant is
             * about to be destroyed. It informs the component that it is about to be
             * destroyed.The component may raise the CCMException with the SYSTEM_ERROR
             * minor code to indicate a failure caused by a system level error.
             */
            //
            // IDL:omg.org/Components/ExtensionComponent/ccm_remove:1.0
            //
            virtual void ccm_remove()
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                // check if component implements the ExtensionComponent local interface
                if (is_a_extension_component)
                {
                    CDMW_INFO(m_logger.in(), "Call ccm_remove()");
                    Components::ExtensionComponent_var extension_comp 
                        = Components::ExtensionComponent::_narrow(m_comp_executor.in());
                    extension_comp->ccm_remove();
                }
            };

        private:
            <%name%>ExecutorLocator(const <%name%>ExecutorLocator&);
            void operator=(const <%name%>ExecutorLocator&);

            bool is_a_extension_component;  // indicates that the component executor inherits
                                            // from the ExtensionComponent local interface.

            <%equivalentLocalInterface%>_var m_comp_executor; // points on Component monolithic executor.
            
            CdmwLogging::Logger_var m_logger;
        };
    };  // end of anonymous namespace


<%openNamespace(generatedServantPackage)%>

    // Constructor
    <%generatedExtensionServantClassname%>::<%generatedExtensionServantClassname%>(const std::string& comp_oid,
            Context*                          ctx,
            CORBA::Object_ptr                 comp_ref,
            Components::EnterpriseComponent*  executor,
            CdmwLifeCycle::ObjectRegistration_ptr object_registration,
            CdmwLogging::LoggerFactory_ptr    logger_factory,
            bool is_created_by_factory_operation)
        throw (Components::CCMException,
               CORBA::SystemException)
        : CCMObject_impl(comp_oid, ctx, COMP_REP_ID, comp_ref, object_registration, logger_factory),
          <%generatedSessionServantClassname%>(comp_oid, ctx, comp_ref, executor, object_registration, logger_factory, is_created_by_factory_operation),
          m_is_created_by_factory_operation(is_created_by_factory_operation)
    {
        // create logger with domain for ccmcif
        m_logger = logger_factory->get_logger(CCMLoggers::CCMGENERATED_LOG_DOMAIN);
       
        CDMW_LOG_FUNCTION(m_logger.in(), "comp_oid="<<comp_oid);
        
        // Check if the home returns an executor locator
        m_executor_locator = Components::ExecutorLocator::_narrow(executor);
    
        if (CORBA::is_nil(m_executor_locator.in()))
        {
            // executor is not an executor locator
            // map monolithic executor to executor locator
            <%equivalentLocalInterface%>_var comp_exec = <%equivalentLocalInterface%>::_narrow(executor);
        
            if (CORBA::is_nil(comp_exec.in()))
            {
                // This is to avoid Assertion failure in debug mode as internal ref count
                // equals 1 (CORBA C++ mapping oblige!)
                _set_ref_count(0);

                CDMW_ERROR(m_logger.in(), "Invalid Component Executor");
                CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidComponentExecutor,
                                    CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            m_executor_locator = new <%name%>ExecutorLocator(comp_exec.in(), m_logger.in());

            m_extension_locator = Components::ExtensionComponent::_narrow(m_executor_locator.in());
        
            if (CORBA::is_nil(m_extension_locator.in()))
            {
                CDMW_ERROR(m_logger.in(), 
                           "Executor Locator should be a extension component!");
                CORBA::INTERNAL ex(OrbSupport::INTERNALInvalidExecutor,
                                   CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        else
        {
            Components::ExtensionComponent_var m_extension_locator 
                = Components::ExtensionComponent::_narrow(m_executor_locator.in());
        
            if (CORBA::is_nil(m_extension_locator.in()))
            {
                CDMW_ERROR(m_logger.in(), 
                           "Executor Locator should be a extension component!");
                CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidComponentExecutor,
                                    CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }

        Components::CCMContext_var ccm_ctx = ctx->get_ccm_context();
        m_context_executor = new <%generatedExtensionContextClassname%>(ccm_ctx.in(), this, logger_factory);
        m_extension_locator->set_extension_context(m_context_executor.in());

    }
    
    <%generatedExtensionServantClassname%>::~<%generatedExtensionServantClassname%>()
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "");
    }

    void
    <%generatedExtensionServantClassname%>::configuration_complete()
        throw(Components::InvalidConfiguration,
              CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "");
        check_is_removed();

        if (!is_configured())
        {
            CDMW_DEBUG(m_logger.in(), "Check if configuration is complete");
         
            SessionLoggerService_impl::configuration_complete();
            m_executor_locator->configuration_complete();
        }
    }
    

    void <%generatedExtensionServantClassname%>::ccm_remove() 
        throw (Components::CCMException, CORBA::SystemException)
    {
        // this is a extension component, call ccm_remove
        m_extension_locator->ccm_remove();
    }

<%closeNamespace(generatedServantPackage)%>
