<%
metamodel http://www.prismtech.com/idlmm

import com.thalesgroup.ccm.codegenerator.corba.Utils
import com.thalesgroup.ccm.codegenerator.corba.cpp.CppMappingServices
import com.thalesgroup.ccm.codegenerator.corba.cpp.Utils
import com.thalesgroup.ccm.codegenerator.container.Utils
import com.thalesgroup.ccm.codegenerator.container.cpp.Utils
import com.thalesgroup.ccm.codegenerator.container.cpp.Names
%>

<%script type="idlmm.ComponentDef" name="filePath" post="trim()"%>
<%generatedSessionServantClassname%>.cpp    

<%--
  -- Generation template  
  --%>
<%script type="idlmm.ComponentDef" name="SessionComponent_cpp" file="<%filePath%>"%>
<%copyrightHeader%>

#include <Foundation/orbsupport/CORBA.hpp>
#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/logging/FunctionLogger.hpp>
#include <CCMContainer/ccmcommon/CCMUtils.hpp>
#include <CCMContainer/ccmcif/CCMUtils.hpp>
#include <CCMContainer/ccmcommon/CCMLoggers.hpp>
#include <CCMContainer/ccmcopi/COPIServiceInit.hpp>

#include "<%generatedSessionServantClassname%>.hpp"

#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <SystemMngt/platforminterface/PlatformInterface.hpp>
#include <Foundation/idllib/CdmwDeployment.stub.hpp>

<%--
    Include all the contained facets.
--%>
<%allProvidesPorts.genIncludeFacetServant.sep("\n")%>

<%--
    Include all the contained consumers.
--%>
<%allConsumesPorts.genIncludeConsumer.sep("\n")%>

<%--
    Include all the contained receptacles.
--%>
<%allUsesPorts.genIncludeReceptacle.sep("\n")%>

#include <CCMContainer/idllib/CdmwCcmCif.skel.hpp>

#include "<%generatedSessionContextClassname%>.hpp"


namespace {
    const char* COMP_REP_ID = "<%ID%>";
    const char* COMP_NAME   = "<%name%>";

    <%--
       Ports constants
    --%>
    <%allProvidesPorts.genPortConstantsDef%>
    <%allUsesPorts.genPortConstantsDef%>
    <%allPublishesPorts.genPortConstantsDef%>
    <%allEmitsPorts.genPortConstantsDef%>
    <%allConsumesPorts.genPortConstantsDef%>
    
    /*
     * If a home, when creating a component, returns an EnterpriseComponent, we map it 
     * in an ExecutorLocator for code optimisation purpose. Thus the code is the same
     * for monolithic executor and segmented executor.
     */
    class <%name%>ExecutorLocator : public CdmwCcmCif::SessionExecutorLocator,
                                    public Cdmw::OrbSupport::RefCountLocalObject
    {
        public:
            <%name%>ExecutorLocator(
                    const ::std::string & component_id,
                    ::Cdmw::CCM::CIF::Context * context,
                    <%equivalentLocalInterface%>_ptr executor,
                    CdmwLogging::Logger_ptr logger)
                : is_a_session_component(false),
                  m_comp_executor(<%equivalentLocalInterface%>::_duplicate(executor)),
                  m_logger(CdmwLogging::Logger::_duplicate(logger))
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                try 
                {
                    Components::SessionComponent_var session_comp
                        = Components::SessionComponent::_narrow(m_comp_executor.in());
            
                    if (!CORBA::is_nil(session_comp.in()))
                    {
                        is_a_session_component = true;
                    }
                }
                catch (...)
                {
                }
                
                <%for (allConsumesPorts){%>
                <%consumerExecutorName%> =
                    new <%consumerExecutorClassname%>(                   
                        component_id,
                        context,
                        executor,
                        logger);                         
                <%}%>
            };

            ~<%name%>ExecutorLocator() 
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                <%for (allConsumesPorts){%>
                delete <%consumerExecutorName%>;
                <%consumerExecutorName%> = 0;
                <%}%>
                
            };

            /*
             * the cif will invoke its obtain_executor operation prior to each invocation to
             * retrieve the implementation for a port. The port name, given in the name
             * parameter, is the same as used in the components interface description 
             * in IDL, or the components name for the main executor. The obtain_executor
             * operation returns a local object reference of the expected type. The 
             * CCMException exception may be raised in case of a system error that 
             * prohibits locating the requested executor.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/obtain_executor:1.0
            //
            virtual CORBA::Object_ptr obtain_executor(const char* name)
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "name="<<name);
                std::string exec_name(name);
                CORBA::Object_var result;

                if (exec_name == "<%name%>")
                {
                    result = <%equivalentLocalInterface%>::_duplicate(m_comp_executor.in());
                }
                <%for (allProvidesPorts){%>
                else if (exec_name == "<%name%>") {        
                    result = m_comp_executor->get_<%name%>();
                }
                <%}%>
                <%for (allConsumesPorts){%>
                else if (exec_name == <%portNameConstName%>) {        
                    result = <%event.equivalentLocalConsumer%>::_duplicate(<%consumerExecutorName%>);
                }
                <%}%>
                else 
                {
                    CDMW_ERROR(m_logger.in(), 
                      "'"<<name<<"' is unkown: cannot obtain executor for it");
                    CORBA::BAD_PARAM ex(Cdmw::OrbSupport::BAD_PARAMInvalidExecutorName, CORBA::COMPLETED_NO);
                    CDMW_LOG_FUNCTION_EXCEPTION(ex);
                    throw ex;
                }

                return result._retn();
            };

            /* 
             * The release_executor operation is called by the cif once the current
             * invocation on an executor that was obtained through the obtain_executor 
             * operation has finished. The locator can thus release any resources that
             * were acquired as part of the obtain_executor operation.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/release_executor:1.0
            //
            virtual void release_executor(CORBA::Object_ptr exc)
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
            };

            /* 
             * The configuration_complete operation is called to propagate the
             * configuration_complete operation on the CCMObject interface to the
             * component implementation.
             */
            //
            // IDL:omg.org/Components/ExecutorLocator/configuration_complete:1.0
            //
            virtual void configuration_complete()
                throw(Components::InvalidConfiguration,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                // No equivalent operation for monolithic executor
            };

            //
            // IDL:omg.org/Components/SessionComponent/set_session_context:1.0
            //
            virtual void set_session_context(Components::SessionContext_ptr ctx)
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                // check if component implements the SessionComponent local interface
                if (is_a_session_component)
                {
                    CDMW_INFO(m_logger.in(), "Call set_session_context()");
                    Components::SessionComponent_var session_comp 
                        = Components::SessionComponent::_narrow(m_comp_executor.in());
                    session_comp->set_session_context(ctx);
                }
            };

            /* 
             * The ccm_activate operation is called by the container to notify a 
             * session component that it has been made active. The component instance
             * should perform any initialization required prior to operation invocation.
             * The component may raise the CCMException with the SYSTEM_ERROR minor code
             * to indicate a failure caused by a system level error.
             */
            //
            // IDL:omg.org/Components/SessionComponent/ccm_activate:1.0
            //
            virtual void ccm_activate()
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                // check if component implements the SessionComponent local interface
                if (is_a_session_component)
                {
                    CDMW_INFO(m_logger.in(), "Call ccm_activate()");
                    Components::SessionComponent_var session_comp 
                        = Components::SessionComponent::_narrow(m_comp_executor.in());
                    session_comp->ccm_activate();
                }
            };

            /* 
             * The ccm_passivate operation is called by the container to notify a session
             * component that it has been made inactive. The component instance should
             * release any resources it acquired at activation time.The component may 
             * raise the CCMException with the SYSTEM_ERROR minor code to indicate a 
             * failure caused by a system level error.
             */
            //
            // IDL:omg.org/Components/SessionComponent/ccm_passivate:1.0
            //
            virtual void ccm_passivate()
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                // check if component implements the SessionComponent local interface
                if (is_a_session_component)
                {
                    CDMW_INFO(m_logger.in(), "Call ccm_passivate()");
                    Components::SessionComponent_var session_comp 
                        = Components::SessionComponent::_narrow(m_comp_executor.in());
                    session_comp->ccm_passivate();
                }
            };

            /* 
             * The ccm_remove operation is called by the container when the servant is
             * about to be destroyed. It informs the component that it is about to be
             * destroyed.The component may raise the CCMException with the SYSTEM_ERROR
             * minor code to indicate a failure caused by a system level error.
             */
            //
            // IDL:omg.org/Components/SessionComponent/ccm_remove:1.0
            //
            virtual void ccm_remove()
                throw(Components::CCMException,
                      CORBA::SystemException)
            {
                CDMW_LOG_FUNCTION(m_logger.in(), "");
                // check if component implements the SessionComponent local interface
                if (is_a_session_component)
                {
                    CDMW_INFO(m_logger.in(), "Call ccm_remove()");
                    Components::SessionComponent_var session_comp 
                        = Components::SessionComponent::_narrow(m_comp_executor.in());
                    session_comp->ccm_remove();
                }
            };

        private:
            <%name%>ExecutorLocator(const <%name%>ExecutorLocator&);
            void operator=(const <%name%>ExecutorLocator&);

            <%self.push()%>
            <%for (allConsumesPorts){%>
            <%genConsumerExecutor(peek)%>
            <%}%>
            <%pop()%>

            bool is_a_session_component;  // indicates that the component executor inherits
                                          // from the SessionComponent local interface.

            <%equivalentLocalInterface%>_var m_comp_executor; // points on Component monolithic executor.

            <%for (allConsumesPorts){%>
            <%consumerExecutorClassname%>* <%consumerExecutorName%>;
            <%}%>
            
            CdmwLogging::Logger_var m_logger;
        };
    };  // end of anonymous namespace


<%openNamespace(generatedServantPackage)%>

    // Constructor
    <%generatedSessionServantClassname%>::<%generatedSessionServantClassname%>(const std::string& comp_oid,
            Context*                          ctx,
            CORBA::Object_ptr                 comp_ref,
            Components::EnterpriseComponent*  executor,
            CdmwLifeCycle::ObjectRegistration_ptr object_registration,
            CdmwLogging::LoggerFactory_ptr    logger_factory,
            bool is_created_by_factory_operation)
        throw (Components::CCMException,
               CORBA::SystemException)
        : CCMObject_impl(comp_oid, ctx, COMP_REP_ID, comp_ref, object_registration, logger_factory),
          m_is_created_by_factory_operation(is_created_by_factory_operation)
          <%for (allAttributes[mode != "ATTR_READONLY"]){%>
          , m_is_<%name%>_configured(false)
          <%}%>
    {
        // create logger with domain for ccmcif
        m_logger = logger_factory->get_logger(CCMLoggers::CCMGENERATED_LOG_DOMAIN);
       
        CDMW_LOG_FUNCTION(m_logger.in(), "comp_oid="<<comp_oid);
        
        // Check if the home returns an executor locator
        m_executor_locator = Components::ExecutorLocator::_narrow(executor);
    
        if (CORBA::is_nil(m_executor_locator.in()))
        {
            // executor is not an executor locator
            // map monolithic executor to executor locator
            <%equivalentLocalInterface%>_var comp_exec = <%equivalentLocalInterface%>::_narrow(executor);
        
            if (CORBA::is_nil(comp_exec.in()))
            {
                // This is to avoid Assertion failure in debug mode as internal ref count
                // equals 1 (CORBA C++ mapping oblige!)
                _set_ref_count(0);

                CDMW_ERROR(m_logger.in(), "Invalid Component Executor");
                CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidComponentExecutor,
                                    CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
            m_executor_locator = 
                new <%name%>ExecutorLocator(
                    get_component_identity(),
                    get_context(),
                    comp_exec.in(),
                    m_logger.in());
                    
            m_session_locator = Components::SessionComponent::_narrow(m_executor_locator.in());
        
            if (CORBA::is_nil(m_session_locator.in()))
            {
                CDMW_ERROR(m_logger.in(), 
                           "Executor Locator should be a session component!");
                CORBA::INTERNAL ex(OrbSupport::INTERNALInvalidExecutor,
                                   CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
        else
        {
            Components::SessionComponent_var m_session_locator 
                = Components::SessionComponent::_narrow(m_executor_locator.in());
        
            if (CORBA::is_nil(m_session_locator.in()))
            {
                CDMW_ERROR(m_logger.in(), 
                           "Executor Locator should be a session component!");
                CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMInvalidComponentExecutor,
                                    CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }

        Components::CCMContext_var ccm_ctx = ctx->get_ccm_context();
        m_context_executor = new <%generatedSessionContextClassname%>(ccm_ctx.in(), this, logger_factory);
        m_session_locator->set_session_context(m_context_executor.in());

        <%-- facets declarations --%>
        
        <%allProvidesPorts.genProvidesDeclaration%>
        
        <%-- receptacles declarations --%>
        <%if (allUsesPorts.nSize > 0){%>
        try {
            <%for (allUsesPorts){%>
            declare_receptacle(<%portNameConstName%>, <%portRepIdConstName%>);
            <%}%>
        }
        catch (const AlreadyDoneException&)
        {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            
            CDMW_ERROR(m_logger.in(), 
                "Receptacle already declared!");
            // Don't propagate while its probably a code generation error!
        }
        catch (const InternalErrorException&)
        {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            
            CDMW_ERROR(m_logger.in(), 
                "Internal exception raised when declaring receptacle!");
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        catch (const CORBA::SystemException& e)
        {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            
            CDMW_ERROR(m_logger.in(), 
                "System exception raised when declaring receptacle: " << e);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        <%}%>      
        
        <%-- publishers & emitters declarations --%>
        <%if (allPublishesPorts.nSize > 0 || allEmitsPorts.nSize > 0){%>
        try {
            <%for (allPublishesPorts){%>
            declare_publisher(<%portNameConstName%>, <%portRepIdConstName%>);
            <%}%>
            <%for (allEmitsPorts){%>
            declare_emitter(<%portNameConstName%>, <%portRepIdConstName%>);
            <%}%>
        }
        catch (const AlreadyDoneException&)
        {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            
            CDMW_ERROR(m_logger.in(), "Event source already declared!");
            // Don't propagate while its probably a code generation error!
        }
        catch (const CORBA::SystemException& e)
        {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            
            CDMW_ERROR(m_logger.in(), 
                "System exception raised when declaring event source: " << e);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        <%}%>

        <%-- consumers declarations --%>
        <%if (allConsumesPorts.nSize > 0){%>
        try {
            <%for (allConsumesPorts){%>
            {
                CORBA::Object_var consumer =
                    declare_consumer(comp_oid.c_str(), <%portNameConstName%>, <%portRepIdConstName%>);
            }
            <%}%>
        }
        catch (const AlreadyDoneException&)
        {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            
            CDMW_ERROR(m_logger.in(), "Consumer already declared!");
            // Don't propagate while its probably a code generation error!
        }
        catch (const InternalErrorException&)
        {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            
            CDMW_ERROR(m_logger.in(), "Internal exception raised when declaring consumer!");
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        catch (const CORBA::SystemException& e)
        {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            
            CDMW_ERROR(m_logger.in(), 
                "System exception raised when declaring consumer: " << e);
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        <%}%>
    }
    
    <%generatedSessionServantClassname%>::~<%generatedSessionServantClassname%>()
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "");
    }

    <%-- push ComponentDef in context to be used by sub-scripts--%>    
    <%self.push()%>

    /**
     * Component attributes
     */
    <%allAttributes.genAttributeOperations.sep("\n\n")%>

    /**
     * Supported interfaces
     */
    <%allSupportedInterfaces.operations.genOperationImpl.sep("\n\n")%>

    /**
     * Equivalent IDL methods
     */

    // Facets
    <%allProvidesPorts.genEquivalentOperations.sep("\n\n")%>

    // Receptacles
    <%allUsesPorts.genEquivalentOperations.sep("\n\n")%>
    
    // Publishers
    <%allPublishesPorts.genEquivalentOperations.sep("\n\n")%>

    // Emitters
    <%allEmitsPorts.genEquivalentOperations.sep("\n\n")%>
    
    // Consumers
    <%allConsumesPorts.genEquivalentOperations.sep("\n\n")%>
    
    <%-- remove ComponentDef from context --%>
    <%pop()%>
    
    void
    <%generatedSessionServantClassname%>::configuration_complete()
        throw(Components::InvalidConfiguration,
              CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "");
        check_is_removed();

        if (!is_configured())
        {
            if (!m_is_created_by_factory_operation)
            {
                CDMW_DEBUG(m_logger.in(), "Check if configuration is complete");
      
                <%allAttributes[mode != "ATTR_READONLY"].genAttributeInitCheck%>
            }
            CCMObject_impl::configuration_complete();
        
            try
            {
                m_executor_locator->configuration_complete();
            
                // change state
                m_state = m_state->configuration_complete(this);
            }
            catch (const Components::CCMException& e)
            {
                CDMW_ERROR(m_logger.in(), "Components::CCMException raised, reason " << e);
                CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                                   CORBA::COMPLETED_NO);
                CDMW_LOG_FUNCTION_EXCEPTION(ex);
                throw ex;
            }
        }
    }
    
    <%equivalentLocalInterface%>_ptr
    <%generatedSessionServantClassname%>::obtain_component_executor()
        throw (CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "");
        check_is_removed();

        try
        {
            CORBA::Object_var obj = m_executor_locator->obtain_executor(COMP_NAME);
            
            if (CORBA::is_nil(obj.in()))
            {
               CDMW_ERROR(m_logger.in(), "Component executor is nil");
               CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXISTInvalidExecutor,
                                          CORBA::COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }
            
            <%equivalentLocalInterface%>_var comp_exec = <%equivalentLocalInterface%>::_narrow(obj.in());
            
            if (CORBA::is_nil(comp_exec.in()))
            {
               CDMW_ERROR(m_logger.in(), 
                   "Component executor is not a <%equivalentLocalInterface%>");
               CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXISTInvalidExecutor,
                                          CORBA::COMPLETED_NO);
               CDMW_LOG_FUNCTION_EXCEPTION(ex);
               throw ex;
            }
            
            return comp_exec._retn();
        }
        catch (const Components::CCMException& e)
        {
            CDMW_ERROR(m_logger.in(), 
               "CCMException obtaining component executor: " <<e);
            CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXISTInvalidExecutor,
                                       CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        catch (const CORBA::BAD_PARAM& e)
        {
            CDMW_ERROR(m_logger.in(), 
               "CORBA::BAD_PARAM obtaining component executor");
            CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXISTInvalidExecutor,
                                       CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }
    
    /**
     * Purpose:
     * <p>This returns the servant for the facet/consumer specified by facet_name.
     * This operation is implemented by the well typed component servant.
     *
     * @param facet_name The facet name
     *
     * @return the facet servant.
     */
    <%if ((allProvidesPorts.nSize > 0) || (allConsumesPorts.nSize > 0)){%>
    PortableServer::Servant
    <%generatedSessionServantClassname%>::get_facet_servant(const std::string& facet_name) 
        throw (CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "facet_name="<<facet_name);
        check_is_removed();
        
        ::std::string component_id = get_component_identity();
        PortableServer::ServantBase_var facet_servant;
        <%for (allProvidesPorts){%>
        if (facet_name == <%portNameConstName%>)
        {
            facet_servant =
                new <%interface_type.generatedFacetServantFullClassname%>(
                    m_executor_locator.in(), 
                    <%portNameConstName%>,
                    m_context,
                    m_comp_ref.in(),
                    this,
                    m_logger.in());
        }
        else
        <%}%>
        <%for (allConsumesPorts){%>
        if (facet_name == <%portNameConstName%>)
        {
            facet_servant =
                new <%event.generatedConsumerFullClassname%>(
                    m_executor_locator.in(), 
                    <%portNameConstName%>,
                    component_id,
                    m_context,
                    this,
                    m_logger.in());
        }
        else
        <%}%>
        {
            // No facet/consumer with this name define for this component
            CDMW_ERROR(m_logger.in(), 
                "No '" << facet_name << 
                "' facet or consumer defined for this component");
            CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                       CORBA::COMPLETED_NO); 
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        
        return facet_servant._retn();
    }

    <%}else{%>
    PortableServer::Servant
    <%generatedSessionServantClassname%>::get_facet_servant(const std::string& facet_name) 
        throw (CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "facet_name="<<facet_name);
        check_is_removed();

        // No facet with this name define for this component
        CDMW_ERROR(m_logger.in(), 
            "No '" << facet_name << 
            "' facet or consumer defined for this component");
                 
        CORBA::OBJECT_NOT_EXIST ex(Cdmw::OrbSupport::OBJECT_NOT_EXIST,
                                   CORBA::COMPLETED_NO); 
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    <%}%>          


    /**
     * Purpose:
     * <p>In case of a session component, this operation will call 
     * it's executor's ccm_activate operation.
     *
     *@exception Components::CCMException Any CCMException thrown by the
     * executor's ccm_activate operation.
     *@exception CORBA::SystemException Any SystemException thrown by the
     * executor's ccm_activate operation.
     */ 
    void <%generatedSessionServantClassname%>::ccm_activate() 
        throw (Components::CCMException, CORBA::SystemException)
    {
        // this is a session component, call ccm_activate
        m_session_locator->ccm_activate();
    }


    /**
     * Purpose:
     * <p>In case of a session component, this operation will call 
     * it's executor's ccm_passivate operation.
     *
     *@exception Components::CCMException Any CCMException thrown by the
     * executor's ccm_passivate operation.
     *@exception CORBA::SystemException Any SystemException thrown by the
     * executor's ccm_passivate operation.
     */ 
    void <%generatedSessionServantClassname%>::ccm_passivate() 
        throw (Components::CCMException, CORBA::SystemException)
    {
        // this is a session component, call ccm_passivate
        m_session_locator->ccm_passivate();
    }


    /**
     * Purpose:
     * <p>In case of a session component, this operation will call 
     * it's executor's ccm_remove operation.
     *
     *@exception Components::CCMException Any CCMException thrown by the
     * executor's ccm_remove operation.
     *@exception CORBA::SystemException Any SystemException thrown by the
     * executor's ccm_remove operation.
     */ 
    void <%generatedSessionServantClassname%>::ccm_remove() 
        throw (Components::CCMException, CORBA::SystemException)
    {
        // this is a session component, call ccm_remove
        m_session_locator->ccm_remove();
    }

    ::CORBA::LocalObject * 
    <%generatedSessionServantClassname%>::get_receptacle_object(const char * receptacle_name, CORBA::Object * facet_object) 
        throw (::CORBA::SystemException) 
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "Receptacle name=" << receptacle_name);
        ::CORBA::LocalObject_var obj = ::CORBA::LocalObject::_nil();
            
        <%if (allUsesPorts.nSize > 0){%>
        ::std::string component_id = get_component_identity();
        <%for (allUsesPorts){%>
        if(0 == strcmp(<%portNameConstName%>, receptacle_name))
        {
            obj = new <%interface_type.generatedCOPIReceptacleFullClassname%>(
                receptacle_name, 
                component_id,
                facet_object, 
                this->m_context,
                m_logger.in());
        }
        else
        <%}%>
        {
            CDMW_ERROR(m_logger.in(), 
                       "No '" <<  receptacle_name << "' receptacle defined for this component");
        }
        <%}else{%>
        CDMW_DEBUG(m_logger.in(), "This component has no receptacles");  
        <%}%>
            
        return obj._retn();
    }
   

    // Change state to REMOVED; this will call ccm_passivate and ccm_remove on executor
    void 
    <%generatedSessionServantClassname%>::advise_executor_of_remove_component()
        throw (CORBA::SystemException)
    {
        CDMW_LOG_FUNCTION(m_logger.in(), "");
        check_is_removed();
        
        try
        {
            m_state = m_state->remove(this);
        }
        catch (const Components::CCMException& e)
        {
            CDMW_ERROR(m_logger.in(), "Components::CCMException raised, reason " << e);
            
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError,
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
    }

<%closeNamespace(generatedServantPackage)%>




<%script type="idlmm.ProvidesDef" name="genIncludeFacetServant"%>
#include "<%interface_type.generatedFacetServantClassname%>.hpp"

<%script type="idlmm.ConsumesDef" name="genIncludeConsumer"%>
#include "<%event.generatedConsumerClassname%>.hpp"

<%script type="idlmm.UsesDef" name="genIncludeReceptacle"%>
#include "<%interface_type.generatedCOPIReceptacleClassname%>.hpp"


<%script type="idlmm.ComponentPortDef" name="genPortConstantsDef"%>
const char* <%portNameConstName%> = "<%name%>";
const char* <%portRepIdConstName%> = "<%portTypeID%>";
    

<%script type="idlmm.ComponentPortDef" name="portNameConstName" post="trim()"%>
    <%portTypeConst%>_<%name.toUpperCase%>_NAME
    
<%script type="idlmm.ComponentPortDef" name="portRepIdConstName" post="trim()"%>
    <%portTypeConst%>_<%name.toUpperCase%>_REP_ID
    
    
<%script type="idlmm.InterfacePortDef" name="portTypeID" post="trim()"%>
    <%-- For provides and uses ports, 
         RepositoryId is the one of the interface type.
    --%>
    <%interface_type.ID%>
    
<%script type="idlmm.EmitsDef" name="portTypeID" post="trim()"%>
    <%-- For emits and publishes ports,
         RepositoryId is the one of the generated Consumer
         (<event_type_name>Consumer)
    --%>
    <%event.ID.replaceFirst(":1.0", "Consumer:1.0")%>
    
<%script type="idlmm.PublishesDef" name="portTypeID" post="trim()"%>
    <%-- For emits and publishes ports,
         RepositoryId is the one of the generated Consumer
         (<event_type_name>Consumer)
    --%>
    <%event.ID.replaceFirst(":1.0", "Consumer:1.0")%>
    
<%script type="idlmm.ConsumesDef" name="portTypeID" post="trim()"%>
    <%-- For consumes ports,
         RepositoryId is the one of Consumer class generated by Cardamom
         (com.thalesgroup.Cdmw_<scoped_event_type_name>Consumer)
    --%>
    IDL:<%generatedPrefix%>/Cdmw_<%event.absoluteName.replaceFirst("::", "").replaceAll("::", "/")%>Consumer:1.0
    
<%script type="idlmm.ComponentPortDef" name="portTypeID" post="trim()"%>
    <%-- fallback script, should not be called --%>
    UNKOWN_PORTTYPE_ERROR!!
    
    
<%script type="idlmm.ProvidesDef" name="portTypeConst" post="trim()"%>
    FACET
<%script type="idlmm.UsesDef" name="portTypeConst" post="trim()"%>
    RECEP
<%script type="idlmm.EmitsDef" name="portTypeConst" post="trim()"%>
    EMITTER
<%script type="idlmm.PublishesDef" name="portTypeConst" post="trim()"%>
    PUBLISHER
<%script type="idlmm.ConsumesDef" name="portTypeConst" post="trim()"%>
    CONSUMER
<%script type="idlmm.ComponentPortDef" name="portTypeConst" post="trim()"%>
    <%-- fallback script, should not be called --%>
    UNKOWN_PORTTYPE_ERROR!!


<%script type="idlmm.ConsumesDef" name="consumerExecutorClassname" post="trim()"%>
    <%name.toU1Case%>ConsumerExecutor_impl
<%script type="idlmm.ConsumesDef" name="consumerExecutorName" post="trim()"%>
    m_<%name%>_consumer_executor


<%script type="idlmm.ConsumesDef" name="genConsumerExecutor"%>
<%-- Required argument:
  --  - 0 : the parent component
  --%>
class <%consumerExecutorClassname%> : 
    virtual public ::CORBA::LocalObject,
    virtual public <%event.equivalentLocalConsumer%>
{
    public:
        <%consumerExecutorClassname%>(
                const ::std::string & component_id,
                ::Cdmw::CCM::CIF::Context * context,
                <%args(0).equivalentLocalInterface%>_ptr comp_executor,
                CdmwLogging::Logger_ptr logger)
            : m_component_id(component_id),
              m_context(context),
              m_comp_executor(<%args(0).equivalentLocalInterface%>::_duplicate(comp_executor)),
              m_logger(::CdmwLogging::Logger::_duplicate(logger))
        {                  
            CDMW_LOG_FUNCTION(m_logger.in(), "");
        }
   
        ~<%consumerExecutorClassname%>() throw() 
        {
            CDMW_LOG_FUNCTION(m_logger.in(), "");
        }

        virtual void push(<%event.asInParameterDef%> event)
            throw (CORBA::SystemException)  
        {
            CDMW_LOG_FUNCTION(m_logger.in(), "");
            push_<%name%>(event);
        }
     
        virtual void push_<%name%>(<%event.asInParameterDef%> event)
            throw (CORBA::SystemException) 
        {
            CDMW_LOG_FUNCTION(m_logger.in(), "");
    
            if(true == Cdmw::CCM::COPI::COPIServiceInit::Is_initialized()) {
                Cdmw::CCM::COPI::PushContainerServantRequest_impl request(
                    "<%name%>", 
                    m_component_id.c_str(),
                    m_comp_executor, 
                    m_context);

                if(!request.is_read_write()) {
                    m_comp_executor->push_<%name%>(event);
                    return;
                }

                // request is a read / write request
                ::CORBA::Any in_arg;
                in_arg <<= event;

                request.add_in_args(in_arg);
                request.pre_invoke();

                // The operation parameters
                <%event.cppFullClassname%> * _event = 0;
                ::CORBA::Any_var event_any;

                try {
                    event_any = request.get_arg(0);
                } catch(::CORBA::SystemException & se) {
                    request.add_system_exception(se);
                    request.throw_exception();
                } catch(...) {
                    CDMW_ERROR(m_logger.in(),"A Stub COPI has corrupted the args");
                    ::CORBA::BAD_OPERATION bad_op;
                    request.add_system_exception(bad_op);
                    request.throw_exception();
                }

                if(::CORBA::tk_null == event_any->type()->kind()) {
                    CDMW_ERROR(m_logger.in(),"A Stub COPI has corrupted the args");
                    ::CORBA::BAD_OPERATION bad_op;
                    request.add_system_exception(bad_op);
                    request.throw_exception();
                }

                if(event_any->type()->equal(<%event.container.cppFullClassname%>::_tc_<%event.name%>)) {
                    if(!(event_any.in() >>= _event)) {
                        CDMW_ASSERT(0);
                    }
                } else {
                    CDMW_ERROR(m_logger.in(),"A Stub COPI has corrupted the args");
                    ::CORBA::BAD_OPERATION bad_op;
                    request.add_system_exception(bad_op);
                    request.throw_exception();
                }

                try {
                    ::CORBA::Any_var the_exception =
                        request.the_exception();
                    if((true == request.m_proceed_call && ::CORBA::tk_null == the_exception->type()->kind())) {
                          m_comp_executor->push_<%name%>(_event);
                    }
                } catch(::CORBA::SystemException & se) {
                    request.add_system_exception(se);
                }
                request.post_invoke();           
                request.throw_exception();
            } else {
                // COPI Service not initialised
                m_comp_executor->push_<%name%>(event);
                return;
            }
        }

    private:
        <%consumerExecutorClassname%>(const <%consumerExecutorClassname%>&);
        void operator=(const <%consumerExecutorClassname%>&);

        const ::std::string m_component_id;
        ::Cdmw::CCM::CIF::Context * m_context;
        <%args(0).equivalentLocalInterface%>_var m_comp_executor;
        CdmwLogging::Logger_var m_logger;
};


<%script type="idlmm.ProvidesDef" name="genProvidesDeclaration"%>
try
{
    CORBA::Object_var <%name%>_facet
        = declare_facet(comp_oid, 
                        <%portNameConstName%>, 
                        <%portRepIdConstName%>);
                      
    if (CORBA::is_nil(<%name%>_facet.in()))
    {
        // This is to avoid Assertion failure in debug mode as internal ref count
        // equals 1 (CORBA C++ mapping oblige!)
        _set_ref_count(0);

        CDMW_ERROR(m_logger.in(), 
            "Internal exception raised when declaring facet <%name%>!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
}
catch (const AlreadyDoneException&)
{
    // This is to avoid Assertion failure in debug mode as internal ref count
    // equals 1 (CORBA C++ mapping oblige!)
    _set_ref_count(0);
  
    CDMW_ERROR(m_logger.in(), "Facet for_clients already declared!");
    // Don't propagate while its probably a code generation error!
}
catch (const InternalErrorException&)
{
    // This is to avoid Assertion failure in debug mode as internal ref count
    // equals 1 (CORBA C++ mapping oblige!)
    _set_ref_count(0);

    CDMW_ERROR(m_logger.in(), 
        "Internal exception raised when declaring facet <%name%>!");
    CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, CORBA::COMPLETED_NO);
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}



<%script type="idlmm.AttributeDef" name="genAttributeOperations"%>
   <%-- REQUIRES parent component in context
        (containingComponent cannot be used since
         attribute could come from a supported interface) 
   --%>
<%type.asReturnParameterDef%> <%peek.generatedSessionServantClassname%>::<%cppName%>()
    throw(<%for (getExceptions){%><%cppFullClassname%>, <%}%>CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();

    <%peek.equivalentLocalInterface%>_var comp_exec = obtain_component_executor();
    <%type.asLocalVariable%> result = comp_exec-><%cppName%>();

    try
    {
        m_executor_locator->release_executor(comp_exec.in());
    }
    catch (const Components::CCMException& e )
    {
        CDMW_ERROR(m_logger.in(), "Components::CCMException raised, reason " << e);
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
         
    return result<%type.localVarReturnSuffix%>;
}

<%if (mode != "ATTR_READONLY"){%>
void <%peek.generatedSessionServantClassname%>::<%cppName%>(<%type.asInParameterDef%> <%cppName%>)
    throw(<%for (setExceptions){%><%cppFullClassname%>, <%}%>CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();

    <%peek.equivalentLocalInterface%>_var comp_exec = obtain_component_executor();
    comp_exec-><%cppName%>(<%cppName%>);

    try
    {
        m_executor_locator->release_executor(comp_exec.in());
    }
    catch (const Components::CCMException& e )
    {
        CDMW_ERROR(m_logger.in(), "Components::CCMException raised, reason " << e);
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    // set boolean to true (needed for configuration_completed)
    m_is_<%name%>_configured = true;
}
<%}%>



<%script type="idlmm.OperationDef" name="genOperationImpl"%>
<%-- REQUIRES parent component in context
     (containingComponent cannot be used since
      operation could come from a supported interface) 
--%>
<%genOperationSignatureWithClassname(peek.generatedSessionServantClassname)%>
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();

    <%peek.equivalentLocalInterface%>_var comp_exec = obtain_component_executor();
    <%if (! type.isVoid){%>
    <%type.asLocalVariable%> __result = 
    <%}%>
    comp_exec-><%cppName%>(<%parameters.cppName.sep(", ")%>);
    
    try
    {
        m_executor_locator->release_executor(comp_exec.in());
    }
    catch (const Components::CCMException& e )
    {
        CDMW_ERROR(m_logger.in(), 
            "CCMException received: " << e);
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    <%if (! type.isVoid){%>
    return __result<%type.localVarReturnSuffix%>;
    <%}%>    
}


<%script type="idlmm.ProvidesDef" name="genEquivalentOperations"%>
<%interface_type.asReturnParameterDef%> <%peek.generatedSessionServantClassname%>::provide_<%name%>()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();

    CORBA::Object_var obj;

    try 
    {
        obj = CCMObject_impl::provide_facet("<%name%>");
    }
    catch (const Components::InvalidName &)
    {
        CDMW_ERROR(m_logger.in(), 
            "'<%name%>' is not a valid facet name");
        CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST,
                                   CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    if (CORBA::is_nil(obj.in()))
    {
        CDMW_ERROR(m_logger.in(), 
            "Facet <%name%> is nil");
        CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                   CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }

    <%interface_type.asLocalVariable%> facet_ref = <%interface_type.cppFullClassname%>::_narrow(obj.in());
       
    if (CORBA::is_nil(facet_ref.in()))
    {
        CDMW_ERROR(m_logger.in(), 
            "Facet <%name%> is not a <%interface_type.cppFullClassname%>");
        CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                   CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
       
    return facet_ref<%interface_type.localVarReturnSuffix%>;
}


<%script type="idlmm.UsesDef" name="genEquivalentOperations"%>
void <%peek.generatedSessionServantClassname%>::connect_<%name%>(<%interface_type.asInParameterDef%> conxn)
    throw(Components::AlreadyConnected, 
          Components::InvalidConnection, 
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();
    
    try
    {
        CCMObject_impl::connect(<%portNameConstName%>, conxn);
    }
    catch (const Components::InvalidName &)
    {
        CDMW_ERROR(m_logger.in(), "Invalid name for receptacle connection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::AlreadyConnected & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (const Components::InvalidConnection & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (const Components::ExceededConnectionLimit &)
    {
        CDMW_ERROR(m_logger.in(), 
            "Exceeded connection limit reached for receptacle connection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);   
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
}

<%interface_type.asReturnParameterDef%> <%peek.generatedSessionServantClassname%>::disconnect_<%name%>()
    throw(Components::NoConnection, 
          CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();
    
    try
    {
        <%interface_type.asLocalVariable%> result = get_connection_<%name%>();
        CCMObject_impl::disconnect(<%portNameConstName%>, 0);
    
        return result._retn();
    }
    catch (const Components::InvalidName &)
    {
        CDMW_ERROR(m_logger.in(), "Invalid name for receptacle disconnection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                          CORBA::COMPLETED_NO);   
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::NoConnection & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (const Components::InvalidConnection &)
    {
        CDMW_ERROR(m_logger.in(), "Invalid connection for receptacle disconnection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::CookieRequired &)
    {
        CDMW_ERROR(m_logger.in(), "Cookie required for receptacle disconnection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);   
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
}

<%interface_type.asReturnParameterDef%> <%peek.generatedSessionServantClassname%>::get_connection_<%name%>()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();
    
    Components::ConnectionDescriptions_var comp_desc;
    
    try
    {
        comp_desc = CCMObject_impl::get_connections(<%portNameConstName%>);
    }
    catch (const Components::InvalidName& e)
    {
        CDMW_ERROR(m_logger.in(), "Invalid name getting receptacle!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);   
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const CORBA::SystemException& e )
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    
    <%interface_type.asLocalVariable%> result;
    
    if (comp_desc->length() == 1)
    {
        CORBA::Object_var obj = CORBA::Object::_duplicate(comp_desc->operator[](0)->objref());
    
        result = <%interface_type.cppFullClassname%>::_narrow(obj.in());
    }
    else
    {
        result = <%interface_type.cppFullClassname%>::_nil();
    }
    
    return result._retn();
}


<%script type="idlmm.ConsumesDef" name="genEquivalentOperations"%>
<%event.equivalentConsumer%>_ptr <%peek.generatedSessionServantClassname%>::get_consumer_<%name%>()
    throw(CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();
    
    CORBA::Object_var obj;
    
    try 
    {
        obj = CCMObject_impl::get_consumer(<%portNameConstName%>);
    }
    catch (const Components::InvalidName & e)
    {
        CDMW_ERROR(m_logger.in(), "Invalid consumer name");
        CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST,
                                   CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    if (CORBA::is_nil(obj.in()))
    {
        CDMW_ERROR(m_logger.in(), "Consumer is nil");
        CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                   CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    
    <%event.equivalentConsumer%>_var consumer_ref =
        <%event.equivalentConsumer%>::_narrow(obj.in());
     
    if (CORBA::is_nil(consumer_ref.in()))
    {
        CDMW_ERROR(m_logger.in(), "Consumer has wrong type");
        CORBA::OBJECT_NOT_EXIST ex(OrbSupport::OBJECT_NOT_EXIST, 
                                   CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
     
    return consumer_ref._retn();
}


<%script type="idlmm.PublishesDef" name="genEquivalentOperations"%>
Components::Cookie* <%peek.generatedSessionServantClassname%>::subscribe_<%name%>(<%event.equivalentConsumer%>_ptr consumer)
    throw(Components::ExceededConnectionLimit, CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();
    
    Components::Cookie_var result;
    
    try
    {
        result = CCMObject_impl::subscribe(<%portNameConstName%>,
                                           consumer);
    }
    catch (const Components::InvalidName &)
    {
        CDMW_ERROR(m_logger.in(), "Invalid name for publisher subscription!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);   
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::InvalidConnection &)
    {
        // should never appear!
        CDMW_ERROR(m_logger.in(), "Invalid Connection");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::ExceededConnectionLimit & e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (const CORBA::SystemException& e)
    {
        throw;
        CDMW_LOG_FUNCTION_EXCEPTION(e);
    }
    
    return result._retn();
}

<%event.equivalentConsumer%>_ptr <%peek.generatedSessionServantClassname%>::unsubscribe_<%name%>(Components::Cookie* ck)
    throw(Components::InvalidConnection, CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();
    
    if (!ck) {
        CDMW_ERROR(m_logger.in(), "Cookie is null");
        CORBA::BAD_PARAM ex(OrbSupport::BAD_PARAMNilObjectReference, CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
     
    <%event.equivalentConsumer%>_var result = <%event.equivalentConsumer%>::_nil();
    try
    {
        Components::EventConsumerBase_var consumer 
            = CCMObject_impl::get_consumer(<%portNameConstName%>, ck);
        
        result = <%event.equivalentConsumer%>::_narrow(consumer.in());
        
        CCMObject_impl::unsubscribe(<%portNameConstName%>, ck);
    }
    catch (const Components::InvalidName&)
    {
        // internal error should not appear
        CDMW_ERROR(m_logger.in(), "Invalid name for publisher unsubscription!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);   
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::InvalidConnection& e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (const CORBA::SystemException& e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    
    return result._retn();
}


<%script type="idlmm.EmitsDef" name="genEquivalentOperations"%>
void <%peek.generatedSessionServantClassname%>::connect_<%name%>(<%event.equivalentConsumer%>_ptr consumer)
    throw(::Components::AlreadyConnected, CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();
    
    try
    {
        CCMObject_impl::connect_consumer(<%portNameConstName%>, consumer);
    }
    catch (const Components::InvalidName&)
    {
        // can not appear!
        CDMW_ERROR(m_logger.in(), "Invalid name for emitter conection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::InvalidConnection&)
    {
        // can not appear!
        CDMW_ERROR(m_logger.in(), "Invalid connection for emitter conection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } 
    catch (const Components::AlreadyConnected& e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (const CORBA::SystemException& e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    } 
}

<%event.equivalentConsumer%>_ptr <%peek.generatedSessionServantClassname%>::disconnect_<%name%>()
    throw(Components::NoConnection, CORBA::SystemException)
{
    CDMW_LOG_FUNCTION(m_logger.in(), "");
    check_is_removed();
    
    try
    {
        Components::EventConsumerBase_var consumer = 
            CCMObject_impl::disconnect_consumer(<%portNameConstName%>);
        
        <%event.equivalentConsumer%>_var result = 
            <%event.equivalentConsumer%>::_narrow(consumer.in());
        
        if (CORBA::is_nil(result.in()))
        {
            // can not appear!
            CDMW_ERROR(m_logger.in(), "Old connection had wrong type");
            CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                               CORBA::COMPLETED_NO);
            CDMW_LOG_FUNCTION_EXCEPTION(ex);
            throw ex;
        }
        
        return result._retn();
    }
    catch (const Components::InvalidName&)
    {
        // can not appear!
        CDMW_ERROR(m_logger.in(), "Invalid name for emitter disconection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    }
    catch (const Components::InvalidConnection&)
    {
        // can not appear!
        CDMW_ERROR(m_logger.in(), "Invalid connection for emitter disconection!");
        CORBA::INTERNAL ex(OrbSupport::INTERNALCCMCIFError, 
                           CORBA::COMPLETED_NO);
        CDMW_LOG_FUNCTION_EXCEPTION(ex);
        throw ex;
    } 
    catch (const Components::AlreadyConnected& e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    }
    catch (const CORBA::SystemException& e)
    {
        CDMW_LOG_FUNCTION_EXCEPTION(e);
        throw;
    } 
}


<%script type="idlmm.AttributeDef" name="genAttributeInitCheck"%>
// check if the attribute have been initialised only if not created by a factory op
if (!m_is_<%name%>_configured)
{
    // component is not well configured
    CDMW_ERROR(m_logger.in(), 
        "Attribut <%name%> not initialised!");
    Components::InvalidConfiguration ex;
    CDMW_LOG_FUNCTION_EXCEPTION(ex);
    throw ex;
}
