<%
metamodel http://www.prismtech.com/idlmm

import com.thalesgroup.cdmw.codegenerator.cdmwinit.ProgramServices
import com.thalesgroup.ccm.codegenerator.container.cpp.Names
%>

<%script type="idlmm.Repository" name="initCCM"%>
// Default Event Channel Factory
CdmwEvent::EventChannelFactory_var event_channel_factory 
    = CdmwEvent::EventChannelFactory::_nil();

<%if (hasEventPorts){%>

<%initEventChannelFactory%>
<%}%>

// Create an Object registration instance
// get the admin root context
CDMW_DEBUG(InitUtils::Get_Logger(), "Get the admin root context");
CdmwNamingAndRepository::Repository_var repository_ref 
    = Cdmw::NamingAndRepository::RepositoryInterface::get_repository();
CosNaming::NamingContext_var admin_root_ctx 
    = repository_ref->resolve_root_context(
        Cdmw::Common::Locations::ADMIN_ROOT_CONTEXT_ID);
Cdmw::CommonSvcs::Naming::NamingInterface ni(admin_root_ctx);
       
// bind the new context where component shall be registered
CosNaming::Name component_name_path;
component_name_path.length(5);
component_name_path[0].id 
    = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
component_name_path[1].id 
    = CORBA::string_dup(application_name.c_str());
component_name_path[2].id 
    = CORBA::string_dup(process_name.c_str());
component_name_path[3].id 
    = CORBA::string_dup("CCM");
component_name_path[4].id 
    = CORBA::string_dup("Components");

CDMW_DEBUG(InitUtils::Get_Logger(), "Bind component context");
std::string component_path
    = ni.to_string(component_name_path);
Cdmw::CommonSvcs::Naming::NamingInterface component_ni 
    = ni.new_context();
ni.rebind_context(component_path, component_ni, true);

CDMW_DEBUG(InitUtils::Get_Logger(), "Create object_registration");
CosNaming::NamingContext_var nc = component_ni.name_context();
CdmwLifeCycle::ObjectRegistration_var  object_registration
    = new Cdmw::LifeCycle::DefaultObjectRegistration(nc.in());

// Register Managed component naming context in the orb
// to make it available 
orb->register_initial_reference("ManagedComponentNamingContext", nc.in());


// Create the ComponentServer
CDMW_DEBUG(InitUtils::Get_Logger(), "Create the ComponentServer");
try {

    CdmwCcmComponentServer::ComponentServer_var the_component_server =
        Cdmw::CCM::ComponentServer::InitUtils::create_component_server(orb,
                                                                       CDMW_rootPOA.in(),
                                                                       event_channel_factory.in(),
                                                                       object_registration.in(),
                                                                       process_name,
                                                                       application_name,
                                                                       argc,argv);
    // Register it as Cdmw::Common::Locations::ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN/
    //      process_name@application_name.ComponentServer
    std::string id = process_name + std::string("@") + application_name;
    std::string kind("ComponentServer");
    CosNaming::Name name(1);
    name.length(1);
    name[0].id   = id.c_str();
    name[0].kind = kind.c_str();

    using namespace Cdmw::NamingAndRepository;             
    using namespace Cdmw::CommonSvcs::Naming;
    using namespace Cdmw::Common;

    std::string key = NamingInterface::to_string(name);

    CdmwNamingAndRepository::NameDomain_var domain =
        RepositoryInterface::get_domain(Locations::ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN);

    try {
        domain->register_new_factory(key.c_str(),the_component_server.in());
    } catch (const CdmwNamingAndRepository::NameDomain::AlreadyExists &) {
        // Re-register it again.
        domain->release_name(key.c_str());
        domain->register_new_factory(key.c_str(),the_component_server.in());
    } 
} catch (const CdmwNamingAndRepository::NoNameDomain &) {
   CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNameDomainNotFound,
                                 CORBA::COMPLETED_NO),
                std::string(Cdmw::Common::Locations::ASSEMBLY_AND_DEPLOYMENT_NAME_DOMAIN) +
                " is not a NameDomain!" );
} catch (const CdmwNamingAndRepository::InvalidName &) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_NO) );
} catch (const CdmwNamingAndRepository::NameDomain::NotRegistered &) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_NO) );
} catch (const CORBA::SystemException & ex) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,ex );
}



<%script type="idlmm.Repository" name="initEventChannelFactory"%>
// *****************************************************************
// ******        Create a local Event Channel Factory         ******
// *****************************************************************
CDMW_DEBUG(InitUtils::Get_Logger(), "Create a local Event Channel Factory");

// the factory name will be: <application_name>.<processname>.EventChannelFactory
std::string tmp_ec_factory_name = 
    application_name + "." + process_name + ".EventChannelFactory";
CosNaming::Name ec_factory_name;
ec_factory_name.length(1);
ec_factory_name[0].id = tmp_ec_factory_name.c_str();
std::string event_channel_factory_name = 
    Cdmw::CommonSvcs::Naming::NamingInterface::to_string(ec_factory_name);

{
    std::string full_name_domain_name;
    full_name_domain_name = Cdmw::Common::Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN;
    full_name_domain_name += "/EVENT_CHANNELS";
    std::string full_factory_name;
    full_factory_name = Cdmw::Common::Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN;
    full_factory_name += "/FACTORIES/";
    full_factory_name += event_channel_factory_name;
    
    // TAO doesn't support thread-pool policy with a persistent POA
    std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_policy(
        new Cdmw::OrbSupport::ThreadPerConnectionPolicy());

    // Get the factory Allocator
    NamedObjectsFactoryAllocatorBase & allocator =
        Cdmw::Event::EventChannelFactory_impl_Allocator::TheAllocator;

    try {
        NamedObjectsFactory_initUtil::setup_named_object_factory(
            allocator,
            orb,
            factories_poa.in(),
            repository.in(),
            root_context,
            full_name_domain_name,
            full_factory_name,
            "",
            *threading_policy,
            argc, argv
            );
    
    } catch (const Cdmw::OutOfMemoryException & ex ) {
        std::string s ("Could not allocate memory for ");
        s += full_factory_name;
        s += "\n";
        s += ex.what();
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     s );
    } catch (const Cdmw::BadParameterException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::InternalErrorException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                     CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                     CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const CORBA::SystemException & ex ) {
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                     ex );
    }
}

try {
    // Retreive reference to the local Event Channel factory 
    using namespace Cdmw::NamingAndRepository;             
    using namespace Cdmw::CommonSvcs::Naming;
    using namespace Cdmw::Common;

    std::string evt_factories_nd(Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN);
    evt_factories_nd += "/FACTORIES";

    NamingInterface evt_ni = 
        RepositoryInterface::get_domain_naming_interface(evt_factories_nd);

    typedef NamingUtil<CdmwEvent::EventChannelFactory> Util;
    event_channel_factory = Util::resolve_name(evt_ni, event_channel_factory_name);
    
} catch (const CdmwNamingAndRepository::NoNameDomain &) {
   CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNameDomainNotFound,
                                 CORBA::COMPLETED_NO),
                std::string(Cdmw::Common::Locations::CDMW_EVENT_SERVICE_NAME_DOMAIN) +
                "/FACTORIES is not a NameDomain!" );
} catch (const CdmwNamingAndRepository::InvalidName &) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_NO) );
} catch (const CosNaming::NamingContext::NotFound &) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_NO) );
} catch (const CosNaming::NamingContext::CannotProceed &) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_NO) );
} catch (const CosNaming::NamingContext::InvalidName &) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_NO) );
} catch (const Cdmw::Common::TypeMismatchException &) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL,
                                 CORBA::COMPLETED_NO) );
} catch (const CORBA::SystemException & ex) {
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,ex );
}



<%script type="idlmm.IRObject" name="genRegisterValuetypeFactories"%>
using namespace Cdmw::CCM::Container;
// Register Valuetype  factories
Cdmw::CCM::CIF::InitUtils::register_valuetype_factories(orb);
Cdmw::CCM::Container::InitUtils::register_valuetype_factories(orb);

<%script type="idlmm.IRObject" name="genUnregisterValuetypeFactories"%>
// Unregister Valuetype  factories
Cdmw::CCM::CIF::InitUtils::unregister_valuetype_factories(orb);
Cdmw::CCM::Container::InitUtils::unregister_valuetype_factories(orb);




<%script type="EObject" name="genRegisterHomeAllocator"%>
// Register <%get("factoryClassname")%> HomeAllocator
{
    typedef HomeAllocator< <%get("factoryClassname")%> > CdmwHome_Allocator;
    HomeAllocatorRegistry::Register("<%get("factoryClassname")%>",
                                    &CdmwHome_Allocator::TheAllocator);
}
<%--
    <%if (hasLoadBalancing){%>
    // Register <%lbFactoryClassname(getCategory(self))%> HomeAllocator
    HomeAllocatorRegistry.register("<%lbFactoryClassname(getCategory(self))%>",
                                   new HomeAllocator("<%lbFactoryClassname(getCategory(self))%>"));
    <%}%>
--%>




<%script type="idlmm.IRObject" name="genUnregisterHomeAllocators" post="trim()"%>
// Unregister HomeAllocators
{
    <%for (getFrameworksIds){%>
    <%if (isHome(self)){%>
    <%getHome(self).factoryClassname(getCategory(self)).put("factoryClassname")%>
    HomeAllocatorRegistry::Unregister("<%get("factoryClassname")%>");
    <%}%>
    <%}%>
}


<%script type="idlmm.HomeDef" name="factoryClassname" post="trim()"%>
<%if (args(0) == "SESSION"){%>
    <%generatedSessionServantFullClassname%>
<%}else if(args(0) == "EXTENSION"){%>
    <%generatedExtensionServantFullClassname%>
<%}else{%>
    ERROR_UNKNOWN_HOME_CATEGORY(<%args(0)%>)
<%}%>





