<%
metamodel http://www.prismtech.com/idlmm

import com.thalesgroup.ccm.codegenerator.corba.CommonServices
import com.thalesgroup.cdmw.codegenerator.cdmwinit.ProgramServices
import com.thalesgroup.cdmw.codegenerator.cdmwinit.DatastoreTypesServices
import com.thalesgroup.cdmw.codegenerator.cdmwinit.cpp.DataStoreUtils
%>



<%script type="idlmm.Repository" name="initFTService"%>
// generate call to FTServiceInit::Init
try {
    <%if (hasFaultToleranceServer()){%>
    Cdmw::FT::FTServiceInit::Init(argc, argv, false);
    <%}else{%>
    Cdmw::FT::FTServiceInit::Init(argc, argv, true);
    <%}%>
} catch (const CORBA::SystemException & e) {
    try {
       CDMW_ERROR(InitUtils::Get_Logger(), 
                  "CORBA Exception : " 
                  << Cdmw::OrbSupport::OrbSupport::exception_to_string(e) );
    } catch(...) {
        CDMW_ASSERT(false);
    }
}
    


<%script type="idlmm.Repository" name="setupFTPolicies"%>
// *****************************************************************
// *************  setup FaultTolerance Policies   ******************
// *****************************************************************

Cdmw::OrbSupport::OrbSupport::set_endpoint_selector_factory(::Cdmw::FT::FTConfiguration::Get_FT_endpoint_selector_factory());

size_t duration_time;
std::string client_id_str;
   
try {
    std::string res;
    try {
        res = xml_data->getServiceAttributeValue("fault-tolerance", "request-duration-policy"); 
        duration_time = atoi(res.c_str());
    } catch (const Cdmw::BadParameterException& ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    }
    
    try  {
        client_id_str = xml_data->getServiceAttributeValue("fault-tolerance", "client-id");
        ::Cdmw::FT::FTConfiguration::Set_client_id(orb, client_id_str.c_str());
    } catch (const Cdmw::BadParameterException& ) {
        // do nothing : parameter is optional
    }
} catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                 CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                  CORBA::COMPLETED_NO),
                 ex.what() );
} catch (const OutOfMemoryException & ex) {
    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                 CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                  CORBA::COMPLETED_NO),
                 ex.what() );
} 

CORBA::Object_var obj;
obj = orb->resolve_initial_references("ORBPolicyManager");

CORBA::PolicyManager_var policy_manager 
    = CORBA::PolicyManager::_narrow(obj.in());

// Create and apply an ORB-wide Routed policy
CDMW_DEBUG(InitUtils::Get_Logger(), "Request duration policy set to " << duration_time);
CORBA::Any any;
TimeBase::TimeT duration = duration_time;

any <<= duration;
CORBA::PolicyList policies(1);
policies.length(1);
policies[0] =
    orb->create_policy(::FT::REQUEST_DURATION_POLICY, any);

policy_manager->set_policy_overrides(policies, CORBA::SET_OVERRIDE);



<%script type="idlmm.Repository" name="setupFTReplicationManager"%>
// *****************************************************************
// ***********  setup FaultTolerance ReplicationManager  ***********
// *****************************************************************

// Get the reference of the Replication Manager and register it in the ORB
CORBA::Object_var objref;
const char* replication_manager_name = "ReplicationManager";
    objref =  InitUtils::get_service(replication_manager_name);  

CdmwFT::ReplicationManager_var replication_manager = CdmwFT::ReplicationManager::_narrow(objref.in());

try 
{
    orb->register_initial_reference("ReplicationManager", replication_manager.in());      
}
catch (const CORBA::SystemException& ex)
{
    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                 ex,
                 "Cannot register ReplicationManager." );
}
catch (const CORBA::ORB::InvalidName& )
{
    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                    "InvalidName => Cannot register ReplicationManager.");
}


<%script type="idlmm.Repository" name="setupFTLocationManager"%>
// *****************************************************************
// *************  setup FaultTolerance LocationManager  ************
// *****************************************************************

// Location information shall be obtain from supervision
::FT::Location loc;
loc.length(3);

loc[0].id = CORBA::string_dup(Cdmw::OsSupport::OS::get_hostname().c_str());
loc[0].kind = "hostname";
loc[1].id = application_name.c_str();
loc[1].kind = "applicationname";
loc[2].id = process_name.c_str();
loc[2].kind = "processname";

CDMW_DEBUG(InitUtils::Get_Logger(), 
           "Setup FaultTolerance LocationManager for location: " 
           << Cdmw::CommonSvcs::Naming::NamingInterface::to_string(loc));
   
// create a reference for the PrimaryBackupGroupRepository
::Cdmw::FT::Location::PrimaryBackupGroupRepository_impl * primary_backup_repository
    = ::Cdmw::FT::Location::PrimaryBackupGroupRepository_impl::Get_instance();
         


<%script type="idlmm.Repository" name="setupStatefullPrimaryBackupAdmin"%>
// get the fault notifier reference
::FT::FaultNotifier_var fault_notifier = replication_manager->get_fault_notifier();

// create the DataStoreTimeoutHandler
CDMW_DEBUG(InitUtils::Get_Logger(), "Create the DataStoreTimeoutHandler");
Cdmw::FT::LocationDataStoreTimeoutHandler* locationDataStoreTimeoutHandler
    = new Cdmw::FT::LocationDataStoreTimeoutHandler(fault_notifier.in());

// Register the timeoutHandler into the DataStoreBase
Cdmw::FT::DataStoreBase::Register_Timeout_Handler(locationDataStoreTimeoutHandler);


// FaultTolerance use datastore
CORBA::ULong datastore_id;
std::string multicast_port;
std::string multicast_domain;
std::string multicast_IP_address;
int max_transaction_in_progress;
int max_transaction_done;
OsSupport::OS::Timeval cohort_timeout;
OsSupport::OS::Timeval coordinator_timeout;
OsSupport::OS::Timeval freeze_timeout;
long number_of_scheduler_thread;
long chunk_size;
long freeze_size;

long dataStore_inc=0;

long localDataStoreInfosSize = <%countAllDatastoreOccurrences%>;
CdmwFT::StateTransfer::LocalDataStoreInfos_var localDataStoreInfos = 
    new CdmwFT::StateTransfer::LocalDataStoreInfos();
localDataStoreInfos->length(1);

std::string hotrestart_arg = 
    ::Cdmw::OsSupport::OS::get_option_value( argc, argv, "--HotRestart");
bool cold_restart = (hotrestart_arg != "yes");

CDMW_DEBUG(InitUtils::Get_Logger(), 
           "Setup StatefullPrimaryBackupAdmin_impl "
           << (cold_restart ? "with cold restart" : "with hot restart") );
           
::Cdmw::FT::Location::StatefullPrimaryBackupAdmin_impl * statefull_primary_backup_admin =
    new ::Cdmw::FT::Location::StatefullPrimaryBackupAdmin_impl(orb, 
                                                               CDMW_rootPOA.in(), 
                                                               loc, 
                                                               primary_backup_repository, 
                                                               cold_restart);

bool reply_logging = false;                                                                            
            
<%setupDatastoreContext%>

<%setupDatastores%>










<%script type="idlmm.Repository" name="setupDatastoreContext"%>
// get information for DataStore context 
try {
    std::string res;
    unsigned long res_long;       
    res = xml_data->getDataStoresAttributeValue("max-transaction-in-progress"); 
    max_transaction_in_progress = atoi(res.c_str());
    
    res = xml_data->getDataStoresAttributeValue("max-transaction-done"); 
    max_transaction_done = atoi(res.c_str());
    
    res = xml_data->getDataStoresAttributeValue("cohort-timeout");
    res_long = atoi(res.c_str());                 
    cohort_timeout.seconds = res_long/1000;
    cohort_timeout.microseconds = (res_long%1000)*1000;
    
    res = xml_data->getDataStoresAttributeValue("coordinator-timeout");                 
    res_long = atoi(res.c_str());                 
    coordinator_timeout.seconds = res_long/1000;
    coordinator_timeout.microseconds = (res_long%1000)*1000;
    
    res = xml_data->getDataStoresAttributeValue("number-of-scheduler-thread");
    number_of_scheduler_thread = atoi(res.c_str());
    
    res = xml_data->getDataStoresAttributeValue("chunk-size");
    chunk_size = atoi(res.c_str());
    
    res = xml_data->getDataStoresAttributeValue("freeze-size");
    freeze_size = atoi(res.c_str());
    
    res = xml_data->getDataStoresAttributeValue("freeze-timeout");
    res_long = atoi(res.c_str());                              
    freeze_timeout.seconds = res_long/1000;
    freeze_timeout.microseconds = (res_long%1000)*1000;

    <%if (faultToleranceDatastoreKind.toUpperCase == "MIOP"){%>
    multicast_port = xml_data->getDataStoreAttributeValue("multicast", 1, "multicast-port");
    multicast_domain = xml_data->getDataStoreAttributeValue("multicast", 1, "multicast-domain");
    multicast_IP_address = xml_data->getDataStoreAttributeValue("multicast", 1, "ip-adresse");
    <%}%>
    
} catch (const Cdmw::BadParameterException& ex) {
    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                 CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                  CORBA::COMPLETED_NO),ex.what() );
}

PortableGroup::GOA_var rootPOA = InitUtils::get_root_GOA(orb);
Cdmw::FT::DataStoreContext *datastore_context = new Cdmw::FT::DataStoreContext(
<%if (faultToleranceDatastoreKind.toUpperCase == "MIOP"){%>
    orb,
    CDMW_rootPOA.in(),
    multicast_port,
    multicast_domain,
    multicast_IP_address,
    max_transaction_in_progress,
    max_transaction_done,
    cohort_timeout,
    coordinator_timeout,
    number_of_scheduler_thread
<%}else{%>
    orb,
    CDMW_rootPOA.in(),
    max_transaction_in_progress,
    max_transaction_done,
    cohort_timeout,
    coordinator_timeout,
    number_of_scheduler_thread
<%}%>
    );
        
// TODO clean up manager (since this can only be done at the end of the program execution
// then this leak is harmless). Also applise to datastore_context and datastores
Cdmw::FT::TXManager* manager = new Cdmw::FT::TXManager(*datastore_context);
        
CdmwFT::StateTransfer::LocalDataStoreInfo localDataStoreInfo;

CdmwFT::StateTransfer::DataStoreGroup_var ftdatastoregroup_ref = 
    (new Cdmw::FT::DataStoreGroup_impl(CDMW_rootPOA.in(), *datastore_context))->_this();

localDataStoreInfo.local_data_store = CdmwFT::StateTransfer::DataStoreGroup::_duplicate(ftdatastoregroup_ref.in());
  
<%if (faultToleranceDatastoreKind.toUpperCase == "MIOP"){%>
localDataStoreInfo.coordinator = CdmwFT::Transaction::TPCCoordinator::_nil();
localDataStoreInfo.cohort = CdmwFT::Transaction::TPCCohort::_nil();
<%}else{%>
localDataStoreInfo.coordinator = datastore_context->get_tpc_coordinator();
localDataStoreInfo.cohort = datastore_context->get_tpc_cohort();
<%}%>
  
<%if (hasFaultToleranceReplyLogging()){%>
// incrementation for the reply logging
localDataStoreInfosSize = localDataStoreInfosSize + 1;
<%}else{%>
// incrementation for the reply logging
localDataStoreInfosSize = localDataStoreInfosSize + 0;
<%}%>
          
localDataStoreInfo.dsids.length(localDataStoreInfosSize);
(*localDataStoreInfos)[0] = localDataStoreInfo;
        
        


<%script type="idlmm.Repository" name="setupDatastores"%>
<%-- for each used datastore --%>
<%for (getUsedDatastoreTypenames){%>
<%push%>
<%-- for each occurrence of the datastore --%>
<%for (intList(getUsedDatastoreOccurrences(peek))){%>
<%setupDatastore(self)%>
<%}%>
<%pop%>
<%}%>

<%--
  -- Setup DataStore for typename in context (<%peek%>).
  -- The number of occurrence should be passed as parameter.
  -- Required arguments:
  --   - 0 : the number of current occurrence
  --%>
<%script type="EObject" name="setupDatastore"%>
// setup datastore <%peek%>  (occurrence <%args(0)%>)
try {
    std::string res;
    InitUtils::get_datastore_id_parameter
        (*(xml_data.get()),
         "<%peek%>",
         <%countAllDatastoreOccurrences%>,
         <%args(0)%>,
         datastore_id);

} catch (const Cdmw::BadParameterException& ex) {
    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                 CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                  CORBA::COMPLETED_NO),ex.what() );
}

(*localDataStoreInfos)[0].dsids[dataStore_inc++] = datastore_id;
typedef Cdmw::FT::TXDataStore< <%getDatastoreOidType(peek).getCppTypeForDataTraits%>,
                              <%getDatastoreDataType(peek).getCppTypeForDataTraits%> >
    <%peek%>;

new <%peek%> (*manager,
              datastore_id,
              orb,
              CDMW_rootPOA.in(),
              freeze_size,
              chunk_size,
              freeze_timeout);



<%script type="idlmm.Repository" name="initReplyLogging"%>
CDMW_DEBUG(InitUtils::Get_Logger(), "Setup ReplyLogging");
reply_logging = true;
typedef Cdmw::FT::TXDataStore<std::string, CdmwFT::ReplyData> 
    ReplyLogging;

datastore_id = Cdmw::FT::FT_REPLY_DATASTORE_ID;

ReplyLogging * logging = new ReplyLogging(*manager,
                                        datastore_id,
                                        orb,
                                        CDMW_rootPOA.in(),
                                        freeze_size,
                                        chunk_size,
                                        freeze_timeout);

(*localDataStoreInfos)[0].dsids.length(dataStore_inc + 1);
(*localDataStoreInfos)[0].dsids[dataStore_inc++] = Cdmw::FT::FT_REPLY_DATASTORE_ID;

Cdmw::FT::FTServiceInit::Set_reply_recorder
    (new Cdmw::FT::RequestLogging::ReplyRecording(logging->get_datastore()));



<%script type="idlmm.Repository" name="registerStatefullLocation"%>
CDMW_DEBUG(InitUtils::Get_Logger(), "Register statefull location");
::CdmwFT::Location::StatefullPrimaryBackupAdmin_var statefull_primary_backup_admin_ref = 
    statefull_primary_backup_admin->_this();

replication_manager->register_statefull_location(
    loc, 
    statefull_primary_backup_admin_ref.in(),
    localDataStoreInfos.in(), 
    reply_logging);

    
<%script type="idlmm.Repository" name="setupPrimaryBackupAdmin"%>
CDMW_DEBUG(InitUtils::Get_Logger(), "Setup PrimaryBackupAdmin_impl");
::Cdmw::FT::Location::PrimaryBackupAdmin_impl * primary_backup_admin =
    new ::Cdmw::FT::Location::PrimaryBackupAdmin_impl(orb, 
                                                      CDMW_rootPOA.in(), 
                                                      loc, 
                                                      primary_backup_repository);
         
::CdmwFT::Location::PrimaryBackupAdmin_var primary_backup_admin_ref = 
    primary_backup_admin->_this();

replication_manager->register_location(loc, 
                                       primary_backup_admin_ref.in());
