<%
metamodel http://www.prismtech.com/idlmm

import com.thalesgroup.cdmw.codegenerator.cdmwinit.ProgramServices
import com.thalesgroup.cdmw.codegenerator.cdmwinit.cpp.initCCM
import com.thalesgroup.ccm.codegenerator.corba.CommonServices
import com.thalesgroup.ccm.codegenerator.corba.cpp.CppMappingServices
%>

<%script type="idlmm.Repository" name="initLifeCycle"%>
// *****************************************************************
// *********  Create CDMW Factories POA (persistent POA)  *********
// ***************************************************************** 
CDMW_DEBUG(InitUtils::Get_Logger(), "Create CDMW Factories POA");
// Raises CORBA::SystemException
PortableServer::POA_var factories_poa 
    = Cdmw::LifeCycle::InitUtils::create_cdmw_LifeCycle_POA(CDMW_rootPOA.in());

// Create CDMW factories

using namespace Cdmw::LifeCycle;
CosNaming::NamingContext_var nc_root_context = CosNaming::NamingContext::_nil();
try {
    nc_root_context
        = repository->resolve_root_context(CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
} catch (const CdmwNamingAndRepository::Repository::NoRootContext& ) {
    std::string s (CdmwNamingAndRepository::DEFAULT_ROOT_CONTEXT);
    s += " is not a Root Context!";
    CDMW_WARN(InitUtils::Get_Logger(), s);
    CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                 CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALCdmwRepositoryError,
                                 CORBA::COMPLETED_NO),
                 s );
} catch (const CORBA::SystemException & ex) {
    CDMW_WARN(InitUtils::Get_Logger(), "CORBA::SystemException received: "
              << Cdmw::OrbSupport::OrbSupport::exception_to_string(ex));
    CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                  ex );
}
// May raise a CORBA::SystemException 
Cdmw::CommonSvcs::Naming::NamingInterface root_context (nc_root_context.in());

<%if (hasHomeImpl){%>
<%genRegisterValuetypeFactories%>
<%}%>

<%for (getFrameworksIds){%>

    <%printFrameworkData%>
    
    // *********************************
    // Init lifecycle-framework <%self%> (<%getCategory(self)%>)
    // *********************************
    
    <%-- compute and store factoryClassname in context as "factoryClassname"--%>
    <%if (isHome(self)){%>
    <%getHome(self).factoryClassname(getCategory(self)).put("factoryClassname")%>
    <%}else{%>
    <%getFactoryClassname(self).withCppScope.put("factoryClassname")%>
    <%}%>
    
    <%--
       Register the home allocator.
    --%>
    <%if (isHome(self)){%>
    
    <%genRegisterHomeAllocator%>
    <%}%>
    
    <%--
       Strategy selection.
    --%>
    <%if (getCategory(self) == "ENTITY"){%>
        <%--
           ENTITY
        --%>
        <%if (isHome(self)){%>
        <%--
           TODO: Add component server code here!
        --%>
        <%}else{%>
        
        <%genEntityAllocatorInit%>
        <%}%>
    
    <%}else if (getCategory(self) == "UNMANAGED_ENTITY"){%>
        <%--
           UNMANAGED_ENTITY.
        --%>
        <%genUnmanagedEntityAllocatorInit%>
    
    <%}else if (getCategory(self) == "SESSION"){%>
        <%--
           SESSION
        --%>
        <%if (isHome(self)){%>
        <%--
           TODO: Add component server code here!
        --%>
        <%}else{%>
        
        <%genSessionAllocatorInit%>
        <%}%>
    
    <%}else if (getCategory(self) == "SERVICE"){%>
        <%--
           SERVICE
        --%>
        <%if (isHome(self)){%>
        <%--
           TODO: Add component server code here!
        --%>
        <%}else{%>
        
        <%genServiceAllocatorInit%>
        <%}%>
    
    <%}else if (getCategory(self) == "PROCESS"){%>
        <%--
           TODO: Add component server code here!
        --%>
    
    <%}else if (getCategory(self) == "USER_DEFINED"){%>
        
        <%genUserDefinedFactoryAllocatorInit%>
    
    <%}else{%>
    <%abort("ERROR UNKOWN CATEGORY: <%getCategory(self)%> (shouldn't happen)")%>
    <%}%>
    
<%}%>




<%--
  -- Generate code for Entity lifecycle-framework
  -- This script is supposed to be called with the lifecycle-framework id as self.
  -- And context should contain a entry for "factoryClassname"
  --  
--%>
<%script type="EObject" name="genEntityAllocatorInit"%>
// *****************************************************************
// ****** <%get("factoryClassname")%>  ******
// ***************************************************************** 
{
    std::string full_name_domain_name;
    std::string full_factory_name;
    typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
    threading_auto_ptr threading_policy(0);
    // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
                               
    int i = 0;
    
    <%if (getOccurrences(self) > 0){%>
    try {
        full_name_domain_name =  xml_data->getFrameworkAttributeValue
            ("<%self%>", i, "domain-name");
        // TODO: If full_name_domain_name and factory_name_domain_name are empty use
        // default name domain
        full_factory_name = xml_data->getFrameworkAttributeValue
            ("<%self%>", i, "full-factory-name");
        
        <%if (getThreading(self) == "SERIALIZE"){%>
        threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
        threading_policy = thrdptr;
        // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
        <%}else if (getThreading(self) == "MULTITHREAD"){%>
        std::string xmlThreadingPolicy("");
        std::string xmlThreadingAttribute("");
        std::string xmlThreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute
            ("<%self%>", i, xmlThreadingPolicy, xmlThreadingAttribute);
                          
        if (!xmlThreadingPolicy.compare("serialize")) {
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
            threading_policy = thrdptr; 
            // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
        } else {
            //threadin_policy shall not be NULL
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
            threading_policy = thrdptr; 
        }
        <%}else{%>
        <%abort("ERROR UNKOWN THREADING POLICY: <%getThreading(self)%> (shouldn't happen)")%>
        <%}%>
           
    } catch (const std::bad_alloc & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const OutOfMemoryException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::BadParameterException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    }
       
    typedef NamedObjectsFactory_initUtil Util;
    typedef NamedObjectsFactoryAllocatorBase AllocatorType;
    
    // Get the factory Allocator
    // TODO: Use a user-provided name rather than a naming convention!
    AllocatorType & allocator =
       <%get("factoryClassname")%>_Allocator::TheAllocator;
    
    try {
        Util::setup_named_object_factory(allocator,
                                         orb,           // ORB
                                         factories_poa.in(),    // PARENT POA
                                         repository.in(),
                                         root_context,
                                         full_name_domain_name,
                                         full_factory_name,
                                         "", // TODO:
                                         *threading_policy,
                                         argc, argv
        );
    
    } catch (const Cdmw::OutOfMemoryException & ex) {
        std::string s ("Could not allocate memory for ");
        s += full_factory_name;
        s += "\n";
        s += ex.what();
       
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     s );
    } catch (const Cdmw::BadParameterException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::InternalErrorException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                     CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                     CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const CORBA::SystemException & ex) {
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                     ex );
    }
    <%}%>
}


<%--
  -- Generate code for UnmanagedEntity lifecycle-framework
  -- This script is supposed to be called with the lifecycle-framework id as self.
  -- And context should contain a entry for "factoryClassname"
  --  
--%>
<%script type="EObject" name="genUnmanagedEntityAllocatorInit"%>
// *****************************************************************
// ****** <%get("factoryClassname")%>  ******
// *****************************************************************
{
    std::string full_name_domain_name;
    std::string full_factory_name;
    
    typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
    threading_auto_ptr threading_policy(0);
    // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
    
    int i = 0;
                  

    <%if (getOccurrences(self) > 0){%>
    try {
        full_name_domain_name = xml_data->getFrameworkAttributeValue
            ("<%self%>", i, "domain-name");
        // TODO: If full_name_domain_name and factory_name_domain_name are empty use
        // default name domain
        full_factory_name = xml_data->getFrameworkAttributeValue
                            ("<%self%>", i, "full-factory-name");

        <%if (getThreading(self) == "SERIALIZE"){%>
        threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
        threading_policy = thrdptr;
        // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
        <%}else if (getThreading(self) == "MULTITHREAD"){%>
        std::string xmlThreadingPolicy("");
        std::string xmlThreadingAttribute("");
        std::string xmlThreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute
            ("<%self%>", i, xmlThreadingPolicy, xmlThreadingAttribute);
        
        if (!xmlThreadingPolicy.compare("serialize")) {
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
            threading_policy = thrdptr;
            // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
        } else {
            //threadin_policy shall not be NULL
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
            threading_policy = thrdptr; 
        }
        <%}else{%>
        <%abort("ERROR UNKOWN THREADING POLICY: <%getThreading(self)%> (shouldn't happen)")%>
        <%}%>

    } catch (const std::bad_alloc & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const OutOfMemoryException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::BadParameterException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    }

    typedef NamedObjectsFactory_initUtil Util;
    typedef NamedObjectsFactoryAllocatorBase AllocatorType;
    
    // Get the factory Allocator
    // TODO: Use a user-provided name rather than a naming convention!
    AllocatorType & allocator =
        <%get("factoryClassname")%>_Allocator::TheAllocator;
    
    try {
        Util::setup_named_object_factory(allocator,
                                         orb,           // ORB
                                         factories_poa.in(),    // PARENT POA
                                         repository.in(),
                                         root_context,
                                         full_name_domain_name,
                                         full_factory_name,
                                         "", // TODO:
                                         *threading_policy,
                                         argc, argv
        );
    
    } catch (const Cdmw::OutOfMemoryException & ex ) {
        std::string s ("Could not allocate memory for ");
        s += full_factory_name;
        s += "\n";
        s += ex.what();
    
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     s );
    } catch (const Cdmw::BadParameterException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::InternalErrorException & ex ){
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                     CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                     CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const CORBA::SystemException & ex ) {
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                     ex );
    }
    <%}%>
}


<%--
  -- Generate code for Session lifecycle-framework
  -- This script is supposed to be called with the lifecycle-framework id as self.
  -- And context should contain a entry for "factoryClassname"
  --  
--%>
<%script type="EObject" name="genSessionAllocatorInit"%>
// *****************************************************************
// ****** <%get("factoryClassname")%>  ******
// *****************************************************************
{
    std::string full_factory_name;
   
    typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
    threading_auto_ptr threading_policy(0);
    // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
     
    int i = 0;
                      
    <%if (getOccurrences(self) > 0){%>
    try {
        full_factory_name = xml_data->getFrameworkAttributeValue
            ("<%self%>", i, "full-factory-name");
                          
        <%if (getThreading(self) == "SERIALIZE"){%>
        threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
        threading_policy = thrdptr;
        // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
        <%}else if (getThreading(self) == "MULTITHREAD"){%>
        std::string xmlThreadingPolicy("");
        std::string xmlThreadingAttribute("");    
        std::string xmlTreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute
            ("<%self%>", i, xmlThreadingPolicy, xmlThreadingAttribute);
        
        if (!xmlThreadingPolicy.compare("serialize")) {
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
            threading_policy = thrdptr; 
            // threading_policy = threading_auto_ptr(new Cdmw::OrbSupport::SingleThreadPolicy());
        } else {
            //threadin_policy shall not be NULL
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
            threading_policy = thrdptr; 
        }
        <%}else{%>
        <%abort("ERROR UNKOWN THREADING POLICY: <%getThreading(self)%> (shouldn't happen)")%>
        <%}%>

    } catch (const std::bad_alloc & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const OutOfMemoryException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::BadParameterException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );       
    } catch (const Cdmw::CdmwInit::XMLErrorException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    }

    typedef AnonymousObjectsFactory_initUtil Util;
    typedef AnonymousObjectsFactoryAllocatorBase AllocatorType;
    
    // Get the factory Allocator
    // TODO: Use a user-provided name rather than a naming convention!
    AllocatorType & allocator =
        <%get("factoryClassname")%>_Allocator::TheAllocator;
    
    try {
        Util::setup_anonymous_object_factory(allocator,
                                             orb,           // ORB
                                             factories_poa.in(),    // PARENT POA
                                             repository.in(),
                                             root_context,
                                             full_factory_name,
                                              "", // TODO:
                                             *threading_policy,
                                             argc, argv
        );
     
    } catch (const Cdmw::OutOfMemoryException & ex ) {
        std::string s ("Could not allocate memory for ");
        s += full_factory_name;
        s += "\n";
        s += ex.what();
    
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     s );
    } catch (const Cdmw::BadParameterException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::InternalErrorException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                     CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                     CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const CORBA::SystemException & ex) {
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                     ex );
    }
    <%}%>
}


<%--
  -- Generate code for Service lifecycle-framework
  -- This script is supposed to be called with the lifecycle-framework id as self.
  -- And context should contain a entry for "factoryClassname"
  --  
--%>
<%script type="EObject" name="genServiceAllocatorInit"%>
// *****************************************************************
// ****** <%get("factoryClassname")%>  ******
// *****************************************************************
{
    std::string full_factory_name;
   
    typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
    threading_auto_ptr threading_policy(0);
    // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
                              
    int i = 0;

    <%if (getOccurrences(self) > 0){%>
    try {
        full_factory_name = xml_data->getFrameworkAttributeValue
            ("<%self%>", i, "full-factory-name");
                          
        <%if (getThreading(self) == "SERIALIZE"){%>
        threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
        threading_policy = thrdptr;
        // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
        <%}else if (getThreading(self) == "MULTITHREAD"){%>
        std::string xmlThreadingPolicy("");
        std::string xmlThreadingAttribute("");
        std::string xmlThreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute
            ("<%self%>", i, xmlThreadingPolicy, xmlThreadingAttribute);
        
        if (!xmlThreadingPolicy.compare("serialize")) {
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
            threading_policy = thrdptr; 
            // threading_policy = threading_auto_ptr(new SingleThreadPolicy());
        } else {
            //threadin_policy shall not be NULL
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
            threading_policy = thrdptr; 
        }
        <%}else{%>
        <%abort("ERROR UNKOWN THREADING POLICY: <%getThreading(self)%> (shouldn't happen)")%>
        <%}%>

    } catch (const std::bad_alloc & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const OutOfMemoryException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::BadParameterException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    }

    typedef AnonymousObjectsFactory_initUtil Util;
    typedef AnonymousObjectsFactoryAllocatorBase AllocatorType;
    
    // Get the factory Allocator
    // TODO: Use a user-provided name rather than a naming convention!
    AllocatorType & allocator =
        <%get("factoryClassname")%>_Allocator::TheAllocator;
    
    try {
        Util::setup_anonymous_object_factory(allocator,
                                             orb,           // ORB
                                             factories_poa.in(),    // PARENT POA
                                             repository.in(),
                                             root_context,
                                             full_factory_name,
                                             "", // TODO:
                                             * threading_policy,
                                             argc, argv
        );
     
    } catch (const Cdmw::OutOfMemoryException & ex ) {
        std::string s ("Could not allocate memory for ");
        s += full_factory_name;
        s += "\n";
        s += ex.what();
        
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     s );
    } catch (const Cdmw::BadParameterException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::InternalErrorException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                     CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                     CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const CORBA::SystemException & ex ) {
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                     ex );
    }
    <%}%>
}


<%--
  -- Generate code for User Defined Factory lifecycle-framework
  -- This script is supposed to be called with the lifecycle-framework id as self.
  -- And context should contain a entry for "factoryClassname"
  --  
--%>
<%script type="EObject" name="genUserDefinedFactoryAllocatorInit"%>
// *****************************************************************
// ******                User defined factory                 ******
// *****************************************************************
{
    std::string full_factory_name;
    <%if (isNamedFactory(self)){%>
    std::string full_name_domain_name;
    <%}%>
    typedef std::auto_ptr<Cdmw::OrbSupport::POAThreadingPolicy> threading_auto_ptr;
    threading_auto_ptr threading_policy(0);
    // Cdmw::LifeCycle::POAThreadingPolicy * threading_policy = 0;
    
    int i = 0;

    <%if (getOccurrences(self) > 0){%>
    try {
        <%if (isNamedFactory(self)){%>
        full_name_domain_name =  xml_data->getFrameworkAttributeValue
            ("<%self%>", i, "domain-name");
        // TODO: If full_name_domain_name and factory_name_domain_name are empty use
        // default name domain
        full_factory_name = xml_data->getFrameworkAttributeValue
            ("<%self%>", i, "full-factory-name");
        <%}else{%>
        full_factory_name = xml_data->getFrameworkAttributeValue
            ("<%self%>", i, "full-factory-name");
        <%}%>

        <%if (getThreading(self) == "SERIALIZE"){%>
        threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
        threading_policy = thrdptr;
        // threading_policy = threading_auto_ptr(new Cdmw::OrbSupport::SingleThreadPolicy());
        <%}else if (getThreading(self) == "MULTITHREAD"){%>
        std::string xmlThreadingPolicy("");
        std::string xmlThreadingAttribute("");
        std::string xmlThreadingAttributeValue = xml_data->getFrameworkMultithreadAttribute
            ("<%self%>", i, xmlThreadingPolicy, xmlThreadingAttribute);
        
        if (!xmlThreadingPolicy.compare("serialize")) {
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::SingleThreadPolicy());
            threading_policy = thrdptr; 
            // threading_policy = threading_auto_ptr(new Cdmw::OrbSupport::SingleThreadPolicy());
        } else {
            //threadin_policy shall not be NULL
            threading_auto_ptr thrdptr(new Cdmw::OrbSupport::ThreadPerConnectionPolicy());
            threading_policy = thrdptr; 
        }
        <%}else{%>
        <%abort("ERROR UNKOWN THREADING POLICY: <%getThreading(self)%> (shouldn't happen)")%>
        <%}%>

    } catch (const std::bad_alloc & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const OutOfMemoryException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::BadParameterException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    }

    <%if (isNamedFactory(self)){%>
    typedef NamedObjectsFactory_initUtil Util;
    typedef NamedObjectsFactoryAllocatorBase AllocatorType;
    <%}else{%>
    typedef AnonymousObjectsFactory_initUtil Util;
    typedef AnonymousObjectsFactoryAllocatorBase AllocatorType;
    <%}%>

    // Get the factory Allocator
    // TODO: Use a user-provided name rather than a naming convention!
    AllocatorType & allocator =
        <%getAllocatorClassname(self).withCppScope%>::TheAllocator;
    
    try {
        Util::setup_<%namedPrefix%>_object_factory(allocator,
            orb,           // ORB
            factories_poa.in(),    // PARENT POA
            repository.in(),
            root_context,
            <%if (isNamedFactory(self)){%>
            full_name_domain_name,
            <%}%>
            full_factory_name,
            "", // TODO:
            *threading_policy,
            argc, argv
        );
    
    } catch (const Cdmw::OutOfMemoryException & ex ) {
        std::string s ("Could not allocate memory for ");
        s += full_factory_name;
        s += "\n";
        s += ex.what();
        
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     s );
    } catch (const Cdmw::BadParameterException & ex) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMLifeCycleFrameworkInit,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const Cdmw::InternalErrorException & ex ) {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                     CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                     CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const CORBA::SystemException & ex ) {
        CDMW_THROW1( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::SystemException>,
                     ex );
    }
    <%}%>
}



<%script type="EObject" name="namedPrefix" post="trim()"%>
<%if (isNamedFactory(self)){%>
named
<%}else{%>
anonymous
<%}%>



<%script type="EObject" name="printFrameworkData"%>
// ***********************************************************
// Framework: <%self%>
//    occurences     <%getOccurrences(self)%>
//    idl-interface  <%getFactoryIdlInterface(self)%>
//    isHome         <%isHome(self)%>
//    isNamed        <%isNamedFactory(self)%>
//    homeDef        <%getHome(self)%>
//    factoryClassname   <%getFactoryClassname(self)%>
//    allocatorClassname <%getAllocatorClassname(self)%>
//    category       <%getCategory(self)%>
//    threading      <%getThreading(self)%>
// ***********************************************************
