<%
metamodel http://www.prismtech.com/idlmm
%>


<%script type="idlmm.Repository" name="initRecordingService"%>
// *****************************************************************
// **********  register initial references for Recording ***********
// *****************************************************************
{
    CDMW_ASSERT(!CORBA::is_nil(local_NameService.in()));
    Cdmw::CommonSvcs::Naming::NamingInterface local_ni(local_NameService.in());
    
    unsigned long callTimeout_millisec = 0;
    try {
        std::string res = 
            xml_data->getServiceAttributeValue("recording", "recorder-timeout"); 
        callTimeout_millisec = atoi(res.c_str());
    } catch (const Cdmw::BadParameterException& ex) {
       CDMW_DEBUG( InitUtils::Get_Logger(),
                   "recorder-timeout was not found. Call timeout set to unlimited." );
    }
    
    // Register initial reference for the recording daemon
    //
    CDMW_DEBUG(InitUtils::Get_Logger(), 
               "CDMW_init: register initial reference of the recording daemon");
    CORBA::Object_var daemon_obj;
    try {
        daemon_obj = local_ni.resolve(CdmwRecording::REC_CONTAINER_MNGR_ID);
    } catch (const CosNaming::NamingContext::NotFound& nf) {
        std::ostringstream ostr;
        ostr << "Could not retrieve reference of the Recording Daemon."
             << "'" << CdmwRecording::REC_CONTAINER_MNGR_ID << "' not found in the naming.";
        CDMW_FATAL(InitUtils::Get_Logger(), ostr.str());
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAM,
                                      CORBA::COMPLETED_NO), ostr.str());
    }
    if (CORBA::is_nil(daemon_obj.in())) {
        std::ostringstream ostr;
        ostr << "Nil Object Reference for '" << CdmwRecording::REC_CONTAINER_MNGR_ID << "'.";
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNilObjectReference,
                                      CORBA::COMPLETED_NO), ostr.str());
    }
    try {
        orb->register_initial_reference(CdmwRecording::REC_CONTAINER_MNGR_ID, daemon_obj.in());
    } catch (const CORBA::ORB::InvalidName& )  {
        std::ostringstream ostr;
        ostr << "InvalidName => Cannot register '" << CdmwRecording::REC_CONTAINER_MNGR_ID 
             <<  "'.";
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                     CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                     ostr.str());
    } catch (const CORBA::SystemException& ex) {
        CDMW_FATAL(InitUtils::Get_Logger(), 
                   "CDMW_init: register_initial_reference of " << 
                   CdmwRecording::REC_CONTAINER_MNGR_ID << " failed.");
        throw;
    }
    // 
    try {
        daemon_obj = local_ni.resolve(CdmwRecording::REC_DAEMON_CTRL_ID);
        if (CORBA::is_nil(daemon_obj.in())) {
            std::ostringstream ostr;
            ostr << "Nil Object Reference for '" << CdmwRecording::REC_DAEMON_CTRL_ID << "'.";
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNilObjectReference,
                                          CORBA::COMPLETED_NO), ostr.str());
        }
    } catch (const CosNaming::NamingContext::NotFound& nf) {
        std::ostringstream ostr;
        ostr << "Could not retrieve reference of the Recording Daemon."
             << "'" << CdmwRecording::REC_DAEMON_CTRL_ID << "' not found in the naming.";
        CDMW_FATAL(InitUtils::Get_Logger(), ostr.str());
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAM,
                                      CORBA::COMPLETED_NO), ostr.str());
    }
    
    try {
        CdmwRecording::DaemonControl_var daemon = 
            CdmwRecording::DaemonControl::_narrow(daemon_obj.in());
        
        CORBA::String_var url__ = daemon->recorder_url();
        std::string recording_server_url(url__.in());
        CDMW_DEBUG(InitUtils::Get_Logger(), 
                   "CDWM_init: recording server URL  = " << recording_server_url);
        if ( recording_server_url.empty() ) {
            CDMW_WARN(InitUtils::Get_Logger(), 
                      "CDMW_init: No Recording server url defined! Thus register_initial_reference() is not perfomed for the Recording server."); 
        } else {
            //
            // Register initial reference for the recording server
            //
            CDMW_DEBUG(InitUtils::Get_Logger(), 
                       "CDMW_init: register initial reference of the recording server");
            
            CORBA::Object_var recording_server_obj = orb->string_to_object(recording_server_url.c_str());
            if (CORBA::is_nil(recording_server_obj.in())) {
                std::ostringstream ostr;
                ostr << "Nil Object Reference for 'RecordingServer' url='" 
                     << recording_server_url << "'.";
                CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                             CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMNilObjectReference,
                                              CORBA::COMPLETED_NO), ostr.str());
            }
            // Use SyncScope and relative roundtrip policies to achieve reliable oneways
                    
            CORBA::Object_var capsule =          
                set_recorder_policies(orb, 
                                      recording_server_obj.in(),
                                      callTimeout_millisec);
            orb->register_initial_reference("RecordingServer", capsule.in());
        }
    } catch (const CORBA::ORB::InvalidName& )  {
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::INTERNAL>,
                     CORBA::INTERNAL(Cdmw::OrbSupport::INTERNAL, CORBA::COMPLETED_NO),
                     "InvalidName => Cannot register 'RecordingServer'.");
    } catch (const CORBA::SystemException& ex) {
        CDMW_FATAL(InitUtils::Get_Logger(), "CDMW_init: SystemException raised. register_initial_reference of RecordingServer not performed. "
                   << ex);
        throw;
    }
}

