<%
metamodel http://www.prismtech.com/idlmm
%>

<%script type="idlmm.Repository" name="initTraceService"%>
#ifdef CDMW_TRACES_ENABLED

// *****************************************************************
// *********         Init The CDMW Trace Library           *********
// *****************************************************************
{
    CDMW_DEBUG(InitUtils::Get_Logger(), "Init Trace Library");
    std::string res;
    size_t flushing_time = 0;
    size_t msg_threshold = 0;
    size_t nb_FlushArea = 0;
    size_t size_FlushArea = 0;
    std::string collectorName;
    std::vector<std::string> collectorNameList;
    
    try {
        try {
            res = xml_data->getServiceAttributeValue("trace", "flushing-time"); //FlushAreaMngr::DEFAULT_FLUSHING_TIME;
            flushing_time = atoi(res.c_str());
            res = xml_data->getServiceAttributeValue("trace", "flush-area-nb"); //FlushAreaMngr::DEFAULT_AREA_NB;
            nb_FlushArea = atoi(res.c_str());
            res = xml_data->getServiceAttributeValue("trace", "flush-area-sz"); //FlushAreaMngr::DEFAULT_AREA_SIZE;
            size_FlushArea = atoi(res.c_str());
            collectorName = xml_data->getServiceAttributeValue("trace", "collector-name");
            collectorNameList.push_back(collectorName);
        }
        catch (const Cdmw::BadParameterException& ex) {
            CDMW_WARN(InitUtils::Get_Logger(), "Cdmw::BadParameterException received: "
                      << ex.what());
            CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                         CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                          CORBA::COMPLETED_NO),
                         ex.what() );
        }
        
        try  {
            res = xml_data->getServiceAttributeValue("trace", "msg-threshold"); //FlushAreaMngr::DEFAULT_MSG_THRESHOLD;
            msg_threshold = atoi(res.c_str());
        
        } catch (const Cdmw::BadParameterException& ) {
            // do nothing : parameter is optional
        }
    } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
        CDMW_WARN(InitUtils::Get_Logger(), "CdmwInit::XMLErrorException received: "
                  << ex.what());
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const OutOfMemoryException & ex) {
        CDMW_WARN(InitUtils::Get_Logger(), "Cdmw::OutOfMemoryException received: "
                  << ex.what());
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    }
    
    // May raise CORBA::SystemException
    Trace::InitUtils::init_trace_library(orb,
                                         CDMW_rootPOA.in(),
                                         application_name,
                                         process_name,
                                         flushing_time,
                                         msg_threshold,
                                         nb_FlushArea,
                                         size_FlushArea,
                                         collectorNameList);
    
    CDMW_TRACE_ACTIVE_FLUSHING();
}

#endif // CDMW_TRACES_ENABLED


<%script type="idlmm.Repository" name="stopTraceService"%>
#ifdef CDMW_TRACES_ENABLED

// Free all allocated memory and stop the trace service
try {
    Cdmw::Trace::FlushAreaMngr::cleanup();
} catch (const Cdmw::InternalErrorException & ) {
    // Do nothing (ignore)
    // TODO: Check if exception has been removed from cleanup() specification.
}
         
#endif // CDMW_TRACES_ENABLED



<%script type="idlmm.Repository" name="initLogManagerCollectorDelegate"%>
// *****************************************************************
// *********   Init The CDMW LogManagerCollectorDelegate   *********
// *****************************************************************
{
    CDMW_DEBUG(InitUtils::Get_Logger(), "Init LogManagerCollectorDelegate");
    size_t msg_threshold = 0; // NOTA: Not used in this version
    std::string collectorName;
    try {
        try {
            collectorName = xml_data->getServiceAttributeValue("trace", "collector-name");
        }
        catch (const Cdmw::BadParameterException& ex) {
            CDMW_DEBUG(InitUtils::Get_Logger(), "Cdmw::BadParameterException received: "
                       << ex.what());
        }
        
        try  {
            std::string res = xml_data->getServiceAttributeValue("trace", "msg-threshold"); //FlushAreaMngr::DEFAULT_MSG_THRESHOLD;
            msg_threshold = atoi(res.c_str());
        
        } catch (const Cdmw::BadParameterException& ) {
            // do nothing : parameter is optional
        }
    } catch (const Cdmw::CdmwInit::XMLErrorException & ex) {
        CDMW_WARN(InitUtils::Get_Logger(), "CdmwInit::XMLErrorException received: "
                  << ex.what());
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::BAD_PARAM>,
                     CORBA::BAD_PARAM(Cdmw::OrbSupport::BAD_PARAMInvalidXMLInitData,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    } catch (const OutOfMemoryException & ex) {
        CDMW_WARN(InitUtils::Get_Logger(), "Cdmw::OutOfMemoryException received: "
                  << ex.what());
        CDMW_THROW2( Cdmw::OrbSupport::CORBASystemExceptionWrapperT<CORBA::NO_MEMORY>,
                     CORBA::NO_MEMORY(Cdmw::OrbSupport::NO_MEMORY,
                                      CORBA::COMPLETED_NO),
                     ex.what() );
    }
    // We have a collector name now. Let's retreive it from PlatformDaemon (NameServer).
    // Use collectorName if provided by user, otherwise use default one.
    if (collectorName.empty()) {
        collectorName = Cdmw::Common::Locations::DEFAULT_LOG_COLLECTOR_ID;
    }
    CdmwLogging::LogCollector_var log_collector__ =
        CdmwLogging::LogCollector::_nil();
    if (CORBA::is_nil(local_NameService.in()))
    {
        CDMW_WARN(InitUtils::Get_Logger(), "No Local NameService available. "
                  << "Trace logs will not be sent to Trace Collector.");
    } else {
        Cdmw::CommonSvcs::Naming::NamingInterface local_ni(local_NameService.in());
        CORBA::Object_var obj__ = CORBA::Object::_nil();
        try {
            obj__ = local_ni.resolve(collectorName);
            log_collector__ = CdmwLogging::LogCollector::_narrow(obj__.in());
            if (CORBA::is_nil(log_collector__.in())) {
                CDMW_WARN(InitUtils::Get_Logger(), 
                          "'" << collectorName << "' does not denote a valid "
                          << "Log Collector object. "
                          << "Trace logs will not be sent to Trace Collector.");
            }
        } catch (const CosNaming::NamingContext::InvalidName& ex) {
            CDMW_WARN(InitUtils::Get_Logger(), 
                      "Could not retrieve reference of the Log Collector "
                      << "'" << collectorName << "' not a valid name. "
                      << "Trace logs will not be sent to Trace Collector.");
        } catch (const CosNaming::NamingContext::NotFound& nf) {
            CDMW_WARN(InitUtils::Get_Logger(), 
                      "Could not retrieve reference of the Log Collector "
                      << "'" << collectorName << "' not found in the naming. "
                      << "Trace logs will not be sent to Trace Collector.");
        }
    }
    if (!CORBA::is_nil(log_collector__.in())) {
        try {
            using namespace Cdmw::Logging;
            LogManagerCollectorDelegate * delegate__ =
                new LogManagerCollectorDelegate(log_collector__.in());
            LogManagerDelegate* d__ = 
                LogManager::Instance()->register_alternate_delegate(delegate__);
            if (d__) delete d__;
        } catch (const Cdmw::OutOfMemoryException& ex) {
            CDMW_WARN(InitUtils::Get_Logger(), "Receive Cdmw::OutOfMemoryException "
                      << ex.what() << ". Trace logs will not be sent to Trace Collector.");
        }
    }
}



<%script type="idlmm.Repository" name="stopLogManagerCollectorDelegate"%>
// Release LogCollectorDelegate
{
    using namespace Cdmw::Logging;
    LogManagerDelegate * d__ = LogManager::Instance()->get_alternate_delegate();
    if (d__) {
        LogManagerCollectorDelegate * delegate__ = dynamic_cast<LogManagerCollectorDelegate*>(d__);
        CDMW_ASSERT(delegate__!=0);
        delete d__;
    }
}
