<%
metamodel http://www.prismtech.com/idlmm

import com.thalesgroup.cdmw.codegenerator.lifecycle.LifeCycleServices
import com.thalesgroup.cdmw.codegenerator.lifecycle.ModelServices
import com.thalesgroup.cdmw.codegenerator.lifecycle.Utils
import com.thalesgroup.cdmw.codegenerator.lifecycle.cpp.Utils
import com.thalesgroup.cdmw.codegenerator.lifecycle.cpp.ServantProvider_hpp
import com.thalesgroup.ccm.codegenerator.corba.CommonServices
import com.thalesgroup.ccm.codegenerator.corba.cpp.Utils
import com.thalesgroup.ccm.codegenerator.corba.cpp.CppMappingServices
import com.thalesgroup.ccm.codegenerator.container.Utils
%>


<%--
  -- Generation template  
  -- This template is not model-based (idlmm.Repository is just specified
  -- to access InterfaceDef data)
  -- All is based on LifeCycleServices which provides access to loaded data
  -- from XML file.
  --%>
<%script type="idlmm.Repository" name="ServiceFactory_cpp" file="<%factoryCppFilePath%>"%>
<%copyrightHeader%>


#include "<%factoryHppFilePath%>"

#include <LifeCycle/lifecycle/ServantProviderAllocator.hpp>

#include "<%servantProviderHppFilePath%>"
#include "<%userServantProviderAllocatorFilename%>"

#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <LifeCycle/lifecycle/ObjectUtil.hpp>
#include <Foundation/common/Assert.hpp>

#include <typeinfo>


namespace
{
    // ----------------------------------------------------------------------
    // Resource Servant locator
    // ----------------------------------------------------------------------

    class TheServantLocator  : 
        public virtual PortableServer::ServantLocator,
        public virtual Cdmw::OrbSupport::RefCountLocalObject
    {
        public:
            TheServantLocator (Cdmw::LifeCycle::ServantProviderBase * provider)
                : m_servant_provider(provider) 
            {
                CDMW_ASSERT(provider != 0);
            }
            
            ~TheServantLocator () {
                <%servantProviderAllocatorClassname.withCppScope%>::TheAllocator.dispose(m_servant_provider);
                m_servant_provider = 0;
            }
    
            PortableServer::Servant 
            preinvoke(const PortableServer::ObjectId& oid,
                      PortableServer::POA_ptr poa,
                      const char* operation,
                      PortableServer::ServantLocator::Cookie&
                      the_cookie)
                throw(CORBA::SystemException) 
            {
                CDMW_ASSERT(m_servant_provider != 0);                      
                return m_servant_provider->lookup_servant(oid);
            }
            
            void 
            postinvoke(const PortableServer::ObjectId& oid,
                       PortableServer::POA_ptr poa,
                       const char* operation,
                       PortableServer::ServantLocator::Cookie the_cookie,
                       PortableServer::Servant the_servant)
                throw(CORBA::SystemException) 
            {
                CDMW_ASSERT(m_servant_provider != 0);
                m_servant_provider -> release_servant (oid, poa, the_servant);
            }

        private:        
            Cdmw::LifeCycle::ServantProviderBase * m_servant_provider;
    };  // End TheServantLocator
};  // End anonymous namespace


<%openNamespace(factoryImplClassnameScope.withCppScope)%>

    <%factoryImplClassname%>::<%factoryImplClassname%>(
            CORBA::ORB_ptr                           orb,
            PortableServer::POA_ptr                  parent,
            const std::string &                      factory_name,
            Cdmw::OrbSupport::POAThreadingPolicy &   threading_policy,
            int & argc, char** argv)
        throw(Cdmw::OutOfMemoryException,
              Cdmw::BadParameterException,
              Cdmw::InternalErrorException,
              CORBA::SystemException)
        : m_orb(CORBA::ORB::_duplicate(orb)), 
          m_poa(PortableServer::POA::_duplicate(parent)),
          m_objects_poa(PortableServer::POA::_nil()),
          m_factory_name(factory_name),
          m_object_deactivator(orb),
          m_servant_provider(0)
    {
        //
        // Create POA for use by the factory
        //
        // Service Life cycle framework use the following POA policies
        //
        // Lifespan policy            = TRANSIENT
        // Id Assignment policy       = USER_ID
        // Id Uniqueness policy       = MULTIPLE_ID ** Not Relevant
        // Implicit Activation policy = NO_IMPLICIT_ACTIVATION
        // Request Processing policy  = USE_SERVANT_MANAGER;
        // Servant Retention policy   = NO_RETAIN;
        //
        PortableServer::POAManager_var poaManager = parent -> the_POAManager();
        CORBA::PolicyList policies;
        policies.length(6);
        policies[0] =
            parent -> create_id_assignment_policy(PortableServer::USER_ID);
        policies[1] =
            parent -> create_lifespan_policy(PortableServer::TRANSIENT);
        policies[2] =
            parent -> create_servant_retention_policy(PortableServer::NON_RETAIN);
        // policies[3] =
        //    parent -> create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
        policies[3] = parent ->
            create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
        policies[4] = parent ->
            create_implicit_activation_policy(PortableServer::NO_IMPLICIT_ACTIVATION);

        Cdmw::OrbSupport::StrategyList poaStrategies;

        <%if (isMultithread){%>
        // Strategies with MULTITHREAD policy will accept Single Thread and
        // Multi Thread strategies. However, SERIALIZE policy will only allow
        // Single Thread policy.
        Cdmw::OrbSupport::SingleThreadPolicy * single_thread
            = dynamic_cast< Cdmw::OrbSupport::SingleThreadPolicy *>(&threading_policy);
        if (single_thread == NULL)
        {
            // Multi Thread strategies
            policies[5] = parent ->
                create_thread_policy(PortableServer::ORB_CTRL_MODEL);
        }
        else
        {
            policies[5] = parent ->
                create_thread_policy(PortableServer::SINGLE_THREAD_MODEL);
        }
        
        threading_policy.append_to_StrategyList(poaStrategies);
        <%}else{%>
        // For Single Thread policy, first check validity of 'threading_policy' using 
        // the code bellow :
        // Begin Threading policy check
        
        policies[5] = parent ->
            create_thread_policy(PortableServer::SINGLE_THREAD_MODEL);
        // Check Multi-Threading Policy
        {
            using namespace Cdmw::OrbSupport;
            try {
                SingleThreadPolicy & single_thread
                    = dynamic_cast<SingleThreadPolicy &>(threading_policy);
                single_thread.append_to_StrategyList(poaStrategies);
            } catch (const std::bad_cast &) {
                // Wrong policy
                std::string parameter("Cdmw::OrbSupport::POAThreadingPolicy");
                parameter += "(";
                parameter += factory_name;
                parameter += ")";
                // This is to avoid Assertion failure in debug mode as internal ref count
                // equals 1 (CORBA C++ mapping oblige!)
                _set_ref_count(0);
                CDMW_THROW2( BadParameterException, parameter, typeid(threading_policy).name() );
            }
        }
        // End Threading policy check
        <%}%>

        try
        {
            m_objects_poa = Cdmw::OrbSupport::OrbSupport::create_POA(m_poa.in(),
                                                                     m_factory_name.c_str(),
                                                                     poaManager.in(), 
                                                                     policies,
                                                                     poaStrategies);
        
            //
            // Create and install servant provider
            //
            Cdmw::LifeCycle::ServantProviderBase * srvprvd 
                = <%servantProviderAllocatorClassname.withCppScope%>::TheAllocator.allocate
                    (orb,
                     m_objects_poa.in(),
                     m_object_deactivator,
                     argc, argv);
            m_servant_provider = dynamic_cast< <%servantProviderFullClassname%>* >(srvprvd);
            CDMW_ASSERT(m_servant_provider != NULL);

            // m_servant_provider->set_cleanup_notifier
            //    (new Cdmw::LifeCycle::CleanupNotifier(m_domain.in()));

            //
            // Create and install servant activator
            //
            PortableServer::ServantLocator_var the_locator
                = new TheServantLocator(m_servant_provider);
            m_objects_poa -> set_servant_manager(the_locator.in());
    
        } catch (const std::bad_alloc & ) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            CDMW_THROW(Cdmw::OutOfMemoryException);
        } catch (const BadParameterException & ) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw;
        } catch (const Cdmw::InternalErrorException & ) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw; 
        } catch (const PortableServer::POA::AdapterAlreadyExists &) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                  CORBA::COMPLETED_YES);
        } catch (const PortableServer::POA::InvalidPolicy &) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                  CORBA::COMPLETED_YES);
        } catch (const CORBA::SystemException &) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw;
        }
    }

    <%factoryImplClassname%>::~<%factoryImplClassname%>()
        throw()
    {
        // <%servantProviderAllocatorClassname.withCppScope%>::TheAllocator.dispose(m_servant_provider);

        // m_objects_poa and m_servant_locator references will be released by their destructors;
        // and the associated POA and ServantLocator implementations will be destroyed following
        // the destruction of the parent POA.
    }

    <%--                
      Create the operation declarations.
    --%>
    <%for (idlInterface.findAllFactoryOperations){%>
    <%generateOperation%>
    
    <%}%>

    //
    // IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/supports:1.0
    //
    CORBA::Boolean 
    <%factoryImplClassname%>::supports(const char* repository_id)
        throw(CORBA::SystemException)
    {
        return CORBA::Boolean(m_servant_provider->supports(repository_id));
    }

    //
    // IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/get_supported_objects:1.0
    //
    CdmwLifeCycle::RepositoryIdSeq* 
    <%factoryImplClassname%>::get_supported_objects()
        throw(CORBA::SystemException)
    {
        return m_servant_provider->get_supported_objects();
    }

<%closeNamespace(factoryImplClassnameScope.withCppScope)%>





<%script type="idlmm.OperationDef" name="generateOperation"%>
<%checkCdmwLifeCycleKeyParameterForAnonymousFactory%>
<%type.asReturnParameterDef%> <%factoryImplClassname%>::<%cppName%>(<%parameters[mode == "PARAM_IN"].genParameterDef.sep(", ")%>)
throw ( 
       <%for (exceptions){%>
       <%cppFullClassname%>,
       <%}%>
       CORBA::SystemException
      )
{
    // [I] Acquire oid from user
    // Operation may throw user exceptions that will re-thrown to caller
    PortableServer::ObjectId_var oid 
         = m_servant_provider-><%cppName%> (
            <%parameters[mode == "PARAM_IN" && !type.isCdmwLifeCycleKey].cppName.sep(", ")%>
           );
        
    // [II] Create its object reference
    typedef Cdmw::LifeCycle::ObjectUtil< <%type.cppFullClassname%> > Utils;
    <%type.asLocalVariable%> ref
        = Utils::create_object_reference(m_objects_poa.in(), oid.in(), m_servant_provider);

    return ref<%type.localVarReturnSuffix%>;
}


