<%
metamodel http://www.prismtech.com/idlmm

import com.thalesgroup.cdmw.codegenerator.lifecycle.LifeCycleServices
import com.thalesgroup.cdmw.codegenerator.lifecycle.ModelServices
import com.thalesgroup.cdmw.codegenerator.lifecycle.Utils
import com.thalesgroup.cdmw.codegenerator.lifecycle.cpp.Utils
import com.thalesgroup.cdmw.codegenerator.lifecycle.cpp.ServantProvider_hpp
import com.thalesgroup.ccm.codegenerator.corba.CommonServices
import com.thalesgroup.ccm.codegenerator.corba.cpp.Utils
import com.thalesgroup.ccm.codegenerator.corba.cpp.CppMappingServices
import com.thalesgroup.ccm.codegenerator.container.Utils
%>


<%--
  -- Generation template  
  -- This template is not model-based (idlmm.Repository is just specified
  -- to access InterfaceDef data)
  -- All is based on LifeCycleServices which provides access to loaded data
  -- from XML file.
  --%>
<%script type="idlmm.Repository" name="UnmanagedEntityFactory_cpp" file="<%factoryCppFilePath%>"%>
<%copyrightHeader%>


#include "<%factoryHppFilePath%>"

#include <LifeCycle/lifecycle/ServantProviderAllocator.hpp>

#include "<%servantProviderHppFilePath%>"
#include "<%userServantProviderAllocatorFilename%>"

#include <Foundation/orbsupport/ExceptionMinorCodes.hpp>
#include <Foundation/orbsupport/OrbSupport.hpp>
#include <Foundation/orbsupport/RefCountLocalObject.hpp>
#include <LifeCycle/lifecycle/ObjectUtil.hpp>
#include <Foundation/common/Assert.hpp>

#include <typeinfo>


namespace
{
    // Maximum number of retries on receiving CORBA::TRANSIENT exception
    // for registering objects.
    const signed long MAX_RETRIES = 1;

    // ----------------------------------------------------------------------
    // Resource Servant activator
    // ----------------------------------------------------------------------

    class TheServantActivator  : 
        public virtual PortableServer::ServantActivator,
        public virtual Cdmw::OrbSupport::RefCountLocalObject
    {
        public:
            TheServantActivator (Cdmw::LifeCycle::ServantProviderBase * provider)
                : m_servant_provider(provider) 
            {
                CDMW_ASSERT(provider != 0);
            }
            
            ~TheServantActivator () {
                <%servantProviderAllocatorClassname.withCppScope%>::TheAllocator.dispose(m_servant_provider);
                m_servant_provider = 0;
            }
    
            virtual PortableServer::Servant 
            incarnate(const PortableServer::ObjectId& oid, 
                      PortableServer::POA_ptr poa)
                throw(PortableServer::ForwardRequest, CORBA::SystemException) 
            {
                CDMW_ASSERT(m_servant_provider != 0);    
                return m_servant_provider->lookup_servant(oid);
            }
    
            virtual void 
            etherealize(const PortableServer::ObjectId& oid,
                        PortableServer::POA_ptr poa,
                        PortableServer::Servant servant,
                        CORBA::Boolean cleanup_in_progress,
                        CORBA::Boolean remaining_activations)
                throw(CORBA::SystemException) 
            {
                CDMW_ASSERT(m_servant_provider != 0);
                if (!remaining_activations)
                {
                    m_servant_provider -> release_servant (oid, poa, servant);
                }
            }

        private:        
            Cdmw::LifeCycle::ServantProviderBase * m_servant_provider;
    };  
};


<%openNamespace(factoryImplClassnameScope.withCppScope)%>

    <%factoryImplClassname%>::<%factoryImplClassname%>(
            CORBA::ORB_ptr                           orb,
            PortableServer::POA_ptr                  parent,
            CdmwLifeCycle::ObjectRegistration_ptr    name_domain,
            const std::string &                      name_domain_name,
            const std::string &                      factory_name,
            Cdmw::OrbSupport::POAThreadingPolicy &   threading_policy,
            int & argc, char** argv)
        throw(Cdmw::OutOfMemoryException,
              Cdmw::BadParameterException,
              Cdmw::InternalErrorException,
              CORBA::SystemException)
        : m_orb(CORBA::ORB::_duplicate(orb)), 
          m_poa(PortableServer::POA::_duplicate(parent)),
          m_objects_poa(PortableServer::POA::_nil()),
          m_domain(CdmwLifeCycle::ObjectRegistration::_duplicate(name_domain)),
          m_domain_name(name_domain_name),
          m_factory_name(factory_name),
          m_object_deactivator(orb),
          m_servant_provider(0)
    {
        //
        // Create POA for use by the factory
        //
        // Entity Life cycle framework use the following POA policies
        //
        // Lifespan policy            = PERSISTENT
        // Id Assignment policy       = USER_ID
        // Id Uniqueness policy       = UNIQUE_ID
        // Implicit Activation policy = NO_IMPLICIT_ACTIVATION
        // Request Processing policy  = USE_SERVANT_MANAGER;
        // Servant Retention policy   = RETAIN;
        // 

        PortableServer::POAManager_var poaManager = parent -> the_POAManager();
        CORBA::PolicyList policies;
        policies.length(7);
        policies[0] =
            parent -> create_id_assignment_policy(PortableServer::USER_ID);
        policies[1] =
            parent -> create_lifespan_policy(PortableServer::PERSISTENT);
        policies[2] =
            parent -> create_servant_retention_policy(PortableServer::RETAIN);
        policies[3] =
            parent -> create_id_uniqueness_policy(PortableServer::UNIQUE_ID);
        policies[4] = parent ->
            create_request_processing_policy(PortableServer::USE_SERVANT_MANAGER);
        policies[5] = parent ->
            create_implicit_activation_policy(PortableServer::NO_IMPLICIT_ACTIVATION);
        
        Cdmw::OrbSupport::StrategyList poaStrategies;

        <%if (isMultithread){%>
        // Strategies with MULTITHREAD policy will accept Single Thread and
        // Multi Thread strategies. However, SERIALIZE policy will only allow
        // Single Thread policy.
        Cdmw::OrbSupport::SingleThreadPolicy * single_thread
            = dynamic_cast< Cdmw::OrbSupport::SingleThreadPolicy *>(&threading_policy);
        if (single_thread == NULL)
        {
            // Multi Thread strategies
            policies[6] = parent ->
                create_thread_policy(PortableServer::ORB_CTRL_MODEL);
        }
        else
        {
            policies[6] = parent ->
                create_thread_policy(PortableServer::SINGLE_THREAD_MODEL);
        }
        
        threading_policy.append_to_StrategyList(poaStrategies);
        <%}else{%>
        // For Single Thread policy, first check validity of 'threading_policy' using 
        // the code bellow :
        // Begin Threading policy check
        
        policies[6] = parent ->
            create_thread_policy(PortableServer::SINGLE_THREAD_MODEL);
        // Check Multi-Threading Policy
        {
            using namespace Cdmw::OrbSupport;
            try {
                SingleThreadPolicy & single_thread
                    = dynamic_cast<SingleThreadPolicy &>(threading_policy);
                single_thread.append_to_StrategyList(poaStrategies);
            } catch (const std::bad_cast &) {
                // Wrong policy
                std::string parameter("Cdmw::OrbSupport::POAThreadingPolicy");
                parameter += "(";
                parameter += factory_name;
                parameter += ")";
                CDMW_THROW2( BadParameterException, parameter, typeid(threading_policy).name() );
            }
        }
        // End Threading policy check
        <%}%>

        try
        {
            // BadParameterException, InternalErrorException
            m_objects_poa = Cdmw::OrbSupport::OrbSupport::create_POA(m_poa.in(),
                                                                     m_factory_name.c_str(),
                                                                     poaManager.in(), 
                                                                     policies,
                                                                     poaStrategies);
        
            //
            // Create and install servant provider
            //
            Cdmw::LifeCycle::ServantProviderBase * srvprvd 
                = <%servantProviderAllocatorClassname.withCppScope%>::TheAllocator.allocate
                    (orb,
                     m_objects_poa.in(),
                     m_object_deactivator,
                     argc, argv);
            m_servant_provider = dynamic_cast< <%servantProviderFullClassname%>* >(srvprvd);
            CDMW_ASSERT(m_servant_provider != NULL);

            // Hand Shaking with the Servant Provider
            // IDs shall be unique
            if (m_servant_provider->may_incarnate_multiple_objects())
            {
                std::string parameter("ObjectId Uniqueness");
                parameter += "(";
                parameter += factory_name;
                parameter += ")";
                CDMW_THROW2( BadParameterException, parameter, "MULTIPLE_ID");
            }
    
            //
            // Create and install servant activator
            //
            PortableServer::ServantActivator_var the_activator
                = new TheServantActivator(m_servant_provider);
            m_objects_poa -> set_servant_manager(the_activator.in());
    
        } catch (const std::bad_alloc & ) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            CDMW_THROW(Cdmw::OutOfMemoryException);
        } catch (const BadParameterException & ) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw;
        } catch (const Cdmw::InternalErrorException & ) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw; 
        } catch (const PortableServer::POA::AdapterAlreadyExists &) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                  CORBA::COMPLETED_YES);
        } catch (const PortableServer::POA::InvalidPolicy &) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw CORBA::INTERNAL(Cdmw::OrbSupport::INTERNALLifeCycleFrameworkError,
                                  CORBA::COMPLETED_YES);
        } catch (const CORBA::SystemException &) {
            // This is to avoid Assertion failure in debug mode as internal ref count
            // equals 1 (CORBA C++ mapping oblige!)
            _set_ref_count(0);
            throw; // rethrow
        }
    }

    <%factoryImplClassname%>::~<%factoryImplClassname%>()
        throw()
    {
        // <%servantProviderAllocatorClassname.withCppScope%>::TheAllocator.dispose(m_servant_provider);

        // m_objects_poa and m_servant_activator references will be released by their destructors;
        // and the associated POA and ServantActivator implementations will be destroyed following
        // the destruction of the parent POA.
    }

    <%--                
      Create the operation declarations.
    --%>
    <%for (idlInterface.findAllFactoryOperations){%>
    <%generateOperation%>
    
    <%}%>

    //
    // IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/supports:1.0
    //
    CORBA::Boolean 
    <%factoryImplClassname%>::supports(const char* repository_id)
        throw(CORBA::SystemException)
    {
        return CORBA::Boolean(m_servant_provider->supports(repository_id));
    }

    //
    // IDL:thalesgroup.com/CdmwLifeCycle/FactoryBase/get_supported_objects:1.0
    //
    CdmwLifeCycle::RepositoryIdSeq* 
    <%factoryImplClassname%>::get_supported_objects()
        throw(CORBA::SystemException)
    {
        return m_servant_provider->get_supported_objects();
    }
    
    
    // Register (name,object) within the repository. 
    // In case of a TRANSIENT system exception
    // There will be a retry max_retries times. if max_retries == 0, then this will do
    // retry forever.
    void
    <%factoryImplClassname%>::register_object
            (CORBA::Object_ptr  the_object,
             const char*        object_name,
             signed long        max_retries)
        throw(CdmwLifeCycle::AlreadyExistant,
              CdmwLifeCycle::InvalidName,
              CORBA::SystemException)
    {
        bool forever = (max_retries <= 0);
    
        signed long counter = max_retries;
    
        do {
            try {
                m_domain->register_object (the_object, object_name);
                break;
            } catch (const CosNaming::NamingContext::AlreadyBound &) {
                CdmwLifeCycle::AlreadyExistant ex;
                // Add more information to help the caller in debuging
                std::string reason = m_domain_name + "/" + object_name;
                ex.name = CORBA::string_dup(reason.c_str());
                throw ex;
            } catch (const CosNaming::NamingContext::InvalidName & e) {
                // no "reason" field available
                throw CdmwLifeCycle::InvalidName();
            } catch (const CORBA::TRANSIENT &) {
                // Retry
            } catch (const CORBA::SystemException &) {
                // Cancel registration ????
                throw;
            }
        } while (forever || (--counter > 0) );
    
        if (!forever && (counter <= 0))
            throw CORBA::TRANSIENT(Cdmw::OrbSupport::TRANSIENTCannotReachRepository,
                                   CORBA::COMPLETED_YES);
    
    }

<%closeNamespace(factoryImplClassnameScope.withCppScope)%>







<%script type="idlmm.OperationDef" name="generateOperation"%>
<%checkCdmwLifeCycleKeyParameterForNamedFactory%>
<%type.asReturnParameterDef%> <%factoryImplClassname%>::<%name%>(<%parameters[mode == "PARAM_IN"].genParameterDef.sep(", ")%>)
throw ( 
       <%for (exceptions){%>
       <%cppFullClassname%>,
       <%}%>
       CORBA::SystemException
      )
{
    // [II] Acquire oid from user for object_key
    // Operation may throw user exceptions that will re-thrown to caller
    PortableServer::ObjectId_var oid 
         = m_servant_provider-><%name%> (
            <%parameters[mode == "PARAM_IN" && !type.isCdmwLifeCycleKey].cppName.sep(", ")%>
           );
        
    // [III] Activate the object immediately :
    //       [a] Acquire a servant from user for odi
    //       [b] Activate the object (using a servant activator)
    //       [c] Create the object reference
    //       [d] Assert the object is really of the expected type
    typedef Cdmw::LifeCycle::ObjectUtil< <%type.cppFullClassname%> > Utils;
    PortableServer::Servant servant = Utils::lookup_servant
        (oid.in(), m_servant_provider);
    <%type.asLocalVariable%> ref;
    try {
        ref = Utils::activate_object(m_objects_poa.in(),oid.in(),servant);   
        // [IV] Register (name,object) within the repository.
        register_object(ref.in(), object_key, MAX_RETRIES);
    } catch (...) {
        m_servant_provider -> release_servant (oid.in(), m_objects_poa.in(), servant);
        throw;
    }

    // [V] Return the object reference
    return ref<%type.localVarReturnSuffix%>;
}


