/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.template.eval;

import org.acceleo.template.core.Template;

import org.acceleo.tools.log.AcceleoException;
import org.acceleo.tools.strings.Int2;

/**
 * ENode Exception.
 * 
 * @author www.obeo.fr
 * 
 */
public class ENodeException extends AcceleoException {

	/**
	 * The identifier of the runtime error marker.
	 */
	public static final String RUNTIME_ERROR_MARKER_ID = "fr.obeo.acceleo.gen.ui.problem"; //$NON-NLS-1$

	/**
	 * The model attribute for the runtime error marker.
	 */
	public static final String RUNTIME_ERROR_MARKER_MODEL = "runmodel"; //$NON-NLS-1$

	/**
	 * The fragment attribute for the runtime error marker.
	 */
	public static final String RUNTIME_ERROR_MARKER_FRAGMENT = "runfragment"; //$NON-NLS-1$

	/**
	 * Position of the evaluation error in the template.
	 */
	protected Int2 pos;

	/**
	 * Template that launch the evaluation error.
	 */
	protected Template template;

	/**
	 * The current object of the model (EObject or ENode).
	 */
	protected Object object;

	/**
	 * Optional runtime exception.
	 */
	protected Throwable exception;

	/**
	 * Indicates if the runtime markers are activated.
	 */
	private static boolean RUNTIME_MARKER = true;

	/**
	 * The trigger to disable markers.
	 */
	private static Object disabledTrigger;

	private String message;

	/**
	 * The default serial version ID.
	 */
	private static final long serialVersionUID = 1;

	/**
	 * Constructor.
	 * 
	 * @param message
	 *            is the message
	 * @param pos
	 *            is the position of the evaluation error in the template
	 * @param template
	 *            is the template
	 * @param object
	 *            is the current object of the model (EObject or ENode)
	 * @param report
	 *            indicates if this exception has to be reported in the error
	 *            log
	 */
	public ENodeException(String message, Int2 pos, Template template, Object object, boolean report) {
		this(message, pos, template, object, report, null);
	}

	/**
	 * Constructor.
	 * 
	 * @param message
	 *            is the message
	 * @param pos
	 *            is the position of the evaluation error in the template
	 * @param template
	 *            is the template
	 * @param object
	 *            is the current object of the model (EObject or ENode)
	 * @param report
	 *            indicates if this exception has to be reported in the error
	 *            log
	 * @param exception
	 *            is an optional runtime exception
	 */
	public ENodeException(String message, Int2 pos, Template template, Object object, boolean report, Throwable exception) {
		this.message = message;
		this.pos = pos;
		this.template = template;
		this.object = object;
		this.exception = exception;
		if (report && RUNTIME_MARKER) {
			getErrorMessage(true);
		}
	}

	/**
	 * The runtime markers are disabled.
	 * 
	 * @param source
	 *            is the trigger object
	 */
	public static void disableRuntimeMarkersFor(Object source) {
		if (RUNTIME_MARKER && disabledTrigger == null) {
			RUNTIME_MARKER = false;
			disabledTrigger = source;
		}
	}

	/**
	 * The runtime markers are enabled.
	 * 
	 * @param source
	 *            is the trigger object
	 */
	public static void enableRuntimeMarkersFor(Object source) {
		if (!RUNTIME_MARKER && source == disabledTrigger) {
			RUNTIME_MARKER = true;
			disabledTrigger = null;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public String getMessage() {
		return getErrorMessage(false);
	}

	private String getErrorMessage(boolean report) {
		// TODO david comment gère-t-on les messages
		
		  String message = super.getMessage();
		  
//		  Map markerAttributes = null; if (template.getFile() != null) { String
//		  path = template.getFile().getAbsolutePath(); IFile workspaceFile =
//		  ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new
//		  Path(path)); if (workspaceFile == null ||
//		  !workspaceFile.isAccessible()) { String relativePath =
//		  AcceleoModuleProvider.getDefault().getRelativePath(template.getFile());
//		  if (relativePath != null) { String pluginId =
//		  AcceleoModuleProvider.getDefault().getPluginId(template.getFile());
//		  if (pluginId != null) { path = new Path("/" +
//		  pluginId).append(relativePath).toString(); } } } else { path =
//		  workspaceFile.getFullPath().toString(); }
//		  
//		  message += ", in file : " + path; Int2[] newLines =
//		  TextSearch.getDefaultSearch().allIndexIn(Resources.getFileContent(template.getFile()).toString(),
//		  "\n", 0, pos.b()); int line = newLines.length + 1; message += ", at line : " + line; int column = pos.b(); if (newLines.length > 0) {
//		  column = column - newLines[newLines.length - 1].e(); } message += ", at column : " + column; if (report) { try { IFile scriptFile =
//		  getScriptFile(); if (scriptFile != null && scriptFile.isAccessible()) {
//		  markerAttributes = getProblemAttributes(scriptFile, line, pos,
//		  message); } } catch (CoreException e) {
//		  AcceleoEcoreGenPlugin.getDefault().log(e, true); } } } else { if
//		  (pos.b() > 0) { message += ", at position [" + pos.b() + "," +
//		  pos.e() + "]"; } } String modelURI = ""; String fragmentURI = ""; if
//		  (object != null) { Object object = this.object; if (object instanceof
//		  ENode) { if (((ENode) object).isEObject()) { try { object = ((ENode)
//		  object).getEObject(); } catch (ENodeCastException e) { // Never
//																	// catch } }
//		  else if (((ENode) object).isList()) { message += ", for node " + "'" +
//		  ((ENode) object).getType() + "'"; object = ((ENode)
//		  object).getContainerEObject(); } else { message += ", for node " +
//		  "'" + ((ENode) object).getType() + "' : " + ((ENode)
//		  object).toString(); object = ((ENode) object).getContainerEObject(); } }
//		  if (object instanceof EObject) { fragmentURI =
//		  ETools.getURI((EObject) object); if (((EObject) object).eResource() !=
//		  null) { URI uri = ((EObject) object).eResource().getURI(); modelURI =
//		  (uri != null) ? uri.path() : ""; message += ", for object : fragment=" + fragmentURI + " file=" + modelURI; } else { message += ", for object : fragment=" + fragmentURI + " type=" + ((EObject)
//		  object).eClass().getName() + " value=" + object.toString(); } } } if
//		  (exception != null) { message += "\n" +
//		  Trace.getStackTrace(exception); } if (markerAttributes != null) { try {
//		  createRuntimeMarker(markerAttributes, modelURI, fragmentURI); } catch
//		  (CoreException e) { AcceleoEcoreGenPlugin.getDefault().log(e, true); } }
		 
		return message;
	}

	// private Map getProblemAttributes(IResource resource, int line, Int2 pos,
	// String message) throws CoreException {
	// boolean exist = false;
	// IMarker[] markers = resource.findMarkers(IMarker.PROBLEM, true, 1);
	// for (int i = 0; !exist && i < markers.length; i++) {
	// IMarker marker = markers[i];
	// Integer otherLine = (Integer) marker.getAttribute(IMarker.LINE_NUMBER);
	// String otherMessage = (String) marker.getAttribute(IMarker.MESSAGE);
	// if (otherLine != null && otherLine.intValue() == line && otherMessage !=
	// null && otherMessage.equals(message)) {
	// exist = true;
	// }
	// }
	// if (!exist) {
	// Map map = new HashMap();
	// map.put(IMarker.LINE_NUMBER, new Integer(line));
	// map.put(IMarker.CHAR_START, new Integer(pos.b()));
	// map.put(IMarker.CHAR_END, new Integer(pos.e()));
	// map.put(IMarker.MESSAGE, message);
	// map.put(IMarker.PRIORITY, new Integer(IMarker.PRIORITY_HIGH));
	// map.put(IMarker.SEVERITY, new Integer(IMarker.SEVERITY_ERROR));
	// return map;
	// } else {
	// return null;
	// }
	// }
	//
	// private void createRuntimeMarker(Map problemAttributes, String modelPath,
	// String fragmentURI) throws CoreException {
	// IFile scriptFile = getScriptFile();
	// if (scriptFile != null && scriptFile.isAccessible()) {
	// problemAttributes.put(RUNTIME_ERROR_MARKER_MODEL, modelPath);
	// problemAttributes.put(RUNTIME_ERROR_MARKER_FRAGMENT, fragmentURI);
	// MarkerUtilities.createMarker(scriptFile, problemAttributes,
	// RUNTIME_ERROR_MARKER_ID);
	// }
	// }
	//
	// private IFile getScriptFile() {
	// if (template.getFile() != null) {
	// return ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new
	// Path(template.getFile().getAbsolutePath()));
	// } else {
	// return null;
	// }
	// }

}
