/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.template.eval;

import org.eclipse.emf.common.util.Monitor;

/**
 * The launch manager : the mode in which to launch - RUN_MODE, DEBUG_MODE, or
 * PREVIEW_MODE.
 * 
 * @author www.obeo.fr
 * 
 */
public final class LaunchManager implements ILauncherManager {

	/**
	 * The mode in which to launch.
	 */
	private final int mode;

	/**
	 * Activates text/model synchronization.
	 */
	private boolean synchronize;

	/**
	 * The progress monitor;
	 */
	private Monitor monitor;

	/**
	 * Constructor.
	 * 
	 * @param mode
	 *            is mode in which to launch
	 * @param synchronize
	 *            activates text/model synchronization
	 */
	private LaunchManager(int mode, boolean synchronize) {
		this.mode = mode;
		this.synchronize = synchronize;
	}

	/**
	 * Creates a new manager.
	 * 
	 * @param mode
	 *            is a string - "run", "debug", "preview", "phantom"
	 * @param synchronize
	 *            indicates if the text/model synchronization is activated
	 * @return the manager or null
	 */
	public static LaunchManager create(String mode, boolean synchronize) {
		LaunchManager newManager;
		if (mode == null) {
			newManager = null;
		} else if (mode.equals("run")) { //$NON-NLS-1$
			newManager = new LaunchManager(RUN_MODE, synchronize);
		} else if (mode.equals("debug")) { //$NON-NLS-1$
			newManager = new LaunchManager(DEBUG_MODE, synchronize);
		} else if (mode.equals("preview")) { //$NON-NLS-1$
			newManager = new LaunchManager(PREVIEW_MODE, synchronize);
		} else if (mode.equals("phantom")) { //$NON-NLS-1$
			newManager = new LaunchManager(PHANTOM_MODE, synchronize);
		} else {
			newManager = null;
		}
		return newManager;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.acceleo.vm.eval.IlauncherManager#getMode()
	 */
	public int getMode() {
		return mode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.acceleo.vm.eval.IlauncherManager#isSynchronize()
	 */
	public boolean isSynchronize() {
		return synchronize;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.acceleo.vm.eval.IlauncherManager#setSynchronize(boolean)
	 */
	public void setSynchronize(boolean synchronize) {
		this.synchronize = synchronize;
	}

	/**
	 * @return the monitor
	 */
	public Monitor getMonitor() {
		return monitor;
	}

	/**
	 * @param monitor
	 *            is the monitor to set
	 */
	public void setMonitor(Monitor monitor) {
		this.monitor = monitor;
	}
}
