/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;

import org.acceleo.engine.context.EvalContext;
import org.acceleo.tools.factory.FactoryException;
import org.eclipse.emf.ecore.EObject;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeException;

/**
 * This class create a runnable to be executed on a given EObject. This runnable
 * will return a mapping between the resource name and the corresponding ENode.
 * 
 * @author Yvan LUSSAUD <a
 *         href="mailto:yvan.lussaud@obeo.fr">yvan.lussaud@obeo.fr</a>
 * 
 */
public class EMTCallableToENode implements Callable<Map<String, ENode>> {
	/** EObjects on which we run. */
	private final List<EObject> eObjects;

	/** The current EMT. */
	private final EMT emt;

	/** The current evaluation context */
	private final EvalContext evalContext = new EvalContext();

	/**
	 * Constructor.
	 * 
	 * @param emt
	 *            the EMT that created this runnable
	 * @param ret
	 *            the map used to store the result of this runnable
	 * @param eObj
	 *            the EObject to generate
	 */
	public EMTCallableToENode(EMT emt, List<EObject> eObjects) {
		this.emt = emt;
		this.eObjects = eObjects;
	}

	/**
	 * {@inheritDoc}
	 */
	public Map<String, ENode> call() throws FactoryException, ENodeException,
			ENodeCastException {
		final Map<String, ENode> ret = new HashMap<String, ENode>();
		for (final EObject eObj : eObjects) {
			ret.putAll(emt.genENodes(eObj, evalContext));
		}
		return ret;
	}
}
