/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.engine.eval.imports.services;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeList;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;
import java.util.concurrent.ConcurrentHashMap;

/**
 * System services for ENode elements.
 * 
 * @author www.obeo.fr
 * 
 */
public final class ContextServices implements IThreadSafeService {
	// TODO YLU fix local thread context when the AcceleoThread will be used.

	/** Context stack. */
	private final Map<Thread, Stack<ENode>> stack = new ConcurrentHashMap<Thread, Stack<ENode>>();

	/**
	 * Context map.
	 */
	private final Map<Thread, Map<String, ENode>> context = new ConcurrentHashMap<Thread, Map<String, ENode>>();

	/**
	 * Constructor.
	 */
	public ContextServices() {
		// nothing to do here
	}

	/**
	 * Puts the current node in the context for the given key, and returns an
	 * empty string.
	 * 
	 * @param node
	 *            is the current node
	 * @param key
	 *            is the key in the context
	 * @deprecated
	 */
	@Deprecated
	public void nPut(ENode node, String key) {
		put(node, key);
	}

	/**
	 * Puts the current node in the context for the given key, and returns an
	 * empty string.
	 * 
	 * @param node
	 *            is the current node
	 * @param key
	 *            is the key in the context
	 */
	public void put(ENode node, String key) {
		getContextMap().put(key, node);
	}

	private Map<String, ENode> getContextMap() {
		Map<String, ENode> map = context.get(Thread.currentThread());
		if (map == null) {
			map = new HashMap<String, ENode>();
			context.put(Thread.currentThread(), map);
		}
		return map;
	}

	private Stack<ENode> getStack() {
		Stack<ENode> s = stack.get(Thread.currentThread());
		if (s == null) {
			s = new Stack<ENode>();
			stack.put(Thread.currentThread(), s);
		}
		return s;
	}

	/**
	 * Gets the node for the given key in the context.
	 * 
	 * @param node
	 *            is the current node
	 * @param key
	 *            is the key in the context
	 * @return the node of the context
	 * @throws ENodeCastException
	 *             if cast error occurs
	 * @deprecated
	 */
	@Deprecated
	public ENode nGet(ENode node, ENode key) throws ENodeCastException {
		ENode associatedNode;
		if (key.isInt()) {
			if (node.isList()) {
				final ENodeList list = node.getList();
				final ENode result = list.get(key.getInt());
				if (result != null) {
					associatedNode = result;
				} else {
					associatedNode = new ENode(ENode.EMPTY, node);
				}
			} else if (key.getInt() == 0) {
				associatedNode = node;
			} else {
				associatedNode = new ENode(ENode.EMPTY, node);
			}
		} else if (key.isString()) {
			// deprecated nGet(ENode node, String key)
			associatedNode = get(node, key.getString());
		} else {
			// deprecated nGet(ENode node, String key)
			associatedNode = get(node, (String) key.getAdapterValue(String.class));
		}
		return associatedNode;
	}

	/**
	 * Gets the node for the given key in the context.
	 * 
	 * @param node
	 *            is the current node
	 * @param key
	 *            is the key in the context
	 * @return the node of the context
	 */
	public ENode get(ENode node, String key) {
		final ENode result = getContextMap().get(key);
		if (result != null) {
			return result.copy();
		} else {
			return new ENode(ENode.EMPTY, node);
		}
	}

	/**
	 * Pushes the current node into the context, and returns an empty string.
	 * 
	 * @param node
	 *            is the current node
	 * @deprecated
	 */
	@Deprecated
	public void nPush(ENode node) {
		push(node);
	}

	/**
	 * Pushes the current node into the context, and returns an empty string.
	 * 
	 * @param node
	 *            is the current node
	 */
	public void push(ENode node) {
		getStack().push(node);
	}

	/**
	 * Pops the context, and returns an empty string.
	 * 
	 * @param node
	 *            is the current node
	 * @deprecated
	 */
	@Deprecated
	public void nPop(ENode node) {
		pop(node);
	}

	/**
	 * Pops the context, and returns an empty string.
	 * 
	 * @param node
	 *            is the current node
	 */
	public void pop(ENode node) {
		getStack().pop();
	}

	/**
	 * Return the last node pushed into the context, and returns an empty
	 * string.
	 * 
	 * @param node
	 *            is the current node
	 * @return an empty string
	 * @deprecated
	 */
	@Deprecated
	public ENode nPeek(ENode node) {
		return peek(node);
	}

	/**
	 * Return the last node pushed into the context, and returns an empty
	 * string.
	 * 
	 * @param node
	 *            is the current node
	 * @return an empty string
	 */
	public ENode peek(ENode node) {
		if (!getStack().isEmpty()) {
			return getStack().peek().copy();
		} else {
			return new ENode("", node); //$NON-NLS-1$
		}
	}

}
