/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.engine.eval.imports.services;

import fr.obeo.acceleo.gen.template.eval.ENode;

import java.io.IOException;

import org.acceleo.engine.context.EvalContext;

/**
 * System services for the properties files.
 * 
 * @author www.obeo.fr
 * 
 */
public final class PropertiesServices implements IContextService {

	/**
	 * The current evaluation context.
	 */
	protected EvalContext evalContext;

	/**
	 * Gets the value for the key in all the property files.
	 * 
	 * @param current
	 *            is the current node
	 * @param key
	 *            is the key
	 * @return the value for the given key
	 * @throws IOException
	 *             if I/O error occurs
	 */
	// TODO revoir la gestion des exceptions
	public ENode getProperty(ENode current, String key) throws IOException {
		final String result = evalContext.getCurrentTemplateContext().getProperties().getProperty(key);
		return new ENode(result, current);
	}

	/**
	 * Gets the property for the key and the property file (without extension).
	 * 
	 * @param current
	 *            is the current node
	 * @param name
	 *            is the name of the property file (without ".properties"
	 *            extension)
	 * @param key
	 *            is the key
	 * @return the value for the given key
	 * @throws IOException
	 *             if I/O error occurs
	 */
	// TODO revoir la gestion des exceptions
	public ENode getProperty(ENode current, String name, String key) throws IOException {
		final String result = evalContext.getCurrentTemplateContext().getProperties().getProperty(name, key);
		return new ENode(result, current);
	}

	/**
	 * Gets the best value for the key in all the property files.
	 * <p>
	 * Sample : key == "aa.bb.cc.dd"
	 * <p>
	 * It returns getProperty("aa.bb.cc.dd") || getProperty("aa.bb.cc") ||
	 * getProperty("aa.bb") || getProperty("aa").
	 * 
	 * @param current
	 *            is the current node
	 * @param key
	 *            is the key
	 * @return the value for the given key
	 * @throws IOException
	 *             if I/O error occurs
	 */
	public ENode getBestProperty(ENode current, String key) throws IOException {
		String result = null;
		String localKey = key;
		while (result == null && localKey != null && localKey.length() > 0) {
			result = evalContext.getCurrentTemplateContext().getProperties().getProperty(localKey);
			if (result == null) {
				final int i = localKey.lastIndexOf("."); //$NON-NLS-1$
				if (i == -1) {
					localKey = null;
				} else {
					localKey = localKey.substring(0, i);
				}
			}
		}
		return new ENode(result, current);
	}

	public void setEvalContext(EvalContext evalContext) {
		this.evalContext = evalContext;
	}
}