/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.engine.eval.imports.services;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeException;
import fr.obeo.acceleo.gen.template.eval.ENodeIterator;
import fr.obeo.acceleo.gen.template.eval.ENodeList;

import org.acceleo.engine.context.EvalContext;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.tools.factory.FactoryException;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Request services.
 * 
 * @author www.obeo.fr
 * 
 */
public final class RequestServices implements IContextService {

	/**
	 * Constant for quote.
	 */
	private static final String QUOTE_LITERAL = "'"; //$NON-NLS-1$

	/**
	 * The current evaluation context.
	 */
	private EvalContext evalContext;

	/**
	 * Evaluates the given expression on all the elements of the current node
	 * and keep the elements validating the condition.
	 * 
	 * @param current
	 *            is the current node of generation
	 * @param call
	 *            is the condition
	 * @return valid elements
	 * @throws TemplateSyntaxException
	 *             if the call expression is not valid.
	 * @throws FactoryException
	 *             if the generation of the call failed.
	 * @throws ENodeCastException
	 *             if a cast exception occurred.
	 * @deprecated
	 */
	@Deprecated
	public ENode select(ENode current, String call) throws FactoryException, ENodeCastException, TemplateSyntaxException {
		return select(current, call, new ENode(true, current));
	}

	/**
	 * Evaluates the given expression on all the elements of the current node
	 * and keep the elements validating the given value.
	 * 
	 * @param current
	 *            is the current node of generation
	 * @param call
	 *            is the expression to apply
	 * @param value
	 *            is the value of the elements to keep
	 * @return valid elements
	 * @throws TemplateSyntaxException
	 *             if the call expression is not valid.
	 * @throws FactoryException
	 *             if the generation of the call failed.
	 * @throws ENodeCastException
	 *             if a cast exception occurred.
	 * @deprecated
	 */
	@Deprecated
	public ENode select(ENode current, String call, ENode value) throws FactoryException, ENodeCastException, TemplateSyntaxException {
		// TODO YLU compute offset
		int offset;
		if (current.getContainerTemplateElement() != null) {
			offset = current.getContainerTemplateElement().getBegin();
		} else {
			offset = 0;
		}
		final IEvaluable expression = evalContext.compile(offset, call.replaceAll(QUOTE_LITERAL, TemplateConstants.SPEC));
		return select(current, expression, value);
	}

	/**
	 * Evaluates the given expression on all the elements of the current node
	 * and keep the elements validating the condition.
	 * 
	 * @param current
	 *            is the current node of generation
	 * @param call
	 *            is the expression to apply
	 * @param value
	 *            is the value of the elements to keep
	 * @return valid elements
	 * @throws FactoryException
	 *             if the generation of the call failed.
	 * @throws ENodeCastException
	 *             if a cast exception occurred.
	 */
	private ENode select(ENode current, IEvaluable call, ENode value) throws FactoryException, ENodeCastException {
		ENode res = null;
		if (current.isList()) {
			final ENodeList list = current.getList();
			final ENodeList resList = new ENodeList();
			final ENodeIterator it = list.iterator();
			while (it.hasNext()) {
				resList.add(select(it.next(), call, value));
			}
			res = new ENode(resList, current);
		} else {
			try {
				final ENode result = call.gen(current, evalContext);
				if (result.equals(value)) {
					res = current;
				} else {
					res = new ENode(ENode.EMPTY, current);
				}
			} catch (final ENodeException e) {
				res = new ENode(ENode.EMPTY, current);
			}
		}
		return res;
	}

	/**
	 * Evaluates the given call on all the elements of the current node and
	 * delete the elements when call evaluation is true.
	 * 
	 * @param current
	 *            is the current node of generation
	 * @param call
	 *            is the expression to apply
	 * @return valid elements
	 * @throws TemplateSyntaxException
	 *             if the call expression is not valid.
	 * @throws FactoryException
	 *             if the generation of the call failed.
	 * @throws ENodeCastException
	 *             if a cast exception occurred.
	 * @deprecated
	 */
	@Deprecated
	public ENode delete(ENode current, String call) throws TemplateSyntaxException, FactoryException, ENodeCastException {
		return delete(current, call, new ENode(true, current));
	}

	/**
	 * Evaluates the given expression on all the elements of the current node
	 * and delete the elements validating the given value.
	 * 
	 * @param current
	 *            is the current node of generation
	 * @param call
	 *            is the expression to apply
	 * @param value
	 *            is the value of the elements to delete
	 * @return valid elements
	 * @throws TemplateSyntaxException
	 *             if the call expression is not valid.
	 * @throws FactoryException
	 *             if the generation of the call failed.
	 * @throws ENodeCastException
	 *             if a cast exception occurred.
	 * @deprecated
	 */
	@Deprecated
	public ENode delete(ENode current, String call, ENode value) throws TemplateSyntaxException, FactoryException, ENodeCastException {
		int offset;
		if (current.getContainerTemplateElement() != null) {
			offset = current.getContainerTemplateElement().getBegin();
		} else {
			offset = 0;
		}
		final IEvaluable expression = evalContext.compile(offset, call.replaceAll(QUOTE_LITERAL, TemplateConstants.SPEC));
		return delete(current, expression, value);
	}

	/**
	 * Evaluates the given expression on all the elements of the current node
	 * and delete the elements validating the given value.
	 * 
	 * @param current
	 *            is the current node of generation
	 * @param call
	 *            is the expression to apply
	 * @param value
	 *            is the value of the elements to delete
	 * @return valid elements
	 * @throws FactoryException
	 *             if the generation of the call failed.
	 * @throws ENodeCastException
	 *             if a cast exception occurred.
	 */
	private ENode delete(ENode current, IEvaluable call, ENode value) throws FactoryException, ENodeCastException {
		ENode res = null;
		if (current.isList()) {
			final ENodeList list = current.getList();
			final ENodeList resList = new ENodeList();
			final ENodeIterator it = list.iterator();
			while (it.hasNext()) {
				resList.add(delete(it.next(), call, value));
			}
			res = new ENode(resList, current);
		} else {
			try {
				final ENode result = call.gen(current, evalContext);
				if (!result.equals(value)) {
					res = current;
				} else {
					res = new ENode(ENode.EMPTY, current);
				}
			} catch (final ENodeException e) {
				res = new ENode(ENode.EMPTY, current);
			}
		}
		return res;
	}

	/**
	 * Evaluates the given text as an acceleo expression.
	 * 
	 * @param current
	 *            is the current node of generation
	 * @param call
	 *            is the expression to evaluate
	 * @return evaluation
	 * @throws FactoryException
	 *             if the generation of the call failed.
	 * @throws TemplateSyntaxException
	 *             if the call expression is not valid.
	 */
	public ENode evaluate(ENode current, String call) throws FactoryException, TemplateSyntaxException {
		ENode evaluatedExpression;
		String localCall = call;
		localCall = localCall.replaceAll(QUOTE_LITERAL, TemplateConstants.SPEC);
		// TODO YLU compute offset
		int offset;
		if (current.getContainerTemplateElement() != null) {
			offset = current.getContainerTemplateElement().getBegin();
		} else {
			offset = 0;
		}
		final IEvaluable expression = evalContext.compile(offset, localCall);
		try {
			evaluatedExpression = expression.gen(current, evalContext);
		} catch (final ENodeException e) {
			evaluatedExpression = new ENode(ENode.EMPTY, current);
		}
		return evaluatedExpression;
	}

	public void setEvalContext(EvalContext evalContext) {
		this.evalContext = evalContext;
	}
}