/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine.template.core.impl.spec;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeException;

import org.acceleo.engine.context.EvalContext;
import org.acceleo.engine.template.impl.spec.EvaluableTools;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.template.core.impl.ScriptImpl;
import org.acceleo.tools.factory.FactoryException;
import org.acceleo.tools.template.TemplateConstants;

/**
 * A representation of the model object '<em><b>Script</b></em>'.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 * 
 */
public class TScript extends ScriptImpl implements IEvaluable {

    /**
     * {@inheritDoc}
     */
    public ENode gen(ENode root, EvalContext evalContext) throws ENodeException, FactoryException {
        final ENode node = EvaluableTools.genStatements(this, root, getStatements(), evalContext);
        if (descriptor.getPost() != null) {
            return ((IEvaluable) descriptor.getPost()).gen(node, evalContext);
        }
        return node;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        final StringBuffer buffer = new StringBuffer(""); //$NON-NLS-1$
        if (descriptor != null) {
            String type = descriptor.getType();
            int dot = type.lastIndexOf("."); //$NON-NLS-1$
            if (dot > -1) {
                dot = type.substring(0, dot).lastIndexOf("."); //$NON-NLS-1$
            } else {
                dot = -1;
            }

            if (dot > -1) {
                type = type.substring(dot + 1);
            }
            // buffer.append(TemplateConstants.FEATURE_BEGIN);
            buffer.append(TemplateConstants.SCRIPT_BEGIN);
            buffer.append(TemplateConstants.SCRIPT_TYPE);
            buffer.append(TemplateConstants.SCRIPT_PROPERTY_ASSIGN);
            buffer.append(TemplateConstants.LITERAL[0]);
            buffer.append(type);
            buffer.append(TemplateConstants.LITERAL[1]);
            buffer.append(' ');
            buffer.append(TemplateConstants.SCRIPT_NAME);
            buffer.append(TemplateConstants.SCRIPT_PROPERTY_ASSIGN);
            buffer.append(TemplateConstants.LITERAL[0]);
            buffer.append(descriptor.getName());
            buffer.append(TemplateConstants.LITERAL[1]);
            if (descriptor.getDescription() != null && descriptor.getDescription().length() > 0) {
                buffer.append(' ');
                buffer.append(TemplateConstants.SCRIPT_DESC);
                buffer.append(TemplateConstants.SCRIPT_PROPERTY_ASSIGN);
                buffer.append(TemplateConstants.LITERAL[0]);
                buffer.append(descriptor.getDescription());
                buffer.append(TemplateConstants.LITERAL[1]);
            }
            if (descriptor.getPost() != null) {
                buffer.append(' ');
                buffer.append(TemplateConstants.SCRIPT_POST);
                buffer.append(TemplateConstants.SCRIPT_PROPERTY_ASSIGN);
                buffer.append(TemplateConstants.LITERAL[0]);
                buffer.append(descriptor.getPost());
                buffer.append(TemplateConstants.LITERAL[1]);
            }
        }
        buffer.append(TemplateConstants.SCRIPT_END);
        buffer.append("\n"); //$NON-NLS-1$
        buffer.append(EvaluableTools.toString(statements));
        return buffer.toString();
    }
}
