/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine.template.expressions.impl.spec;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeException;

import org.acceleo.engine.AcceleoEngineMessages;
import org.acceleo.engine.context.EvalContext;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.template.expressions.impl.NotImpl;
import org.acceleo.tools.factory.FactoryException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.template.TemplateConstants;

/**
 * A representation of the model object '<em><b>Not</b></em>'.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 * 
 */
public class TNot extends NotImpl implements IEvaluable {

	/**
	 * {@inheritDoc}
	 */
	public ENode gen(ENode root, EvalContext evalContext) throws ENodeException, FactoryException {

		// TODO startDebug(evalContext.getLaunchManager());
		try {
			ENode test = ((IEvaluable) expression).gen(root, evalContext);
			ENode result;
			try {
				boolean value = test.getBoolean();
				result = new ENode(!value, root);
			} catch (ENodeCastException e1) {
				try {
					Object boolValue = test.getAdapterValue(boolean.class);
					if (boolValue instanceof Boolean) {
						result = new ENode(!((Boolean) boolValue).booleanValue(), root);
					} else {
						throw new ENodeException(
								AcceleoEngineMessages.getString("ENodeError.BooleanRequired", new Object[] { test.getType(), }), new Int2(expression.getBegin(), expression.getEnd()), getTemplate(), root, true); //$NON-NLS-1$
					}
				} catch (ENodeCastException e2) {
					throw new ENodeException(e2.getMessage(), new Int2(expression.getBegin(), expression.getEnd()), getTemplate(), root, true);
				}
			}
			// TODO stepDebugOutput(evalContext.getLaunchManager(), root,
			// result);
			return result;
		} finally {
			// TODO endDebug(evalContext.getLaunchManager());
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return TemplateConstants.NOT + expression.toString();
	}
}
