/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine.template.expressions.impl.spec;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeException;

import java.util.Iterator;

import org.acceleo.engine.context.EvalContext;
import org.acceleo.engine.eval.tools.ExpressionTools;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.template.expressions.Expression;
import org.acceleo.template.expressions.impl.OperatorImpl;
import org.acceleo.tools.factory.FactoryException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.template.TemplateConstants;

/**
 * A representation of the model object '<em><b>Operator</b></em>'.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 * 
 */
public class TOperator extends OperatorImpl implements IEvaluable {

	/**
	 * {@inheritDoc}
	 */
	public ENode gen(ENode root, EvalContext evalContext) throws ENodeException, FactoryException {
		// TODO startDebug(mode);
		ENode ret = null;
		ENode last = null;
		try {
			for (Expression operand : getOperands()) {
				ENode node = ((IEvaluable) operand).gen(root, evalContext);
				if (last == null || last.isBoolean()) {
					if (operator.equals(TemplateConstants.OPERATOR_OR)) {
						if (node.isBoolean() && node.getBoolean()) {
							// true or x = true
							last = node;
							break;
						}
					} else if (operator.equals(TemplateConstants.OPERATOR_AND)) {
						if (node.isBoolean() && !node.getBoolean()) {
							// false and x = false
							last = node;
							break;
						}
					}
				}
				if (last != null) {
					if (operator.equals(TemplateConstants.OPERATOR_OR)) {
						node = ExpressionTools.or(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_AND)) {
						node = ExpressionTools.and(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_EQUALS)) {
						node = ExpressionTools.equals(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_NOT_EQUALS)) {
						node = ExpressionTools.notEquals(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_SUP_EQUALS)) {
						node = ExpressionTools.supE(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_INF_EQUALS)) {
						node = ExpressionTools.infE(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_SUP)) {
						node = ExpressionTools.sup(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_INF)) {
						node = ExpressionTools.inf(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_ADD)) {
						node = ExpressionTools.add(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_SUB)) {
						node = ExpressionTools.sub(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_DIV)) {
						node = ExpressionTools.div(last, node);
					} else if (operator.equals(TemplateConstants.OPERATOR_MUL)) {
						node = ExpressionTools.mul(last, node);
					}
				}
				last = node;
			}
			if (last != null) {
				ret = last;
			} else {
				ret = new ENode(ENode.EMPTY, root);
			}
			return ret;
		} catch (ENodeCastException e) {
			throw new ENodeException(e.getMessage(), new Int2(begin, end), getTemplate(), root, true);
		} finally {
			// TODO endDebug(mode);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer(""); //$NON-NLS-1$
		Iterator<Expression> children = operands.iterator();
		while (children.hasNext()) {
			Expression child = children.next();
			buffer.append(child.toString());
			if (children.hasNext()) {
				buffer.append(' ' + operator + ' ');
			}
		}
		return buffer.toString();
	}
}
