/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine.template.impl.spec;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.acceleo.engine.template.core.impl.spec.TCoreFactory;
import org.acceleo.engine.template.expressions.impl.spec.TExpressionsFactory;
import org.acceleo.engine.template.statements.impl.spec.TStatementsFactory;
import org.acceleo.template.TemplateFactory;
import org.acceleo.template.core.CoreFactory;
import org.acceleo.template.expressions.ExpressionsFactory;
import org.acceleo.template.statements.StatementsFactory;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;

/**
 * A tool class for EMT element to standalone engine element conversion.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 * 
 */
public class TemplateConverter {

	/**
	 * EMT factories to engine factories mapping.
	 */
	private static final Map<EFactory, EFactory> factoryToFactory = new HashMap<EFactory, EFactory>();

	static {
		factoryToFactory.put(TemplateFactory.eINSTANCE, new TTemplateFactory());
		factoryToFactory.put(CoreFactory.eINSTANCE, new TCoreFactory());
		factoryToFactory.put(ExpressionsFactory.eINSTANCE, new TExpressionsFactory());
		factoryToFactory.put(StatementsFactory.eINSTANCE, new TStatementsFactory());
	}

	/** Hidden constructor. */
	private TemplateConverter() {
		// nothing to do here
	}

	/**
	 * Convert an EMT model elements tree to an engine elements tree.
	 * 
	 * @param toConvert
	 *            EMF element to convert
	 * @return engine element
	 */
	public static EObject convertToEngineTemplate(EObject toConvert) {
		return convert(toConvert, new HashMap<EObject, EObject>());
	}

	private static EObject convert(EObject eObject, Map<EObject, EObject> sourcesToTarget) {
		if (sourcesToTarget.containsKey(eObject)) {
			return sourcesToTarget.get(eObject);
		}

		EObject result = null;
		EFactory factory = factoryToFactory.get(eObject.eClass().getEPackage().getEFactoryInstance());
		EClass eClass = eObject.eClass();
		result = factory.create(eClass);

		sourcesToTarget.put(eObject, result);

		copyFeatures(eObject, result, sourcesToTarget);

		return result;
	}

	private static void copyFeatures(EObject source, EObject target, Map<EObject, EObject> sourcesToTarget) {
		for (EStructuralFeature feature : source.eClass().getEAllStructuralFeatures()) {
			if (feature.isChangeable() && !(feature instanceof EReference && ((EReference) feature).isContainer())) {
				if (factoryToFactory.containsKey(feature.getEType().getEPackage().getEFactoryInstance())) {
					//
					// We should convert.
					if (feature.isMany()) {
						Collection<Object> collection = (Collection<Object>) target.eGet(feature);
						for (EObject eObject : (Iterable<EObject>) source.eGet(feature)) {
							collection.add(convert(eObject, sourcesToTarget));
						}
					} else if (source.eGet(feature) != null) {
						target.eSet(feature, convert((EObject) source.eGet(feature), sourcesToTarget));
					}
				} else {
					//
					// Copy.
					target.eSet(feature, source.eGet(feature));
				}
			}
		}
	}
}