/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine.template.statements.impl.spec;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeException;
import fr.obeo.acceleo.gen.template.eval.ENodeIterator;
import fr.obeo.acceleo.gen.template.eval.ENodeList;
import fr.obeo.acceleo.gen.template.eval.ILauncherManager;

import org.acceleo.engine.context.EvalContext;
import org.acceleo.engine.template.core.impl.spec.TTemplate;
import org.acceleo.engine.template.impl.spec.EvaluableTools;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.template.statements.impl.ForImpl;
import org.acceleo.tools.factory.FactoryException;
import org.acceleo.tools.template.TemplateConstants;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EcoreFactory;

/**
 * A representation of the model object '<em><b>For</b></em>'.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 * 
 */
public class TFor extends ForImpl implements IEvaluable {
	/**
	 * {@inheritDoc}
	 */
	public ENode gen(ENode root, EvalContext evalContext) throws ENodeException, FactoryException {
		ENode res = null;
		EObject eObj;
		try {
			eObj = root.getEObject();
		} catch (final ENodeCastException e) {
			eObj = ENode.EMPTY;
		}
		final ENode node = new ENode(ENode.EMPTY, eObj, this, evalContext.getLaunchManager().isSynchronize());
		final ENode evalIterator = ((IEvaluable) iterator).gen(root, evalContext);
		if (evalContext.getLaunchManager().getMode() == ILauncherManager.DEBUG_MODE) {
			node.log().getAll(evalIterator.log(), false);
		}
		if (!evalIterator.isNull()) {
			boolean containsEObject;
			ENodeIterator it;
			if (evalIterator.isList()) {
				try {
					containsEObject = false;
					it = evalIterator.getList().iterator();
					while (it.hasNext() && !containsEObject) {
						final ENode child = it.next();
						if (child.isEObject()) {
							containsEObject = true;
						}
					}
					it = evalIterator.getList().iterator();
				} catch (final ENodeCastException e) {
					// Never catch
					containsEObject = false;
					it = new ENodeList().iterator();
				}
			} else if (evalIterator.isInt()) {
				try {
					containsEObject = true;
					final ENodeList list = new ENodeList();
					final int count = evalIterator.getInt();
					for (int i = 0; i < count; i++) {
						list.add(new ENode(eObj, (IEvaluable) iterator, evalContext.getLaunchManager().isSynchronize()));
					}
					it = list.iterator();
				} catch (final ENodeCastException e) {
					// Never catch
					containsEObject = false;
					it = new ENodeList().iterator();
				}
			} else {
				containsEObject = evalIterator.isEObject();
				final ENodeList list = new ENodeList();
				list.add(evalIterator);
				it = list.iterator();
			}
			res = iterate(node, it, containsEObject, evalContext);
		} else {
			res = new ENode(ENode.EMPTY, eObj, this, evalContext.getLaunchManager().isSynchronize());
		}
		return res;
	}

	/**
	 * Iterate on the given iterator to generate the code corresponding to the
	 * statements.
	 * 
	 * @param node
	 *            the node to return
	 * @param it
	 *            the iterator created from the iterator expression
	 * @param containsEObject
	 *            true if the list pointed by "it" contains EObject
	 * @param evalContext
	 *            Current evaluation context
	 * @return the ENode containing the resulting generation of the statement
	 *         over the iterator list
	 * @throws ENodeException
	 *             if the statements generation failed
	 * @throws FactoryException
	 *             if the statements generation failed
	 */
	private ENode iterate(ENode node, ENodeIterator it, boolean containsEObject, EvalContext evalContext) throws ENodeException, FactoryException {
		int i = 0;
		while (it.hasNext()) {
			ENode child = it.next();
			if (!containsEObject) {
				if (child.isString()) {
					try {
						final EEnumLiteral literal = EcoreFactory.eINSTANCE.createEEnumLiteral();
						final String value = child.getString();
						literal.setName(value);
						literal.setLiteral(value);
						child = new ENode(literal, child);
					} catch (final ENodeCastException e) {
						// Never catch
					}
				} else if (child.isInt()) {
					try {
						final EEnumLiteral literal = EcoreFactory.eINSTANCE.createEEnumLiteral();
						final String value = String.valueOf(child.getInt());
						literal.setName(value);
						literal.setLiteral(value);
						child = new ENode(literal, child);
					} catch (final ENodeCastException e) {
						// Never catch
					}
				} else if (child.isDouble()) {
					try {
						final EEnumLiteral literal = EcoreFactory.eINSTANCE.createEEnumLiteral();
						final String value = String.valueOf(child.getDouble());
						literal.setName(value);
						literal.setLiteral(value);
						child = new ENode(literal, child);
					} catch (final ENodeCastException e) {
						// Never catch
					}
				} else if (child.isBoolean()) {
					try {
						final EEnumLiteral literal = EcoreFactory.eINSTANCE.createEEnumLiteral();
						final String value = String.valueOf(child.getBoolean());
						literal.setName(value);
						literal.setLiteral(value);
						child = new ENode(literal, child);
					} catch (final ENodeCastException e) {
						// Never catch
					}
				}
			}
			evalContext.contextPush(TTemplate.WHILE_INDEX, i);
			evalContext.contextPush(TTemplate.CURRENT_NODE, child);
			try {
				if (child.isEObject()) {
					final ENode childRet = EvaluableTools.genStatements(this, child, getStatements(), evalContext);
					node.append(childRet);
					++i;
				} else if (containsEObject) {
					node.append(child);
				}
			} finally {
				evalContext.contextPop(TTemplate.CURRENT_NODE);
				evalContext.contextPop(TTemplate.WHILE_INDEX);
			}
		}
		return node;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return TemplateConstants.FOR_BEGIN + iterator.toString() + TemplateConstants.FOR_THEN + "\n" + EvaluableTools.toString(statements) + TemplateConstants.FOR_END; //$NON-NLS-1$
	}
}
