/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine.template.statements.impl.spec;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeException;

import java.util.Iterator;

import org.acceleo.engine.context.EvalContext;
import org.acceleo.engine.template.impl.spec.EvaluableTools;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.template.statements.If;
import org.acceleo.template.statements.impl.IfImpl;
import org.acceleo.tools.factory.FactoryException;
import org.acceleo.tools.template.TemplateConstants;
import org.eclipse.emf.ecore.EObject;

/**
 * A representation of the model object '<em><b>If</b></em>'.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 * 
 */
public class TIf extends IfImpl implements IEvaluable {
	/**
	 * {@inheritDoc}
	 */
	public ENode gen(ENode root, EvalContext evalContext) throws ENodeException, FactoryException {
		ENode ret = null;
		boolean conditionTest = adaptConditionResult(((IEvaluable) condition).gen(root, evalContext));
		if (conditionTest) {
			ret = EvaluableTools.genStatements(this, root, getThenStatements(), evalContext);
		} else {
			for (final Iterator<If> it = getElseIf().iterator(); it.hasNext() && !conditionTest;) {
				final If currentElsif = it.next();
				conditionTest = adaptConditionResult(((IEvaluable) currentElsif.getCondition()).gen(root, evalContext));
				if (conditionTest) {
					ret = EvaluableTools.genStatements(this, root, currentElsif.getThenStatements(), evalContext);
				}
			}
			if (!conditionTest) {
				if (getElseStatements() != null && getElseStatements().size() > 0) {
					ret = EvaluableTools.genStatements(this, root, getElseStatements(), evalContext);
				} else {
					EObject eObj;
					try {
						eObj = root.getEObject();
					} catch (final ENodeCastException e) {
						eObj = ENode.EMPTY;
					}
					ret = new ENode(ENode.EMPTY, eObj, this, evalContext.getLaunchManager().isSynchronize());
				}
			}
		}
		return ret;
	}

	/**
	 * This method adapts condition result to boolean value.
	 * 
	 * @param conditionValue
	 *            is the evaluated condition
	 * @return boolean value of condition result
	 */
	private boolean adaptConditionResult(ENode conditionValue) {
		boolean conditionTest = false;
		try {
			if (conditionValue != null) {
				if (!conditionValue.isBoolean()) {
					conditionTest = (Boolean) conditionValue.getAdapterValue(boolean.class);
				} else {
					conditionTest = conditionValue.getBoolean();
				}
			}
		} catch (final ENodeCastException e) {
			// can't happen
		}
		return conditionTest;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String optElse = EvaluableTools.toString(elseStatements);
		if (optElse.length() > 0) {
			optElse = TemplateConstants.IF_ELSE + optElse;
		}
		final StringBuffer optElseIf = new StringBuffer(""); //$NON-NLS-1$
		if (elseIf != null) {
			for (final If eIf : elseIf) {
				optElseIf.append(TemplateConstants.IF_ELSE_IF);
				optElseIf.append(eIf.getCondition().toString());
				optElseIf.append(TemplateConstants.IF_THEN);
				optElseIf.append(EvaluableTools.toString(eIf.getThenStatements()));
			}
		}
		return TemplateConstants.IF_BEGIN + condition.toString() + TemplateConstants.IF_THEN
				+ "\n" + EvaluableTools.toString(thenStatements) + optElseIf.toString() + optElse + TemplateConstants.IF_END; //$NON-NLS-1$
	}
}
