/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.gen;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.acceleo.engine.IModel;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

public class EcoreModel implements IModel {

	/**
	 * The metamodel nsURI for this model.
	 */
	private String nsURI;

	/**
	 * The resource representing this model.
	 */
	private Resource resource;

	/**
	 * Constructor.
	 * 
	 * @param resourceSet
	 *            the resource set to use for resources loading
	 * @param ecoreURI
	 *            the URI of the metamodel resource
	 * @param modelURI
	 *            the URI of the model resource
	 * @throws Exception
	 */
	public EcoreModel(ResourceSet resourceSet, URI ecoreURI, URI modelURI) {
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("ecore", new EcoreResourceFactoryImpl()); //$NON-NLS-1$
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl()); //$NON-NLS-1$
		Resource eCoreResource = resourceSet.getResource(ecoreURI, true);
		if (eCoreResource.getContents().size() > 0) {
			EObject result = eCoreResource.getContents().get(0);

			List<EObject> ePackageList = new ArrayList<EObject>();
			ePackageList.add(result);
			while (ePackageList.size() > 0) {
				Object current = ePackageList.remove(0);
				if (current instanceof EPackage) {
					EPackage ePackage = (EPackage) current;
					nsURI = ePackage.getNsURI();
					EPackage.Registry.INSTANCE.put(nsURI, ePackage);
					for (Iterator<EPackage> itSubPackage = ePackage.getESubpackages().iterator(); itSubPackage.hasNext();) {
						EPackage subPackage = itSubPackage.next();
						ePackageList.add(subPackage);
					}
				}
			}
		}
		resource = resourceSet.getResource(modelURI, true);
	}

	/**
	 * {@inheritDoc}
	 */
	public String getNsURI() {
		return nsURI;
	}

	/**
	 * {@inheritDoc}
	 */
	public List<EObject> getRoots() {
		List<EObject> ret = new ArrayList<EObject>();
		for (int i = 0; i < resource.getContents().size(); ++i) {
			ret.add(resource.getContents().get(i));
		}
		return ret;
	}

}
