/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.gen;

import java.util.ArrayList;
import java.util.List;

import org.acceleo.engine.IModel;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;

/**
 * Utility class to handle source model.
 * 
 * @author David Bonneau
 * 
 */
public class Model implements IModel {

	/**
	 * Model resource.
	 */
	private Resource resource;

	/**
	 * Model nsUri.
	 */
	private String nsURI;

	/**
	 * Constructor.
	 * 
	 * @param resourceSet
	 *            EMF resourceSet containing source model resource
	 * @param metaModelclassName
	 *            Metamodel class name
	 * @param modelURI
	 *            URI containing source model
	 * @throws Exception
	 */
	public Model(org.eclipse.emf.ecore.resource.ResourceSet resourceSet, String metaModelclassName, URI modelURI) throws Exception {
		// Register the metamodel
		Class<?> vClass = Class.forName(metaModelclassName);
		Object vObject = vClass.getField("eINSTANCE"); //$NON-NLS-1$

		nsURI = (String) vClass.getField("eNS_URI").get(vObject); //$NON-NLS-1$

		// Demand load the resource for this file.
		resource = resourceSet.getResource(modelURI, true);

	}

	/**
	 * {@inheritDoc}
	 */
	public String getNsURI() {
		return nsURI;
	}

	/**
	 * {@inheritDoc}
	 */
	public List<EObject> getRoots() {
		List<EObject> ret = new ArrayList<EObject>();
		for (int i = 0; i < resource.getContents().size(); ++i) {
			ret.add(resource.getContents().get(i));
		}
		return ret;
	}

}
