/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.parser.template;

import org.acceleo.template.core.Template;

import org.acceleo.tools.log.AcceleoException;
import org.acceleo.tools.strings.Int2;

/**
 * Template syntax exception.
 * 
 * @author www.obeo.fr
 * 
 */
public class TemplateSyntaxException extends AcceleoException {

	/** Serial used for unserialization checks. */
	private static final long serialVersionUID = 1;

	/**
	 * The script.
	 */
	protected final Template script;

	/**
	 * Position of the syntax error.
	 */
	protected final Int2 pos;

	/**
	 * Constructor.
	 * 
	 * @param message
	 *            is the message
	 * @param script
	 *            is the script
	 * @param pos
	 *            is the position of the syntax error
	 */
	public TemplateSyntaxException(String message, Template script, Int2 pos) {
		super(message);
		this.script = script;
		this.pos = pos;
	}

	/**
	 * Constructor.
	 * 
	 * @param message
	 *            is the message
	 * @param script
	 *            is the script
	 * @param pos
	 *            is the position of the syntax error
	 */
	public TemplateSyntaxException(String message, Template script, int pos) {
		super(message);
		this.script = script;
		this.pos = new Int2(pos, pos + 1);
	}

	/**
	 * Returns the position of the syntax error.
	 * 
	 * @return the position of the syntax error
	 */
	public Int2 getPos() {
		return pos;
	}

	/**
	 * Returns the script.
	 * 
	 * @return the script
	 */
	public Template getScript() {
		return script;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getMessage() {
		return super.getMessage();
	}

}
