/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.core;

import org.acceleo.template.core.CoreFactory;
import org.acceleo.template.core.FilePath;
import org.acceleo.template.core.ScriptDescriptor;
import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.Expression;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.AbstractParser;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.parser.template.expressions.ExpressionParser;
import org.acceleo.parser.template.statements.StatementParser;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo's ScriptDescriptor parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class ScriptDescriptorParser extends AbstractParser {
	/**
	 * {@inheritDoc}
	 */
	public static ScriptDescriptor fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		FilePath templateFile = CoreFactory.eINSTANCE.createFilePath();
		Expression templatePost = null;
		String templateType = null;
		String templateName = null;
		String templateDescription = null;
		Int2 rangeFileTemplate = null;
		Int2 rangePostExpression = null;
		Int2[] properties = TextSearch.getDefaultSearch().splitPositionsIn(buffer, range.b(), range.e(), TemplateConstants.SCRIPT_PROPERTIES_SEPARATORS, false, TemplateConstants.SPEC,
				TemplateConstants.INHIBS_SCRIPT_DECLA);
		for (int i = 0; i < properties.length; i += 2) {
			String key = buffer.substring(properties[i].b(), properties[i].e());
			if ((i + 1) < properties.length) {
				String value = buffer.substring(properties[i + 1].b(), properties[i + 1].e());
				if (value.length() >= 2 && value.startsWith(TemplateConstants.LITERAL[0]) && value.endsWith(TemplateConstants.LITERAL[1])) {
					value = value.substring(1, value.length() - 1);
					if (key.equals(TemplateConstants.SCRIPT_TYPE)) {
						templateType = value;
					} else if (key.equals(TemplateConstants.SCRIPT_NAME)) {
						templateName = value;
					} else if (key.equals(TemplateConstants.SCRIPT_DESC)) {
						templateDescription = value;
					} else if (key.equals(TemplateConstants.SCRIPT_FILE)) {
						rangeFileTemplate = new Int2(properties[i + 1].b() + 1, properties[i + 1].e() - 1);
						templateFile.getStatements().addAll(StatementParser.read(offset, buffer, rangeFileTemplate, template));
					} else if (key.equals(TemplateConstants.SCRIPT_POST)) {
						rangePostExpression = new Int2(properties[i + 1].b() + 1, properties[i + 1].e() - 1);
						templatePost = ExpressionParser.fromString(offset, buffer, rangePostExpression, template);
					} else {
						throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.InvalidKey", new Object[] { key, }), template, properties[i].b()); //$NON-NLS-1$
					}
				} else {
					throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.InvalidKeyValue", new Object[] { key, }), template, properties[i].e()); //$NON-NLS-1$
				}
			} else {
				throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingKeyValue", new Object[] { key, }), template, properties[i].e()); //$NON-NLS-1$
			}
		}
		if (templateType == null) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingType"), template, range.b()); //$NON-NLS-1$
		}
		if (templateName == null) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingName"), template, range.b()); //$NON-NLS-1$
		}
		ScriptDescriptor scriptDescriptor = CoreFactory.eINSTANCE.createScriptDescriptor();
		scriptDescriptor.setType(templateType);
		scriptDescriptor.setName(templateName);
		scriptDescriptor.setDescription(templateDescription);
		scriptDescriptor.setFile(templateFile);
		scriptDescriptor.setPost(templatePost);
		scriptDescriptor.setBegin(range.b());
		scriptDescriptor.setEnd(range.e());
		return scriptDescriptor;
	}
}
