/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.core;

import org.acceleo.template.core.CoreFactory;
import org.acceleo.template.core.Script;
import org.acceleo.template.core.ScriptDescriptor;
import org.acceleo.template.core.Template;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.AbstractParser;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.parser.template.statements.StatementParser;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo's Script parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class ScriptParser extends AbstractParser {
	/**
	 * {@inheritDoc}
	 */
	public static Script fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		TemplateConstants.initConstants(buffer);
		Script script = null;
		ScriptDescriptor descriptor = null;
		Int2 end = new Int2(0, 0);
		Int2 begin = TextSearch.getDefaultSearch().indexOf(buffer, TemplateConstants.SCRIPT_BEGIN, end.e(), null, TemplateConstants.INHIBS_SCRIPT_CONTENT);

		// Prepare new script
		end = TextSearch.getDefaultSearch().blockIndexEndOf(buffer, TemplateConstants.SCRIPT_BEGIN, TemplateConstants.SCRIPT_END, begin.b(), false, TemplateConstants.SPEC,
				TemplateConstants.INHIBS_SCRIPT_DECLA);
		if (end.e() == -1) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingScriptEndTag"), template, begin.b()); //$NON-NLS-1$
		} else {
			Int2 limitsTextTemplate = StatementParser.formatTemplate(buffer, new Int2(begin.b(), end.e()), 2);
			descriptor = ScriptDescriptorParser.fromString(offset, buffer, limitsTextTemplate, template);
			script = CoreFactory.eINSTANCE.createScript();
			script.setDescriptor(descriptor);
			script.getStatements().addAll(StatementParser.read(offset, buffer, limitsTextTemplate, template));
			script.setBegin(limitsTextTemplate.b());
			script.setEnd(limitsTextTemplate.e());
		}
		return script;
	}
}
