/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.expressions;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.Call;
import org.acceleo.template.expressions.Expression;
import org.acceleo.template.expressions.ExpressionsFactory;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.AbstractParser;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo Call parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class CallParser extends AbstractParser {

	/**
	 * {@inheritDoc}
	 */
	public static Call fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Int2 localRange = range;
		Int2 trim = TextSearch.getDefaultSearch().trim(buffer, localRange.b(), localRange.e());
		if (trim.b() == -1) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingElement"), template, localRange); //$NON-NLS-1$
		} else {
			localRange = trim;
		}
		Expression filter = null;
		Int2 begin = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.BRACKETS[0], localRange.b(), localRange.e(), TemplateConstants.SPEC,
				new String[][] { TemplateConstants.LITERAL, TemplateConstants.PARENTH });
		if (begin.b() > -1) {
			Int2 end = TextSearch.getDefaultSearch().blockIndexEndIn(buffer, TemplateConstants.BRACKETS[0], TemplateConstants.BRACKETS[1], begin.b(), localRange.e(), false, TemplateConstants.SPEC,
					TemplateConstants.INHIBS_EXPRESSION);
			if (end.b() == -1) {
				throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingCloseBracket"), template, begin.b()); //$NON-NLS-1$
			}
			if (buffer.substring(end.e(), localRange.e()).trim().length() > 0) {
				throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.InvalidSequence"), template, end.e()); //$NON-NLS-1$
			}
			Int2 pos = new Int2(begin.e(), end.b());
			filter = ExpressionParser.fromString(offset, buffer, pos, template);
			filter.setBegin(offset + pos.b());
			filter.setEnd(offset + pos.e());
			localRange = new Int2(localRange.b(), begin.b());
			trim = TextSearch.getDefaultSearch().trim(buffer, localRange.b(), localRange.e());
			if (trim.b() == -1) {
				throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingElement"), template, localRange); //$NON-NLS-1$
			} else {
				localRange = trim;
			}
		}
		Call call;
		begin = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.PARENTH[0], localRange.b(), localRange.e(), TemplateConstants.SPEC, new String[][] { TemplateConstants.LITERAL });
		if (begin.b() > -1) {
			Int2 end = TextSearch.getDefaultSearch().blockIndexEndIn(buffer, TemplateConstants.PARENTH[0], TemplateConstants.PARENTH[1], begin.b(), localRange.e(), false, TemplateConstants.SPEC,
					TemplateConstants.INHIBS_EXPRESSION);
			if (end.b() == -1) {
				throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingCloseParenthesis"), template, begin.b()); //$NON-NLS-1$
			}
			if (buffer.substring(end.e(), localRange.e()).trim().length() > 0) {
				throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.InvalidSequence"), template, end.e()); //$NON-NLS-1$
			}
			Int2[] positions = TextSearch.getDefaultSearch().splitPositionsIn(buffer, begin.e(), end.b(), new String[] { TemplateConstants.ARG_SEP }, false, TemplateConstants.SPEC,
					TemplateConstants.INHIBS_EXPRESSION);
			call = setCall(buffer.substring(localRange.b(), begin.b()));
			call.setBegin(offset + localRange.b());
			call.setEnd(offset + localRange.e());
			for (int i = 0; i < positions.length; i++) {
				Int2 pos = positions[i];
				call.getArguments().add(ExpressionParser.fromString(offset, buffer, pos, template));
			}
		} else {
			call = setCall(buffer.substring(localRange.b(), localRange.e()));
			call.setBegin(offset + localRange.b());
			call.setEnd(offset + localRange.e());
		}
		call.setBegin(offset + localRange.b());
		call.setEnd(offset + localRange.e());
		if (filter != null) {
			call.setFilter(filter);
		}
		return call;
	}

	/**
	 * Set the call for the given link string.
	 * 
	 * @param link
	 *            the link string representing the call
	 * @return the corresponding Call
	 */
	private static Call setCall(String link) {
		Call call = ExpressionsFactory.eINSTANCE.createCall();
		if (link != null) {
			int iDot = link.indexOf(TemplateConstants.LINK_PREFIX_SEPARATOR);
			if (iDot > -1) {
				call.setPrefix(link.substring(0, iDot).trim());
				call.setName(link.substring(iDot + TemplateConstants.LINK_PREFIX_SEPARATOR.length()).trim());
			} else {
				call.setPrefix(""); //$NON-NLS-1$
				call.setName(link.trim());
			}
		} else {
			call.setPrefix(""); //$NON-NLS-1$
			call.setName(""); //$NON-NLS-1$
		}
		return call;
	}
}
