/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.expressions;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.CallSet;
import org.acceleo.template.expressions.ExpressionsFactory;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo CallSet parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class CallSetParser extends ExpressionParser {

	/**
	 * {@inheritDoc}
	 */
	public static CallSet fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Int2 localRange = range;
		Int2 trim = TextSearch.getDefaultSearch().trim(buffer, localRange.b(), localRange.e());
		if (trim.b() == -1) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingElement"), template, localRange); //$NON-NLS-1$
		} else {
			localRange = trim;
		}
		Int2[] positions = TextSearch.getDefaultSearch().splitPositionsIn(buffer, localRange.b(), localRange.e(), new String[] { TemplateConstants.CALL_SEP }, false, TemplateConstants.SPEC,
				TemplateConstants.INHIBS_EXPRESSION);
		CallSet callSet = ExpressionsFactory.eINSTANCE.createCallSet();
		callSet.setBegin(offset + localRange.b());
		callSet.setEnd(offset + localRange.e());
		for (int i = 0; i < positions.length; i++) {
			Int2 pos = positions[i];
			callSet.getCalls().add(CallParser.fromString(offset, buffer, pos, template));
		}
		return callSet;
	}
}
