/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.expressions;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.Expression;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.AbstractParser;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;

/**
 * Acceleo Expression parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class ExpressionParser extends AbstractParser {

	/**
	 * {@inheritDoc}
	 */
	public static Expression fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Expression res = null;
		Int2 localRange = range;
		Int2 trim = TextSearch.getDefaultSearch().trim(buffer, localRange.b(), localRange.e());
		if (trim.b() == -1) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingElement"), template, localRange); //$NON-NLS-1$
		} else {
			localRange = trim;
		}
		Expression expression = OperatorParser.fromString(offset, buffer, localRange, template);
		if (expression != null) {
			res = expression;
		}
		if (expression == null) {
			expression = NotParser.fromString(offset, buffer, localRange, template);
			if (expression != null) {
				res = expression;
			}
		}
		if (expression == null) {
			expression = ParenthesisParser.fromString(offset, buffer, localRange, template);
			if (expression != null) {
				res = expression;
			}
		}
		if (expression == null) {
			expression = LiteralParser.fromString(offset, buffer, localRange, template);
			if (expression != null) {
				res = expression;
			}
		}
		if (expression == null) {
			expression = CallSetParser.fromString(offset, buffer, localRange, template);
			res = expression;
		}
		return res;
	}
}
