/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.expressions;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.BooleanLiteral;
import org.acceleo.template.expressions.DoubleLiteral;
import org.acceleo.template.expressions.ExpressionsFactory;
import org.acceleo.template.expressions.IntegerLiteral;
import org.acceleo.template.expressions.Literal;
import org.acceleo.template.expressions.StringLiteral;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.tools.format.Conventions;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo Literal parser.
 * 
 * @author Yvan LUSSAUD <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 * 
 */
public class LiteralParser extends ExpressionParser {
	/** Boolean literal : true. */
	protected static final String LITERAL_TRUE = TemplateConstants.LITERAL_TRUE;

	/** Boolean literal : false. */
	protected static final String LITERAL_FALSE = TemplateConstants.LITERAL_FALSE;

	/** Language literal : null. */
	protected static final String LITERAL_NULL = TemplateConstants.LITERAL_NULL;

	/**
	 * {@inheritDoc}
	 */
	public static Literal fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Int2 localRange = range;
		Int2 trim = TextSearch.getDefaultSearch().trim(buffer, localRange.b(), localRange.e());
		if (trim.b() == -1) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingElement"), template, localRange); //$NON-NLS-1$
		} else {
			localRange = trim;
		}
		Literal expression;
		String text = buffer.substring(localRange.b(), localRange.e());
		if ((text.length() >= (TemplateConstants.LITERAL[0].length() + TemplateConstants.LITERAL[1].length())) && (text.startsWith(TemplateConstants.LITERAL[0]))) {
			if (!text.endsWith(TemplateConstants.LITERAL[1])) {
				throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.InvalidLitteral"), template, localRange); //$NON-NLS-1$
			}
			String value = text.substring(TemplateConstants.LITERAL[0].length(), text.length() - TemplateConstants.LITERAL[1].length());
			expression = ExpressionsFactory.eINSTANCE.createStringLiteral();
			((StringLiteral) expression).setValue(Conventions.unformatString(value));
		} else {
			if (text.equals(LITERAL_TRUE)) {
				expression = ExpressionsFactory.eINSTANCE.createBooleanLiteral();
				((BooleanLiteral) expression).setValue(true);
			} else if (text.equals(LITERAL_FALSE)) {
				expression = ExpressionsFactory.eINSTANCE.createBooleanLiteral();
				((BooleanLiteral) expression).setValue(false);
			} else if (text.equals(LITERAL_NULL)) {
				expression = ExpressionsFactory.eINSTANCE.createNullLiteral();
			} else if (text.indexOf(".") > -1) { //$NON-NLS-1$
				try {
					expression = ExpressionsFactory.eINSTANCE.createDoubleLiteral();
					((DoubleLiteral) expression).setValue(Double.parseDouble(text));
				} catch (NumberFormatException e) {
					expression = null;
				}
			} else {
				try {
					expression = ExpressionsFactory.eINSTANCE.createIntegerLiteral();
					((IntegerLiteral) expression).setValue(Integer.parseInt(text));
				} catch (NumberFormatException e) {
					expression = null;
				}
			}
		}
		if (expression != null) {
			expression.setBegin(offset + localRange.b());
			expression.setEnd(offset + localRange.e());
		}
		return expression;
	}
}
