/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.expressions;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.Expression;
import org.acceleo.template.expressions.ExpressionsFactory;
import org.acceleo.template.expressions.Not;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo Not parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class NotParser extends ExpressionParser {

	/**
	 * {@inheritDoc}
	 */
	public static Not fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Int2 localRange = range;
		Int2 trim = TextSearch.getDefaultSearch().trim(buffer, localRange.b(), localRange.e());
		if (trim.b() == -1) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingElement"), template, localRange); //$NON-NLS-1$
		} else {
			localRange = trim;
		}
		String text = buffer.substring(localRange.b(), localRange.e());
		if (text.startsWith(TemplateConstants.NOT)) {
			Expression expression = ExpressionParser.fromString(offset, buffer, new Int2(localRange.b() + TemplateConstants.NOT.length(), localRange.e()), template);
			Not notExpression = ExpressionsFactory.eINSTANCE.createNot();
			notExpression.setExpression(expression);
			notExpression.setBegin(offset + localRange.b());
			notExpression.setEnd(offset + localRange.e());
			return notExpression;
		} else {
			return null;
		}
	}
}
