/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.expressions;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.ExpressionsFactory;
import org.acceleo.template.expressions.Operator;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo Operator parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class OperatorParser extends ExpressionParser {

	/**
	 * {@inheritDoc}
	 */
	public static Operator fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Int2 localRange = range;
		Int2 trim = TextSearch.getDefaultSearch().trim(buffer, localRange.b(), localRange.e());
		if (trim.b() == -1) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingElement"), template, localRange); //$NON-NLS-1$
		} else {
			localRange = trim;
		}

		for (int i = 0; i < TemplateConstants.OPERATORS.length; i++) {
			Int2[] positions = TextSearch.getDefaultSearch().splitPositionsIn(buffer, localRange.b(), localRange.e(), new String[] { TemplateConstants.OPERATORS[i] }, false, TemplateConstants.SPEC,
					TemplateConstants.INHIBS_EXPRESSION);

			if (validateOperands(buffer, TemplateConstants.OPERATORS[i], positions)) {
				Operator operator = ExpressionsFactory.eINSTANCE.createOperator();
				operator.setOperator(TemplateConstants.OPERATORS[i]);
				operator.setBegin(offset + localRange.b());
				operator.setEnd(offset + localRange.e());
				for (int j = 0; j < positions.length; j++) {
					Int2 pos = positions[j];
					operator.getOperands().add(ExpressionParser.fromString(offset, buffer, pos, template));
				}
				return operator;
			}
		}
		return null;
	}

	/**
	 * Validate the operands.
	 * 
	 * @param buffer
	 *            the current buffer
	 * @param operator
	 *            the current operator
	 * @param operandPos
	 *            positions of operands
	 * @return true if operands can be used false otherwise
	 */
	private static boolean validateOperands(String buffer, String operator, Int2[] operandPos) {
		boolean res = operandPos.length > 1;
		for (int i = 0; i < operandPos.length && res; ++i) {
			if (TemplateConstants.OPERATOR_INF.equals(operator) || TemplateConstants.OPERATOR_SUP.equals(operator)) {
				// Test if the first operand char is "=" when parsing "<" or ">"
				// operators to avoid "<=" and ">=" conflict
				res = res && !buffer.substring(operandPos[i].b(), operandPos[i].e()).startsWith("="); //$NON-NLS-1$
			}
		}
		return res;
	}
}
