/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.expressions;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.Expression;
import org.acceleo.template.expressions.ExpressionsFactory;
import org.acceleo.template.expressions.Parenthesis;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo Parenthesis parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class ParenthesisParser extends ExpressionParser {

	/**
	 * {@inheritDoc}
	 */
	public static Parenthesis fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Int2 localRange = range;
		Int2 trim = TextSearch.getDefaultSearch().trim(buffer, localRange.b(), localRange.e());
		if (trim.b() == -1) {
			throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingElement"), template, localRange); //$NON-NLS-1$
		} else {
			localRange = trim;
		}
		Int2 begin = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.PARENTH[0], localRange.b(), localRange.e());
		if (begin.b() == localRange.b()) {
			Int2 end = TextSearch.getDefaultSearch().blockIndexEndIn(buffer, TemplateConstants.PARENTH[0], TemplateConstants.PARENTH[1], begin.b(), localRange.e(), true, TemplateConstants.SPEC,
					TemplateConstants.INHIBS_EXPRESSION);
			if (end.e() == localRange.e()) {
				Expression expression = ExpressionParser.fromString(offset, buffer, new Int2(localRange.b() + TemplateConstants.PARENTH[0].length(), localRange.e()
						- TemplateConstants.PARENTH[1].length()), template);
				Parenthesis parenthesis = ExpressionsFactory.eINSTANCE.createParenthesis();
				parenthesis.setExpression(expression);
				parenthesis.setBegin(offset + localRange.b());
				parenthesis.setEnd(offset + localRange.e());
				return parenthesis;
			}
		}
		return null;
	}
}
