/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.statements;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.Expression;
import org.acceleo.template.statements.For;
import org.acceleo.template.statements.Statement;
import org.acceleo.template.statements.StatementsFactory;

import java.util.ArrayList;
import java.util.List;

import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.parser.template.expressions.ExpressionParser;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo Expression parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class ForParser extends StatementParser {

	/**
	 * {@inheritDoc}
	 */
	public static For fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Expression iteratorExpression;
		List<Statement> block;
		Int2 iThen = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.FOR_THEN, range.b(), range.e(), TemplateConstants.SPEC, TemplateConstants.INHIBS_EXPRESSION);
		if (iThen.b() > -1) {
			iteratorExpression = ExpressionParser.fromString(offset, buffer, new Int2(range.b(), iThen.b()), template);
			Int2 blockPos = new Int2(iThen.e(), range.e());
			blockPos = formatTemplate(buffer, blockPos, 0);
			block = read(offset, buffer, blockPos, template);
		} else {
			iteratorExpression = ExpressionParser.fromString(offset, buffer, new Int2(range.b(), range.e()), template);
			block = new ArrayList<Statement>();
		}
		For eFor = StatementsFactory.eINSTANCE.createFor();
		eFor.setIterator(iteratorExpression);
		eFor.getStatements().addAll(block);
		eFor.setBegin(offset + range.b());
		eFor.setEnd(offset + range.e());
		return eFor;
	}
}
