/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.parser.template.statements;

import org.acceleo.template.core.Template;
import org.acceleo.template.expressions.Expression;
import org.acceleo.template.statements.If;
import org.acceleo.template.statements.Statement;
import org.acceleo.template.statements.StatementsFactory;

import java.util.ArrayList;
import java.util.List;

import org.acceleo.parser.AcceleoParserMessages;
import org.acceleo.parser.template.TemplateSyntaxException;
import org.acceleo.parser.template.expressions.ExpressionParser;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.strings.TextSearch;
import org.acceleo.tools.template.TemplateConstants;

/**
 * Acceleo Text parser.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 */
public class IfParser extends StatementParser {
	/**
	 * No public access to the constructor.
	 */
	protected IfParser() {

	}

	/**
	 * {@inheritDoc}
	 */
	public static If fromString(int offset, String buffer, Int2 range, Template template) throws TemplateSyntaxException {
		Expression condition;
		List<Statement> ifBlock;
		List<Expression> elseIfConditions = new ArrayList<Expression>();
		List<List<Statement>> elseIfBlocks = new ArrayList<List<Statement>>();
		List<Statement> elseBlock;
		// If -> Then
		Int2 iThen = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.IF_THEN, range.b(), range.e(), TemplateConstants.SPEC, TemplateConstants.INHIBS_EXPRESSION);
		if (iThen.b() > -1) {
			condition = ExpressionParser.fromString(offset, buffer, new Int2(range.b(), iThen.b()), template);
			int iThenEnd;
			// Else -> End
			Int2 iElse = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.IF_ELSE, iThen.e(), range.e(), null, TemplateConstants.INHIBS_STATEMENT);
			if (iElse.b() > -1) {
				iThenEnd = iElse.b();
				Int2 elsePos = new Int2(iElse.e(), range.e());
				elsePos = formatTemplate(buffer, elsePos, 0);
				elseBlock = read(offset, buffer, elsePos, template);
			} else {
				iThenEnd = range.e();
				elseBlock = new ArrayList<Statement>();
			}
			// ElseIf -> ElseIf|Else|End
			int iElseIfEndMax = iThenEnd;
			Int2 iElseIf = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.IF_ELSE_IF, iThen.e(), iElseIfEndMax, null, TemplateConstants.INHIBS_STATEMENT);
			if (iElseIf.b() > -1) {
				iThenEnd = iElseIf.b();
				while (iElseIf.b() > -1) {
					// ElseIf test
					Expression elseIfTest;
					Int2 iTestEnd = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.IF_THEN, iElseIf.e(), iElseIfEndMax, TemplateConstants.SPEC, TemplateConstants.INHIBS_EXPRESSION);
					if (iTestEnd.b() > -1) {
						elseIfTest = ExpressionParser.fromString(offset, buffer, new Int2(iElseIf.e(), iTestEnd.b()), template);
					} else {
						throw new TemplateSyntaxException(AcceleoParserMessages.getString("TemplateSyntaxError.MissingCloseIf"), template, new Int2(iElseIf.e(), iElseIfEndMax)); //$NON-NLS-1$
					}
					iElseIf = TextSearch.getDefaultSearch().indexIn(buffer, TemplateConstants.IF_ELSE_IF, iTestEnd.e(), iElseIfEndMax, null, TemplateConstants.INHIBS_STATEMENT);
					Int2 elseIfPos;
					if (iElseIf.b() > -1) {
						elseIfPos = new Int2(iTestEnd.e(), iElseIf.b());
					} else {
						elseIfPos = new Int2(iTestEnd.e(), iElseIfEndMax);
					}
					elseIfPos = formatTemplate(buffer, elseIfPos, 0);
					List<Statement> elseIfBlock = read(offset, buffer, elseIfPos, template);
					elseIfConditions.add(elseIfTest);
					elseIfBlocks.add(elseIfBlock);
				}
			}
			// Then -> ElseIf|Else|End
			Int2 ifPos = new Int2(iThen.e(), iThenEnd);
			ifPos = formatTemplate(buffer, ifPos, 0);
			ifBlock = read(offset, buffer, ifPos, template);
		} else {
			condition = ExpressionParser.fromString(offset, buffer, new Int2(range.b(), range.e()), template);
			ifBlock = new ArrayList<Statement>();
			elseBlock = new ArrayList<Statement>();
		}
		If eIf = StatementsFactory.eINSTANCE.createIf();
		eIf.setCondition(condition);
		eIf.getThenStatements().addAll(ifBlock);
		eIf.getElseStatements().addAll(elseBlock);
		for (int i = 0; i < elseIfConditions.size(); ++i) {
			if (elseIfConditions.get(i) != null) {
				If eElseIf = StatementsFactory.eINSTANCE.createIf();
				eElseIf.setCondition(elseIfConditions.get(i));
				if (elseIfBlocks.get(i) != null) {
					eElseIf.getThenStatements().addAll(elseIfBlocks.get(i));
				}
				eIf.getElseIf().add(eElseIf);
			}
		}

		eIf.setBegin(offset + range.b());
		eIf.setEnd(offset + range.e());
		return eIf;
	}
}
