/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.tools.template;

import org.acceleo.tools.AcceleoToolsMessages;
import org.acceleo.tools.strings.TextSearch;

/**
 * Constants for the generation tool.
 * 
 * @author www.obeo.fr
 * 
 */
public final class TemplateConstants {

	// Template's global constants.
	/** String representing SPEC. */
	public static String SPEC = ""; //$NON-NLS-1$

	// Template's user code.
	/** Default tag used to start the user code. */
	public static final String DEFAULT_USER_BEGIN = AcceleoToolsMessages.getString("TemplateConstants.UserCodeStart"); //$NON-NLS-1$

	/** Default tag used to stop the user code. */
	public static final String DEFAULT_USER_END = AcceleoToolsMessages.getString("TemplateConstants.UserCodeEnd"); //$NON-NLS-1$

	// Template's literals.
	/** Boolean literal : true. */
	public static final String LITERAL_TRUE = "true"; //$NON-NLS-1$

	/** Boolean literal : false. */
	public static final String LITERAL_FALSE = "false"; //$NON-NLS-1$

	/** Language literal : null. */
	public static final String LITERAL_NULL = "null"; //$NON-NLS-1$

	// Template's comments.
	/** String representing comment begin. */
	public static String COMMENT_BEGIN;

	/** String representing comment end. */
	public static String COMMENT_END;

	/** Strings array containing comment begin and comment end. */
	public static String[] COMMENT;

	/** Strings matrix of comments constants. */
	public static String[][] INHIBS_COMMENT;

	// Template's imports.
	/** String representing import begin. */
	public static String IMPORT_BEGIN;

	/** String representing import end. */
	public static String IMPORT_END;

	/** String representing import keyword. */
	public static final String IMPORT_WORD = "import"; //$NON-NLS-1$

	/** String representing model type keyword. */
	public static final String MODELTYPE_WORD = "metamodel"; //$NON-NLS-1$

	// Template's user code.
	/** String representing start user section keyword. */
	public static final String USER_BEGIN_NAME = "startUserCode"; //$NON-NLS-1$

	/** String representing start user section tag. */
	public static String USER_BEGIN;

	/** String representing end user section keyword. */
	public static String USER_END_NAME = "endUserCode"; //$NON-NLS-1$

	/** String representing end user section tag. */
	public static String USER_END;

	// Template's expressions.
	/** String array representing parenthesis begin and end. */
	public static String[] PARENTH;

	/** Strings array representing brackets begin and end. */
	public static String[] BRACKETS;

	/** Strings array representing literals. */
	public static String[] LITERAL;

	/** Strings representing literal spec. */
	public static String LITERAL_SPEC;

	/** String matrix of expressions constants. */
	public static String[][] INHIBS_EXPRESSION;

	/** String representing not operator. */
	public static final String NOT = "!"; //$NON-NLS-1$

	/** String representing OR operator. */
	public static final String OPERATOR_OR = "||"; //$NON-NLS-1$

	/** String representing AND operator. */
	public static final String OPERATOR_AND = "&&"; //$NON-NLS-1$

	/** String representing EQUALS operator. */
	public static final String OPERATOR_EQUALS = "=="; //$NON-NLS-1$

	/** String representing NOT EQUALS operator. */
	public static final String OPERATOR_NOT_EQUALS = "!="; //$NON-NLS-1$

	/** String representing SUP EQUALS operator. */
	public static final String OPERATOR_SUP_EQUALS = ">="; //$NON-NLS-1$

	/** String representing INF EQUALS operator. */
	public static final String OPERATOR_INF_EQUALS = "<="; //$NON-NLS-1$

	/** String representing SUP operator. */
	public static final String OPERATOR_SUP = ">"; //$NON-NLS-1$

	/** String representing INF operator. */
	public static final String OPERATOR_INF = "<"; //$NON-NLS-1$

	/** String representing ADD operator. */
	public static final String OPERATOR_ADD = "+"; //$NON-NLS-1$

	/** String representing SUB operator. */
	public static final String OPERATOR_SUB = "-"; //$NON-NLS-1$

	/** String representing DIV operator. */
	public static final String OPERATOR_DIV = "/"; //$NON-NLS-1$

	/** String representing MUL operator. */
	public static final String OPERATOR_MUL = "*"; //$NON-NLS-1$

	/** Strings array containing operators representation. */
	public static String[] OPERATORS;

	/** String representing call separator. */
	public static String CALL_SEP;

	/** String representing argument separator. */
	public static String ARG_SEP;

	// Template's statements.
	/** String representing if beginning. */
	public static String IF_BEGIN;

	/** String representing then declaration. */
	public static String IF_THEN;

	/** String representing else declaration. */
	public static String IF_ELSE;

	/** String representing else if declaration. */
	public static String IF_ELSE_IF;

	/** String representing end declaration. */
	public static String IF_END;

	/** Strings array with if begin and if end. */
	public static String[] IF;

	/** String representing for begin. */
	public static String FOR_BEGIN;

	/** String representing for then. */
	public static String FOR_THEN;

	/** String representing for end. */
	public static String FOR_END;

	/** Strings array with for begin and for end. */
	public static String[] FOR;

	/** String representing feature begin. */
	public static String FEATURE_BEGIN;

	/** String representing feature end. */
	public static String FEATURE_END;

	/** Strings array with feature begin and feature end. */
	public static String[] FEATURE;

	/** Strings matrix of statements constants. */
	public static String[][] INHIBS_STATEMENT;

	// Template's declaration line.

	/** String representing script begin. */
	public static String SCRIPT_BEGIN;

	/** String representing script type keyword. */
	public static final String SCRIPT_TYPE = "type"; //$NON-NLS-1$

	/** String representing script name keyword. */
	public static final String SCRIPT_NAME = "name"; //$NON-NLS-1$

	/** String representing script desc keyword. */
	public static final String SCRIPT_DESC = "description"; //$NON-NLS-1$

	/** String representing script file keyword. */
	public static final String SCRIPT_FILE = "file"; //$NON-NLS-1$

	/** String representing script post keyword. */
	public static final String SCRIPT_POST = "post"; //$NON-NLS-1$

	/** String representing script property assignment operator. */
	public static final String SCRIPT_PROPERTY_ASSIGN = "="; //$NON-NLS-1$

	/** String representing script end. */
	public static String SCRIPT_END;

	/** Strings array containing properties separators. */
	public static String[] SCRIPT_PROPERTIES_SEPARATORS;

	/** Strings matrix of scripts declaration constants. */
	public static String[][] INHIBS_SCRIPT_DECLA;

	/** Strings matrix of script content constants. */
	public static String[][] INHIBS_SCRIPT_CONTENT;

	// Link prefix
	/** String representing script keyword. */
	public static final String LINK_PREFIX_SCRIPT = "script"; //$NON-NLS-1$

	/** String representing metamodel keyword. */
	public static final String LINK_PREFIX_METAMODEL = "metamodel"; //$NON-NLS-1$

	/** String representing short metamodel keyword. */
	public static final String LINK_PREFIX_METAMODEL_SHORT = "m"; //$NON-NLS-1$

	/** String representing service keyword. */
	public static final String LINK_PREFIX_JAVA = "service"; //$NON-NLS-1$

	/** String representing separator keyword. */
	public static final String LINK_PREFIX_SEPARATOR = "::"; //$NON-NLS-1$

	// Predefined links
	/** String representing args keyword. */
	public static final String LINK_NAME_ARGS = "args"; //$NON-NLS-1$

	/** String representing i keyword. */
	public static final String LINK_NAME_INDEX = "i"; //$NON-NLS-1$

	/** String representing sep keyword. */
	public static final String SERVICE_SEP = "sep"; //$NON-NLS-1$

	/** String representing sepStr keyword. */
	public static final String SERVICE_SEPSTR = "sepStr"; //$NON-NLS-1$

	/** String representing first char. */
	private static Character lastFirstChar;

	/**
	 * Private Constructor. Utility class should not have default or public
	 * constructor.
	 */
	private TemplateConstants() {
		// nothing to do here
	}

	/**
	 * Initializes all the constants for the generation tool.
	 */
	public static void initConstants() {
		initConstants(""); //$NON-NLS-1$
	}

	/**
	 * Initializes all the constants for the generation tool.
	 * 
	 * @param content
	 *            is the text that is used to determine the first and the last
	 *            character of the tags
	 */
	public static void initConstants(String content) {
		String localContent;
		// Choose tag
		if (content == null) {
			localContent = ""; //$NON-NLS-1$
		} else {
			localContent = content.trim();
		}
		boolean tab = localContent.length() > 0 && localContent.charAt(0) == '[';
		char first;
		char last;
		if (tab) {
			first = '[';
			last = ']';
		} else {
			first = '<';
			last = '>';
		}

		if (lastFirstChar != null && first == lastFirstChar.charValue()) {
			return;
		} else {
			lastFirstChar = new Character(first);
		}

		// contains String equal to first + "%" and "%" + last, avoid too much
		// concatenations later on
		final String tagStart = Character.toString(first) + '%';
		final String tagEnd = '%' + Character.toString(last);

		// Update constants
		SPEC = "\\\""; //$NON-NLS-1$
		COMMENT_BEGIN = tagStart + "--"; //$NON-NLS-1$
		COMMENT_END = "--" + tagEnd; //$NON-NLS-1$
		COMMENT = new String[] { COMMENT_BEGIN, COMMENT_END, TextSearch.FORCE_NOT_RECURSIVE };
		INHIBS_COMMENT = new String[][] { COMMENT };
		IMPORT_BEGIN = tagStart;
		IMPORT_END = tagEnd;
		USER_BEGIN = tagStart + USER_BEGIN_NAME + tagEnd;
		USER_END = tagStart + USER_END_NAME + tagEnd;
		PARENTH = new String[] { "(", ")" }; //$NON-NLS-1$ //$NON-NLS-2$
		BRACKETS = new String[] { "[", "]" }; //$NON-NLS-1$ //$NON-NLS-2$
		LITERAL = new String[] { "\"", "\"", TextSearch.FORCE_NOT_RECURSIVE }; //$NON-NLS-1$ //$NON-NLS-2$
		LITERAL_SPEC = "\\\""; //$NON-NLS-1$
		INHIBS_EXPRESSION = new String[][] { PARENTH, LITERAL, BRACKETS };
		OPERATORS = new String[] { OPERATOR_OR, OPERATOR_AND, OPERATOR_EQUALS, OPERATOR_NOT_EQUALS, OPERATOR_SUP_EQUALS, OPERATOR_INF_EQUALS, OPERATOR_SUP, OPERATOR_INF, OPERATOR_ADD, OPERATOR_SUB,
				OPERATOR_DIV, OPERATOR_MUL, };
		CALL_SEP = "."; //$NON-NLS-1$
		ARG_SEP = ","; //$NON-NLS-1$
		IF_BEGIN = tagStart + "if "; //$NON-NLS-1$
		IF_THEN = '{' + tagEnd;
		IF_ELSE = tagStart + "}else{" + tagEnd; //$NON-NLS-1$
		IF_ELSE_IF = tagStart + "}else if"; //$NON-NLS-1$
		IF_END = tagStart + '}' + tagEnd;
		IF = new String[] { IF_BEGIN, IF_END };
		FOR_BEGIN = tagStart + "for "; //$NON-NLS-1$
		FOR_THEN = '{' + tagEnd;
		FOR_END = tagStart + '}' + tagEnd;
		FOR = new String[] { FOR_BEGIN, FOR_END };
		FEATURE_BEGIN = tagStart;
		FEATURE_END = tagEnd;
		FEATURE = new String[] { FEATURE_BEGIN, FEATURE_END };
		INHIBS_STATEMENT = new String[][] { COMMENT, IF, FOR, FEATURE, };
		SCRIPT_BEGIN = tagStart + "script "; //$NON-NLS-1$
		SCRIPT_END = tagEnd;
		SCRIPT_PROPERTIES_SEPARATORS = new String[] { SCRIPT_PROPERTY_ASSIGN, " ", "\t" }; //$NON-NLS-1$ //$NON-NLS-2$
		INHIBS_SCRIPT_DECLA = new String[][] { LITERAL };
		INHIBS_SCRIPT_CONTENT = new String[][] { COMMENT, FEATURE };
	}

}
